@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -529,13 +529,13 @@ bool ExportHelper::XSLTConvertFile(QString inputfilename, QString outputfilename
   XSqlQuery q;
   q.prepare("SELECT fetchMetricText(:xsltdir) AS dir,"
             "       fetchMetricText(:xsltcmd) AS cmd;");
-#if defined Q_WS_MACX
+#if defined Q_OS_MAC
   q.bindValue(":xsltdir", "XSLTDefaultDirMac");
   q.bindValue(":xsltcmd", "XSLTProcessorMac");
-#elif defined Q_WS_WIN
+#elif defined Q_OS_WIN
   q.bindValue(":xsltdir", "XSLTDefaultDirWindows");
   q.bindValue(":xsltcmd", "XSLTProcessorWindows");
-#elif defined Q_WS_X11
+#elif defined Q_OS_LINUX
   q.bindValue(":xsltdir", "XSLTDefaultDirLinux");
   q.bindValue(":xsltcmd", "XSLTProcessorLinux");
 #endif
