@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -25,7 +25,7 @@
 #include <QVariant>
 
 #include "dbtools.h"
-#include "login2Options.h"
+#include "errorReporter.h"
 #include "qmd5.h"
 #include "storedProcErrorLookup.h"
 #include "xsqlquery.h"
@@ -36,7 +36,7 @@
 /* TODO: rename _nonxTupleDB to _isxTupleDB internally and
          set it based on db contents, not command line parameter input
  */
-login2::login2(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+login2::login2(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : QDialog(parent, modal ? (fl | Qt::Dialog) : fl)
 {
   setObjectName(name);
@@ -51,9 +51,8 @@ login2::login2(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
 
   connect(_buttonBox, SIGNAL(accepted()), this, SLOT(sLogin()));
   connect(_buttonBox, SIGNAL(helpRequested()), this, SLOT(sOpenHelp()));
-  //connect(_options, SIGNAL(clicked()), this, SLOT(sOptions()));
   connect(_server, SIGNAL(editingFinished()), this, SLOT(sChangeURL()));
-  connect(_database, SIGNAL(editTextChanged(QString)), this, SLOT(sChangeURL()));
+  connect(_database->lineEdit(), SIGNAL(editingFinished()), this, SLOT(sChangeURL()));
   connect(_port, SIGNAL(editingFinished()), this, SLOT(sChangeURL()));
   //connect(_otherOption, SIGNAL(toggled(bool)), _options, SLOT(setEnabled(bool)));
   //connect(_otherOption, SIGNAL(toggled(bool)), _recent, SLOT(setEnabled(bool)));
@@ -110,19 +109,19 @@ int login2::set(const ParameterList &pParams, QSplashScreen *pSplash)
   {
     _username->setText(param.toString());
     _password->setFocus();
-    _captive = TRUE;
+    _captive = true;
   }
   else
   {
     _username->setFocus();
-    _captive = FALSE;
+    _captive = false;
   }
 
   param = pParams.value("password", &valid);
   if (valid)
   {
     _password->setText(param.toString());
-    _captive = TRUE;
+    _captive = true;
   }
 
   param = pParams.value("copyright", &valid);
@@ -171,6 +170,13 @@ int login2::set(const ParameterList &pParams, QSplashScreen *pSplash)
   if (valid)
     _setSearchPath = true;
 
+  param = pParams.value("applicationName", &valid);
+  if (valid) {
+    _connAppName = param.toString().trimmed();
+  } else {
+    _connAppName = "xTuple ERP (unknown)";
+  }
+
   if(pParams.inList("login"))
     sLogin();
 
@@ -310,7 +316,7 @@ void login2::sLogin()
 
   if(isCloud || isXtuple)
   {
-    salt = "private";
+    salt = "j3H44uadEI#8#kSmkh#H%JSLAKDOHImklhdfsn3#432?%^kjasdjla3uy989apa3uipoweurw-03235##+=-lhkhdNOHA?%@mxncvbwoiwerNKLJHwe278NH28shNeGc";
   }
   else
   {
@@ -329,13 +335,64 @@ void login2::sLogin()
   int methodidx; // not declared in for () because we'll need it later
   for (methodidx = 0; methodidx < method.size(); methodidx++)
   {
-    db.setConnectOptions(method.at(methodidx).first);
+    db.setConnectOptions(QString("application_name='%1';%2").arg(_connAppName).arg(method.at(methodidx).first));
     db.setPassword(method.at(methodidx).second);
     if (db.open())
       break;  // break instead of for-loop condition to preserve methodidx
   }
 
-  // if connected using OpenMFG enhanced auth, remangle the password
+  if (db.isOpen())
+  {
+    QString earliest = "9.3.0",
+            latest   = "9.6.0";
+    XSqlQuery checkVersion;   // include earliest in the range but exclude latest
+    checkVersion.prepare("SELECT compareVersion(:earliest) <= 0"
+                         "   AND compareVersion(:latest)   > 0 AS ok,"
+                         "       version() AS version;");
+    checkVersion.bindValue(":earliest", earliest);
+    checkVersion.bindValue(":latest",   latest);
+    checkVersion.exec();
+    if (checkVersion.first() && ! checkVersion.value("ok").toBool() &&
+        QMessageBox::question(this, tr("Unsupported Database Server Version"),
+                              tr("<p>The database server is at version %1 but "
+                                 "xTuple ERP only supports from %2 up to but "
+                                 "not including %3.</p><p>Continue anyway?</p>")
+                                .arg(checkVersion.value("version").toString(),
+                                     earliest, latest),
+                              QMessageBox::Yes | QMessageBox::No,
+                              QMessageBox::No) == QMessageBox::No) {
+      if (_splash) {
+        _splash->hide();
+      }
+      setCursor(QCursor(Qt::ArrowCursor));
+      db.close();
+      if (! _captive) {
+        _username->setText("");
+        _username->setFocus();
+      } else
+        _password->setFocus();
+
+      _password->setText("");
+      return;
+    } else if (ErrorReporter::error(QtCriticalMsg, this,
+                                    tr("Cannot Connect to xTuple ERP Server"),
+                                    checkVersion, __FILE__, __LINE__)) {
+      if (_splash)
+        _splash->hide();
+      db.close();
+      if (! _captive) {
+        _username->setText("");
+        _username->setFocus();
+      } else
+        _password->setFocus();
+
+      _password->setText("");
+      return;
+    }
+  }
+
+   // if connected using OpenMFG enhanced auth, remangle the password
+
   if (db.isOpen() && (methodidx == 2 || methodidx == 5))
       XSqlQuery chgpass(QString("ALTER USER \"%1\" WITH PASSWORD '%2'")
                       .arg(_cUsername, QMd5(QString(_cPassword + salt + _cUsername))));
@@ -362,15 +419,7 @@ void login2::sLogin()
                              "<p>This may be due to a problem with your user name, password, or server connection information. "
                              "<p>Below is more detail on the connection problem: "
                              "<p>%1" ).arg(db.lastError().text()));
-
-    if (!_captive)
-    {
-      _username->setText("");
-      _username->setFocus();
-    }
-    else
-      _password->setFocus();
-
+    _password->setFocus();
     _password->setText("");
     return;
   }
@@ -389,6 +438,36 @@ void login2::sLogin()
   
   if(!_nonxTupleDB)
   {
+    // check that the connected database at least looks like a valid xTuple database.
+    XSqlQuery checkSchema(
+                "SELECT EXISTS ("
+                "   SELECT 1"
+                "   FROM pg_catalog.pg_class AS c"
+                "   JOIN  pg_catalog.pg_namespace AS n ON n.oid = c.relnamespace"
+                "   WHERE n.nspname = 'public'"
+                "   AND c.relname = 'metric'"
+                ") AS has_metric"
+              );
+
+    if (!(checkSchema.first() ? checkSchema.value("has_metric").toBool() : false))
+    {
+        setCursor(QCursor(Qt::ArrowCursor));
+
+        if (_splash) {
+          _splash->hide();
+        }
+
+        QString msg = QObject::tr(
+           "<p>The database \"%1\" does not appear to be a valid xTuple database.</p>"
+           "<p>Please check the database name and try again.</p>"
+        ).arg(dbName);
+
+        QMessageBox::critical(this, tr("Invalid Database"), msg, QMessageBox::Ok | QMessageBox::Escape | QMessageBox::Default);
+
+        return;
+    }
+
+
     QString loginqry = "";
     if (_setSearchPath)
       loginqry="SELECT login(true) AS result, CURRENT_USER AS user;";
@@ -449,27 +528,6 @@ void login2::sLogin()
   }
 }
 
-void login2::sOptions()
-{
-  ParameterList params;
-  params.append("databaseURL", _databaseURL);
-
-  if (_multipleConnections)
-    params.append("dontSaveSettings");
-
-  login2Options newdlg(this, "", TRUE);
-  newdlg.set(params);
-  if (newdlg.exec() != QDialog::Rejected)
-  {
-    updateRecentOptions();
-    _databaseURL = newdlg._databaseURL;
-    populateDatabaseInfo();
-    updateRecentOptions();
-    updateRecentOptionsActions();
-    _username->setFocus();
-  }
-}
-
 void login2::populateDatabaseInfo()
 {
   QString protocol;
@@ -533,8 +591,8 @@ void login2::updateRecentOptionsActions()
   {
     list.takeFirst();
     int size = list.size();
-    if (size > 5)
-      size = 5;
+    if (size > 10)
+      size = 10;
 
     QString protocol;
     QString hostName;
