@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -14,6 +14,7 @@
 #include "guiclient.h"
 #include "xdialog.h"
 #include <parameter.h>
+#include "lotSerialUtils.h"
 
 #include "ui_assignLotSerial.h"
 
@@ -22,7 +23,7 @@ class assignLotSerial : public XDialog, public Ui::assignLotSerial
     Q_OBJECT
 
 public:
-    assignLotSerial(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    assignLotSerial(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~assignLotSerial();
 
     QPushButton* _print;
@@ -45,6 +46,7 @@ private:
     int _itemlocSeries;
     int _itemlocdistid;
     bool _trapClose;
+    LotSerialUtils _lschars;
 };
 
 #endif // ASSIGNLOTSERIAL_H
