@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,12 +11,14 @@
 /* TODO:
    support Visa ECI, MasterCard UCAF for x_authentication_indicator?
    support Visa CAVV or MasterCard UCAF for x_cardholder_authentication_value?
+   switch completely to the API @ http://developer.authorize.net/api/reference/
 */
 /** @ingroup creditcards
     @class   AuthorizeDotNetProcessor
     @brief   The implementation of Authorize.Net-specific credit card handling
  */
 
+#include <QDebug>
 #include <QSqlError>
 
 #include <currcluster.h>
@@ -39,9 +41,9 @@ AuthorizeDotNetProcessor::AuthorizeDotNetProcessor() : CreditCardProcessor()
 {
   _company = "Authorize.Net";
   _defaultLivePort   = 443;
-  _defaultLiveServer = "https://secure.authorize.net/gateway/transact.dll";
+  _defaultLiveServer = "https://secure2.authorize.net/gateway/transact.dll";
   _defaultTestPort   = 443;
-  _defaultTestServer = "https://certification.authorize.net/gateway/transact.dll";
+  _defaultTestServer = "https://test.authorize.net/gateway/transact.dll";
 
   _msgHash.insert(-200, tr("A Login is required."));
   _msgHash.insert(-201, tr("The Login must be 20 characters or less."));
@@ -66,6 +68,12 @@ AuthorizeDotNetProcessor::AuthorizeDotNetProcessor() : CreditCardProcessor()
 
 }
 
+/** Build a basic outgoing request for an initial transaction - that is, one
+    that can stand alone and does not depend on the success of a prior
+    transaction.
+
+    @see buildFollowup
+ */
 int AuthorizeDotNetProcessor::buildCommon(const int pccardid, const QString &pcvv, const double pamount, const int pcurrid, QString &prequest, QString pordertype)
 {
   // TODO: if check and not credit card transaction do something else
@@ -115,7 +123,7 @@ int AuthorizeDotNetProcessor::buildCommon(const int pccardid, const QString &pcv
     APPENDFIELD(prequest, "x_delim_char", _metrics->value("CCANDelim"));
 
   APPENDFIELD(prequest, "x_version",      _metrics->value("CCANVer"));
-  APPENDFIELD(prequest, "x_delim_data", "TRUE");
+  APPENDFIELD(prequest, "x_delim_data", "true");
 
   if (! _metrics->value("CCANEncap").isEmpty())
     APPENDFIELD(prequest, "x_encap_char", _metrics->value("CCANEncap"));
@@ -125,7 +133,12 @@ int AuthorizeDotNetProcessor::buildCommon(const int pccardid, const QString &pcv
   APPENDFIELD(prequest, "x_amount",   QString::number(pamount, 'f', 2));
   // TODO: if check and not credit card transaction do something else
   APPENDFIELD(prequest, "x_card_num", anq.value("ccard_number").toString());
-  APPENDFIELD(prequest, "x_test_request", isLive() ? "FALSE" : "TRUE");
+
+  if (_metrics->value("CCServer").contains("test.authorize.net")) {
+    APPENDFIELD(prequest, "x_test_request", "false");
+  } else {
+    APPENDFIELD(prequest, "x_test_request", isLive() ? "false" : "true");
+  }
 
   // TODO: if check and not credit card transaction do something else
   QString work_month;
@@ -135,7 +148,7 @@ int AuthorizeDotNetProcessor::buildCommon(const int pccardid, const QString &pcv
   APPENDFIELD(prequest, "x_exp_date",
 	      work_month + anq.value("ccard_year_expired").toString().right(2));
 
-  APPENDFIELD(prequest, "x_relay_response", "FALSE");
+  APPENDFIELD(prequest, "x_relay_response", "false");
   APPENDFIELD(prequest, "x_duplicate_window",
 			_metrics->value("CCANDuplicateWindow"));
 
@@ -167,11 +180,6 @@ int AuthorizeDotNetProcessor::buildCommon(const int pccardid, const QString &pcv
     _errorMsg = anq.lastError().databaseText();
     return -1;
   }
-  else
-  {
-    _errorMsg = errorMsg(-17).arg(pccardid);
-    return -17;
-  }
 
   // TODO: if check and not credit card transaction do something else
   APPENDFIELD(prequest, "x_method",     "CC");
@@ -181,7 +189,89 @@ int AuthorizeDotNetProcessor::buildCommon(const int pccardid, const QString &pcv
     APPENDFIELD(prequest, "x_card_code", pcvv);
 
   if (DEBUG)
-    qDebug("AN:buildCommon built %s\n", prequest.toAscii().data());
+    qDebug("AN:buildCommon built %s\n", prequest.toLatin1().data());
+  return 0;
+}
+
+/** @brief Build an outgoing request for a follow-on transaction -
+           that is, one that requires a previous transaction to be meaningful.
+
+    Some Authorize.Net transactions can only occur after being set
+    up by a prior transaction. For example, @em capturing a
+    pre-authorization requires an existing valid pre-authorization.
+    These follow-on transactions pass less data and are somewhat
+    more secure than full messages.
+
+    @param         pccpayid the internal id of the @c ccpay record
+    @param         ptransid the transaction id, typically the order number
+    @param         pamount  in: the amount in the ERP transaction currency,
+                            out: the amount in the Authorize.Net transaction
+    @param[in,out] pcurrid  in: the currency of the ERP transaction,
+                            out: the currency of the Authorize.Net transaction
+    @param[in,out] prequest   The message to send to Authorize.Net
+    @param         pordertype The type of Authorize.Net transaction
+
+    @see buildCommon
+ */
+int AuthorizeDotNetProcessor::buildFollowup(const int pccpayid, const QString &ptransid, double &pamount, int &pcurrid, QString &prequest, QString pordertype)
+{
+  Q_UNUSED(pccpayid);
+  int returnValue = 0;
+  int currid      = pcurrid;
+
+  if (_metrics->value("CCANCurrency") != "TRANS")
+  {
+    currid = _metrics->value("CCANCurrency").toInt();
+    pamount = currToCurr(pcurrid, currid, pamount, &returnValue);
+    pcurrid = currid;
+    if (returnValue < 0)
+      return returnValue;
+  }
+
+  if (! _metrics->value("CCANDelim").isEmpty())
+    APPENDFIELD(prequest, "x_delim_char", _metrics->value("CCANDelim"));
+
+  if (! _metrics->value("CCANEncap").isEmpty())
+    APPENDFIELD(prequest, "x_encap_char", _metrics->value("CCANEncap"));
+
+  APPENDFIELD(prequest, "x_version",      _metrics->value("CCANVer"));
+  APPENDFIELD(prequest, "x_delim_data", "true");
+  APPENDFIELD(prequest, "x_login",    _metricsenc->value("CCLogin"));
+  APPENDFIELD(prequest, "x_tran_key", _metricsenc->value("CCPassword"));
+  APPENDFIELD(prequest, "x_trans_id", ptransid);
+  APPENDFIELD(prequest, "x_amount",   QString::number(pamount, 'f', 2));
+  APPENDFIELD(prequest, "x_method",   "CC");
+
+  if (_metrics->value("CCServer").contains("test.authorize.net")) {
+    APPENDFIELD(prequest, "x_test_request", "false");
+  } else {
+    APPENDFIELD(prequest, "x_test_request", isLive() ? "false" : "true");
+  }
+
+  APPENDFIELD(prequest, "x_relay_response", "false");
+  APPENDFIELD(prequest, "x_duplicate_window",
+			_metrics->value("CCANDuplicateWindow"));
+
+  XSqlQuery anq;
+  anq.prepare("SELECT curr_abbr FROM curr_symbol WHERE (curr_id=:currid);");
+  anq.bindValue(":currid", pcurrid);
+  anq.exec();
+  if (anq.first())
+  {
+    APPENDFIELD(prequest, "x_currency_code", anq.value("curr_abbr").toString());
+  }
+  else if (anq.lastError().type() != QSqlError::NoError)
+  {
+    _errorMsg = anq.lastError().databaseText();
+    return -1;
+  }
+
+  // TODO: if check and not credit card transaction do something else
+  APPENDFIELD(prequest, "x_method",     "CC");
+  APPENDFIELD(prequest, "x_type",       pordertype);
+
+  if (DEBUG)
+    qDebug("AN:buildFollowup built %s\n", prequest.toLatin1().data());
   return 0;
 }
 
@@ -190,7 +280,7 @@ int  AuthorizeDotNetProcessor::doAuthorize(const int pccardid, const QString &pc
   if (DEBUG)
     qDebug("AN:doAuthorize(%d, pcvv, %f, %f, %d, %f, %f, %d, %s, %s, %d)",
 	   pccardid, pamount, ptax, ptaxexempt,  pfreight,  pduty, pcurrid,
-	   pneworder.toAscii().data(), preforder.toAscii().data(), pccpayid);
+	   pneworder.toLatin1().data(), preforder.toLatin1().data(), pccpayid);
 
   int    returnValue = 0;
   double amount  = pamount;
@@ -222,7 +312,7 @@ int  AuthorizeDotNetProcessor::doAuthorize(const int pccardid, const QString &pc
     return returnValue;
 
   APPENDFIELD(request, "x_tax",        QString::number(tax, 'f', 2));
-  APPENDFIELD(request, "x_tax_exempt", ptaxexempt ? "TRUE" : "FALSE");
+  APPENDFIELD(request, "x_tax_exempt", ptaxexempt ? "true" : "false");
   APPENDFIELD(request, "x_freight",    QString::number(freight, 'f', 2));
   APPENDFIELD(request, "x_duty",       QString::number(duty,    'f', 2));
 
@@ -246,7 +336,7 @@ int  AuthorizeDotNetProcessor::doCharge(const int pccardid, const QString &pcvv,
   if (DEBUG)
     qDebug("AN:doCharge(%d, pcvv, %f, %f, %d, %f, %f, %d, %s, %s, %d)",
 	   pccardid, pamount,  ptax, ptaxexempt,  pfreight,  pduty, pcurrid,
-	   pneworder.toAscii().data(), preforder.toAscii().data(), pccpayid);
+	   pneworder.toLatin1().data(), preforder.toLatin1().data(), pccpayid);
 
   int    returnValue = 0;
   double amount  = pamount;
@@ -279,13 +369,15 @@ int  AuthorizeDotNetProcessor::doCharge(const int pccardid, const QString &pcvv,
     return returnValue;
 
   APPENDFIELD(request, "x_tax",        QString::number(tax, 'f', 2));
-  APPENDFIELD(request, "x_tax_exempt", ptaxexempt ? "TRUE" : "FALSE");
+  APPENDFIELD(request, "x_tax_exempt", ptaxexempt ? "true" : "false");
   APPENDFIELD(request, "x_freight",    QString::number(freight, 'f', 2));
   APPENDFIELD(request, "x_duty",       QString::number(duty,    'f', 2));
 
   if (! preforder.isEmpty())
     APPENDFIELD(request, "x_po_num",   preforder);
 
+  APPENDFIELD(request, "x_invoice_num",   pneworder);
+
   QString response;
 
   returnValue = sendViaHTTP(request, response);
@@ -300,33 +392,22 @@ int  AuthorizeDotNetProcessor::doCharge(const int pccardid, const QString &pcvv,
 
 int AuthorizeDotNetProcessor::doChargePreauthorized(const int pccardid, const QString &pcvv, const double pamount, const int pcurrid, QString &pneworder, QString &preforder, int &pccpayid, ParameterList &pparams)
 {
+  Q_UNUSED(pcvv);
   if (DEBUG)
     qDebug("AN:doChargePreauthorized(%d, pcvv, %f, %d, %s, %s, %d)",
 	   pccardid, pamount,  pcurrid,
-	   pneworder.toAscii().data(), preforder.toAscii().data(), pccpayid);
+	   pneworder.toLatin1().data(), preforder.toLatin1().data(), pccpayid);
 
   int    returnValue = 0;
   double amount  = pamount;
   int    currid  = pcurrid;
 
-  if (_metrics->value("CCANCurrency") != "TRANS")
-  {
-    currid = _metrics->value("CCANCurrency").toInt();
-    amount = currToCurr(pcurrid, currid, pamount, &returnValue);
-    if (returnValue < 0)
-      return returnValue;
-  }
-
   QString request;
-
-  returnValue = buildCommon(pccardid, pcvv, amount, currid, request, "PRIOR_AUTH_CAPTURE");
-  if (returnValue !=  0)
+  returnValue = buildFollowup(pccpayid, preforder, amount, currid, request, "PRIOR_AUTH_CAPTURE");
+  if (returnValue != 0)
     return returnValue;
 
-  APPENDFIELD(request, "x_trans_id", preforder);
-
   QString response;
-
   returnValue = sendViaHTTP(request, response);
   if (returnValue < 0)
     return returnValue;
@@ -339,46 +420,61 @@ int AuthorizeDotNetProcessor::doChargePreauthorized(const int pccardid, const QS
 
 int AuthorizeDotNetProcessor::doCredit(const int pccardid, const QString &pcvv, const double pamount, const double ptax, const bool ptaxexempt, const double pfreight, const double pduty, const int pcurrid, QString &pneworder, QString &preforder, int &pccpayid, ParameterList &pparams)
 {
+  Q_UNUSED(pcvv);
   if (DEBUG)
     qDebug("AN:doCredit(%d, pcvv, %f, %f, %d, %f, %f, %d, %s, %s, %d)",
 	   pccardid, pamount, ptax, ptaxexempt,  pfreight,  pduty, pcurrid,
-	   pneworder.toAscii().data(), preforder.toAscii().data(), pccpayid);
-
-  int    returnValue = 0;
-  double amount  = pamount;
-  double tax     = ptax;
-  double freight = pfreight;
-  double duty    = pduty;
-  int    currid  = pcurrid;
-
-  if (_metrics->value("CCANCurrency") != "TRANS")
-  {
-    currid = _metrics->value("CCANCurrency").toInt();
-    amount = currToCurr(pcurrid, currid, pamount, &returnValue);
-    if (returnValue < 0)
-      return returnValue;
-    tax = currToCurr(pcurrid, currid, ptax, &returnValue);
-    if (returnValue < 0)
-      return returnValue;
-    freight = currToCurr(pcurrid, currid, pfreight, &returnValue);
-    if (returnValue < 0)
-      return returnValue;
-    duty = currToCurr(pcurrid, currid, pduty, &returnValue);
-    if (returnValue < 0)
-      return returnValue;
-  }
+	   pneworder.toLatin1().data(), preforder.toLatin1().data(), pccpayid);
 
+  int     returnValue = 0;
+  double  amount  = pamount;
+  int     currid  = pcurrid;
+  bool    tryVoid = false;
+  QString approvalCode;
   QString request;
 
-  returnValue = buildCommon(pccardid, pcvv, amount, currid, request, "CREDIT");
-  if (returnValue !=  0)
+  returnValue = buildFollowup(pccpayid, preforder, amount, currid, request, "CREDIT");
+  if (returnValue != 0)
     return returnValue;
 
-  APPENDFIELD(request, "x_trans_id",   preforder);
-  APPENDFIELD(request, "x_tax",        QString::number(tax, 'f', 2));
-  APPENDFIELD(request, "x_tax_exempt", ptaxexempt ? "TRUE" : "FALSE");
-  APPENDFIELD(request, "x_freight",    QString::number(freight, 'f', 2));
-  APPENDFIELD(request, "x_duty",       QString::number(duty,    'f', 2));
+  XSqlQuery anq;
+  anq.prepare("SELECT ccpay_card_pan_trunc,"
+              "  (ccpay_transaction_datetime > CURRENT_DATE"
+              "  AND ccpay_amount = :amount) AS tryVoid,"
+              "  ccpay_r_code,"
+              "  formatbytea(decrypt(setbytea(ccard_number),"
+              "              setbytea(:key),'bf')) AS ccard_number,"
+              "  formatbytea(decrypt(setbytea(ccard_month_expired),"
+              "              setbytea(:key),'bf')) AS ccard_month_expired,"
+              "  formatbytea(decrypt(setbytea(ccard_year_expired),"
+              "              setbytea(:key),'bf')) AS ccard_year_expired"
+              "  FROM ccpay LEFT OUTER JOIN ccard ON (ccpay_ccard_id=ccard_id)"
+              " WHERE (ccpay_id=:ccpayid);");
+  anq.bindValue(":ccpayid", pccpayid);
+  anq.bindValue(":key",     omfgThis->_key);
+  anq.bindValue(":now",     QDateTime::currentDateTime());
+  anq.bindValue(":amount",  amount);
+  anq.exec();
+  if (anq.first())
+  {
+    QString cardnum = anq.value("ccpay_card_pan_trunc").toString();
+    if (cardnum.isEmpty())
+      cardnum = anq.value("ccard_number").toString();
+    APPENDFIELD(request, "x_card_num", cardnum);
+    QString work_month;
+    work_month.setNum(anq.value("ccard_month_expired").toDouble());
+    if (work_month.length() == 1)
+      work_month = "0" + work_month;
+    APPENDFIELD(request, "x_exp_date",
+                work_month + anq.value("ccard_year_expired").toString().right(2));
+    tryVoid = anq.value("tryVoid").toBool();
+    approvalCode = anq.value("ccpay_r_code").toString();
+  }
+  else if (anq.lastError().type() != QSqlError::NoError)
+  {
+    _errorMsg = anq.lastError().databaseText();
+    return -1;
+  }
 
   QString response;
   returnValue = sendViaHTTP(request, response);
@@ -388,44 +484,47 @@ int AuthorizeDotNetProcessor::doCredit(const int pccardid, const QString &pcvv,
   returnValue = handleResponse(response, pccardid, "R", amount, currid,
 			       pneworder, preforder, pccpayid, pparams);
 
+  // TODO: make more precise - look for return code 54
+  if (returnValue < 0 && tryVoid) {
+    int voidResult = 0;
+    QString tmpErrorMsg = _errorMsg;
+    ParameterList voidParams;
+    _errorMsg.clear();
+    voidResult = doVoidPrevious(pccardid,   pcvv,      amount,       currid,
+                                pneworder,  preforder, approvalCode, pccpayid,
+                                voidParams);
+    if (voidResult >= 0) {
+      returnValue = voidResult;
+      pparams.clear();
+      while (! voidParams.isEmpty())
+        pparams.append(voidParams.takeFirst());
+    }
+    else
+      _errorMsg = tmpErrorMsg;
+  }
+
   return returnValue;
 }
 
 int AuthorizeDotNetProcessor::doVoidPrevious(const int pccardid, const QString &pcvv, const double pamount, const int pcurrid, QString &pneworder, QString &preforder, QString &papproval, int &pccpayid, ParameterList &pparams)
 {
+  Q_UNUSED(pcvv);
   if (DEBUG)
     qDebug("AN:doVoidPrevious(%d, pcvv, %f, %d, %s, %s, %s, %d)",
 	   pccardid, pamount, pcurrid,
-	   pneworder.toAscii().data(), preforder.toAscii().data(),
-	   papproval.toAscii().data(), pccpayid);
+	   pneworder.toLatin1().data(), preforder.toLatin1().data(),
+	   papproval.toLatin1().data(), pccpayid);
 
   QString tmpErrorMsg = _errorMsg;
 
-  int    returnValue = 0;
-  double amount = pamount;
-  int    currid = pcurrid;
-
-  if (_metrics->value("CCANCurrency") != "TRANS")
-  {
-    currid = _metrics->value("CCANCurrency").toInt();
-    amount = currToCurr(pcurrid, currid, pamount, &returnValue);
-    if (returnValue < 0)
-    {
-      _errorMsg = tmpErrorMsg;
-      return returnValue;
-    }
-  }
-
+  int     returnValue = 0;
+  double  amount = pamount;
+  int     currid = pcurrid;
   QString request;
 
-  returnValue = buildCommon(pccardid, pcvv, amount, currid, request, "VOID");
-  if (returnValue != 0)
-    return returnValue;
-
-  APPENDFIELD(request, "x_trans_id", preforder);
+  returnValue = buildFollowup(pccpayid, preforder, amount, currid, request, "VOID");
 
   QString response;
-
   returnValue = sendViaHTTP(request, response);
   _errorMsg = tmpErrorMsg;
   if (returnValue < 0)
@@ -457,17 +556,15 @@ int AuthorizeDotNetProcessor::fieldValue(const QStringList plist, const int pind
     pvalue = pvalue.mid(firstPos + 1, lastPos - firstPos - 1);
   }
   if (DEBUG)
-    qDebug("AN:fieldValue of %d is %s", pindex, pvalue.toAscii().data());
+    qDebug("AN:fieldValue of %d is %s", pindex, pvalue.toLatin1().data());
   return 0;
 }
 
 int AuthorizeDotNetProcessor::handleResponse(const QString &presponse, const int pccardid, const QString &ptype, const double pamount, const int pcurrid, QString &pneworder, QString &preforder, int &pccpayid, ParameterList &pparams)
 {
   if (DEBUG)
-    qDebug("AN::handleResponse(%s, %d, %s, %f, %d, %s, %d, pparams)",
-	   presponse.toAscii().data(), pccardid,
-	   ptype.toAscii().data(), pamount, pcurrid,
-	   preforder.toAscii().data(), pccpayid);
+    qDebug() << "AN::handleResponse(" << presponse << pccardid << ptype << pamount
+             << pcurrid << preforder << pccpayid << "pparams)";
 
   // if we got an error msg very early on
   if (presponse.startsWith("<HTML>"))
@@ -491,8 +588,12 @@ int AuthorizeDotNetProcessor::handleResponse(const QString &presponse, const int
   QString r_ref;
   QString r_shipping;
   QString r_tax;
+  QString r_pantrunc;
+  QString r_cardtype;
+  QString r_hash;
 
   QString status;
+  bool    isCP=false; // looks like an (unsupported) card-present transaction
 
   // TODO: explore using encap here and code from CSV Import to properly split
 
@@ -502,12 +603,22 @@ int AuthorizeDotNetProcessor::handleResponse(const QString &presponse, const int
      starting at 0.
    */
   QStringList responseFields = presponse.split(delim);
-  
+
   QString r_response;
   int returnValue = fieldValue(responseFields, 1, r_response);
   if (returnValue < 0)
     return returnValue;
 
+  if (! QRegExp("[1-4]").exactMatch(r_response) &&
+      QRegExp("\\d+\\.\\d+").exactMatch(r_response))
+  {
+    qDebug() << "Looks like A.N returned a unsupported Card-Present response:" << r_response;
+    isCP = true;
+    returnValue = fieldValue(responseFields, 2, r_response);
+    if (returnValue < 0)
+      return returnValue;
+  }
+
   if (r_response.toInt() == 1)
     r_approved = "APPROVED";
   else if (r_response.toInt() == 2)
@@ -532,36 +643,60 @@ int AuthorizeDotNetProcessor::handleResponse(const QString &presponse, const int
   returnValue = fieldValue(responseFields, 6, r_avs);	 	// avs result
   if (returnValue < 0)
     return returnValue;
+
+  if (isCP)
+  {
+    returnValue = fieldValue(responseFields, 8, r_ordernum);	// transaction id
+    if (returnValue < 0)
+      return returnValue;
+
+    returnValue = fieldValue(responseFields, 7, r_cvv); // ccv response code
+    if (returnValue < 0 && ptype == "CP") // may not get cvv on preauth capture
+      returnValue = 0;
+    else if (returnValue < 0)
+      return returnValue;
+
+    returnValue = fieldValue(responseFields, 9, r_hash);	// md5 hash
+
+  } else {
   returnValue = fieldValue(responseFields, 7, r_ordernum);	// transaction id
 
   if (returnValue < 0)
     return returnValue;
 
-  // fieldValue(responseFields, 8-10);	// echo invoice_number description amount 
-  // fieldValue(responseFields, 11-13);	// echo method type cust_id
-  // fieldValue(responseFields, 14-24);	// echo name, company, and address info
-  // fieldValue(responseFields, 25-32);	// echo ship_to fields
-
   returnValue = fieldValue(responseFields, 33, r_tax);		// echo x_tax
   if (returnValue < 0)
     return returnValue;
 
-  // fieldValue(responseFields, 34);				// echo x_duty
-
   returnValue = fieldValue(responseFields, 35, r_shipping);	// echo x_freight
   if (returnValue < 0)
     return returnValue;
 
-  // fieldValue(responseFields, 36);		// echo x_tax_exempt
-  // fieldValue(responseFields, 37);		// echo x_po_num
-
-  returnValue = fieldValue(responseFields, 39, r_cvv); // ccv response code
+  returnValue = fieldValue(responseFields, 38, r_hash);	// md5 hash
   if (returnValue < 0)
     return returnValue;
 
-  // fieldValue(responseFields, 40);		// cavv response code
-  // fieldValue(responseFields, 41-68);		// reserved for future use
-  // fieldValue(responseFields, 69+);		// echo of merchant-defined fields
+  returnValue = fieldValue(responseFields, 39, r_cvv); // ccv response code
+  if (returnValue < 0 && ptype == "CP") // may not get cvv on preauth capture
+    returnValue = 0;
+  else if (returnValue < 0)
+    return returnValue;
+
+  returnValue = fieldValue(responseFields, 51, r_pantrunc);
+  if (returnValue < 0)
+    return returnValue;
+  r_pantrunc = r_pantrunc.right(4);
+
+  returnValue = fieldValue(responseFields, 52, r_cardtype);
+  if (returnValue < 0)
+    return returnValue;
+  }
+
+  if (r_cardtype == "Discover" || r_cardtype == "MasterCard"
+      || r_cardtype == "Visa"  || r_cardtype == "American Express")
+    r_cardtype.remove(1, r_cardtype.length());
+  else
+    r_cardtype = "O";
 
   /* treat heldforreview as approved because the AIM doc says response
      reason codes 252 and 253 are both approved but being reviewed.
@@ -632,6 +767,8 @@ int AuthorizeDotNetProcessor::handleResponse(const QString &presponse, const int
   pparams.append("tax",         r_tax);
   pparams.append("ref",         r_ref);
   pparams.append("message",     r_message);
+  pparams.append("pantrunc",    r_pantrunc);
+  pparams.append("cardtype",    r_cardtype);
 
   pparams.append("auth", QVariant(ptype == "A"));
 
@@ -647,9 +784,6 @@ int AuthorizeDotNetProcessor::handleResponse(const QString &presponse, const int
   if (returnValue == 0 && _metrics->boolean("CCANMD5HashSetOnGateway"))
   {
     QString expected_hash;
-    QString r_hash;
-
-    returnValue = fieldValue(responseFields, 38, r_hash);	// md5 hash
     XSqlQuery anq;
     anq.prepare("SELECT UPPER(MD5(:inputstr)) AS expected;");
     anq.bindValue(":inputstr", _metricsenc->value("CCANMD5Hash") +
@@ -666,7 +800,7 @@ int AuthorizeDotNetProcessor::handleResponse(const QString &presponse, const int
     }
     if (DEBUG)
       qDebug("AN:handleResponse expected md5 %s and got %s",
-	      expected_hash.toAscii().data(), r_hash.toAscii().data());
+	      expected_hash.toLatin1().data(), r_hash.toLatin1().data());
 
     if (_metrics->value("CCANMD5HashAction") == "F" && expected_hash != r_hash)
     {
@@ -682,7 +816,7 @@ int AuthorizeDotNetProcessor::handleResponse(const QString &presponse, const int
 
   if (DEBUG)
     qDebug("AN::handleResponse returning %d %s",
-           returnValue, errorMsg().toAscii().data());
+           returnValue, errorMsg().toLatin1().data());
   return returnValue;
 }
 
