@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -23,7 +23,7 @@ class bankAccount : public XDialog, public Ui::bankAccount
     Q_OBJECT
 
 public:
-    bankAccount(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    bankAccount(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~bankAccount();
 
 public slots:
@@ -40,6 +40,7 @@ protected:
 protected slots:
     virtual void languageChange();
     virtual void sHandleTransmitGroup();
+    virtual void sHandleType();
 
 private:
     int _mode;
