@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -24,8 +24,9 @@
 #include <openreports.h>
 
 #include "bomItem.h"
+#include "errorReporter.h"
 
-BOM::BOM(QWidget* parent, const char* name, Qt::WFlags fl)
+BOM::BOM(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
@@ -40,7 +41,7 @@ BOM::BOM(QWidget* parent, const char* name, Qt::WFlags fl)
   connect(_showExpired, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
   connect(_showFuture, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
   connect(_bomitem, SIGNAL(populateMenu(QMenu*,QTreeWidgetItem*,int)), this, SLOT(sPopulateMenu(QMenu*)));
-  connect(_close, SIGNAL(clicked()), this, SLOT(sClose()));
+  connect(_close, SIGNAL(clicked()), this, SLOT(close()));
   connect(_view, SIGNAL(clicked()), this, SLOT(sView()));
   connect(_save, SIGNAL(clicked()), this, SLOT(sSave()));
   connect(_expire, SIGNAL(clicked()), this, SLOT(sExpire()));
@@ -92,6 +93,7 @@ BOM::BOM(QWidget* parent, const char* name, Qt::WFlags fl)
   connect(omfgThis, SIGNAL(bomsUpdated(int, bool)), SLOT(sFillList(int, bool)));
   _revision->setMode(RevisionLineEdit::Maintain);
   _revision->setType("BOM");
+  _bomheadid = -1;
 }
 
 BOM::~BOM()
@@ -107,6 +109,7 @@ void BOM::languageChange()
 enum SetResponse BOM::set(const ParameterList &pParams)
 {
   XWidget::set(pParams);
+  XSqlQuery bomet;
   QVariant param;
   bool     valid;
   
@@ -116,28 +119,49 @@ enum SetResponse BOM::set(const ParameterList &pParams)
     if (param.toString() == "new")
     {
       _mode = cNew;
-      _new->setEnabled(FALSE);
+      _new->setEnabled(false);
       _revision->setId(-1);
+
+      bomet.exec("SELECT NEXTVAL('bomhead_bomhead_id_seq') AS bomhead_id");
+      if (bomet.first())
+      {
+        _bomheadid = bomet.value("bomhead_id").toInt();
+        _documents->setId(_bomheadid);
+        _comments->setId(_bomheadid);
+      }
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                    bomet, __FILE__, __LINE__))
+      {
+        return UndefinedError;
+      }
+      
+      emit newMode(_mode);
+      emit newId(_bomheadid);
     }
     else if (param.toString() == "edit")
     {
       _mode = cEdit;
-      _item->setReadOnly(TRUE);
+      _item->setReadOnly(true);
+      
+      emit newMode(_mode);
     }
     else if (param.toString() == "view")
     {
       _mode = cView;
-      _new->setEnabled(FALSE);
-      _item->setReadOnly(TRUE);
-      _documentNum->setEnabled(FALSE);
-      _revision->setEnabled(FALSE);
-      _revisionDate->setEnabled(FALSE);
-      _batchSize->setEnabled(FALSE);
-      _doRequireQtyPer->setEnabled(FALSE);
-      _requiredQtyPer->setEnabled(FALSE);
-      _save->setEnabled(FALSE);
+      _new->setEnabled(false);
+      _item->setReadOnly(true);
+      _documentNum->setEnabled(false);
+//      _revision->setEnabled(false);
+      _revisionDate->setEnabled(false);
+      _batchSize->setEnabled(false);
+      _doRequireQtyPer->setEnabled(false);
+      _requiredQtyPer->setEnabled(false);
+      _documents->setReadOnly(true);
+      _save->setEnabled(false);
       
       connect(_bomitem, SIGNAL(itemSelected(int)), _view, SLOT(animateClick()));
+      
+      emit newMode(_mode);
     }
 
     param = pParams.value("item_id", &valid);
@@ -153,7 +177,29 @@ enum SetResponse BOM::set(const ParameterList &pParams)
   return NoError;
 }
 
+int BOM::id() const
+{
+  return _bomheadid;
+}
+
+int BOM::mode() const
+{
+  return _mode;
+}
+
 bool BOM::sSave()
+{
+  if (save(false))
+  {
+    emit saved(_bomheadid);
+    close();
+    return true;
+  }
+  else
+    return false;
+}
+
+bool BOM::save(bool partial)
 {
   XSqlQuery BSave;
   if(_item->id() == -1)
@@ -168,45 +214,41 @@ bool BOM::sSave()
   else if(_batchSize->toDouble() == 0.0)
   {
     QMessageBox::warning( this, tr("Batch Size Error"),
-      tr("<p>The Batch Size quantity must be greater than zero.") );
+                         tr("<p>The Batch Size quantity must be greater than zero.") );
     return false;
   }
-
-  if(!sCheckRequiredQtyPer())
-    return false;
   
-  BSave.prepare( "SELECT bomhead_id "
-             "FROM bomhead "
-             "WHERE ((bomhead_item_id=:item_id) "
-			 "AND (bomhead_rev_id=:bomhead_rev_id));" );
-  BSave.bindValue(":item_id", _item->id());
-  BSave.bindValue(":bomhead_rev_id", _revision->id());
-  BSave.exec();
-  if (BSave.first())
+  if (!partial)
+  {
+    if(!sCheckRequiredQtyPer())
+      return false;
+  }
+  
+  if (_mode == cEdit)
   {   
     BSave.prepare( "UPDATE bomhead "
                "SET bomhead_docnum=:bomhead_docnum,"
-               "    bomhead_revision=:bomhead_revision, bomhead_revisiondate=:bomhead_revisiondate,"
+               "    bomhead_revision=:bomhead_revision,"
+               "    bomhead_revisiondate=:bomhead_revisiondate,"
                "    bomhead_batchsize=:bomhead_batchsize,"
                "    bomhead_requiredqtyper=:bomhead_requiredqtyper "
-               "WHERE ((bomhead_item_id=:bomhead_item_id) "
-			   "AND (bomhead_rev_id=:bomhead_rev_id));" );
-    BSave.bindValue(":bomhead_item_id", _item->id());
-    BSave.bindValue(":bomhead_rev_id", _revision->id());
+               "WHERE (bomhead_id=:bomhead_id);" );
   }
   else
   {
     BSave.prepare( "INSERT INTO bomhead "
-               "( bomhead_item_id, bomhead_docnum,"
+               "( bomhead_id, bomhead_item_id, bomhead_docnum,"
                "  bomhead_revision, bomhead_revisiondate,"
                "  bomhead_batchsize, bomhead_requiredqtyper ) "
                "VALUES "
-               "( :bomhead_item_id, :bomhead_docnum,"
+               "( :bomhead_id, :bomhead_item_id, :bomhead_docnum,"
                "  :bomhead_revision, :bomhead_revisiondate, "
                "  :bomhead_batchsize, :bomhead_requiredqtyper ) " );
-    BSave.bindValue(":bomhead_item_id", _item->id());
   }
   
+  BSave.bindValue(":bomhead_id", _bomheadid);
+  BSave.bindValue(":bomhead_item_id", _item->id());
+  BSave.bindValue(":bomhead_rev_id", _revision->id());
   BSave.bindValue(":bomhead_docnum", _documentNum->text());
   BSave.bindValue(":bomhead_revision", _revision->number());
   BSave.bindValue(":bomhead_revisiondate", _revisionDate->date());
@@ -214,9 +256,14 @@ bool BOM::sSave()
   if(_doRequireQtyPer->isChecked())
     BSave.bindValue(":bomhead_requiredqtyper", _requiredQtyPer->text().toDouble());
   BSave.exec();
-  
-  close();
 
+  if (_mode == cNew)
+  {
+    _mode=cEdit;
+    emit newMode(_mode);
+  }
+
+  omfgThis->sBOMsUpdated(_bomheadid, true);
   return true;
 }
 
@@ -299,12 +346,19 @@ void BOM::sPopulateMenu(QMenu *menuThis)
 
 void BOM::sNew()
 {
+  if(_mode == cNew)
+  {
+    if(!save(true))
+      return;
+  }
+  
   ParameterList params;
   params.append("mode", "new");
+  params.append("bomhead_id", _bomheadid);
   params.append("item_id", _item->id());
   params.append("revision_id", _revision->id());
   
-  bomItem newdlg(this, "", TRUE);
+  bomItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -315,7 +369,7 @@ void BOM::sEdit()
   params.append("mode", "edit");
   params.append("bomitem_id", _bomitem->id());
   
-  bomItem newdlg(this, "", TRUE);
+  bomItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -326,7 +380,7 @@ void BOM::sView()
   params.append("mode", "view");
   params.append("bomitem_id", _bomitem->id());
   
-  bomItem newdlg(this, "", TRUE);
+  bomItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -340,7 +394,7 @@ void BOM::sExpire()
   BExpire.bindValue(":bomitem_id", _bomitem->id());
   BExpire.exec();
   
-  omfgThis->sBOMsUpdated(_item->id(), TRUE);
+  omfgThis->sBOMsUpdated(_item->id(), true);
 }
 
 void BOM::sDelete()
@@ -357,7 +411,7 @@ void BOM::sDelete()
     BDelete.bindValue(":bomitem_id", _bomitem->id());
     BDelete.exec();
 
-    omfgThis->sBOMsUpdated(_item->id(), TRUE);
+    omfgThis->sBOMsUpdated(_item->id(), true);
   }
 }
 
@@ -368,7 +422,7 @@ void BOM::sReplace()
   params.append("bomitem_id", _bomitem->id());
   params.append("revision_id", _revision->id());
   
-  bomItem newdlg(this, "", TRUE);
+  bomItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -380,7 +434,7 @@ void BOM::sMoveUp()
   BMoveUp.bindValue(":bomitem_id", _bomitem->id());
   BMoveUp.exec();
   
-  omfgThis->sBOMsUpdated(_item->id(), TRUE);
+  omfgThis->sBOMsUpdated(_item->id(), true);
 }
 
 void BOM::sMoveDown()
@@ -390,13 +444,13 @@ void BOM::sMoveDown()
   BMoveDown.bindValue(":bomitem_id", _bomitem->id());
   BMoveDown.exec();
   
-  omfgThis->sBOMsUpdated(_item->id(), TRUE);
+  omfgThis->sBOMsUpdated(_item->id(), true);
 }
 
 void BOM::sFillList()
 {
   XSqlQuery BFillList;
-  sFillList(_item->id(), TRUE);
+  sFillList(_item->id(), true);
 }
 
 void BOM::sFillList(int pItemid, bool)
@@ -413,6 +467,15 @@ void BOM::sFillList(int pItemid, bool)
     BFillList.exec();
     if (BFillList.first())
     {
+      _bomheadid = BFillList.value("bomhead_id").toInt();
+      if (_mode == cNew)
+      {
+        _mode = cEdit;
+        emit newMode(_mode);
+        emit newId(_bomheadid);
+      }
+      _comments->setId(_bomheadid);
+      _documents->setId(_bomheadid);
       _documentNum->setText(BFillList.value("bomhead_docnum"));
       _revisionDate->setDate(BFillList.value("bomhead_revisiondate").toDate());
       _batchSize->setDouble(BFillList.value("bomhead_batchsize").toDouble());
@@ -424,15 +487,15 @@ void BOM::sFillList(int pItemid, bool)
       _revision->setNumber(BFillList.value("bomhead_revision").toString());
       if ( (_revision->description() == "Inactive") || (_mode == cView) )
       {
-        _save->setEnabled(FALSE);
-        _documentNum->setEnabled(FALSE);
-        _revisionDate->setEnabled(FALSE);
-        _batchSize->setEnabled(FALSE);
-        _new->setEnabled(FALSE);
-        _edit->setEnabled(FALSE);
-        _expire->setEnabled(FALSE);
-        _moveUp->setEnabled(FALSE);
-        _moveDown->setEnabled(FALSE);
+        _save->setEnabled(false);
+        _documentNum->setEnabled(false);
+        _revisionDate->setEnabled(false);
+        _batchSize->setEnabled(false);
+        _new->setEnabled(false);
+        _edit->setEnabled(false);
+        _expire->setEnabled(false);
+        _moveUp->setEnabled(false);
+        _moveDown->setEnabled(false);
         disconnect(_bomitem, SIGNAL(valid(bool)), _edit, SLOT(setEnabled(bool)));
         disconnect(_bomitem, SIGNAL(valid(bool)), _expire, SLOT(setEnabled(bool)));
         disconnect(_bomitem, SIGNAL(valid(bool)), _moveUp, SLOT(setEnabled(bool)));
@@ -441,15 +504,15 @@ void BOM::sFillList(int pItemid, bool)
       }
       else
       {
-        _save->setEnabled(TRUE);
-        _documentNum->setEnabled(TRUE);
-        _revisionDate->setEnabled(TRUE);
-        _batchSize->setEnabled(TRUE);
-        _new->setEnabled(TRUE);
-        _edit->setEnabled(FALSE);
-        _expire->setEnabled(FALSE);
-        _moveUp->setEnabled(FALSE);
-        _moveDown->setEnabled(FALSE);
+        _save->setEnabled(true);
+        _documentNum->setEnabled(true);
+        _revisionDate->setEnabled(true);
+        _batchSize->setEnabled(true);
+        _new->setEnabled(true);
+        _edit->setEnabled(false);
+        _expire->setEnabled(false);
+        _moveUp->setEnabled(false);
+        _moveDown->setEnabled(false);
         connect(_bomitem, SIGNAL(valid(bool)), _edit, SLOT(setEnabled(bool)));
         connect(_bomitem, SIGNAL(valid(bool)), _expire, SLOT(setEnabled(bool)));
         connect(_bomitem, SIGNAL(valid(bool)), _moveUp, SLOT(setEnabled(bool)));
@@ -461,15 +524,15 @@ void BOM::sFillList(int pItemid, bool)
     {
       if (_mode != cView)
       {
-        _save->setEnabled(TRUE);
-        _documentNum->setEnabled(TRUE);
-        _revisionDate->setEnabled(TRUE);
-        _batchSize->setEnabled(TRUE);
-        _new->setEnabled(TRUE);
-        _edit->setEnabled(FALSE);
-        _expire->setEnabled(FALSE);
-        _moveUp->setEnabled(FALSE);
-        _moveDown->setEnabled(FALSE);
+        _save->setEnabled(true);
+        _documentNum->setEnabled(true);
+        _revisionDate->setEnabled(true);
+        _batchSize->setEnabled(true);
+        _new->setEnabled(true);
+        _edit->setEnabled(false);
+        _expire->setEnabled(false);
+        _moveUp->setEnabled(false);
+        _moveDown->setEnabled(false);
         connect(_bomitem, SIGNAL(valid(bool)), _edit, SLOT(setEnabled(bool)));
         connect(_bomitem, SIGNAL(valid(bool)), _expire, SLOT(setEnabled(bool)));
         connect(_bomitem, SIGNAL(valid(bool)), _moveUp, SLOT(setEnabled(bool)));
@@ -487,9 +550,9 @@ void BOM::sFillList(int pItemid, bool)
     MetaSQLQuery mql = mqlLoad("bomItems", "detail");
     BFillList = mql.toQuery(params);
     _bomitem->populate(BFillList);
-    if (BFillList.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                  BFillList, __FILE__, __LINE__))
     {
-      systemError(this, BFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     
@@ -509,8 +572,8 @@ void BOM::sFillList(int pItemid, bool)
           "GROUP BY item_picklist;");
     BFillList = picklistmql.toQuery(params);
     
-    bool   foundPick    = FALSE;
-    bool   foundNonPick = FALSE;
+    bool   foundPick    = false;
+    bool   foundNonPick = false;
     int    totalNumber  = 0;
     double totalQtyPer  = 0.0;
     while (BFillList.next())
@@ -520,20 +583,20 @@ void BOM::sFillList(int pItemid, bool)
       
       if (BFillList.value("item_picklist").toBool())
       {
-        foundPick = TRUE;
+        foundPick = true;
         _pickNumber->setDouble(BFillList.value("total").toDouble());
         _pickQtyPer->setDouble(BFillList.value("qtyper").toDouble());
       }
       else
       {
-        foundNonPick = TRUE;
+        foundNonPick = true;
         _nonPickNumber->setDouble(BFillList.value("total").toDouble());
         _nonPickQtyPer->setDouble(BFillList.value("qtyper").toDouble());
       }
     }
-    if (BFillList.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                  BFillList, __FILE__, __LINE__))
     {
-      systemError(this, BFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     
@@ -577,12 +640,14 @@ void BOM::sFillList(int pItemid, bool)
         _currentActCost->setDouble(BFillList.value("actcost").toDouble());
         _maxCost->setDouble(BFillList.value("item_maxcost").toDouble());
       }
-      if (BFillList.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                    BFillList, __FILE__, __LINE__))
       {
-        systemError(this, BFillList.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
+    
+    emit populated();
   }
   else if (!_item->isValid())
   {
@@ -595,12 +660,6 @@ void BOM::sFillList(int pItemid, bool)
   }
 }
 
-void BOM::sClose()
-{
-  if(sCheckRequiredQtyPer())
-    close();
-}
-
 bool BOM::sCheckRequiredQtyPer()
 {
   if(cView == _mode || !_doRequireQtyPer->isChecked() || _totalQtyPerCache == 0.0)
