@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -26,7 +26,7 @@
 
 const char *_issueMethods[] = { "S", "L", "M" };
 
-bomItem::bomItem(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+bomItem::bomItem(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
@@ -37,7 +37,7 @@ bomItem::bomItem(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   _substituteGroupInt->addButton(_bomDefinedSubstitutes);
 
   connect(_buttonBox, SIGNAL(accepted()), this, SLOT(sSaveClick()));
-  connect(_buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
+  connect(_buttonBox, SIGNAL(rejected()), this, SLOT(sClose()));
   connect(_item, SIGNAL(typeChanged(const QString&)), this, SLOT(sItemTypeChanged(const QString&)));
   connect(_item, SIGNAL(newId(int)), this, SLOT(sItemIdChanged()));
   connect(_newSubstitution, SIGNAL(clicked()), this, SLOT(sNewSubstitute()));
@@ -48,15 +48,13 @@ bomItem::bomItem(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   connect(_editCost, SIGNAL(clicked()), this, SLOT(sEditCost()));
   connect(_deleteCost, SIGNAL(clicked()), this, SLOT(sDeleteCost()));
   connect(_char, SIGNAL(activated(int)), this, SLOT(sCharIdChanged()));
+  connect(_uom, SIGNAL(newID(int)), this, SLOT(sUOMChanged()));
 
-  if (_metrics->boolean("AllowInactiveBomItems"))
-    _item->setType(ItemLineEdit::cGeneralComponents);
-  else
-    _item->setType(ItemLineEdit::cGeneralComponents | ItemLineEdit::cActive);
+  _item->setType(ItemLineEdit::cGeneralComponents);
 
-  _dates->setStartNull(tr("Always"), omfgThis->startOfTime(), TRUE);
+  _dates->setStartNull(tr("Always"), omfgThis->startOfTime(), true);
   _dates->setStartCaption(tr("Effective"));
-  _dates->setEndNull(tr("Never"), omfgThis->endOfTime(), TRUE);
+  _dates->setEndNull(tr("Never"), omfgThis->endOfTime(), true);
   _dates->setEndCaption(tr("Expires"));
 
   _qtyFxd->setValidator(omfgThis->qtyVal());
@@ -72,20 +70,16 @@ bomItem::bomItem(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   _itemcost->addColumn(tr("Element"),     -1,           Qt::AlignLeft,   true, "costelem_type");
   _itemcost->addColumn(tr("Lower"),       _costColumn,  Qt::AlignCenter, true, "itemcost_lowlevel" );
   _itemcost->addColumn(tr("Std. Cost"),   _costColumn,  Qt::AlignRight,  true, "itemcost_stdcost"  );
-  _itemcost->addColumn(tr("Currency"),    _currencyColumn, Qt::AlignLeft,true, "baseCurr" );
+  _itemcost->addColumn(tr("Currency"),    _currencyColumn, Qt::AlignLeft,!omfgThis->singleCurrency(), "baseCurr" );
   _itemcost->addColumn(tr("Posted"),      _dateColumn,  Qt::AlignCenter, true, "itemcost_posted" );
   _itemcost->addColumn(tr("Act. Cost"),   _costColumn,  Qt::AlignRight,  true, "itemcost_actcost"  );
-  _itemcost->addColumn(tr("Currency"),    _currencyColumn, Qt::AlignLeft,true, "costCurr" );
+  _itemcost->addColumn(tr("Currency"),    _currencyColumn, Qt::AlignLeft,!omfgThis->singleCurrency(), "costCurr" );
   _itemcost->addColumn(tr("Updated"),     _dateColumn,  Qt::AlignCenter, true, "itemcost_updated" );
 
-  if (omfgThis->singleCurrency())
-  {
-      _itemcost->hideColumn(3);
-      _itemcost->hideColumn(6);
-  }
-
+  _invuomid=-1;
   _parentitemid=0;
-  _saved=FALSE;
+  _bomheadid=0;
+  _saved=false;
   adjustSize();
 }
 
@@ -113,6 +107,10 @@ enum SetResponse bomItem::set(const ParameterList &pParams)
     populate();
   }
 
+  param = pParams.value("bomhead_id", &valid);
+  if (valid)
+    _bomheadid = param.toInt();
+  
   param = pParams.value("item_id", &valid);
   if (valid)
     _parentitemid = param.toInt();
@@ -128,6 +126,9 @@ enum SetResponse bomItem::set(const ParameterList &pParams)
     {
       _mode = cNew;
 
+      if (!_metrics->boolean("AllowInactiveBomItems"))
+        _item->setType(ItemLineEdit::cGeneralComponents | ItemLineEdit::cActive);
+
       QString issueMethod = _metrics->value("DefaultWomatlIssueMethod");
       for (int counter = 0; counter < _issueMethod->count(); counter++)
       {
@@ -140,10 +141,10 @@ enum SetResponse bomItem::set(const ParameterList &pParams)
       bomet.exec("SELECT NEXTVAL('bomitem_bomitem_id_seq') AS bomitem_id");
       if (bomet.first())
         _bomitemid = bomet.value("bomitem_id").toInt();
-      else if (bomet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                    bomet, __FILE__, __LINE__))
       {
-	systemError(this, bomet.lastError().databaseText(), __FILE__, __LINE__);
-	return UndefinedError;
+        return UndefinedError;
       }
   
       //Set up configuration tab if parent item is configured or kit
@@ -191,16 +192,16 @@ enum SetResponse bomItem::set(const ParameterList &pParams)
       bomet.exec("SELECT NEXTVAL('bomitem_bomitem_id_seq') AS bomitem_id");
       if (bomet.first())
         _bomitemid = bomet.value("bomitem_id").toInt();
-      else if (bomet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                    bomet, __FILE__, __LINE__))
       {
-	systemError(this, bomet.lastError().databaseText(), __FILE__, __LINE__);
-	return UndefinedError;
+        return UndefinedError;
       }
     }
     else if (param.toString() == "edit")
     {
       _mode = cEdit;
-      _item->setReadOnly(TRUE);
+      _item->setReadOnly(true);
       connect(_bomDefinedCosts, SIGNAL(toggled(bool)), this, SLOT(sHandleBomitemCost()));
     }
     else if (param.toString() == "copy")
@@ -211,10 +212,10 @@ enum SetResponse bomItem::set(const ParameterList &pParams)
       bomet.exec("SELECT NEXTVAL('bomitem_bomitem_id_seq') AS bomitem_id");
       if (bomet.first())
         _bomitemid = bomet.value("bomitem_id").toInt();
-      else if (bomet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                    bomet, __FILE__, __LINE__))
       {
-	systemError(this, bomet.lastError().databaseText(), __FILE__, __LINE__);
-	return UndefinedError;
+        return UndefinedError;
       }
 
       _dates->setStartDate(omfgThis->dbDate());
@@ -223,20 +224,20 @@ enum SetResponse bomItem::set(const ParameterList &pParams)
     {
       _mode = cView;
 
-      _item->setReadOnly(TRUE);
-      _qtyFxd->setEnabled(FALSE);
-      _qtyPer->setEnabled(FALSE);
-      _scrap->setEnabled(FALSE);
-      _dates->setEnabled(FALSE);
-      _createWo->setEnabled(FALSE);
-      _issueWo->setEnabled(FALSE);
-      _issueMethod->setEnabled(FALSE);
-      _uom->setEnabled(FALSE);
-      _comments->setReadOnly(TRUE);
-      _ecn->setEnabled(FALSE);
-      _substituteGroup->setEnabled(FALSE);
-      _notes->setEnabled(FALSE);
-      _ref->setEnabled(FALSE);
+      _item->setReadOnly(true);
+      _qtyFxd->setEnabled(false);
+      _qtyPer->setEnabled(false);
+      _scrap->setEnabled(false);
+      _dates->setEnabled(false);
+      _createWo->setEnabled(false);
+      _issueWo->setEnabled(false);
+      _issueMethod->setEnabled(false);
+      _uom->setEnabled(false);
+      _comments->setReadOnly(true);
+      _ecn->setEnabled(false);
+      _substituteGroup->setEnabled(false);
+      _notes->setEnabled(false);
+      _ref->setEnabled(false);
       _buttonBox->setStandardButtons(QDialogButtonBox::Close);
 
       _newCost->setEnabled(false);
@@ -367,12 +368,12 @@ void bomItem::sSave()
 //  Not used?
 //  bomitem.bindValue(":configType", "N");
 //  bomitem.bindValue(":configId", -1);
-//  bomitem.bindValue(":configFlag", QVariant(FALSE));
+//  bomitem.bindValue(":configFlag", QVariant(false));
 
   bomitem.exec();
-  if (bomitem.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving BOM Information"),
+                                bomitem, __FILE__, __LINE__))
   {
-    systemError(this, bomitem.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -391,9 +392,9 @@ void bomItem::sSave()
     replace.bindValue(":sourcebomitemid", _sourceBomitemid);
     replace.bindValue(":bomitem_id", _bomitemid);
     replace.exec();
-    if (replace.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving BOM Information"),
+                                  replace, __FILE__, __LINE__))
     {
-      systemError(this, replace.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -408,18 +409,18 @@ void bomItem::sSave()
     replace.bindValue(":bomitem_expires", _dates->startDate());
     replace.bindValue(":sourcebomitemid", _sourceBomitemid);
     replace.exec();
-    if (replace.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving BOM Information"),
+                                  replace, __FILE__, __LINE__))
     {
-      systemError(this, replace.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
 
-  omfgThis->sBOMsUpdated(_parentitemid, TRUE);
+  omfgThis->sBOMsUpdated(_parentitemid, true);
   
   emit saved(_bomitemid);
 
-  _saved=TRUE;
+  _saved=true;
 }
 
 void bomItem::sSaveClick()
@@ -440,9 +441,9 @@ void bomItem::sClose()
                "WHERE (bomitemcost_bomitem_id=:bomitem_id);" );
     bomClose.bindValue("bomitem_id", _bomitemid);
     bomClose.exec();
-    if (bomClose.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error cancelling BOM Operation"),
+                                  bomClose, __FILE__, __LINE__))
     {
-      systemError(this, bomClose.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -453,27 +454,105 @@ void bomItem::sClose()
 void bomItem::sItemTypeChanged(const QString &type)
 {
   if (type == "M")
-    _createWo->setEnabled(TRUE);
+    _createWo->setEnabled(true);
   else
   {
-    _createWo->setEnabled(FALSE);
-    _createWo->setChecked(FALSE);
+    _createWo->setEnabled(false);
+    _createWo->setChecked(false);
+  }
+}
+
+void bomItem::sPopulateUOM()
+{
+  if (_item->id() != -1)
+  {
+    // Get list of active, valid Material Issue UOMs
+    MetaSQLQuery muom = mqlLoad("uoms", "item");
+    
+    ParameterList params;
+    params.append("uomtype", "MaterialIssue");
+    params.append("item_id", _item->id());
+    
+    // Include Global UOMs
+    if (_privileges->check("MaintainUOMs"))
+    {
+      params.append("includeGlobal", true);
+      params.append("global", tr("-Global"));
+    }
+    
+    // Also have to factor UOMs previously used on BOM Item now inactive
+    if (_bomitemid != -1)
+    {
+      XSqlQuery bomuom;
+      bomuom.prepare("SELECT bomitem_uom_id "
+                     "  FROM bomitem"
+                     " WHERE(bomitem_id=:bomitem_id);");
+      bomuom.bindValue(":bomitem_id", _bomitemid);
+      bomuom.exec();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting BOM Item UOMs"),
+                               bomuom, __FILE__, __LINE__))
+        return;
+      else if (bomuom.first())
+        params.append("uom_id", bomuom.value("bomitem_uom_id"));
+    }
+
+    XSqlQuery quom = muom.toQuery(params);
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting UOMs"),
+                             quom, __FILE__, __LINE__))
+      return;
+
+    int saveuomid = _uom->id();
+    disconnect(_uom, SIGNAL(newID(int)), this, SLOT(sUOMChanged()));
+    _uom->populate(quom);
+    _uom->setId(saveuomid);
+    connect(_uom, SIGNAL(newID(int)), this, SLOT(sUOMChanged()));
+  }
+}
+
+void bomItem::sUOMChanged()
+{
+  // Check for Global UOM Conversion that must be setup for Item
+  if (_uom->code() == "G")
+  {
+    if (QMessageBox::question(this, tr("Use Global UOM?"),
+                              tr("<p>This Global UOM Conversion is not setup for this Item."
+                                 "<p>Do you want to add this UOM conversion to this Item?"),
+                              QMessageBox::Yes | QMessageBox::Default,
+                              QMessageBox::No  | QMessageBox::Escape) == QMessageBox::Yes)
+    {
+      // create itemuomconv and itemuom
+      XSqlQuery adduom;
+      adduom.prepare("SELECT createItemUomConv(:item_id, :uom_id, :uom_type) AS result;");
+      adduom.bindValue(":item_id", _item->id());
+      adduom.bindValue(":uom_id", _uom->id());
+      adduom.bindValue(":uom_type", "MaterialIssue");
+      adduom.exec();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Creating Item UOM Conv"),
+                               adduom, __FILE__, __LINE__))
+        return;
+      
+      // repopulate uom combobox
+      sPopulateUOM();
+    }
+    else
+    {
+      _uom->setId(_invuomid);
+    }
   }
 }
 
 void bomItem::populate()
 {
   XSqlQuery qbomitem;
-  qbomitem.prepare( "SELECT bomitem.*, item.* "
-                    "FROM bomitem JOIN item ON (item_id=bomitem_parent_item_id) "
-                    "WHERE (bomitem_id=:bomitem_id);" );
+  qbomitem.prepare("SELECT bomitem.*, item_type, item_config"
+                   "  FROM bomitem JOIN item ON (item_id=bomitem_parent_item_id) "
+                   " WHERE (bomitem_id=:bomitem_id);" );
   qbomitem.bindValue(":bomitem_id", _bomitemid);
   qbomitem.exec();
-  if (qbomitem.lastError().type() != QSqlError::NoError)
-  {
-    systemError(this, qbomitem.lastError().databaseText(), __FILE__, __LINE__);
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting BOM Item"),
+                           qbomitem, __FILE__, __LINE__))
     return;
-  }
+
   if (qbomitem.first())
   {
     _parentitemid = qbomitem.value("bomitem_parent_item_id").toInt();
@@ -548,7 +627,7 @@ void bomItem::sNewSubstitute()
   if (_mode == cNew)
     params.append("bomitem_item_id", _item->id());
 
-  itemSubstitute newdlg(this, "", TRUE);
+  itemSubstitute newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -561,7 +640,7 @@ void bomItem::sEditSubstitute()
   params.append("mode", "edit");
   params.append("bomitemsub_id", _bomitemsub->id());
 
-  itemSubstitute newdlg(this, "", TRUE);
+  itemSubstitute newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -575,9 +654,9 @@ void bomItem::sDeleteSubstitute()
              "WHERE (bomitemsub_id=:bomitemsub_id);" );
   bomDeleteSubstitute.bindValue(":bomitemsub_id", _bomitemsub->id());
   bomDeleteSubstitute.exec();
-  if (bomDeleteSubstitute.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting BOM Substitute"),
+                                bomDeleteSubstitute, __FILE__, __LINE__))
   {
-    systemError(this, bomDeleteSubstitute.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -596,23 +675,17 @@ void bomItem::sFillSubstituteList()
   bomFillSubstituteList.bindValue(":bomitem_id", _bomitemid);
   bomFillSubstituteList.exec();
   _bomitemsub->populate(bomFillSubstituteList);
-  if (bomFillSubstituteList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                bomFillSubstituteList, __FILE__, __LINE__))
   {
-    systemError(this, bomFillSubstituteList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
 
 void bomItem::sItemIdChanged()
 {
-  MetaSQLQuery muom = mqlLoad("uoms", "item");
-
-  ParameterList params;
-  params.append("uomtype", "MaterialIssue");
-  params.append("item_id", _item->id());
-
-  XSqlQuery quom = muom.toQuery(params);
-  _uom->populate(quom);
+  // Get list of active, valid Material Issue UOMs
+  sPopulateUOM();
 
   XSqlQuery qitem;
   qitem.prepare("SELECT item_inv_uom_id, item_type "
@@ -622,7 +695,28 @@ void bomItem::sItemIdChanged()
   qitem.exec();
   if(qitem.first())
   {
-    _uom->setId(qitem.value("item_inv_uom_id").toInt());
+    if (qitem.value("item_type").toString() != "P" && qitem.value("item_type").toString() != "O")
+    {
+      // Prevent job costed items from being added to the BOM
+      XSqlQuery qry;
+      qry.prepare("SELECT itemsite_id FROM itemsite WHERE itemsite_item_id = :item_id AND itemsite_costmethod = 'J';");
+      qry.bindValue(":item_id", _item->id());
+      qry.exec();
+      if (qry.first())
+      {
+        QMessageBox::critical(this, tr("Job Costed Item"),
+                              tr("You cannot add a Job Costed item to a Bill of Material"));
+        _item->setId(-1);
+        _item->setFocus();
+        return;
+      }
+      else if (qry.lastError().type() != QSqlError::NoError)
+        ErrorReporter::error(QtCriticalMsg, this, tr("Item Job cost check"),
+                             qry, __FILE__, __LINE__);
+    }
+
+    _invuomid = qitem.value("item_inv_uom_id").toInt();
+    _uom->setId(_invuomid);
     if (qitem.value("item_type").toString() != "T" && qitem.value("item_type").toString() != "R")
     {
       if (_qtyPer->text().length() == 0)
@@ -636,8 +730,10 @@ void bomItem::sItemIdChanged()
       _qtyFxd->setDouble(1.0);
       _qtyPer->setDouble(0.0);
     }
+
     if (_scrap->text().length() == 0)
       _scrap->setDouble(0.0);
+    
   }
 }
 
@@ -687,9 +783,9 @@ void bomItem::sFillCostList()
     }
 
     _itemcost->populate(qry, true);
-    if (qry.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                                  qry, __FILE__, __LINE__))
     {
-      systemError(this, qry.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -741,9 +837,9 @@ void bomItem::sFillCostList()
                             "",
                             baseKnown ? formatCost(actualCost) : tr("?????"),
                             convert.value("currConcat"));
-    else if (convert.lastError().type() != QSqlError::NoError)
-        systemError(this, convert.lastError().databaseText(), __FILE__, __LINE__);
-
+    else
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving BOM Information"),
+                             convert, __FILE__, __LINE__);
   }
   else
     _itemcost->clear();
@@ -756,8 +852,8 @@ void bomItem::sHandleBomitemCost()
   qry.bindValue(":bomitem_id", _bomitemid);
   qry.bindValue(":enabled", _bomDefinedCosts->isChecked());
   qry.exec();
-  if (qry.lastError().type() != QSqlError::NoError)
-    systemError(this, qry.lastError().databaseText(), __FILE__, __LINE__);
+  ErrorReporter::error(QtCriticalMsg, this, tr("Error Toggling BOM Cost Setting"),
+                                qry, __FILE__, __LINE__);
   sFillCostList();
 }
 
@@ -781,7 +877,7 @@ void bomItem::sNewCost()
   if (_mode == cNew)
     params.append("bomitem_item_id", _item->id());
 
-  itemCost newdlg(this, "", TRUE);
+  itemCost newdlg(this, "", true);
   if (newdlg.set(params) == NoError && newdlg.exec())
     sFillCostList();
 }
@@ -795,7 +891,7 @@ void bomItem::sEditCost()
   if (_mode == cView)
     params.append("mode", "view");
 
-  itemCost newdlg(this, "", TRUE);
+  itemCost newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec())
@@ -808,8 +904,8 @@ void bomItem::sDeleteCost()
   bomDeleteCost.prepare( "DELETE FROM bomitemcost WHERE (bomitemcost_id=:bomitemcost_id);" );
   bomDeleteCost.bindValue(":bomitemcost_id", _itemcost->id());
   bomDeleteCost.exec();
-  if (bomDeleteCost.lastError().type() != QSqlError::NoError)
-    systemError(this, bomDeleteCost.lastError().databaseText(), __FILE__, __LINE__);
+  ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting BOM Cost Component"),
+                                bomDeleteCost, __FILE__, __LINE__);
 
   sFillCostList();
 }
