@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,7 +22,7 @@ class cashReceiptMiscDistrib : public XDialog, public Ui::cashReceiptMiscDistrib
     Q_OBJECT
 
 public:
-    cashReceiptMiscDistrib(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    cashReceiptMiscDistrib(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~cashReceiptMiscDistrib();
 
 public slots:
@@ -30,6 +30,9 @@ public slots:
     virtual void populate();
     virtual void sSave();
 
+    virtual void showCustomers(int group, int customer);
+    virtual void setGroup(const ParameterList &pParams);
+
 protected slots:
     virtual void languageChange();
 
@@ -37,7 +40,8 @@ private:
     int _mode;
     int _cashrcptmiscid;
     int _cashrcptid;
-
+    int _custgrp;
+    int _cashmisc;
 };
 
 #endif // CASHRECEIPTMISCDISTRIB_H
