@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,27 +19,18 @@
 
 #include "ui_characteristic.h"
 
+class characteristicPrivate;
+
 class characteristic : public XDialog, public Ui::characteristic
 {
     Q_OBJECT
 
 public:
-    characteristic(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    characteristic(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~characteristic();
 
     enum Type { Text, List, Date };
 
-    enum Use { Addresses     = 0x00000001,
-               Contacts      = 0x00000002,
-               CRMAccounts   = 0x00000004,
-               Customers     = 0x00000008,
-               Employees     = 0x00000010,
-               Incidents     = 0x00000020,
-               Items         = 0x00000040,
-               LotSerial     = 0x00000080,
-               Opportunities = 0x00000100,
-             };
-
 public slots:
     virtual enum SetResponse set(const ParameterList & pParams );
     virtual void populate();
@@ -56,9 +47,7 @@ protected slots:
 
 
 private:
-    int _mode;
-    int _charid;
-    QSqlTableModel *_charoptModel;
+    characteristicPrivate *_d;
 };
 
 #endif // CHARACTERISTIC_H
