@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,7 +22,7 @@ class commentType : public XDialog, public Ui::commentType
     Q_OBJECT
 
 public:
-    commentType(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    commentType(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~commentType();
 
 public slots:
@@ -32,7 +32,8 @@ public slots:
 protected slots:
     virtual void languageChange();
 
-    virtual void sSave();
+    virtual void sSaveClicked();
+    virtual bool sSave();
     virtual void sCheck();
     virtual void sModuleSelected( const QString & pModule );
     virtual void sAdd();
