@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,10 +19,13 @@
 
 #include "login2.h"
 #include "currcluster.h"
+#include "version.h"
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
 #define DEBUG false
 
-company::company(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+company::company(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
@@ -54,6 +57,9 @@ company::company(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   _discrepancy->setType(GLCluster::cExpense);
   _discrepancy->setShowExternal(true);
   _discrepancy->setIgnoreCompany(true);
+  _unassigned->setType(GLCluster::cLiability);
+  _unassigned->setShowExternal(true);
+  _unassigned->setIgnoreCompany(true);
 }
 
 company::~company()
@@ -92,10 +98,10 @@ enum SetResponse company::set(const ParameterList &pParams)
     {
       _mode = cView;
       
-      _number->setEnabled(FALSE);
-      _descrip->setEnabled(FALSE);
-      _external->setEnabled(FALSE);
-      _authGroup->setEnabled(FALSE);
+      _number->setEnabled(false);
+      _descrip->setEnabled(false);
+      _external->setEnabled(false);
+      _authGroup->setEnabled(false);
       _buttonBox->setStandardButtons(QDialogButtonBox::Close);
     }
   }
@@ -105,44 +111,22 @@ enum SetResponse company::set(const ParameterList &pParams)
 void company::sSave()
 {
   XSqlQuery companySave;
-  if (_number->text().length() == 0)
-  {
-      QMessageBox::warning( this, tr("Cannot Save Company"),
-                            tr("You must enter a valid Number.") );
-      _number->setFocus();
-      return;
-  }
-  
-  struct {
-    bool	condition;
-    QString	msg;
-    QWidget*	widget;
-  } error[] = {
-    { _external->isChecked() && _extServer->text().isEmpty(),
-      tr("<p>You must enter a Server if this is an external Company."),
-      _extServer
-    },
-    { _external->isChecked() && _extPort->value() == 0,
-      tr("<p>You must enter a Port if this is an external Company."),
-      _extPort
-    },
-    { _external->isChecked() && _extDB->text().isEmpty(),
-      tr("<p>You must enter a Database if this is an external Company."),
-      _extDB
-    },
-    { true, "", NULL }
-  }; // error[]
+  QList<GuiErrorCheck> errors;
+  errors << GuiErrorCheck(_number->text().trimmed().isEmpty(), _number,
+                          tr("You must enter a valid Number.") )
+         << GuiErrorCheck(_external->isChecked() && _extServer->text().trimmed().isEmpty(),
+                          _extServer, 
+                          tr("<p>You must enter a Server if this is an external Company.") )
+         << GuiErrorCheck(_external->isChecked() && _extPort->value() == 0,
+                          _extPort, 
+                          tr("<p>You must enter a Port if this is an external Company.") )
+         << GuiErrorCheck(_external->isChecked() && _extDB->text().trimmed().isEmpty(),
+                          _extDB, 
+                          tr("<p>You must enter a Database if this is an external Company.") )
+  ;
 
-  int errIndex;
-  for (errIndex = 0; ! error[errIndex].condition; errIndex++)
-    ;
-  if (! error[errIndex].msg.isEmpty())
-  {
-    QMessageBox::critical(this, tr("Cannot Save Company"),
-			  error[errIndex].msg);
-    error[errIndex].widget->setFocus();
+  if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Company"), errors))
     return;
-  }
 
   companySave.prepare("SELECT company_id"
             "  FROM company"
@@ -151,58 +135,41 @@ void company::sSave()
   companySave.bindValue(":company_id",       _companyid);
   companySave.bindValue(":company_number",   _number->text());
   companySave.exec();
-  if(companySave.first())
-  {
-    QMessageBox::critical(this, tr("Duplicate Company Number"),
-      tr("A Company Number already exists for the one specified.") );
-    _number->setFocus();
-    return;
-  }
 
-  if (_yearend->isValid() &&
-      _companyid != _yearend->companyId())
-  {
-    QMessageBox::critical(this, tr("Company Account Mismatch"),
-                          tr("The Retained Earnings Account must belong to this Company.") );
-    _yearend->setFocus();
-    return;
-  }
+  errors << GuiErrorCheck(companySave.first(), _number,
+                          tr("A Company Number already exists for the one specified.") );
 
-  if (_gainloss->isValid() &&
-      _companyid != _gainloss->companyId())
-  {
-    QMessageBox::critical(this, tr("Company Account Mismatch"),
-                          tr("The Currency Gain/Loss Account must belong to this Company.") );
-    _gainloss->setFocus();
+  if (GuiErrorCheck::reportErrors(this, tr("Duplicate Company Number"), errors))
     return;
-  }
 
-  if (_discrepancy->isValid() &&
-      _companyid != _discrepancy->companyId())
-  {
-    QMessageBox::critical(this, tr("Company Account Mismatch"),
-                          tr("The G/L Discrepancy Account must belong to this Company.") );
-    _discrepancy->setFocus();
-    return;
-  }
+  errors << GuiErrorCheck(_yearend->isValid() && _companyid != _yearend->companyId(),
+                          _yearend,
+                          tr("The Retained Earnings Account must belong to this Company.") )
+         << GuiErrorCheck(_gainloss->isValid() && _companyid != _gainloss->companyId(),
+                          _gainloss,
+                          tr("The Currency Gain/Loss Account must belong to this Company.") )
+         << GuiErrorCheck(_discrepancy->isValid() && _companyid != _discrepancy->companyId(),
+                          _discrepancy,
+                          tr("The G/L Discrepancy Account must belong to this Company.") )
+         << GuiErrorCheck(_unassigned->isValid() && _companyid != _unassigned->companyId(),
+                          _unassigned,
+                          tr("The Unassigned G/L Account must belong to this Company.") )
+         << GuiErrorCheck(_unrlzgainloss->isValid() && _companyid != _unrlzgainloss->companyId(),
+                          _unrlzgainloss,
+                          tr("The Unrealized Currency Gain/LossL Account must belong to this Company.") )
+  ;
 
-  if (_unrlzgainloss->isValid() &&
-      _companyid != _unrlzgainloss->companyId())
-  {
-    QMessageBox::critical(this, tr("Company Account Mismatch"),
-                          tr("The Unrealized Currency Gain/Loss Account must belong to this Company.") );
-    _unrlzgainloss->setFocus();
+  if (GuiErrorCheck::reportErrors(this, tr("Company Account Mismatch"), errors))
     return;
-  }
 
   if (_mode == cNew)
   {
     companySave.exec("SELECT NEXTVAL('company_company_id_seq') AS company_id;");
     if (companySave.first())
       _companyid = companySave.value("company_id").toInt();
-    else if (companySave.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Company"),
+                                  companySave, __FILE__, __LINE__))
     {
-      systemError(this, companySave.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     
@@ -211,13 +178,13 @@ void company::sSave()
                "  company_external, company_server, company_port,"
                "  company_database, company_curr_id, company_yearend_accnt_id, "
                "  company_gainloss_accnt_id, company_dscrp_accnt_id, "
-               "  company_unrlzgainloss_accnt_id) "
+               "  company_unrlzgainloss_accnt_id, company_unassigned_accnt_id) "
                "VALUES "
                "( :company_id, :company_number, :company_descrip,"
                "  :company_external, :company_server, :company_port, "
                "  :company_database, :company_curr_id, :company_yearend_accnt_id, "
                "  :company_gainloss_accnt_id, :company_dscrp_accnt_id, "
-               "  :company_unrlzgainloss_accnt_id);" );
+               "  :company_unrlzgainloss_accnt_id, :company_unassigned_accnt_id);" );
   }
   else if (_mode == cEdit)
   {
@@ -262,12 +229,13 @@ void company::sSave()
                "    company_yearend_accnt_id=:company_yearend_accnt_id, "
                "    company_gainloss_accnt_id=:company_gainloss_accnt_id, "
                "    company_dscrp_accnt_id=:company_dscrp_accnt_id, "
-               "    company_unrlzgainloss_accnt_id=:company_unrlzgainloss_accnt_id "
+               "    company_unrlzgainloss_accnt_id=:company_unrlzgainloss_accnt_id, "
+               "    company_unassigned_accnt_id=:company_unassigned_accnt_id "
                "WHERE (company_id=:company_id);" );
   }
   
   companySave.bindValue(":company_id",       _companyid);
-  companySave.bindValue(":company_number",   _number->text());
+  companySave.bindValue(":company_number",   _number->text().trimmed());
   companySave.bindValue(":company_descrip",  _descrip->toPlainText());
   companySave.bindValue(":company_external", _external->isChecked());
   companySave.bindValue(":company_server",   _extServer->text());
@@ -277,6 +245,8 @@ void company::sSave()
     companySave.bindValue(":company_gainloss_accnt_id", _gainloss->id());
   if (_discrepancy->isValid())
     companySave.bindValue(":company_dscrp_accnt_id", _discrepancy->id());
+  if (_unassigned->isValid())
+    companySave.bindValue(":company_unassigned_accnt_id", _unassigned->id());
   if (_yearend->isValid())
     companySave.bindValue(":company_yearend_accnt_id", _yearend->id());
   if (_external->isChecked())
@@ -286,15 +256,16 @@ void company::sSave()
       companySave.bindValue(":company_unrlzgainloss_accnt_id", _unrlzgainloss->id());
   }
   companySave.exec();
-  if (companySave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Company"),
+                                companySave, __FILE__, __LINE__))
   {
-    systemError(this, companySave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
   if ((!_yearend->isValid()) ||
      (!_gainloss->isValid()) ||
      (!_discrepancy->isValid()) ||
+     (!_unassigned->isValid()) ||
      (_external->isChecked() &&
       _currency->id() != CurrCluster::baseId() &&
       !_unrlzgainloss->isValid()))
@@ -327,6 +298,7 @@ void company::populate()
     _yearend->setId(companypopulate.value("company_yearend_accnt_id").toInt());
     _gainloss->setId(companypopulate.value("company_gainloss_accnt_id").toInt());
     _discrepancy->setId(companypopulate.value("company_dscrp_accnt_id").toInt());
+    _unassigned->setId(companypopulate.value("company_unassigned_accnt_id").toInt());
     if (_external->isChecked())
     {
       _cachedCurrid = companypopulate.value("company_curr_id").toInt();
@@ -343,15 +315,15 @@ void company::populate()
     companypopulate.exec();
     if (companypopulate.first())
       _external->setEnabled(companypopulate.value("result").toInt() == 0);
-    else if (companypopulate.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Company Information"),
+                                  companypopulate, __FILE__, __LINE__))
     {
-      systemError(this, companypopulate.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
-  else if (companypopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Company Information"),
+                                companypopulate, __FILE__, __LINE__))
   {
-    systemError(this, companypopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sHandleTest();
@@ -383,6 +355,7 @@ void company::sTest()
   ParameterList params;
   params.append("databaseURL", dbURL);
   params.append("multipleConnections");
+  params.append("applicationName", _ConnAppName);
 
   login2 newdlg(this, "testLogin", false);
   newdlg.set(params);
@@ -422,9 +395,9 @@ void company::sTest()
         return;
       }
     }
-    else if (rmq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Connection Information"),
+                                  rmq, __FILE__, __LINE__))
     {
-      systemError(this, rmq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -457,14 +430,14 @@ void company::sTest()
         return;
       }
     }
-    else if (rmq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Connection Information"),
+                                  rmq, __FILE__, __LINE__))
     {
-      systemError(this, rmq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
-    else if (companyTest.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Connection Information"),
+                                  companyTest, __FILE__, __LINE__))
     {
-      systemError(this, companyTest.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else if (!rmq.first())
@@ -492,9 +465,9 @@ void company::sTest()
       QMessageBox::warning(this, windowTitle(), tr("Test Successful."));
       return;
     }
-    else if (rmq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Connection Information"),
+                                  rmq, __FILE__, __LINE__))
     {
-      systemError(this, rmq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else
@@ -543,11 +516,9 @@ void company::sCurrencyChanged()
                     "  WHERE (company_id=:company_id)));");
         qry.bindValue(":company_id", _companyid);
         qry.exec();
-        if (qry.lastError().type() != QSqlError::NoError)
+        if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Changing Currency"),
+                                 qry, __FILE__, __LINE__))
         {
-          systemError(this, tr("A System Error occurred at %1::%2.")
-                      .arg(__FILE__)
-                      .arg(__LINE__) );
           return;
         }
       }
