@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,8 +16,10 @@
 
 #include "configureEncryption.h"
 #include "guiclient.h"
+#include "errorReporter.h"
+#include "guiErrorCheck.h"
 
-configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::WFlags fl)
+configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::WindowFlags fl)
     : XAbstractConfigure(parent, fl)
 {
   XSqlQuery configureconfigureGL;
@@ -29,6 +31,7 @@ configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::
   _yearend->setType(GLCluster::cEquity);
   _gainLoss->setType(GLCluster::cExpense);
   _discrepancy->setType(GLCluster::cExpense);
+  _unassigned->setType(GLCluster::cLiability);
 
   // AP
   _nextAPMemoNumber->setValidator(omfgThis->orderVal());
@@ -95,6 +98,7 @@ configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::
   _reqInvoiceReg->setChecked(_metrics->boolean("ReqInvRegVoucher"));
   _reqInvoiceMisc->setChecked(_metrics->boolean("ReqInvMiscVoucher"));
   _recurringVoucherBuffer->setValue(_metrics->value("RecurringVoucherBuffer").toInt());
+  _reprint->setChecked(_metrics->boolean("ReprintPaymentNumbers"));
 
   // AR
   _nextARMemoNumber->setValidator(omfgThis->orderVal());
@@ -103,18 +107,21 @@ configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::
   configureconfigureGL.exec("SELECT currentARMemoNumber() AS result;");
   if (configureconfigureGL.first())
     _nextARMemoNumber->setText(configureconfigureGL.value("result"));
-  else if (configureconfigureGL.lastError().type() != QSqlError::NoError)
-    systemError(this, configureconfigureGL.lastError().databaseText(), __FILE__, __LINE__);
+  else
+     ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Next AR Memo Number"),
+                       configureconfigureGL, __FILE__, __LINE__);
 
   configureconfigureGL.exec("SELECT currentCashRcptNumber() AS result;");
   if (configureconfigureGL.first())
     _nextCashRcptNumber->setText(configureconfigureGL.value("result"));
-  else if (configureconfigureGL.lastError().type() != QSqlError::NoError)
-    systemError(this, configureconfigureGL.lastError().databaseText(), __FILE__, __LINE__);
+  else
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Next Cash Receipt Number"),
+                                 configureconfigureGL, __FILE__, __LINE__);
 
   _hideApplyto->setChecked(_metrics->boolean("HideApplyToBalance"));
   _customerDeposits->setChecked(_metrics->boolean("EnableCustomerDeposits"));
   _discountTax->setChecked(_metrics->boolean("CreditTaxDiscount"));
+  _altCashExchangeRate->setChecked(_metrics->boolean("AltCashExchangeRate"));
 
   _name->setText(_metrics->value("remitto_name"));
   _address->setLine1(_metrics->value("remitto_address1"));
@@ -140,15 +147,16 @@ configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::
   _externalConsolidation->setVisible(extConsolAllowed);
   if (_metrics->value("GLCompanySize").toInt() == 0)
   {
-    _useCompanySegment->setChecked(FALSE);
-    _externalConsolidation->setChecked(FALSE);
+    _useCompanySegment->setChecked(false);
+    _externalConsolidation->setChecked(false);
     _yearend->setId(_metrics->value("YearEndEquityAccount").toInt());
     _gainLoss->setId(_metrics->value("CurrencyGainLossAccount").toInt());
     _discrepancy->setId(_metrics->value("GLSeriesDiscrepancyAccount").toInt());
+    _unassigned->setId(_metrics->value("UnassignedAccount").toInt());
   }
   else
   {
-    _useCompanySegment->setChecked(TRUE);
+    _useCompanySegment->setChecked(true);
     _companySegmentSize->setValue(_metrics->value("GLCompanySize").toInt());
 
     _externalConsolidation->setChecked(_metrics->boolean("MultiCompanyFinancialConsolidation") &&
@@ -157,12 +165,12 @@ configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::
 
   if (_metrics->value("GLProfitSize").toInt() == 0)
   {
-    _useProfitCenters->setChecked(FALSE);
+    _useProfitCenters->setChecked(false);
     _cacheuseProfitCenters = false;
   }
   else
   {
-    _useProfitCenters->setChecked(TRUE);
+    _useProfitCenters->setChecked(true);
     _profitCenterSize->setValue(_metrics->value("GLProfitSize").toInt());
     _ffProfitCenters->setChecked(_metrics->boolean("GLFFProfitCenters"));
     _cacheuseProfitCenters = true;
@@ -170,12 +178,12 @@ configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::
 
   if (_metrics->value("GLSubaccountSize").toInt() == 0)
   {
-    _useSubaccounts->setChecked(FALSE);
+    _useSubaccounts->setChecked(false);
     _cacheuseSubaccounts = false;
   }
   else
   {
-    _useSubaccounts->setChecked(TRUE);
+    _useSubaccounts->setChecked(true);
     _subaccountSize->setValue(_metrics->value("GLSubaccountSize").toInt());
     _ffSubaccounts->setChecked(_metrics->boolean("GLFFSubaccounts"));
     _cacheuseSubaccounts = true;
@@ -184,18 +192,35 @@ configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::
   switch(_metrics->value("CurrencyExchangeSense").toInt())
   {
     case 1:
-      _localToBase->setChecked(TRUE);
+      _localToBase->setChecked(true);
       break;
     case 0:
     default:
-      _baseToLocal->setChecked(TRUE);
+      _baseToLocal->setChecked(true);
   }
+  _fxChangeLog->setChecked(_metrics->boolean("FXChangeLog"));
 
   _mandatoryNotes->setChecked(_metrics->boolean("MandatoryGLEntryNotes"));
   _manualFwdUpdate->setChecked(_metrics->boolean("ManualForwardUpdate"));
+  
   _taxauth->setId(_metrics->value("DefaultTaxAuthority").toInt());
   // TODO hide default tax authority, not used?
-  _taxGroup->setVisible(FALSE);
+  _taxauthLit->setVisible(false);
+  _taxauth->setVisible(false);
+  _cashBasedTax->setChecked(_metrics->boolean("CashBasedTax"));
+  _importBankRecon->setChecked(_metrics->boolean("ImportBankReconciliation"));
+  _debitBankadjtype->populate("SELECT bankadjtype_id,"
+                              "       (bankadjtype_name || '-' || bankadjtype_descrip),"
+                              "       bankadjtype_name "
+                              "FROM bankadjtype "
+                              "ORDER BY bankadjtype_name;");
+  _debitBankadjtype->setId(_metrics->value("ImportBankRecDebitAdj").toInt());
+  _creditBankadjtype->populate("SELECT bankadjtype_id,"
+                               "       (bankadjtype_name || '-' || bankadjtype_descrip),"
+                               "       bankadjtype_name "
+                               "FROM bankadjtype "
+                               "ORDER BY bankadjtype_name;");
+  _creditBankadjtype->setId(_metrics->value("ImportBankRecCreditAdj").toInt());
 
   _int2gl->setChecked(_metrics->boolean("InterfaceToGL"));
   _cacheint2gl = _int2gl->isChecked();
@@ -237,6 +262,37 @@ configureGL::configureGL(QWidget* parent, const char* name, bool /*modal*/, Qt::
     _useSubaccounts->setChecked(true);
     _cacheuseSubaccounts = true;
   }
+
+  // FC
+  
+  _annualInterestRate->setValidator(omfgThis->percentVal());
+  _financeChargeAccount->setType(GLCluster::cRevenue);
+  
+  _finchargid = -1;
+  XSqlQuery fcquery;
+  fcquery.exec("SELECT * FROM fincharg LIMIT 1;");
+  if(fcquery.first())
+  {
+    _finchargid = fcquery.value("fincharg_id").toInt();
+    _annualInterestRate->setDouble(fcquery.value("fincharg_air").toDouble());
+    _minFinanceCharge->setBaseValue(fcquery.value("fincharg_mincharg").toDouble());
+    _gracePeriod->setValue(fcquery.value("fincharg_graceperiod").toDouble());
+    if (fcquery.value("fincharg_calcfrom").toInt() == 1)
+    {
+      _dueDate->setChecked(true);
+    }
+    else
+    {
+      _invoiceDate->setChecked(true);
+    }
+    _assignOnOverdue->setChecked(fcquery.value("fincharg_assessoverdue").toBool());
+    _financeChargeLabel->setText(fcquery.value("fincharg_markoninvoice").toString());
+    _financeChargeAccount->setId(fcquery.value("fincharg_accnt_id").toInt());
+    _salesCat->setId(fcquery.value("fincharg_salescat_id").toInt());
+  }
+  else
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Finance Charge Information"),
+                    fcquery, __FILE__, __LINE__);
   
   adjustSize();
 }
@@ -286,7 +342,7 @@ bool configureGL::sSave()
     if (configureSave.first())
     {
       QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                            "You must assign G/L Accounts to all Cost Categories");
+                            "You must assign Ledger Accounts to all Cost Categories");
       return false;
     }
     if (_metrics->boolean("MultiWhs") && _metrics->boolean("Transforms"))
@@ -299,7 +355,7 @@ bool configureGL::sSave()
       if (configureSave.first())
       {
         QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                              "You must assign a Transform Clearing G/L Account to all Cost Categories");
+                              "You must assign a Transform Clearing Ledger Account to all Cost Categories");
         return false;
       }
     }
@@ -313,7 +369,7 @@ bool configureGL::sSave()
       if (configureSave.first())
       {
         QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                              "You must assign a Transfer Order Liability Clearing G/L Account to all Cost Categories");
+                              "You must assign a Transfer Order Liability Clearing Ledger Account to all Cost Categories");
         return false;
       }
     }
@@ -327,7 +383,7 @@ bool configureGL::sSave()
       if (configureSave.first())
       {
         QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                              "You must assign a Labor and Overhead Costs G/L Account to all Cost Categories");
+                              "You must assign a Labor and Overhead Costs Ledger Account to all Cost Categories");
         return false;
       }
     }
@@ -349,7 +405,7 @@ bool configureGL::sSave()
     if (configureSave.first())
     {
       QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                            "You must assign G/L Accounts to all Expense Categories");
+                            "You must assign Ledger Accounts to all Expense Categories");
       return false;
     }
     configureSave.exec("SELECT apaccnt_id "
@@ -364,7 +420,7 @@ bool configureGL::sSave()
     if (configureSave.first())
     {
       QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                            "You must assign G/L Accounts to all Payables Assignments");
+                            "You must assign Ledger Accounts to all Payables Assignments");
       return false;
     }
   }
@@ -385,7 +441,7 @@ bool configureGL::sSave()
     if (configureSave.first())
     {
       QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                            "You must assign G/L Accounts to all Receivables Assignments");
+                            "You must assign Ledger Accounts to all Receivables Assignments");
       return false;
     }
     if (_metrics->boolean("EnableCustomerDeposits"))
@@ -398,7 +454,7 @@ bool configureGL::sSave()
       if (configureSave.first())
       {
         QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                              "You must assign a Deferred Revenue G/L Account to all Receivables Assignments");
+                              "You must assign a Deferred Revenue Ledger Account to all Receivables Assignments");
         return false;
       }
     }
@@ -414,7 +470,7 @@ bool configureGL::sSave()
     if (configureSave.first())
     {
       QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                            "You must assign G/L Accounts to all Sales Assignments");
+                            "You must assign Ledger Accounts to all Sales Assignments");
       return false;
     }
     if (_metrics->boolean("EnableReturnAuth"))
@@ -431,7 +487,7 @@ bool configureGL::sSave()
       if (configureSave.first())
       {
         QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                              "You must assign a Returns G/L Account to all Sales Assignments");
+                              "You must assign a Returns Ledger Account to all Sales Assignments");
         return false;
       }
     }
@@ -447,7 +503,7 @@ bool configureGL::sSave()
     if (configureSave.first())
     {
       QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                            "You must assign G/L Accounts to all Sales Categories");
+                            "You must assign Ledger Accounts to all Sales Categories");
       return false;
     }
   }
@@ -455,41 +511,28 @@ bool configureGL::sSave()
   if (_metrics->boolean("ACHSupported"))
   {
     QString tmpCompanyId = _companyId->text();
-    struct {
-      bool    condition;
-      QString msg;
-      QWidget *widget;
-    } error[] = {
-      { _achGroup->isChecked() && _companyId->text().isEmpty(),
-        tr("Please enter a default Company Id if you are going to create "
-           "ACH files."),
-        _companyId },
-      { _achGroup->isChecked() &&
-        (_companyIdIsEIN->isChecked() || _companyIdIsDUNS->isChecked()) && 
-        tmpCompanyId.remove("-").size() != 9,
-        tr("EIN, TIN, and DUNS numbers are all 9 digit numbers. Other "
-           "characters (except dashes for readability) are not allowed."),
-        _companyId },
-      { _achGroup->isChecked() &&
-        _companyIdIsOther->isChecked() && _companyId->text().size() > 10,
-        tr("Company Ids must be 10 characters or shorter (not counting dashes "
-           "in EIN's, TIN's, and DUNS numbers)."),
-        _companyId },
-      { _achGroup->isChecked() &&
-        ! (_companyIdIsEIN->isChecked() || _companyIdIsDUNS->isChecked() ||
-           _companyIdIsOther->isChecked()),
-        tr("Please mark whether the Company Id is an EIN, TIN, DUNS number, "
-           "or Other."),
-        _companyIdIsEIN }
-    };
-    for (unsigned int i = 0; i < sizeof(error) / sizeof(error[0]); i++)
-      if (error[i].condition)
-      {
-        QMessageBox::critical(this, tr("Cannot Save Accounting Configuration"),
-                              error[i].msg);
-        error[i].widget->setFocus();
+
+    QList<GuiErrorCheck>errors;
+    errors<<GuiErrorCheck(_achGroup->isChecked() && _companyId->text().isEmpty(), _companyId,
+                          tr("Please enter a default Company Id if you are going to create ACH files."))
+         <<GuiErrorCheck(_achGroup->isChecked() &&
+                         (_companyIdIsEIN->isChecked() || _companyIdIsDUNS->isChecked()) &&
+                         tmpCompanyId.remove("-").size() != 9, _companyId,
+                         tr("EIN, TIN, and DUNS numbers are all 9 digit numbers. Other "
+                            "characters (except dashes for readability) are not allowed."))
+        <<GuiErrorCheck(_achGroup->isChecked() &&
+                        _companyIdIsOther->isChecked() && _companyId->text().size() > 10, _companyId,
+                        tr("Company Ids must be 10 characters or shorter (not counting dashes "
+                           "in EIN's, TIN's, and DUNS numbers)."))
+       <<GuiErrorCheck(_achGroup->isChecked() &&
+                       ! (_companyIdIsEIN->isChecked() || _companyIdIsDUNS->isChecked() ||
+                          _companyIdIsOther->isChecked()), _companyIdIsEIN,
+                        tr("Please mark whether the Company Id is an EIN, TIN, DUNS number, "
+                           "or Other."));
+
+    if(GuiErrorCheck::reportErrors(this,tr("Cannot Post Transaction"),errors))
         return false;
-      }
+
   }
 
   if (!_useProfitCenters->isChecked() && _cacheuseProfitCenters)
@@ -526,7 +569,7 @@ bool configureGL::sSave()
       if (check.first())
       {
         QMessageBox::critical(this, tr("Cannot turn off Profit Centers"),
-                              "Turning off Profit Centers would result in duplicate G/L Accounts.");
+                              "Turning off Profit Centers would result in duplicate Ledger Accounts.");
         return false;
       }
     }
@@ -568,7 +611,7 @@ bool configureGL::sSave()
       if (check.first())
       {
         QMessageBox::critical(this, tr("Cannot turn off Subaccounts"),
-                              "Turning off Subaccounts would result in duplicate G/L Accounts.");
+                              "Turning off Subaccounts would result in duplicate Ledger Accounts.");
         return false;
       }
     }
@@ -589,7 +632,7 @@ bool configureGL::sSave()
     if (check.first())
     {
       QMessageBox::critical(this, tr("Cannot turn off Profit Centers and Subaccounts"),
-                            "Turning off both Profit Centers and Subaccounts would result in duplicate G/L Accounts.");
+                            "Turning off both Profit Centers and Subaccounts would result in duplicate Ledger Accounts.");
       return false;
     }
   }
@@ -646,14 +689,15 @@ bool configureGL::sSave()
   _metrics->set("ReqInvRegVoucher", _reqInvoiceReg->isChecked());
   _metrics->set("ReqInvMiscVoucher", _reqInvoiceMisc->isChecked());
   _metrics->set("RecurringVoucherBuffer", _recurringVoucherBuffer->value());
+  _metrics->set("ReprintPaymentNumbers", _reprint->isChecked());
 
   // AR
   configureSave.prepare("SELECT setNextARMemoNumber(:armemo_number) AS result;");
   configureSave.bindValue(":armemo_number", _nextARMemoNumber->text().toInt());
   configureSave.exec();
-  if (configureSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Setting Next AR Memo Number"),
+                      configureSave, __FILE__, __LINE__))
   {
-    systemError(this, configureSave.lastError().databaseText(), __FILE__, __LINE__);
     _nextARMemoNumber->setFocus();
     return false;
   }
@@ -661,9 +705,9 @@ bool configureGL::sSave()
   configureSave.prepare("SELECT setNextCashRcptNumber(:cashrcpt_number) AS result;");
   configureSave.bindValue(":cashrcpt_number", _nextCashRcptNumber->text().toInt());
   configureSave.exec();
-  if (configureSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Setting Next AR Cash Receipt Number"),
+                      configureSave, __FILE__, __LINE__))
   {
-    systemError(this, configureSave.lastError().databaseText(), __FILE__, __LINE__);
     _nextCashRcptNumber->setFocus();
     return false;
   }
@@ -671,6 +715,7 @@ bool configureGL::sSave()
   _metrics->set("HideApplyToBalance", _hideApplyto->isChecked());
   _metrics->set("EnableCustomerDeposits", _customerDeposits->isChecked());
   _metrics->set("CreditTaxDiscount", _discountTax->isChecked());
+  _metrics->set("AltCashExchangeRate", _altCashExchangeRate->isChecked());
 
   _metrics->set("remitto_name", 	_name->text().trimmed());
   _metrics->set("remitto_address1",	_address->line1().trimmed());
@@ -710,6 +755,7 @@ bool configureGL::sSave()
     _metrics->set("YearEndEquityAccount", _yearend->id());
     _metrics->set("CurrencyGainLossAccount", _gainLoss->id());
     _metrics->set("GLSeriesDiscrepancyAccount", _discrepancy->id());
+    _metrics->set("UnassignedAccount", _unassigned->id());
   }
   if(companyseg)
     companyseg->setEnabled(_useCompanySegment->isChecked());
@@ -724,14 +770,14 @@ bool configureGL::sSave()
   else
   {
     _metrics->set("GLProfitSize", 0);
-    _metrics->set("GLFFProfitCenters", FALSE);
+    _metrics->set("GLFFProfitCenters", false);
     if(profitcenter)
-      profitcenter->setEnabled(FALSE);
+      profitcenter->setEnabled(false);
     XSqlQuery update;
     update.exec("UPDATE accnt SET accnt_profit=NULL,"
                 "                 accnt_sub=CASE WHEN (accnt_sub='') THEN NULL ELSE accnt_sub END;");
-    if (update.lastError().type() != QSqlError::NoError)
-        systemError(this, update.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Accounting Setup Information"),
+                      update, __FILE__, __LINE__);
   }
 
   if (_useSubaccounts->isChecked())
@@ -744,14 +790,14 @@ bool configureGL::sSave()
   else
   {
     _metrics->set("GLSubaccountSize", 0);
-    _metrics->set("GLFFSubaccounts", FALSE);
+    _metrics->set("GLFFSubaccounts", false);
     if(subaccounts)
-      subaccounts->setEnabled(FALSE);
+      subaccounts->setEnabled(false);
     XSqlQuery update;
     update.exec("UPDATE accnt SET accnt_sub=NULL,"
                 "                 accnt_profit=CASE WHEN (accnt_profit='') THEN NULL ELSE accnt_profit END;");
-    if (update.lastError().type() != QSqlError::NoError)
-        systemError(this, update.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Accounting Setup Information"),
+                      update, __FILE__, __LINE__);
   }
 
   _metrics->set("UseJournals", _journal->isChecked());
@@ -761,10 +807,17 @@ bool configureGL::sSave()
   else // if(_baseToLocal->isChecked())
     _metrics->set("CurrencyExchangeSense", 0);
 
+  _metrics->set("FXChangeLog", _fxChangeLog->isChecked());
+
   _metrics->set("MandatoryGLEntryNotes", _mandatoryNotes->isChecked());
   _metrics->set("ManualForwardUpdate", _manualFwdUpdate->isChecked());
+  
   _metrics->set("DefaultTaxAuthority", _taxauth->id());
-
+  _metrics->set("CashBasedTax", _cashBasedTax->isChecked());
+  _metrics->set("ImportBankReconciliation", _importBankRecon->isChecked());
+  _metrics->set("ImportBankRecDebitAdj", _debitBankadjtype->id());
+  _metrics->set("ImportBankRecCreditAdj", _creditBankadjtype->id());
+  
   _metrics->set("InterfaceToGL", _int2gl->isChecked());
   _metrics->set("InterfaceAPToGL", _intap2gl->isChecked());
   _metrics->set("InterfaceARToGL", _intar2gl->isChecked());
@@ -794,5 +847,50 @@ bool configureGL::sSave()
                                "system is configured to perform encryption."));
   }
 
+  // FC
+  if (_financeChargeAccount->id() > 0)
+  {
+    XSqlQuery fcSave;
+    if (_finchargid > -1)
+      fcSave.prepare("UPDATE fincharg"
+                     "  SET fincharg_mincharg=:mincharg,"
+                     "      fincharg_graceperiod=:graceperiod,"
+                     "      fincharg_assessoverdue=:assessoverdue,"
+                     "      fincharg_calcfrom=:calcfrom,"
+                     "      fincharg_markoninvoice=:markoninvoice,"
+                     "      fincharg_air=:air,"
+                     "      fincharg_accnt_id=:glaccnt,"
+                     "      fincharg_salescat_id=:salescat "
+                     "WHERE (fincharg_id=:finchargid);");
+    else
+      fcSave.prepare("INSERT INTO fincharg"
+                     "  (fincharg_id, fincharg_mincharg, fincharg_graceperiod,"
+                     "   fincharg_assessoverdue, fincharg_calcfrom, fincharg_markoninvoice,"
+                     "   fincharg_air, fincharg_accnt_id, fincharg_salescat_id) "
+                     "VALUES"
+                     "  (1, :mincharg, :graceperiod,"
+                     "   :assessoverdue, :calcfrom, :markoninvoice,"
+                     "   :air, :glaccnt, :salescat);");
+    fcSave.bindValue(":mincharg", _minFinanceCharge->baseValue());
+    fcSave.bindValue(":graceperiod", _gracePeriod->value());
+    fcSave.bindValue(":assessoverdue", _assignOnOverdue->isChecked());
+    if (_dueDate->isChecked())
+      fcSave.bindValue(":calcfrom", 1);
+    else
+      fcSave.bindValue(":calcfrom", 2);
+    fcSave.bindValue(":markoninvoice", _financeChargeLabel->text());
+    fcSave.bindValue(":air", _annualInterestRate->toDouble());
+    fcSave.bindValue(":glaccnt", _financeChargeAccount->id());
+    fcSave.bindValue(":salescat", _salesCat->id());
+    fcSave.bindValue(":finchargid", _finchargid);
+    fcSave.exec();
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Accounting Setup Information"),
+                                  fcSave, __FILE__, __LINE__))
+    {
+      return false;
+    }
+  }
+
   return true;
 }
+
