@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -12,8 +12,10 @@
 
 #include <QSqlError>
 #include <QMessageBox>
+#include <errorReporter.h>
+#include <guiErrorCheck.h>
 
-configureSO::configureSO(QWidget* parent, const char* name, bool /*modal*/, Qt::WFlags fl)
+configureSO::configureSO(QWidget* parent, const char* name, bool /*modal*/, Qt::WindowFlags fl)
     : XAbstractConfigure(parent, fl)
 {
   XSqlQuery configureconfigureSO;
@@ -65,9 +67,9 @@ configureSO::configureSO(QWidget* parent, const char* name, bool /*modal*/, Qt::
     _nextCmNumber->setText(configureconfigureSO.value("cmnumber"));
     _nextInNumber->setText(configureconfigureSO.value("innumber"));
   }
-  else if (configureconfigureSO.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving SO Setting Information"),
+                                configureconfigureSO, __FILE__, __LINE__))
   {
-    systemError(this, configureconfigureSO.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -84,16 +86,21 @@ configureSO::configureSO(QWidget* parent, const char* name, bool /*modal*/, Qt::
   _hideSOMiscChrg->setChecked(_metrics->boolean("HideSOMiscCharge"));
   _enableSOShipping->setChecked(_metrics->boolean("EnableSOShipping"));
   _printSO->setChecked(_metrics->boolean("DefaultPrintSOOnSave"));
+  _creditCheckSO->setChecked(_metrics->boolean("CreditCheckSOOnSave"));
+  _lineItemsSO->setChecked(_metrics->boolean("DefaultSOLineItemsTab"));
   _enablePromiseDate->setChecked(_metrics->boolean("UsePromiseDate"));
   _calcFreight->setChecked(_metrics->boolean("CalculateFreight"));
   _includePkgWeight->setChecked(_metrics->boolean("IncludePackageWeight"));
   _quoteafterSO->setChecked(_metrics->boolean("ShowQuotesAfterSO"));
   _itemPricingPrecedence->setChecked(_metrics->boolean("ItemPricingPrecedence"));
+  _useListPriceSchedules->setChecked(_metrics->boolean("AllowListPriceSchedules"));
   _wholesalePriceCosting->setChecked(_metrics->boolean("WholesalePriceCosting"));
   _long30Markups->setChecked(_metrics->boolean("Long30Markups"));
 
   _shipform->setId(_metrics->value("DefaultShipFormId").toInt());
   _shipvia->setId(_metrics->value("DefaultShipViaId").toInt());
+  _shipchrg->setId(_metrics->value("DefaultShipChrgId").toInt());
+  _sellingWarehouse->setId(_metrics->value("DefaultSellingWarehouseId").toInt());
 
   if (_metrics->value("DefaultBalanceMethod") == "B")
     _balanceMethod->setCurrentIndex(0);
@@ -126,7 +133,7 @@ configureSO::configureSO(QWidget* parent, const char* name, bool /*modal*/, Qt::
 
   this->setWindowTitle("Sales Configuration");
 
-  //Set status of Returns Authorization based on context
+  //Set status of Returns Authorization and Reservations based on context
   if(_metrics->value("Application") == "PostBooks")
   {
     _authNumGenerationLit->setVisible(false);
@@ -137,11 +144,14 @@ configureSO::configureSO(QWidget* parent, const char* name, bool /*modal*/, Qt::
     _enableReturns->setChecked(false);
     _enableReservations->hide();
     _enableReservations->setChecked(false);
+    _requireReservations->hide();
+    _requireReservations->setChecked(false);
     _locationGroup->hide();
     _locationGroup->setChecked(false);
     _lowest->setChecked(false);
     _highest->setChecked(false);
     _alpha->setChecked(false);
+    _manualReservations->setChecked(false);
   }
   else
   {
@@ -205,10 +215,13 @@ configureSO::configureSO(QWidget* parent, const char* name, bool /*modal*/, Qt::
 
     _returnAuthChangeLog->setChecked(_metrics->boolean("ReturnAuthorizationChangeLog"));
     _printRA->setChecked(_metrics->boolean("DefaultPrintRAOnSave"));
+    _closeRA->setChecked(_metrics->boolean("CloseRAOnCredit"));
 
     _enableReservations->setChecked(_metrics->boolean("EnableSOReservations"));
+    _requireReservations->setChecked(_metrics->boolean("RequireSOReservations"));
 
     _locationGroup->setChecked(_metrics->boolean("EnableSOReservationsByLocation"));
+    _manualReservations->setChecked(_metrics->boolean("SOManualReservations"));
     if(_metrics->value("SOReservationLocationMethod").toInt() == 1)
       _lowest->setChecked(true);
     else if (_metrics->value("SOReservationLocationMethod").toInt() == 2)
@@ -216,6 +229,21 @@ configureSO::configureSO(QWidget* parent, const char* name, bool /*modal*/, Qt::
     else if(_metrics->value("SOReservationLocationMethod").toInt() == 3)
       _alpha->setChecked(true);
   }
+  
+  _ssosCust->setType(CLineEdit::ActiveCustomers);
+  _enableSSOS->setChecked(_metrics->boolean("SSOSEnabled"));
+  if (_enableSSOS->isChecked())
+  {
+    _ssosCust->setId(_metrics->value("SSOSDefaultCustId").toInt());
+    _ssosSaleType->setId(_metrics->value("SSOSDefaultSaleTypeId").toInt());
+    _ssosRequireInv->setChecked(_metrics->boolean("SSOSRequireInv"));
+    _ssosPrintSOAck->setChecked(_metrics->boolean("SSOSPrintSOAck"));
+    _ssosPrintPackList->setChecked(_metrics->boolean("SSOSPrintPackList"));
+    _ssosPrintInvoice->setChecked(_metrics->boolean("SSOSPrintInvoice"));
+  }
+  else
+    _ssosGroup->setEnabled(false);
+  
   adjustSize();
 }
 
@@ -242,15 +270,15 @@ bool configureSO::sSave()
        (!_locationGroup->isChecked()) )
   {
     if (QMessageBox::warning(this, tr("Reserve by Location Disabled"),
-                             tr("<p>All existing location reservations will be removed. Are you sure you want to continue?"),
+                             tr("<p>All existing Sales Order location reservations will be removed. Are you sure you want to continue?"),
                              QMessageBox::Yes, QMessageBox::No | QMessageBox::Default) == QMessageBox::No)
     {
       return false;
     }
     else
     {
-      configureSave.prepare("DELETE FROM itemlocrsrv "
-                " WHERE (itemlocrsrv_source='SO');");
+      configureSave.prepare("DELETE FROM reserve "
+                " WHERE (reserve_demand_type='SO');");
       configureSave.exec();
     }	
   }
@@ -274,6 +302,8 @@ bool configureSO::sSave()
   _metrics->set("InvcNumberGeneration", _invoiceNumGeneration->methodCode());
   _metrics->set("DefaultShipFormId", _shipform->id());
   _metrics->set("DefaultShipViaId", _shipvia->id());
+  _metrics->set("DefaultShipChrgId", _shipchrg->id());
+  _metrics->set("DefaultSellingWarehouseId", _sellingWarehouse->id());
   _metrics->set("DefaultCustType", _custtype->id());
   _metrics->set("DefaultSalesRep", _salesrep->id());
   _metrics->set("DefaultTerms", _terms->id());
@@ -281,17 +311,22 @@ bool configureSO::sSave()
   _metrics->set("DefaultBackOrders", _backorders->isChecked());
   _metrics->set("DefaultFreeFormShiptos", _freeFormShiptos->isChecked());
   _metrics->set("DefaultPrintSOOnSave", _printSO->isChecked());
+  _metrics->set("CreditCheckSOOnSave", _creditCheckSO->isChecked());
+  _metrics->set("DefaultSOLineItemsTab", _lineItemsSO->isChecked());
   _metrics->set("UsePromiseDate", _enablePromiseDate->isChecked());
   _metrics->set("CalculateFreight", _calcFreight->isChecked());
   _metrics->set("IncludePackageWeight", _includePkgWeight->isChecked());
   _metrics->set("EnableReturnAuth", _enableReturns->isChecked());
   _metrics->set("EnableSOReservations", _enableReservations->isChecked());
+  _metrics->set("RequireSOReservations", _enableReservations->isChecked() && _requireReservations->isChecked());
   _metrics->set("ItemPricingPrecedence", _itemPricingPrecedence->isChecked());
+  _metrics->set("AllowListPriceSchedules", _useListPriceSchedules->isChecked());
   _metrics->set("WholesalePriceCosting", _wholesalePriceCosting->isChecked());
   _metrics->set("Long30Markups", _long30Markups->isChecked());
 
-  _metrics->set("EnableSOReservationsByLocation", _locationGroup->isChecked());
-  //SOReservationLocationMethod are three Options Either 
+  _metrics->set("EnableSOReservationsByLocation", _enableReservations->isChecked() && _locationGroup->isChecked());
+  _metrics->set("SOManualReservations", _enableReservations->isChecked() && _locationGroup->isChecked() && _manualReservations->isChecked());
+  //SOReservationLocationMethod are three Options Either
   // Lowest quantity first,
   // Highest quantity first,
   // Alpha by Location Name
@@ -354,9 +389,9 @@ bool configureSO::sSave()
   configureSave.bindValue(":cmnumber", _nextCmNumber->text().toInt());
   configureSave.bindValue(":innumber", _nextInNumber->text().toInt());
   configureSave.exec();
-  if (configureSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving SO Setting Information"),
+                                configureSave, __FILE__, __LINE__))
   {
-    systemError(this, configureSave.lastError().databaseText(), __FILE__, __LINE__);
     return false;
   }
 
@@ -367,18 +402,48 @@ bool configureSO::sSave()
     _metrics->set("DefaultRaCreditMethod", QString(creditMethodTypes[_creditBy->currentIndex()]));
     _metrics->set("ReturnAuthorizationChangeLog", _returnAuthChangeLog->isChecked());
     _metrics->set("DefaultPrintRAOnSave", _printRA->isChecked());
+    _metrics->set("CloseRAOnCredit", _closeRA->isChecked());
     _metrics->set("RANumberGeneration", _returnAuthorizationNumGeneration->methodCode());
 
     configureSave.prepare( "SELECT setNextRaNumber(:ranumber);" );
     configureSave.bindValue(":ranumber", _nextRaNumber->text().toInt());
     configureSave.exec();
-    if (configureSave.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving SO Setting Information"),
+                                  configureSave, __FILE__, __LINE__))
     {
-      systemError(this, configureSave.lastError().databaseText(), __FILE__, __LINE__);
       return false;
     }
   }
   _metrics->set("EnableReturnAuth", (_enableReturns->isChecked() || !_enableReturns->isCheckable()));
+  
+  if (_enableSSOS->isChecked())
+  {
+
+    QList<GuiErrorCheck>errors;
+    errors<<GuiErrorCheck(!_ssosCust->isValid(), _ssosCust,
+                          tr("<p>You must select a Cash Customer # if Simple S/O is enabled."));
+
+    if(GuiErrorCheck::reportErrors(this,tr("Unable To Apply Sales Settings"),errors))
+        return false;
+    
+    _metrics->set("SSOSEnabled", true);
+    _metrics->set("SSOSDefaultCustId", _ssosCust->id());
+    _metrics->set("SSOSDefaultSaleTypeId", _ssosSaleType->id());
+    _metrics->set("SSOSRequireInv", _ssosRequireInv->isChecked());
+    _metrics->set("SSOSPrintSOAck", _ssosPrintSOAck->isChecked());
+    _metrics->set("SSOSPrintPackList", _ssosPrintPackList->isChecked());
+    _metrics->set("SSOSPrintInvoice", _ssosPrintInvoice->isChecked());
+  }
+  else
+  {
+    _metrics->set("SSOSEnabled", false);
+    _metrics->set("SSOSDefaultCustId", -1);
+    _metrics->set("SSOSDefaultSaleTypeId", -1);
+    _metrics->set("SSOSRequireInv", false);
+    _metrics->set("SSOSPrintSOAck", false);
+    _metrics->set("SSOSPrintPackList", false);
+    _metrics->set("SSOSPrintInvoice", false);
+  }
 
   return true;
 }
