@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -10,7 +10,7 @@
 
 #include "confirmAchOK.h"
 
-confirmAchOK::confirmAchOK(QWidget* parent, const char* name, Qt::WFlags fl)
+confirmAchOK::confirmAchOK(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XDialog(parent, name, fl)
 {
   setupUi(this);
