@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,7 +19,7 @@
 #include "errorReporter.h"
 #include "guiErrorCheck.h"
 
-countSlip::countSlip(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+countSlip::countSlip(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
@@ -34,13 +34,13 @@ countSlip::countSlip(QWidget* parent, const char* name, bool modal, Qt::WFlags f
   connect(_item, SIGNAL(newId(int)), this, SLOT(sPopulateItemSiteInfo()));
   connect(_warehouse, SIGNAL(newID(int)), this, SLOT(sPopulateItemSiteInfo()));
 
-  _captive = FALSE;
+  _captive = false;
 
-#ifndef Q_WS_MAC
+#ifndef Q_OS_MAC
   _countTagList->setMaximumWidth(25);
 #endif
 
-  _item->setReadOnly(TRUE);
+  _item->setReadOnly(true);
   _qty->setValidator(omfgThis->qtyVal());
   _expiration->setAllowNullDate(true);
   _expiration->setAllowNullDate(false);
@@ -80,7 +80,7 @@ enum SetResponse countSlip::set(const ParameterList &pParams)
   param = pParams.value("cnttag_id", &valid);
   if (valid)
   {
-    _captive = TRUE;
+    _captive = true;
 
     _cnttagid = param.toInt();
     populateTagInfo();
@@ -89,7 +89,7 @@ enum SetResponse countSlip::set(const ParameterList &pParams)
   param = pParams.value("cntslip_id", &valid);
   if (valid)
   {
-    _captive = TRUE;
+    _captive = true;
 
     _cntslipid = param.toInt();
     populate();
@@ -107,7 +107,7 @@ enum SetResponse countSlip::set(const ParameterList &pParams)
     else if (param.toString() == "edit")
     {
       _mode = cEdit;
-      _captive = TRUE;
+      _captive = true;
 
       countet.prepare( "SELECT cntslip_posted "
                  "FROM cntslip "
@@ -124,16 +124,16 @@ enum SetResponse countSlip::set(const ParameterList &pParams)
       }
 
       _countTagList->hide();
-      _number->setEnabled(FALSE);
+      _number->setEnabled(false);
     }
     else if (param.toString() == "view")
     {
       _mode = cView;
 
       _countTagList->hide();
-      _number->setEnabled(FALSE);
-      _qty->setEnabled(FALSE);
-      _comments->setEnabled(FALSE);
+      _number->setEnabled(false);
+      _qty->setEnabled(false);
+      _comments->setEnabled(false);
       _close->setText(tr("&Close"));
       _save->hide();
     }
@@ -141,15 +141,15 @@ enum SetResponse countSlip::set(const ParameterList &pParams)
     {
       _mode = cPost;
 
-      _location->setEnabled(FALSE);
-      _lotSerial->setEnabled(FALSE);
+      _location->setEnabled(false);
+      _lotSerial->setEnabled(false);
       _countTagList->hide();
-      _number->setEnabled(FALSE);
-      _qty->setEnabled(FALSE);
-      _comments->setEnabled(FALSE);
+      _number->setEnabled(false);
+      _qty->setEnabled(false);
+      _comments->setEnabled(false);
       _save->setText(tr("&Post"));
-      _expiration->setEnabled(FALSE);
-      _warranty->setEnabled(FALSE);
+      _expiration->setEnabled(false);
+      _warranty->setEnabled(false);
     }
   }
 
@@ -303,6 +303,28 @@ void countSlip::sSave()
       }
     }
 
+    // Check for duplicate Serial # in different Site from Site being counted
+    if (_metrics->boolean("MultiWhs"))
+    {
+      countSave.prepare( "SELECT itemloc_id "
+                        "FROM itemsite JOIN itemloc ON (itemloc_itemsite_id=itemsite_id)"
+                        "              JOIN ls ON (ls_id=itemloc_ls_id)"
+                        "WHERE ( (itemsite_item_id=:item_id)"
+                        "  AND   (itemsite_warehous_id != :warehous_id)"
+                        "  AND   (itemsite_controlmethod='S')"
+                        "  AND   (UPPER(ls_number)=UPPER(:cntslip_lotserial)) );" );
+      countSave.bindValue(":item_id", _item->id());
+      countSave.bindValue(":warehous_id", _warehouse->id());
+      countSave.bindValue(":cntslip_lotserial", _lotSerial->text());
+      countSave.exec();
+      if (countSave.first())
+      {
+        errors << GuiErrorCheck(true, _lotSerial,
+                                tr( "This Serial # exists in a different Site.\n"
+                                   "Please verify the Serial # of the Count Slip you are entering." ));
+      }
+    }
+    
     if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Count Slip"), errors))
       return;
 
@@ -320,7 +342,7 @@ void countSlip::sSave()
                "  cntslip_lotserial_warrpurc,"
                "  cntslip_comments ) "
                "SELECT :cntslip_id, :cnttag_id,"
-               "       getEffectiveXtUser(), CURRENT_TIMESTAMP, FALSE,"
+               "       getEffectiveXtUser(), CURRENT_TIMESTAMP, false,"
                "       :cntslip_number, :cntslip_qty,"
                "       :cntslip_location_id, :cntslip_lotserial,"
                "       :cntslip_lotserial_expiration,"
@@ -374,9 +396,9 @@ void countSlip::sSave()
     _qty->clear();
     _comments->clear();
     _location->clear();
-    _location->setEnabled(FALSE);
+    _location->setEnabled(false);
     _lotSerial->clear();
-    _lotSerial->setEnabled(FALSE);
+    _lotSerial->setEnabled(false);
     _close->setText(tr("&Close"));
 
     _countTagList->setFocus();
@@ -389,7 +411,7 @@ void countSlip::sCountTagList()
   params.append("cnttag_id", _cnttagid);
   params.append("tagType", cUnpostedCounts);
 
-  countTagList newdlg(this, "", TRUE);
+  countTagList newdlg(this, "", true);
   newdlg.set(params);
   _cnttagid = newdlg.exec();
 
@@ -456,7 +478,8 @@ void countSlip::sPopulateItemSiteInfo()
                         "FROM location, itemsite "
                         "WHERE ( (validLocation(location_id, itemsite_id))"
                         " AND (itemsite_warehous_id=:warehous_id)"
-                        " AND (itemsite_item_id=:item_id) ) "
+                        " AND (itemsite_item_id=:item_id) "
+                        " AND (location_active) ) "
                         "ORDER BY locationname;" );
       location.bindValue(":warehous_id", _warehouse->id());
       location.bindValue(":item_id", _item->id());
@@ -464,29 +487,29 @@ void countSlip::sPopulateItemSiteInfo()
 
       _location->populate(location);
       _location->setId(countPopulateItemSiteInfo.value("itemsite_location_id").toInt());
-      _location->setEnabled(TRUE);
+      _location->setEnabled(true);
     }
     else
     {
       _location->clear();
-      _location->setEnabled(FALSE);
+      _location->setEnabled(false);
     }
 
     if (controlMethod == "L")
     {
-      _lotSerial->setEnabled(TRUE);
-      _qty->setEnabled(TRUE);
+      _lotSerial->setEnabled(true);
+      _qty->setEnabled(true);
     }
     else if (controlMethod == "S")
     {
-      _lotSerial->setEnabled(TRUE);
+      _lotSerial->setEnabled(true);
       _qty->setText("1.0");
-      _qty->setEnabled(FALSE);
+      _qty->setEnabled(false);
     }
     else
     {
-      _lotSerial->setEnabled(FALSE);
-      _qty->setEnabled(TRUE);
+      _lotSerial->setEnabled(false);
+      _qty->setEnabled(true);
     }
   }
 }
