@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -14,8 +14,9 @@
 #include <QSqlError>
 #include <QValidator>
 #include <QVariant>
+#include "guiErrorCheck.h"
 
-createItemSitesByClassCode::createItemSitesByClassCode(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+createItemSitesByClassCode::createItemSitesByClassCode(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
@@ -69,10 +70,10 @@ createItemSitesByClassCode::createItemSitesByClassCode(QWidget* parent, const ch
 
   _classCode->setType(ParameterGroup::ClassCode);
 
-  _plannerCode->setAllowNull(TRUE);
+  _plannerCode->setAllowNull(true);
   _plannerCode->setType(XComboBox::PlannerCodes);
 
-  _costcat->setAllowNull(TRUE);
+  _costcat->setAllowNull(true);
   _costcat->setType(XComboBox::CostCategories);
 
   _controlMethod->append(0, "None",     "N");
@@ -102,7 +103,7 @@ createItemSitesByClassCode::createItemSitesByClassCode(QWidget* parent, const ch
   }
   else
   {
-    _warehouse->setAllowNull(TRUE);
+    _warehouse->setAllowNull(true);
     _warehouse->setNull();
   }
 
@@ -141,67 +142,38 @@ void createItemSitesByClassCode::languageChange()
 void createItemSitesByClassCode::sSave()
 {
   XSqlQuery createSave;
-  if (_warehouse->id() == -1)
-  {
-    QMessageBox::critical( this, tr("Select a Site"),
-                           tr("<p>You must select a Site for this Item Site "
-                              "before creating it." ) );
-    _warehouse->setFocus();
-    return;
-  }
 
-//  if ( (_metrics->boolean("InterfaceToGL")) && (_costcat->id() == -1) )
-  if (_costcat->id() == -1)
-  {
-    QMessageBox::critical( this, tr("Cannot Create Item Sites"),
-                           tr("<p>You must select a Cost Category for these "
-                              "Item Sites before you may create them.") );
-    _costcat->setFocus();
-    return;
-  } 
+  QList<GuiErrorCheck>errors;
+  errors<<GuiErrorCheck(_warehouse->id() == -1, _warehouse,
+                        tr("<p>You must select a Site for this Item Site "
+                           "before creating it."))
+       <<GuiErrorCheck(_costcat->id() == -1, _costcat,
+                       tr("<p>You must select a Cost Category for these "
+                          "Item Sites before you may create them."))
+      <<GuiErrorCheck(_plannerCode->id() == -1, _plannerCode,
+                      tr("<p>You must select a Planner Code for these "
+                         "Item Sites before you may create them."))
+      <<GuiErrorCheck(_controlMethod->currentIndex() == -1, _controlMethod,
+                      tr("<p>You must select a Control Method for these "
+                         "Item Sites before you may create them."))
+      <<GuiErrorCheck(!_costNone->isChecked() && !_costAvg->isChecked()
+                                        && !_costStd->isChecked(), _costNone,
+                       tr("<p>You must select a Cost Method for this "
+                          "Item Site before you may save it."))
+      <<GuiErrorCheck(_stocked->isChecked() && _reorderLevel->toDouble() == 0, _reorderLevel,
+                      tr("<p>You must set a reorder level "
+                         "for a stocked item before you may save it."));
 
-  if (_plannerCode->id() == -1)
-  {
-    QMessageBox::critical( this, tr("Cannot Create Item Sites"),
-                           tr("<p>You must select a Planner Code for these "
-                              "Item Sites before you may create them.") );
-    _plannerCode->setFocus();
-    return;
-  } 
-
-  if (_controlMethod->currentIndex() == -1)
-  {
-    QMessageBox::critical( this, tr("Cannot Create Item Sites"),
-                           tr("<p>You must select a Control Method for these "
-                              "Item Sites before you may create them.") );
-    _controlMethod->setFocus();
-    return;
-  }
-
-  if(!_costNone->isChecked() && !_costAvg->isChecked()
-   && !_costStd->isChecked())
-  {
-    QMessageBox::critical(this, tr("Cannot Save Item Site"),
-                          tr("<p>You must select a Cost Method for this "
-                             "Item Site before you may save it.") );
-    return;
-  }
-
-  if (_stocked->isChecked() && _reorderLevel->toDouble() == 0)
-  {
-    QMessageBox::critical( this, tr("Cannot Save Item Site"),
-                           tr("<p>You must set a reorder level "
-			      "for a stocked item before you may save it.") );
-    _reorderLevel->setFocus();
-    return;
-  }
+  if(GuiErrorCheck::reportErrors(this,tr("Cannot Save Item Site"),errors))
+      return;
   
   if ( _locationControl->isChecked() )
   {
     XSqlQuery locationid;
     locationid.prepare( "SELECT location_id "
                         "FROM location "
-                        "WHERE (location_warehous_id=:warehous_id)"
+                        "WHERE ((location_warehous_id=:warehous_id) "
+                        " AND (location_active) ) " 
                         "LIMIT 1;" );
     locationid.bindValue(":warehous_id", _warehouse->id());
     locationid.exec();
@@ -238,9 +210,9 @@ void createItemSitesByClassCode::sSave()
                "  itemsite_cosdefault, itemsite_lsseq_id ) "
                "SELECT item_id,"
                "       :warehous_id, 0.0, 0.0,"
-               "       CASE WHEN item_type IN ('B', 'F', 'R', 'L', 'K') THEN FALSE "
+               "       CASE WHEN item_type IN ('B', 'F', 'R', 'L', 'K') THEN false "
                "            ELSE :itemsite_useparams       END,"
-               "       CASE WHEN item_type IN ('B', 'F', 'R', 'L', 'K') THEN FALSE "
+               "       CASE WHEN item_type IN ('B', 'F', 'R', 'L', 'K') THEN false "
                "            ELSE :itemsite_useparamsmanual END,"
                "       CASE WHEN item_type IN ('B', 'F', 'R', 'L', 'K') THEN 0"
                "            ELSE :itemsite_reorderlevel END,"
@@ -265,9 +237,9 @@ void createItemSitesByClassCode::sSave()
                "       :itemsite_createpr, :itemsite_createwo,"
                "       :itemsite_sold, :itemsite_soldranking,"
                "       :itemsite_stocked, :itemsite_disallowblankwip,"
-               "       :itemsite_controlmethod, :itemsite_perishable, TRUE,"
+               "       :itemsite_controlmethod, :itemsite_perishable, true,"
                "       CASE WHEN item_type IN ('B', 'F', 'R', 'L', 'K') "
-               "                 THEN FALSE "
+               "                 THEN false "
                "            ELSE :itemsite_loccntrl END,"
                "       CASE WHEN item_type IN ('B', 'F', 'R', 'L', 'K') "
                "                 THEN -1 "
@@ -276,9 +248,9 @@ void createItemSitesByClassCode::sSave()
                "                 THEN '' "
                "            ELSE :itemsite_location END,"
                "       :itemsite_location_comments, '',"
-               "       :itemsite_abcclass, FALSE, startOfTime(),"
+               "       :itemsite_abcclass, false, startOfTime(),"
                "       :itemsite_ordergroup, :itemsite_ordergroup_first, :itemsite_mps_timefence, "
-               "       FALSE, "
+               "       false, "
                "       CASE WHEN(item_type='L') THEN 'M' ELSE :itemsite_planning_type END, "
                "       CASE WHEN(item_type='R') THEN 'N' ELSE :itemsite_costmethod END, "
                "       :itemsite_cosdefault,  :itemsite_lsseq_id "
@@ -382,11 +354,11 @@ void createItemSitesByClassCode::sHandlePOSupply(bool /*pSupplied*/)
 {
     /*
   if (pSupplied)
-    _createPr->setEnabled(TRUE);
+    _createPr->setEnabled(true);
   else
   {
-    _createPr->setEnabled(FALSE);
-    _createPr->setChecked(FALSE);
+    _createPr->setEnabled(false);
+    _createPr->setChecked(false);
   }
   */
 } 
@@ -394,11 +366,11 @@ void createItemSitesByClassCode::sHandlePOSupply(bool /*pSupplied*/)
 void createItemSitesByClassCode::sHandleWOSupply(bool pSupplied)
 {
   if (pSupplied)
-    _createWo->setEnabled(TRUE);
+    _createWo->setEnabled(true);
   else
   {
-    _createWo->setEnabled(FALSE);
-    _createWo->setChecked(FALSE);
+    _createWo->setEnabled(false);
+    _createWo->setChecked(false);
   }
 } 
 
@@ -425,11 +397,11 @@ void createItemSitesByClassCode::sHandleControlMethod()
 
   if ( (_controlMethod->currentIndex() == 2) ||
        (_controlMethod->currentIndex() == 3) )
-    _perishable->setEnabled(TRUE);
+    _perishable->setEnabled(true);
   else
   {
-    _perishable->setChecked(FALSE);
-    _perishable->setEnabled(FALSE);
+    _perishable->setChecked(false);
+    _perishable->setEnabled(false);
   }
 }
 
@@ -437,21 +409,21 @@ void createItemSitesByClassCode::sHandlePlanningType()
 {
   if (_planningType->code() == "M" || _planningType->code() == "S")
   {
-    _createPlannedTransfers->setEnabled(TRUE);
-    _orderGroup->setEnabled(TRUE);
-    _orderGroupFirst->setEnabled(TRUE);
+    _createPlannedTransfers->setEnabled(true);
+    _orderGroup->setEnabled(true);
+    _orderGroupFirst->setEnabled(true);
 
     if (_planningType->code() == "S")
-      _mpsTimeFence->setEnabled(TRUE);
+      _mpsTimeFence->setEnabled(true);
     else
-      _mpsTimeFence->setEnabled(FALSE);
+      _mpsTimeFence->setEnabled(false);
   }
   else
   {
-    _createPlannedTransfers->setEnabled(FALSE);
-    _orderGroup->setEnabled(FALSE);
-    _orderGroupFirst->setEnabled(FALSE);
-    _mpsTimeFence->setEnabled(FALSE);
+    _createPlannedTransfers->setEnabled(false);
+    _orderGroup->setEnabled(false);
+    _orderGroupFirst->setEnabled(false);
+    _mpsTimeFence->setEnabled(false);
   }
 
 } 
@@ -462,7 +434,8 @@ void createItemSitesByClassCode::populateLocations()
   createpopulateLocations.prepare( "SELECT location_id, formatLocationName(location_id) AS locationname "
              "FROM location "
              "WHERE ( (location_warehous_id=:warehous_id)"
-             " AND (NOT location_restrict) ) "
+             " AND (NOT location_restrict) "
+             " AND (location_active) ) "
              "ORDER BY locationname;" );
   createpopulateLocations.bindValue(":warehous_id", _warehouse->id());
   createpopulateLocations.exec();
@@ -470,20 +443,20 @@ void createItemSitesByClassCode::populateLocations()
 
   if (_locations->count())
   {
-    _location->setEnabled(TRUE);
-    _locations->setEnabled(TRUE);
+    _location->setEnabled(true);
+    _locations->setEnabled(true);
   }
   else
   {
-    _location->setEnabled(FALSE);
-    _locations->setEnabled(FALSE);
+    _location->setEnabled(false);
+    _locations->setEnabled(false);
   }
 }
 
 void createItemSitesByClassCode::clear()
 {
-  _useParameters->setChecked(FALSE);
-  _useParametersOnManual->setChecked(FALSE);
+  _useParameters->setChecked(false);
+  _useParametersOnManual->setChecked(false);
   _reorderLevel->setDouble(0.0);
   _orderUpToQty->setDouble(0.0);
   _minimumOrder->setDouble(0.0);
@@ -492,7 +465,7 @@ void createItemSitesByClassCode::clear()
   _safetyStock->setDouble(0.0);
 
   _orderGroup->setValue(1);
-  _orderGroupFirst->setChecked(FALSE);
+  _orderGroupFirst->setChecked(false);
 
   _cycleCountFreq->setValue(0);
   _leadTime->setValue(0);
@@ -500,19 +473,19 @@ void createItemSitesByClassCode::clear()
 
   _controlMethod->setCurrentIndex(1);
   sHandleControlMethod();
-  _poSupply->setChecked(TRUE);
-  _woSupply->setChecked(TRUE);
-  _sold->setChecked(TRUE);
-  _stocked->setChecked(FALSE);
+  _poSupply->setChecked(true);
+  _woSupply->setChecked(true);
+  _sold->setChecked(true);
+  _stocked->setChecked(false);
 
-  _locationControl->setChecked(FALSE);
-  _useDefaultLocation->setChecked(FALSE);
+  _locationControl->setChecked(false);
+  _useDefaultLocation->setChecked(false);
   _miscLocationName->clear();
   _locationComments->clear();
 
   _costcat->setId(-1);
   
-  _woCostGroup->setChecked(FALSE);
+  _woCostGroup->setChecked(false);
 
   populateLocations();
 }
