@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -14,6 +14,7 @@
 #include "guiclient.h"
 #include "xdialog.h"
 #include <parameter.h>
+#include "lotSerialUtils.h"
 
 #include "ui_createLotSerial.h"
 
@@ -22,13 +23,15 @@ class createLotSerial : public XDialog, public Ui::createLotSerial
     Q_OBJECT
 
 public:
-    createLotSerial(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    createLotSerial(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~createLotSerial();
 
 public slots:
     virtual enum SetResponse set(const ParameterList & pParams );
+    virtual void clearCharacteristics();
     virtual void sAssign();
     virtual void sHandleLotSerial();
+    virtual void sHandleCharacteristics();
     virtual void sLotSerialSelected();
 
 protected slots:
@@ -41,6 +44,10 @@ private:
     int  _itemsiteid;
     bool _serial;
     bool _preassigned;
+    LotSerialUtils _lschars;
+    QList<QWidget *> _charWidgets;
+    bool _lotsFound;
+
 };
 
 #endif // CREATELOTSERIAL_H
