@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -17,7 +17,6 @@
 
 #include <metasql.h>
 
-#include "characteristicAssignment.h"
 #include "customer.h"
 #include "employee.h"
 #include "errorReporter.h"
@@ -32,11 +31,10 @@
 #include "taxAuthority.h"
 #include "user.h"
 #include "vendor.h"
-#include "vendorWorkBench.h"
 
 #define DEBUG false
 
-crmaccount::crmaccount(QWidget* parent, const char* name, Qt::WFlags fl)
+crmaccount::crmaccount(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
@@ -69,12 +67,9 @@ crmaccount::crmaccount(QWidget* parent, const char* name, Qt::WFlags fl)
   connect(_close,               SIGNAL(clicked()), this, SLOT(sClose()));
   connect(_competitor,          SIGNAL(clicked()), this, SLOT(sCompetitor()));
   connect(_customerButton,      SIGNAL(clicked()), this, SLOT(sCustomer()));
-  connect(_deleteCharacteristic,SIGNAL(clicked()), this, SLOT(sDeleteCharacteristic()));
   connect(_deleteReg,           SIGNAL(clicked()), this, SLOT(sDeleteReg()));
-  connect(_editCharacteristic,  SIGNAL(clicked()), this, SLOT(sEditCharacteristic()));
   connect(_editReg,             SIGNAL(clicked()), this, SLOT(sEditReg()));
   connect(_employeeButton,      SIGNAL(clicked()), this, SLOT(sEmployee()));
-  connect(_newCharacteristic,   SIGNAL(clicked()), this, SLOT(sNewCharacteristic()));
   connect(_newReg,              SIGNAL(clicked()), this, SLOT(sNewReg()));
   connect(_partner,             SIGNAL(clicked()), this, SLOT(sPartner()));
   connect(_prospectButton,      SIGNAL(clicked()), this, SLOT(sProspect()));
@@ -99,8 +94,7 @@ crmaccount::crmaccount(QWidget* parent, const char* name, Qt::WFlags fl)
   connect(_allButton, SIGNAL(toggled(bool)), this, SLOT(sHandleButtons()));
   connect(_contacts, SIGNAL(cntctDetached(int)), this, SLOT(sHandleCntctDetach(int)));
 
-  _charass->addColumn(tr("Characteristic"), _itemColumn, Qt::AlignLeft, true, "char_name");
-  _charass->addColumn(tr("Value"),          -1,          Qt::AlignLeft, true, "charass_value");
+  _charass->setType("CRMACCT");
 
   _reg->addColumn(tr("Lot/Serial")  ,  _itemColumn,  Qt::AlignLeft, true, "ls_number" );
   _reg->addColumn(tr("Item")        ,  _itemColumn,  Qt::AlignLeft, true, "item_number" );
@@ -127,6 +121,7 @@ crmaccount::crmaccount(QWidget* parent, const char* name, Qt::WFlags fl)
   _vendId       = -1;
   _comments->setId(-1);
   _documents->setId(-1);
+  _charass->setId(-1);
   _modal        = parent && (parent->inherits("customer") ||
                              parent->inherits("employee") ||
                              parent->inherits("prospect") ||
@@ -211,29 +206,15 @@ enum SetResponse crmaccount::set(const ParameterList &pParams)
         return UndefinedError;
 
       _mode = cNew;
-
-      connect(_charass, SIGNAL(valid(bool)), _editCharacteristic, SLOT(setEnabled(bool)));
-      connect(_charass, SIGNAL(valid(bool)), _deleteCharacteristic, SLOT(setEnabled(bool)));
     }
     else if (param.toString() == "edit")
     {
       _mode = cEdit;
-
-      connect(_charass, SIGNAL(valid(bool)), _editCharacteristic, SLOT(setEnabled(bool)));
-      connect(_charass, SIGNAL(valid(bool)), _deleteCharacteristic, SLOT(setEnabled(bool)));
     }
     else if (param.toString() == "view")
     {
       _mode = cView;
-
-      _save->hide();
-      disconnect(_reg, SIGNAL(valid(bool)), _editReg, SLOT(setEnabled(bool)));
-      disconnect(_reg, SIGNAL(valid(bool)), _deleteReg, SLOT(setEnabled(bool)));
-      disconnect(_reg, SIGNAL(valid(bool)), _editReg, SLOT(animateClick()));
-
-      ParameterList params;
-      params.append("mode", "view");
-      _contacts->set(params);
+      setViewMode();
     }
   }
 
@@ -249,13 +230,26 @@ enum SetResponse crmaccount::set(const ParameterList &pParams)
   if(pParams.inList("scripted"))
     _tab->setCurrentIndex(_tab->indexOf(_todoListTab));
 
+  if (_mode == cEdit && !_lock.acquire("crmacct", _crmacctId, AppLock::Interactive))
+  {
+    _mode = cView;
+    setViewMode();
+  }
+
+  sHandleChildButtons();
+
+  return NoError;
+}
+
+void crmaccount::setViewMode()
+{
   bool canEdit = (_mode == cNew || _mode == cEdit);
 
   _active->setEnabled(canEdit);
   _comments->setReadOnly(! canEdit);
   _documents->setReadOnly(! canEdit);
+  _charass->setReadOnly(! canEdit);
   _name->setEnabled(canEdit);
-  _newCharacteristic->setEnabled(canEdit);
   _newReg->setEnabled(canEdit);
   _notes->setEnabled(canEdit);
   _number->setEnabled(canEdit && _metrics->value("CRMAccountNumberGeneration") != "A");
@@ -265,9 +259,17 @@ enum SetResponse crmaccount::set(const ParameterList &pParams)
   _secondary->setEnabled(canEdit);
   _typeBox->setEnabled(canEdit);
 
-  sHandleChildButtons();
+  if (_mode == cView)
+  {
+    _save->hide();
+    disconnect(_reg, SIGNAL(valid(bool)), _editReg, SLOT(setEnabled(bool)));
+    disconnect(_reg, SIGNAL(valid(bool)), _deleteReg, SLOT(setEnabled(bool)));
+    disconnect(_reg, SIGNAL(valid(bool)), _editReg, SLOT(animateClick()));
 
-  return NoError;
+    ParameterList params;
+    params.append("mode", "view");
+    _contacts->set(params);
+  }
 }
 
 void crmaccount::sClose()
@@ -276,7 +278,7 @@ void crmaccount::sClose()
   {
     if (QMessageBox::question(this, tr("Close without saving?"),
                               tr("<p>Are you sure you want to close this window "
-                                 "without saving the new CRM Account?"),
+                                 "without saving the new Account?"),
                               QMessageBox::Yes,
                               QMessageBox::No | QMessageBox::Default) == QMessageBox::No)
       return;
@@ -299,7 +301,7 @@ void crmaccount::sClose()
       {
         rollback.exec();
         ErrorReporter::error(QtCriticalMsg, this,
-                             tr("Error detaching Contact from CRM Account."),
+                             tr("Error detaching Contact from Account."),
                              storedProcErrorLookup("detachContact", returnVal),
                              __FILE__, __LINE__);
         return;
@@ -309,7 +311,7 @@ void crmaccount::sClose()
     {
       rollback.exec();
       ErrorReporter::error(QtCriticalMsg, this,
-                           tr("Error detaching Contact from CRM Account"),
+                           tr("Error detaching Contact from Account"),
                            detachq, __FILE__, __LINE__);
       return;
     }
@@ -354,7 +356,7 @@ void crmaccount::sClose()
     {
       rollback.exec();
       ErrorReporter::error(QtCriticalMsg, this,
-                           tr("Error deleting the initial CRM Account record"),
+                           tr("Error deleting the initial Account record"),
                            deleteq, __FILE__, __LINE__);
       return;
     }
@@ -381,7 +383,7 @@ void crmaccount::sSave()
   if (dupq.first())
     numberIsUsed = true;
   else if (ErrorReporter::error(QtCriticalMsg, this,
-                          tr("Error looking for duplicate CRM Account Number"),
+                          tr("Error looking for duplicate Account Number"),
                           dupq, __FILE__, __LINE__))
     return;
 
@@ -390,20 +392,20 @@ void crmaccount::sSave()
                           " to enter the data or unselect the check box.");
   QList<GuiErrorCheck> errors;
   errors<< GuiErrorCheck(numberIsUsed, _number,
-                         tr("This CRM Account Number is already in use by an "
-                            "existing CRM Account. Please choose a different "
+                         tr("This Account Number is already in use by an "
+                            "existing Account. Please choose a different "
                             "number and save again."))
         << GuiErrorCheck(_number->text().isEmpty(), _number,
-                         tr("You must enter a number for this CRM Account "
+                         tr("You must enter a number for this Account "
                             "before saving it."))
         << GuiErrorCheck(_name->text().isEmpty(), _name,
-                         tr("You must enter a name for this CRM Account before "
+                         tr("You must enter a name for this Account before "
                             "saving it."))
         << GuiErrorCheck(_parentCrmacct->id() == _crmacctId, _parentCrmacct,
-                         tr("This CRM Account cannot be a parent "
+                         tr("This Account cannot be a parent "
                             "to itself."))
         << GuiErrorCheck(_customer->isChecked() && _prospectId > 0, _customerButton,
-                         tr("This CRM Account is a Prospect but it is marked "
+                         tr("This Account is a Prospect but it is marked "
                             "as a Customer. Either mark it as a Prospect or "
                             "click %1 to convert it before saving.")
                          .arg(_customerButton->text()))
@@ -414,7 +416,7 @@ void crmaccount::sSave()
                          incomplete.arg(tr("Employee"),
                                         _employeeButton->text()))
         << GuiErrorCheck(_prospect->isChecked() && _custId > 0, _prospectButton,
-                         tr("This CRM Account is a Customer but it is marked "
+                         tr("This Account is a Customer but it is marked "
                             "as a Prospect. Either mark it as a Customer or "
                             "click %1 to convert it before saving.")
                          .arg(_prospectButton->text()))
@@ -435,7 +437,7 @@ void crmaccount::sSave()
         << GuiErrorCheck(_vendor->isChecked() && _vendId <= 0, _vendorButton,
                          incomplete.arg(tr("Vendor"), _vendorButton->text()))
   ;
-  if (GuiErrorCheck::reportErrors(this, tr("Cannot Save CRM Account"), errors))
+  if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Account"), errors))
     return;
 
   // TODO: handle _username
@@ -500,7 +502,7 @@ void crmaccount::sSave()
   if (err.type() != QSqlError::NoError)
   {
     rollback.exec();
-    ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                          err, __FILE__, __LINE__);
     return;
   }
@@ -514,8 +516,16 @@ void crmaccount::sSave()
     return;
   }
 
+  disconnect(omfgThis, SIGNAL(customersUpdated(int, bool)), this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(employeeUpdated(int)), this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(prospectsUpdated()),   this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(salesRepUpdated(int)), this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(taxAuthsUpdated(int)), this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(vendorsUpdated()),     this, SLOT(sUpdateRelationships()));
+  disconnect(omfgThis, SIGNAL(userUpdated(QString)), this, SLOT(sUpdateRelationships()));
+
   omfgThis->sCrmAccountsUpdated(_crmacctId);
-  omfgThis->sCustomersUpdated(-1, TRUE);
+  omfgThis->sCustomersUpdated(-1, true);
   omfgThis->sEmployeeUpdated(-1);
   omfgThis->sProspectsUpdated();
   omfgThis->sTaxAuthsUpdated(-1);
@@ -595,71 +605,6 @@ QSqlError crmaccount::saveNoErrorCheck(bool pInTxn)
   return updateq.lastError();
 }
 
-void crmaccount::sNewCharacteristic()
-{
-  ParameterList params;
-  params.append("mode", "new");
-  params.append("crmacct_id", _crmacctId);
-
-  characteristicAssignment newdlg(this, "", TRUE);
-  newdlg.set(params);
-
-  if (newdlg.exec() != XDialog::Rejected)
-    sGetCharacteristics();
-}
-
-void crmaccount::sEditCharacteristic()
-{
-  ParameterList params;
-  params.append("mode", "edit");
-  params.append("charass_id", _charass->id());
-
-  characteristicAssignment newdlg(this, "", TRUE);
-  newdlg.set(params);
-
-  if (newdlg.exec() != XDialog::Rejected)
-    sGetCharacteristics();
-}
-
-void crmaccount::sDeleteCharacteristic()
-{
-  XSqlQuery delcaq;
-  delcaq.prepare( "DELETE FROM charass "
-             "WHERE (charass_id=:charass_id);" );
-  delcaq.bindValue(":charass_id", _charass->id());
-  delcaq.exec();
-  if (ErrorReporter::error(QtCriticalMsg, this,
-                           tr("Error deleting Characteristic Assignment"),
-                           delcaq, __FILE__, __LINE__))
-    return;
-
-  sGetCharacteristics();
-}
-
-void crmaccount::sGetCharacteristics()
-{
-  XSqlQuery getcaq;
-  getcaq.prepare( "SELECT charass_id, char_name, "
-             " CASE WHEN char_type < 2 THEN "
-             "   charass_value "
-             " ELSE "
-             "   formatDate(charass_value::date) "
-             "END AS charass_value "
-             "FROM charass, char "
-             "WHERE ( (charass_target_type='CRMACCT')"
-             " AND (charass_char_id=char_id)"
-             " AND (charass_target_id=:crmacct_id) ) "
-             "ORDER BY char_order, char_name;" );
-  getcaq.bindValue(":crmacct_id", _crmacctId);
-  getcaq.exec();
-  _charass->populate(getcaq);
-
-  if (ErrorReporter::error(QtCriticalMsg, this,
-                           tr("Error getting Characteristic Assignments"),
-                           getcaq, __FILE__, __LINE__))
-    return;
-}
-
 int crmaccount::id()
 {
   return _crmacctId;
@@ -668,8 +613,9 @@ int crmaccount::id()
 void crmaccount::setId(int id)
 {
   _crmacctId = id;
-  _todoList->parameterWidget()->setDefault(tr("CRM Account"), _crmacctId, true);
+  _todoList->parameterWidget()->setDefault(tr("Account"), _crmacctId, true);
   _contacts->setCrmacctid(_crmacctId);
+  _charass->setId(_crmacctId);
   sPopulate();
 
   if (cView != _mode)
@@ -718,6 +664,7 @@ void crmaccount::sPopulate()
     _parentCrmacct->setId(getq.value("crmacct_parent_id").toInt());
     _comments->setId(_crmacctId);
     _documents->setId(_crmacctId);
+    _charass->setId(_crmacctId);
     _primary->setSearchAcct(_crmacctId);
     _secondary->setSearchAcct(_crmacctId);
     _owner->setUsername(getq.value("crmacct_owner_username").toString());
@@ -725,7 +672,7 @@ void crmaccount::sPopulate()
     sHandleChildButtons();
   }
   else if (ErrorReporter::error(QtCriticalMsg, this,
-                                tr("Error reading the CRM Account"),
+                                tr("Error reading the Account"),
                                 getq, __FILE__, __LINE__))
     return;
   else
@@ -746,12 +693,12 @@ void crmaccount::sPopulate()
     _vendId       = -1;
     _comments->setId(-1);
     _documents->setId(-1);
+    _charass->setId(-1);
     _canCreateUsers = false;
 
     sHandleChildButtons();
   }
 
-  sGetCharacteristics();
   sPopulateRegistrations();
   _contacts->sFillList();
   _todoList->sFillList();
@@ -768,7 +715,7 @@ void crmaccount::sCustomer()
       _privileges->check("MaintainAllCRMAccounts"))
   {
     QSqlError err = saveNoErrorCheck();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                              err, __FILE__, __LINE__))
       return;
   }
@@ -858,7 +805,7 @@ void crmaccount::sProspect()
       _privileges->check("MaintainAllCRMAccounts"))
   {
     QSqlError err = saveNoErrorCheck();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                              err, __FILE__, __LINE__))
       return;
   }
@@ -920,7 +867,7 @@ void crmaccount::sTaxAuth()
       _privileges->check("MaintainAllCRMAccounts"))
   {
     QSqlError err = saveNoErrorCheck();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                              err, __FILE__, __LINE__))
       return;
   }
@@ -951,7 +898,7 @@ void crmaccount::sEditVendor()
       _privileges->check("MaintainAllCRMAccounts"))
   {
     QSqlError err = saveNoErrorCheck();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                              err, __FILE__, __LINE__))
       return;
   }
@@ -982,7 +929,7 @@ void crmaccount::sViewVendor()
       _privileges->check("MaintainAllCRMAccounts"))
   {
     QSqlError err = saveNoErrorCheck();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                              err, __FILE__, __LINE__))
       return;
   }
@@ -1023,7 +970,7 @@ void crmaccount::sUpdateRelationships()
     _owner->setUsername(getq.value("crmacct_owner_username").toString());
   }
   else if (ErrorReporter::error(QtCriticalMsg, this,
-                                tr("Error reading the CRM Account"),
+                                tr("Error reading the Account"),
                                 getq, __FILE__, __LINE__))
     return;
 
@@ -1037,26 +984,16 @@ void crmaccount::doDialog(QWidget *parent, const ParameterList & pParams)
   omfgThis->handleNewWindow(ci);
 }
 
-void crmaccount::sVendorInfo()
-{
-  ParameterList params;
-  params.append("vend_id", _vendId);
-
-  vendorWorkBench *newdlg = new vendorWorkBench(this, "vendorWorkBench", Qt::Window);
-  newdlg->set(params);
-  omfgThis->handleNewWindow(newdlg);
-}
-
 void crmaccount::sCustomerToggled()
 {
   if (_customer->isChecked())
-    _prospect->setChecked(FALSE);
+    _prospect->setChecked(false);
 }
 
 void crmaccount::sProspectToggled()
 {
   if (_prospect->isChecked())
-    _customer->setChecked(FALSE);
+    _customer->setChecked(false);
 }
 
 void crmaccount::sPopulateRegistrations()
@@ -1091,7 +1028,7 @@ void crmaccount::sNewReg()
   params.append("mode", "new");
   params.append("crmacct_id", _crmacctId);
 
-  lotSerialRegistration newdlg(this, "", TRUE);
+  lotSerialRegistration newdlg(this, "", true);
   newdlg.set(params);
 
   newdlg.exec();
@@ -1104,7 +1041,7 @@ void crmaccount::sEditReg()
   params.append("mode", "edit");
   params.append("lsreg_id", _reg->id());
 
-  lotSerialRegistration newdlg(this, "", TRUE);
+  lotSerialRegistration newdlg(this, "", true);
   newdlg.set(params);
 
   newdlg.exec();
@@ -1160,16 +1097,24 @@ void crmaccount::sCheckNumber()
       query.bindValue(":crmacct_id", _crmacctId);
       query.exec();
       if (ErrorReporter::error(QtCriticalMsg, this,
-                               tr("Error deleting temporary CRM Account"),
+                               tr("Error deleting temporary Account"),
                                query, __FILE__, __LINE__))
         return;
 
+      if (! _lock.release())
+        ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                           _lock.lastError(), __FILE__, __LINE__);
+
       setId(newq.value("crmacct_id").toInt());
       sPopulate();
 
-      connect(_charass, SIGNAL(valid(bool)), _editCharacteristic, SLOT(setEnabled(bool)));
-      connect(_charass, SIGNAL(valid(bool)), _deleteCharacteristic, SLOT(setEnabled(bool)));
-      _number->setEnabled(FALSE);
+      if (_mode == cEdit && !_lock.acquire("crmacct", _crmacctId, AppLock::Interactive))
+      {
+        _mode = cView;
+        setViewMode();
+      }
+
+      _number->setEnabled(false);
     }
     else if (ErrorReporter::error(QtCriticalMsg, this, tr("Database Error"),
                                   newq, __FILE__, __LINE__))
@@ -1288,8 +1233,6 @@ void crmaccount::sHandleChildButtons()
     menuItem=vendorMenu->addAction(tr("View..."), this, SLOT(sViewVendor()));
     menuItem->setEnabled(_privileges->check("ViewVendors") ||
                          _privileges->check("MaintainVendors"));
-    menuItem=vendorMenu->addAction(tr("Workbench..."), this, SLOT(sVendorInfo()));
-    menuItem->setEnabled(_privileges->check("MaintainVendors"));
     _vendorButton->setMenu(vendorMenu);
   }
   else
@@ -1310,7 +1253,7 @@ void crmaccount::sEmployee()
       _privileges->check("MaintainAllCRMAccounts"))
   {
     QSqlError err = saveNoErrorCheck();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                              err, __FILE__, __LINE__))
       return;
   }
@@ -1341,7 +1284,7 @@ void crmaccount::sSalesRep()
       _privileges->check("MaintainAllCRMAccounts"))
   {
     QSqlError err = saveNoErrorCheck();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                              err, __FILE__, __LINE__))
       return;
   }
@@ -1372,7 +1315,7 @@ void crmaccount::sUser()
       _privileges->check("MaintainAllCRMAccounts"))
   {
     QSqlError err = saveNoErrorCheck();
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving CRM Account"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error saving Account"),
                              err, __FILE__, __LINE__))
       return;
   }
