@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -25,7 +25,7 @@ class CrmaccountMergePurgePage : public QWizardPage, public Ui::crmaccountMergeP
     virtual void initializePage();
 
   signals:
-    virtual void mergeSetChanged();
+    void mergeSetChanged();
 
   protected slots:
     virtual void sDelete();
