@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -246,11 +246,11 @@ int CyberSourceProcessor::buildCommon(const int pccardid, const QString &pcvv, c
   createChildTextNode(_requestMessage, CPDATA_NS ":clientLibrary", "xTuple ERP");
   createChildTextNode(_requestMessage, CPDATA_NS ":clientLibraryVersion", _Version);
   createChildTextNode(_requestMessage, CPDATA_NS ":clientEnvironment",
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
                       "Mac OS X"
-#elif defined Q_WS_WIN
+#elif defined Q_OS_WIN
                       "Windows"
-#elif defined Q_WS_X11
+#elif defined Q_OS_LINUX
                       "X Windows"
 #else
                       "undefined Qt client"
