@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,7 +19,6 @@
 #include <QPrintDialog>
 #include <QShortcut>
 #include <QToolButton>
-#include <QDebug>
 
 #include <metasql.h>
 #include <mqlutil.h>
@@ -30,6 +29,7 @@
 #include <previewdialog.h>
 
 #include "../scriptapi/parameterlistsetup.h"
+#include "errorReporter.h"
 
 class displayPrivate : public Ui::display
 {
@@ -82,6 +82,18 @@ public:
     _sep2 = _toolBar->addSeparator();
     _sep2->setVisible(false);
 
+    _expandBtn = new QToolButton(_toolBar);
+    _expandBtn->setObjectName("_expandBtn");
+    _expandBtn->setFocusPolicy(Qt::NoFocus);
+    _expandAct = _toolBar->addWidget(_expandBtn);
+    _expandAct->setVisible(false);
+
+    _collapseBtn = new QToolButton(_toolBar);
+    _collapseBtn->setObjectName("_collapseBtn");
+    _collapseBtn->setFocusPolicy(Qt::NoFocus);
+    _collapseAct = _toolBar->addWidget(_collapseBtn);
+    _collapseAct->setVisible(false);
+
     // Print buttons
     _printBtn = new QToolButton(_toolBar);
     _printBtn->setObjectName("_printBtn");
@@ -125,7 +137,7 @@ public:
   }
 
   bool setParams(ParameterList &);
-  void setupCharacteristics(unsigned int use);
+  void setupCharacteristics(QStringList uses);
   void print(ParameterList, bool, bool);
 
   QString reportName;
@@ -143,6 +155,8 @@ public:
   QAction* _filterAct;
   QAction* _moreAct;
   QAction* _sep2;
+  QAction* _expandAct;
+  QAction* _collapseAct;
   QAction* _printAct;
   QAction* _previewAct;
   QAction* _sep3;
@@ -159,6 +173,8 @@ public:
   QToolButton * _queryBtn;
   QToolButton * _previewBtn;
   QToolButton * _printBtn;
+  QToolButton * _expandBtn;
+  QToolButton * _collapseBtn;
 
   QList<QVariant> _charidstext;
   QList<QVariant> _charidslist;
@@ -245,38 +261,22 @@ void displayPrivate::print(ParameterList pParams, bool showPreview, bool forceSe
   }
 }
 
-void displayPrivate::setupCharacteristics(unsigned int use)
+void displayPrivate::setupCharacteristics(QStringList uses)
 {
-  QStringList uses;
-  if (use & characteristic::Addresses)
-    uses << "char_addresses";
-  if (use & characteristic::Contacts)
-    uses << "char_contacts";
-  if (use & characteristic::CRMAccounts)
-    uses << "char_crmaccounts";
-  if (use & characteristic::Customers)
-    uses << "char_customers";
-  if (use & characteristic::Employees)
-    uses << "char_employees";
-  if (use & characteristic::Incidents)
-    uses << "char_incidents";
-  if (use & characteristic::Items)
-    uses << "char_items";
-  if (use & characteristic::LotSerial)
-    uses << "char_lotserial";
-  if (use & characteristic::Opportunities)
-    uses << "char_opportunity";
-
-  // Add columns and parameters for characteristics
   QString column;
   QString name;
   QString sql = QString("SELECT char_id, char_name, char_type "
                         "FROM char "
-                        "WHERE (%1) "
-                        " AND (char_search) "
-                        "ORDER BY char_name;").arg(uses.join(" AND "));
+                        "  JOIN charuse ON char_id = charuse_char_id"
+                        " WHERE char_search "
+                        "   AND charuse_target_type IN ('%1')"
+                        " ORDER BY char_name;").arg(uses.join("','"));
   XSqlQuery chars;
   chars.exec(sql);
+  if (chars.size() <= 0)
+  {
+    qWarning() << "Could not find any characteristics matching" << uses;
+  }
   while (chars.next())
   {
     characteristic::Type chartype = (characteristic::Type)chars.value("char_type").toInt();
@@ -300,8 +300,8 @@ void displayPrivate::setupCharacteristics(unsigned int use)
     else if (chartype == characteristic::Date)
     {
       _charidsdate.append(chars.value("char_id").toInt());
-      QString start = QApplication::translate("display", "Start Date", 0, QApplication::UnicodeUTF8);
-      QString end = QApplication::translate("display", "End Date", 0, QApplication::UnicodeUTF8);
+      QString start = QApplication::translate("display", "Start Date", 0);
+      QString end = QApplication::translate("display", "End Date", 0);
       _parameterWidget->append(name + " " + start, column + "startDate", ParameterWidget::Date);
       _parameterWidget->append(name + " " + end, column + "endDate", ParameterWidget::Date);
     }
@@ -315,7 +315,7 @@ bool displayPrivate::setParams(ParameterList &params)
   if (!_search->isNull())
   {
     params.append("search_pattern", _search->text());
-    QString searchon =  QApplication::translate("display", "Search On:", 0, QApplication::UnicodeUTF8);
+    QString searchon =  QApplication::translate("display", "Search On:", 0);
     filter.prepend(searchon + " " + _search->text() + "\n");
   }
   params.append("filter", filter);
@@ -382,6 +382,8 @@ display::display(QWidget* parent, const char* name, Qt::WindowFlags flags)
   _data->_newBtn->setText(tr("New"));
   _data->_closeBtn->setText(tr("Close"));
   _data->_moreBtn->setText(tr("More"));
+  _data->_expandBtn->setText(tr("Expand All"));
+  _data->_collapseBtn->setText(tr("Collapse All"));
   _data->_printBtn->setText(tr("Print"));
   _data->_previewBtn->setText(tr("Preview"));
   _data->_queryBtn->setText(tr("Query"));
@@ -403,10 +405,14 @@ display::display(QWidget* parent, const char* name, Qt::WindowFlags flags)
   _data->_closeBtn->setToolTip(_data->_closeBtn->text() + " " + _data->_closeAct->shortcut().toString(QKeySequence::NativeText));
   _data->_queryBtn->setToolTip(_data->_queryBtn->text() + " " + _data->_queryAct->shortcut().toString(QKeySequence::NativeText));
   _data->_printBtn->setToolTip(_data->_printBtn->text() + " " + _data->_printAct->shortcut().toString(QKeySequence::NativeText));
+  _data->_expandBtn->setToolTip(_data->_expandBtn->text());
+  _data->_collapseBtn->setToolTip(_data->_collapseBtn->text());
 
   connect(_data->_newBtn, SIGNAL(clicked()), _data->_newAct, SLOT(trigger()));
   connect(_data->_closeBtn, SIGNAL(clicked()), _data->_closeAct, SLOT(trigger()));
   connect(_data->_moreBtn, SIGNAL(clicked(bool)), filterButton, SLOT(setChecked(bool)));
+  connect(_data->_expandBtn, SIGNAL(clicked()), _data->_expandAct, SLOT(trigger()));
+  connect(_data->_collapseBtn, SIGNAL(clicked()), _data->_collapseAct, SLOT(trigger()));
   connect(_data->_printBtn, SIGNAL(clicked()), _data->_printAct, SLOT(trigger()));
   connect(_data->_previewBtn, SIGNAL(clicked()), _data->_previewAct, SLOT(trigger()));
   connect(_data->_queryBtn, SIGNAL(clicked()), _data->_queryAct, SLOT(trigger()));
@@ -420,6 +426,8 @@ display::display(QWidget* parent, const char* name, Qt::WindowFlags flags)
   connect(_data->_newAct, SIGNAL(triggered()), this, SLOT(sNew()));
   connect(_data->_closeAct, SIGNAL(triggered()), this, SLOT(close()));
   connect(_data->_queryAct, SIGNAL(triggered()), this, SLOT(sFillList()));
+  connect(_data->_expandAct, SIGNAL(triggered()), this, SLOT(sExpand()));
+  connect(_data->_collapseAct, SIGNAL(triggered()), this, SLOT(sCollapse()));
   connect(_data->_printAct, SIGNAL(triggered()), this, SLOT(sPrint()));
   connect(_data->_previewAct, SIGNAL(triggered()), this, SLOT(sPreview()));
   connect(_data->_searchAct, SIGNAL(triggered()), this, SLOT(sFillList()));
@@ -479,6 +487,16 @@ QAction * display::closeAction()
   return _data->_closeAct;
 }
 
+QAction * display::expandAction()
+{
+  return _data->_expandAct;
+}
+
+QAction * display::collapseAction()
+{
+  return _data->_collapseAct;
+}
+
 QAction * display::filterSeparator()
 {
   return _data->_sep1;
@@ -535,11 +553,18 @@ bool display::setParams(ParameterList & params)
   return ret;
 }
 
-void display::setupCharacteristics(unsigned int uses)
+void display::setupCharacteristics(QStringList uses)
 {
   _data->setupCharacteristics(uses);
 }
 
+void display::setupCharacteristics(QString uses)
+{
+  QStringList ulist;
+  ulist << uses;
+  _data->setupCharacteristics(ulist);
+}
+
 void display::setReportName(const QString & reportName)
 {
   _data->reportName = reportName;
@@ -598,6 +623,26 @@ bool display::closeVisible() const
   return _data->_closeAct->isVisible();
 }
 
+void display::setExpandVisible(bool show)
+{
+  _data->_expandAct->setVisible(show);
+}
+
+bool display::expandVisible() const
+{
+  return _data->_expandAct->isVisible();
+}
+
+void display::setCollapseVisible(bool show)
+{
+  _data->_collapseAct->setVisible(show);
+}
+
+bool display::collapseVisible() const
+{
+  return _data->_collapseAct->isVisible();
+}
+
 void display::setParameterWidgetVisible(bool show)
 {
   _data->_parameterWidget->setVisible(show);
@@ -694,6 +739,19 @@ void display::sNew()
 {
 }
 
+void display::sExpand()
+{
+    if (_data->_list)
+        _data->_list->expandAll();
+}
+
+void display::sCollapse()
+{
+    if (_data->_list)
+        _data->_list->collapseAll();
+}
+
+
 void display::sPrint()
 {
   sPrint(ParameterList());
@@ -733,14 +791,16 @@ void display::sFillList(ParameterList pParams, bool forceSetParams)
   MetaSQLQuery mql = MQLUtil::mqlLoad(_data->metasqlGroup, _data->metasqlName, errorString, &ok);
   if(!ok)
   {
-    systemError(this, errorString, __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Information"),
+                         errorString, __FILE__, __LINE__);
     return;
   }
   XSqlQuery xq = mql.toQuery(pParams);
   _data->_list->populate(xq, itemid, _data->_useAltId);
   if (xq.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, xq.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Information"),
+                           xq, __FILE__, __LINE__);
     return;
   }
   emit fillListAfter();
