@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,7 +22,7 @@ class dspOrderActivityByProject : public display, public Ui::dspOrderActivityByP
     Q_OBJECT
 
 public:
-    dspOrderActivityByProject(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    dspOrderActivityByProject(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
 
     virtual bool setParams(ParameterList &);
 
@@ -32,6 +32,10 @@ public slots:
     virtual void sEdit();
     virtual void sView();
     virtual void sFillList();
+    virtual void sNewSalesOrder();
+    virtual void sNewPurchaseOrder();
+    virtual void sNewWorkOrder();
+    virtual void sNewProjectTask();
 
 protected slots:
     virtual void languageChange();
