@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,7 +21,7 @@ class dspRunningAvailability : public display, public Ui::dspRunningAvailability
     Q_OBJECT
 
 public:
-    dspRunningAvailability(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    dspRunningAvailability(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
 
     virtual bool setParams(ParameterList&);
 
@@ -38,6 +38,10 @@ public slots:
     virtual void sViewTo();
     virtual void sViewWo();
     virtual void sViewPo();
+    virtual void sEditSo();
+    virtual void sEditTo();
+    virtual void sEditWo();
+    virtual void sEditPo();
     virtual void sFillList();
 
 protected slots:
