@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,11 +21,13 @@
 #include "assignLotSerial.h"
 #include "distributeToLocation.h"
 #include "inputManager.h"
+#include "errorReporter.h"
+#include "storedProcErrorLookup.h"
 
 #define cIncludeLotSerial   0x01
 #define cNoIncludeLotSerial 0x02
 
-distributeInventory::distributeInventory(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+distributeInventory::distributeInventory(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
@@ -39,10 +41,11 @@ distributeInventory::distributeInventory(QWidget* parent, const char* name, bool
   connect(_taggedOnly,  SIGNAL(toggled(bool)), this, SLOT(sFillList()));
   connect(_bc,   SIGNAL(textChanged(QString)), this, SLOT(sBcChanged(QString)));
   connect(_qtyOnly,     SIGNAL(toggled(bool)), this, SLOT(sFillList()));
+  connect(_zone, SIGNAL(currentIndexChanged(int)), this, SLOT(sFillList()));
 
   omfgThis->inputManager()->notify(cBCLotSerialNumber, this, this, SLOT(sCatchLotSerialNumber(QString)));
 
-  _item->setReadOnly(TRUE);
+  _item->setReadOnly(true);
   _qtyToDistribute->setPrecision(omfgThis->qtyVal());
   _qtyTagged->setPrecision(omfgThis->qtyVal());
   _qtyRemaining->setPrecision(omfgThis->qtyVal());
@@ -50,6 +53,7 @@ distributeInventory::distributeInventory(QWidget* parent, const char* name, bool
   _itemloc->addColumn(tr("Location"),     _itemColumn, Qt::AlignLeft,  true, "locationname");
   _itemloc->addColumn(tr("Default"),      _ynColumn,   Qt::AlignLeft,  true, "defaultlocation");
   _itemloc->addColumn(tr("Netable"),      _ynColumn,   Qt::AlignCenter,true, "location_netable");
+  _itemloc->addColumn(tr("Usable"),       _ynColumn,   Qt::AlignCenter,true, "location_usable");
   _itemloc->addColumn(tr("Lot/Serial #"), -1,          Qt::AlignLeft,  true, "lotserial");
   _itemloc->addColumn(tr("Expiration"),   _dateColumn, Qt::AlignCenter,true, "f_expiration");
   _itemloc->addColumn(tr("Qty. Before"),  _qtyColumn,  Qt::AlignRight, true, "qty");
@@ -62,7 +66,10 @@ distributeInventory::distributeInventory(QWidget* parent, const char* name, bool
     _warehouseLit->hide();
     _warehouse->hide();
   }
-  
+
+// Populate Zone filter dropdown
+  updateZoneList();  
+
   //If not lot serial control, hide info
   if (!_metrics->boolean("LotSerialControl"))
   {
@@ -108,15 +115,16 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
                      "       COALESCE(itemlocdist_source_id,-1) AS itemlocdist_source_id,"
                      "       CASE WHEN (invhist_transtype IN ('RM','RP','RR','RX')) THEN 'R'"
                      "            WHEN (invhist_transtype = 'IM') THEN 'I'"
+                     "            WHEN (invhist_transtype = 'SH' AND invhist_ordtype='SO') THEN 'I'"
                      "            ELSE 'O'"
                      "       END AS trans_type,"
                      "       CASE WHEN (invhist_transtype IN ('RM','RP','RR','RX')"
-                     "                  AND itemsite_recvlocation_dist) THEN TRUE"
+                     "                  AND itemsite_recvlocation_dist) THEN true"
                      "            WHEN (invhist_transtype = 'IM'"
-                     "                  AND itemsite_issuelocation_dist) THEN TRUE"
+                     "                  AND itemsite_issuelocation_dist) THEN true"
                      "            WHEN (invhist_transtype NOT IN ('RM','RP','RR','RX','IM')"
-                     "                  AND itemsite_location_dist) THEN TRUE"
-                     "            ELSE FALSE"
+                     "                  AND itemsite_location_dist) THEN true"
+                     "            ELSE false"
                      "       END AS auto_dist "
                      "FROM itemlocdist JOIN itemsite ON (itemlocdist_itemsite_id=itemsite_id) "
                      "                 LEFT OUTER JOIN invhist ON (itemlocdist_invhist_id=invhist_id) "
@@ -194,9 +202,9 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
               fromlots.bindValue(":itemlocdist_series", itemlocSeries);
               fromlots.bindValue(":itemlocdist_id", itemloc.value("itemlocdist_id"));
               fromlots.exec();
-              if (fromlots.lastError().type() != QSqlError::NoError)
+              if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                            fromlots, __FILE__, __LINE__))
               {
-                systemError(0, fromlots.lastError().databaseText(), __FILE__, __LINE__);
                 return XDialog::Rejected;
               }
             }
@@ -218,14 +226,14 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
             autocreatels.exec();
             if (autocreatels.first())
               params.append("itemlocseries", autocreatels.value("itemlocseries").toInt());
-            else if (autocreatels.lastError().type() != QSqlError::NoError)
+            else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                          autocreatels, __FILE__, __LINE__))
             {
-              systemError(0, autocreatels.lastError().databaseText(), __FILE__, __LINE__);
               return XDialog::Rejected;
             }
           }
 
-          assignLotSerial newdlg(pParent, "", TRUE);
+          assignLotSerial newdlg(pParent, "", true);
           newdlg.set(params);
           itemlocSeries = newdlg.exec();
           if (itemlocSeries == XDialog::Rejected)
@@ -245,7 +253,7 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
             ParameterList params;
             params.append("itemlocdist_id", query.value("itemlocdist_id").toInt());
             params.append("trans_type", itemloc.value("trans_type").toString());
-            distributeInventory newdlg(pParent, "", TRUE);
+            distributeInventory newdlg(pParent, "", true);
             newdlg.set(params);
             if (itemloc.value("auto_dist").toBool())
             {
@@ -291,7 +299,7 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
         if (itemloc.value("itemlocdist_distlotserial").toBool())
           params.append("includeLotSerialDetail");
 
-        distributeInventory newdlg(pParent, "", TRUE);
+        distributeInventory newdlg(pParent, "", true);
         newdlg.set(params);
         if (itemloc.value("auto_dist").toBool())
         {
@@ -324,9 +332,9 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
       post.prepare( "SELECT distributeItemlocSeries(:itemlocdist_series) AS result;");
       post.bindValue(":itemlocdist_series", ildsList.at(i));
       post.exec();
-      if (post.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                    post, __FILE__, __LINE__))
       {
-        systemError(0, post.lastError().databaseText(), __FILE__, __LINE__);
         return XDialog::Rejected;
       }
     }
@@ -336,9 +344,9 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
       post.prepare("SELECT distributeToLocations(:itemlocdist_id) AS result;");
       post.bindValue(":itemlocdist_id", ildList.at(i));
       post.exec();
-      if (post.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                    post, __FILE__, __LINE__))
       {
-        systemError(0, post.lastError().databaseText(), __FILE__, __LINE__);
         return XDialog::Rejected;
       }
     }
@@ -353,9 +361,9 @@ int distributeInventory::SeriesAdjust(int pItemlocSeries, QWidget *pParent, cons
             QMessageBox::warning( 0, tr("Inventory Distribution"), 
         tr("There was an error posting the transaction.  Contact your administrator") );
     }
-    else if (post.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Lot/Serial Information"),
+                                  post, __FILE__, __LINE__))
     {
-      systemError(0, post.lastError().databaseText(), __FILE__, __LINE__);
       return XDialog::Rejected;
     }
   }
@@ -420,9 +428,9 @@ void distributeInventory::populate()
     else
       _bcQty->clear();
   }
-  else if (distributepopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                distributepopulate, __FILE__, __LINE__))
   {
-    systemError(this, distributepopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -450,7 +458,9 @@ void distributeInventory::sSelectLocation()
   else if (_itemloc->altId() == cItemloc)
     params.append("itemlocdist_id", _itemloc->id());
 
-  distributeToLocation newdlg(this, "", TRUE);
+  params.append("itemsite_controlmethod", _controlMethod);
+
+  distributeToLocation newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() == XDialog::Accepted)
@@ -537,14 +547,15 @@ bool distributeInventory::sDefault()
         int result = distributeDefault.value("result").toInt();
         if (result < 0)
         {
-          QMessageBox::warning( 0, tr("Inventory Distribution"),
-                                tr("There was an error distributing to default location."));
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                 storedProcErrorLookup("distributetodefault", result),
+                                 __FILE__, __LINE__);
           return false;
         }
       }
-      else if (distributeDefault.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtWarningMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                              distributeDefault, __FILE__, __LINE__))
       {
-        systemError(this, distributeDefault.lastError().databaseText(), __FILE__, __LINE__);
         return false;
       }
       sFillList();
@@ -601,13 +612,13 @@ void distributeInventory::sFillList()
     if ( (distributeFillList.value("itemsite_location_id").toInt() != -1) &&
          ( (_mode == cNoIncludeLotSerial) || ( (_mode == cIncludeLotSerial) && (!distributeFillList.value("lscontrol").toBool()) ) ) )
     {
-      _default->setEnabled(TRUE);
-      _defaultAndPost->setEnabled(TRUE);
+      _default->setEnabled(true);
+      _defaultAndPost->setEnabled(true);
     }
     else
     {
-      _default->setEnabled(FALSE);
-      _defaultAndPost->setEnabled(FALSE);
+      _default->setEnabled(false);
+      _defaultAndPost->setEnabled(false);
     }
 
     ParameterList params;
@@ -633,19 +644,22 @@ void distributeInventory::sFillList()
     params.append("itemsite_id",    distributeFillList.value("itemsite_id").toInt());
     params.append("transtype",      _transtype);
 
+    if (_zone->id() > 0)
+      params.append("zone",         _zone->id());
+
     MetaSQLQuery mql = mqlLoad("distributeInventory", "locations");
     distributeFillList = mql.toQuery(params);
 
     _itemloc->populate(distributeFillList, true);
-    if (distributeFillList.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                  distributeFillList, __FILE__, __LINE__))
     {
-      systemError(this, distributeFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
-  else if (distributeFillList.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                distributeFillList, __FILE__, __LINE__))
   {
-    systemError(this, distributeFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -680,9 +694,9 @@ void distributeInventory::sBcDistribute()
 
   if(!distributeBcDistribute.first())
   {
-    if (distributeBcDistribute.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Lot/Serial Information"),
+                                  distributeBcDistribute, __FILE__, __LINE__))
     {
-      systemError(this, distributeBcDistribute.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     QMessageBox::warning(this, tr("No Match Found"),
@@ -697,7 +711,9 @@ void distributeInventory::sBcDistribute()
   params.append("qty",                   _bcQty->text());
   params.append("distribute");
 
-  distributeToLocation newdlg(this, "", TRUE);
+  params.append("itemsite_controlmethod", _controlMethod);
+
+  distributeToLocation newdlg(this, "", true);
   if (newdlg.set(params) != NoError)
     return;
 
@@ -747,11 +763,13 @@ void distributeInventory::sPopulateDefaultSelector()
             loclist.prepare( " SELECT location_id, formatLocationName(location_id) AS locationname "
                            " FROM location "
                            " WHERE ( (location_warehous_id=:warehous_id)"
-                           "   AND (NOT location_restrict) ) "
+                           "   AND (NOT location_restrict) "
+                           "   AND (location_active) ) "
                            " UNION SELECT location_id, formatLocationName(location_id) AS locationname "
                            "  FROM location, locitem, itemsite"
                            "  WHERE ( (location_warehous_id=:warehous_id)"
                            "    AND (location_restrict)"
+                           "    AND (location_active)"
                            "    AND (locitem_location_id=location_id)"
                            "    AND (locitem_item_id=itemsite_item_id)"
                            "    AND (itemsite_id=:itemsite_id))"
@@ -810,9 +828,24 @@ void distributeInventory::sChangeDefaultLocation()
    query.bindValue(":itemsite_id", _itemsite_id);
    query.exec();
    sFillList();
-   if (query.lastError().type() != QSqlError::NoError)
-      {
-        systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
-        return;
-      }
+   if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Item Site Information"),
+                                 query, __FILE__, __LINE__))
+   {
+     return;
+   }
+}
+
+void distributeInventory::updateZoneList()
+{
+  XSqlQuery zoneFillList;
+  QString zoneSql( "SELECT whsezone_id, whsezone_name||'-'||whsezone_descrip "
+             " FROM whsezone  "
+             " WHERE (whsezone_warehous_id=:warehous_id)"
+             " ORDER BY whsezone_name;");
+
+  zoneFillList.prepare(zoneSql);
+  zoneFillList.bindValue(":warehous_id", _warehouse->id());
+  zoneFillList.exec();
+  _zone->populate(zoneFillList);
+  
 }
