@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -30,6 +30,7 @@
 #include "cashReceipt.h"
 #include "creditMemo.h"
 #include "creditcardprocessor.h"
+#include "customer.h"
 #include "distributeInventory.h"
 #include "dspInvoiceInformation.h"
 #include "dspSalesOrderStatus.h"
@@ -44,7 +45,9 @@
 #include "salesOrder.h"
 #include "storedProcErrorLookup.h"
 
-dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WFlags fl)
+#include "errorReporter.h"
+
+dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspAROpenItems", fl)
 {
   setupUi(optionsWidget());
@@ -59,8 +62,8 @@ dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WFlags fl)
   connect(_customerSelector, SIGNAL(updated()), this, SLOT(sHandlePrintGroup()));
   connect(_closed, SIGNAL(toggled(bool)), this, SLOT(sClosedToggled(bool)));
 
-  _dates->setStartNull(tr("Earliest"), omfgThis->startOfTime(), TRUE);
-  _dates->setEndNull(tr("Latest"), omfgThis->endOfTime(), TRUE);
+  _dates->setStartNull(tr("Earliest"), omfgThis->startOfTime(), true);
+  _dates->setEndNull(tr("Latest"), omfgThis->endOfTime(), true);
 
   QString baseBalanceTitle(tr("Balance"));
   QString baseAmountTitle(tr("Amount"));
@@ -88,8 +91,8 @@ dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WFlags fl)
   list()->addColumn(baseAmountTitle,    _moneyColumn, Qt::AlignRight,  false,  "base_amount");
   list()->addColumn(tr("Paid"),         _moneyColumn, Qt::AlignRight,  true,  "paid");
   list()->addColumn(basePaidTitle,      _moneyColumn, Qt::AlignRight,  false,  "base_paid");
-  list()->addColumn(tr("Balance"),      _moneyColumn, Qt::AlignRight,  true,  "balance");
-  list()->addColumn(tr("Currency"),  _currencyColumn, Qt::AlignLeft,   true,  "currAbbr");
+  list()->addColumn(tr("Balance"),      _moneyColumn, Qt::AlignRight,  !omfgThis->singleCurrency(),  "balance");
+  list()->addColumn(tr("Currency"),  _currencyColumn, Qt::AlignLeft,   !omfgThis->singleCurrency(),  "currAbbr");
   list()->addColumn(baseBalanceTitle,   _moneyColumn, Qt::AlignRight,  true,  "base_balance");
   list()->addColumn(tr("Credit Card"),            -1, Qt::AlignLeft,   false, "ccard_number");
   list()->addColumn(tr("Notes"),                  -1, Qt::AlignLeft,   false, "notes");
@@ -97,12 +100,6 @@ dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WFlags fl)
   connect(omfgThis, SIGNAL(creditMemosUpdated()), this, SLOT(sFillList()));
   connect(omfgThis, SIGNAL(invoicesUpdated(int, bool)), this, SLOT(sFillList()));
 
-  if (omfgThis->singleCurrency())
-  {
-    list()->hideColumn("currAbbr");
-    list()->hideColumn("balance");
-  }
-
   disconnect(newAction(), SIGNAL(triggered()), this, SLOT(sNew()));
   connect(newAction(), SIGNAL(triggered()), this, SLOT(sCreateInvoice()));
 
@@ -115,7 +112,7 @@ dspAROpenItems::dspAROpenItems(QWidget* parent, const char*, Qt::WFlags fl)
   menuItem = newMenu->addAction(tr("Misc. Debit Memo"),   this, SLOT(sEnterMiscArDebitMemo()));
   menuItem->setEnabled(_privileges->check("MaintainARMemos"));
   newMenu->addSeparator();
-  menuItem = newMenu->addAction(tr("Credit Memo"), this, SLOT(sNewCreditMemo()));
+  menuItem = newMenu->addAction(tr("Sales Credit"), this, SLOT(sNewCreditMemo()));
   menuItem->setEnabled(_privileges->check("MaintainCreditMemos"));
   menuItem = newMenu->addAction(tr("Misc. Credit Memo"),   this, SLOT(sEnterMiscArCreditMemo()));
   menuItem->setEnabled(_privileges->check("MaintainARMemos"));
@@ -151,7 +148,7 @@ enum SetResponse dspAROpenItems::set(const ParameterList &pParams)
   if (valid)
   {
     _asOf->setDate(param.toDate());
-    _asOf->setEnabled(FALSE);
+    _asOf->setEnabled(false);
   }
 
   if (pParams.inList("run"))
@@ -167,6 +164,9 @@ void dspAROpenItems::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *pItem, int)
 {
   QAction *menuItem = 0;
 
+  if ( _privileges->check("MaintainCustomerMasters") || _privileges->check("ViewCustomerMasters") )
+    menuItem = pMenu->addAction(tr("Open Customer..."), this, SLOT(sOpenCustomer()));
+
   if (((XTreeWidgetItem *)pItem)->altId() < 4)
   {
     menuItem = pMenu->addAction(tr("Print..."), this, SLOT(sPrintItem()));
@@ -191,7 +191,7 @@ void dspAROpenItems::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *pItem, int)
   else if (((XTreeWidgetItem *)pItem)->altId() == 1 && ((XTreeWidgetItem *)pItem)->id("docnumber") > -1)
   // Credit Memo
   {
-    menuItem = pMenu->addAction(tr("Edit Credit Memo..."), this, SLOT(sEdit()));
+    menuItem = pMenu->addAction(tr("Edit Sales Credit..."), this, SLOT(sEdit()));
     menuItem->setEnabled(_privileges->check("MaintainCreditMemos"));
   }
   else if (((XTreeWidgetItem *)pItem)->id() > 0)
@@ -238,11 +238,11 @@ void dspAROpenItems::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *pItem, int)
   {
     if(((XTreeWidgetItem *)pItem)->rawValue("posted") != 0)
     {
-      menuItem = pMenu->addAction(tr("Void Posted Credit Memo..."), this, SLOT(sVoidCreditMemo()));
+      menuItem = pMenu->addAction(tr("Void Posted Sales Credit..."), this, SLOT(sVoidCreditMemo()));
       menuItem->setEnabled(_privileges->check("VoidPostedARCreditMemos"));
     }
 
-    menuItem = pMenu->addAction(tr("View Credit Memo..."), this, SLOT(sViewCreditMemo()));
+    menuItem = pMenu->addAction(tr("View Sales Credit..."), this, SLOT(sViewCreditMemo()));
     menuItem->setEnabled(_privileges->check("MaintainCreditMemos") || _privileges->check("ViewCreditMemos"));
   }
   else if (((XTreeWidgetItem *)pItem)->altId() == 4)
@@ -326,12 +326,55 @@ void dspAROpenItems::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *pItem, int)
 
 }
 
+void dspAROpenItems::sOpenCustomer()
+{
+  XSqlQuery customerq;
+  ParameterList params;
+  if ( _privileges->check("MaintainCustomerMasters") )
+    params.append("mode", "edit");
+  else
+    params.append("mode", "view");
+  
+  if (_customerSelector->isSelectedCust())
+    params.append("cust_id", _customerSelector->custId());
+  else if (list()->id() > -1)
+  {
+    customerq.prepare("SELECT aropen_cust_id FROM aropen WHERE aropen_id=:aropen_id;");
+    customerq.bindValue(":aropen_id", list()->id());
+    customerq.exec();
+    if (customerq.first())
+      params.append("cust_id", customerq.value("aropen_cust_id").toInt());
+  }
+  else if (list()->id() == -1 && list()->altId() == 0)
+  {
+    customerq.prepare("SELECT invchead_cust_id FROM invchead WHERE invchead_id=:invchead_id;");
+    customerq.bindValue(":invchead_id", list()->currentItem()->id("docnumber"));
+    customerq.exec();
+    if (customerq.first())
+      params.append("cust_id", customerq.value("invchead_cust_id").toInt());
+  }
+  else if (list()->id() == -1 && list()->altId() == 1)
+  {
+    customerq.prepare("SELECT cmhead_cust_id FROM cmhead WHERE cmhead_id=:cmhead_id;");
+    customerq.bindValue(":cmhead_id", list()->currentItem()->id("docnumber"));
+    customerq.exec();
+    if (customerq.first())
+      params.append("cust_id", customerq.value("cmhead_cust_id").toInt());
+  }
+  else
+    return;
+
+  customer *newdlg = new customer();
+  newdlg->set(params);
+  omfgThis->handleNewWindow(newdlg);
+}
+
 void dspAROpenItems::sApplyAropenCM()
 {
   ParameterList params;
   params.append("aropen_id", list()->id());
 
-  applyARCreditMemo newdlg(this, "", TRUE);
+  applyARCreditMemo newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -377,9 +420,9 @@ void dspAROpenItems::sCCRefundCM()
     currid  = dspCCRefundCM.value("aropen_curr_id").toInt();
     docnum  = dspCCRefundCM.value("aropen_docnumber").toString();
   }
-  else if (dspCCRefundCM.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving AR Information"),
+                                dspCCRefundCM, __FILE__, __LINE__))
   {
-    systemError(this, dspCCRefundCM.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -431,13 +474,14 @@ void dspAROpenItems::sDeleteCreditMemo()
       if (delq.first())
       {
             if (! delq.value("result").toBool())
-              systemError(this, tr("Could not delete Credit Memo."),
-                          __FILE__, __LINE__);
+              ErrorReporter::error(QtCriticalMsg, this,
+                                 tr("Error Deleting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                                 delq, __FILE__, __LINE__);
       }
       else if (delq.lastError().type() != QSqlError::NoError)
-            systemError(this,
-                        tr("Error deleting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")) +
-                        delq.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg,
+                         this, tr("Error Deleting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                         delq, __FILE__, __LINE__);
     }
 
     omfgThis->sCreditMemosUpdated();
@@ -463,17 +507,19 @@ void dspAROpenItems::sDeleteInvoice()
             int result = dspDeleteInvoice.value("result").toInt();
             if (result < 0)
             {
-              systemError(this, storedProcErrorLookup("deleteInvoice", result),
-                          __FILE__, __LINE__);
+              ErrorReporter::error(QtCriticalMsg, this,
+                                   tr("Error Deleting Invoice %1\n").arg(list()->currentItem()->text("docnumber")),
+                                   storedProcErrorLookup("deleteInvoice", result),
+                                   __FILE__, __LINE__);
             }
       }
-      else if (dspDeleteInvoice.lastError().type() != QSqlError::NoError)
-            systemError(this,
-                        tr("Error deleting Invoice %1\n").arg(list()->currentItem()->text("docnumber")) +
-                        dspDeleteInvoice.lastError().databaseText(), __FILE__, __LINE__);
+      else
+          ErrorReporter::error(QtCriticalMsg, this,
+                               tr("Error Deleting Invoice %1\n").arg(list()->currentItem()->text("docnumber")),
+                               dspDeleteInvoice, __FILE__, __LINE__);
     }
 
-    omfgThis->sInvoicesUpdated(-1, TRUE);
+    omfgThis->sInvoicesUpdated(-1, true);
     omfgThis->sBillingSelectionUpdated(-1, -1);
   }
 }
@@ -529,7 +575,7 @@ void dspAROpenItems::sEdit()
   {
     params.append("mode", "edit");
     params.append("aropen_id", list()->id());
-    arOpenItem newdlg(this, "", TRUE);
+    arOpenItem newdlg(this, "", true);
     newdlg.set(params);
 
     if (newdlg.exec() != XDialog::Rejected)
@@ -540,7 +586,7 @@ void dspAROpenItems::sEdit()
   {
     params.append("mode", "edit");
     params.append("incdt_id", list()->currentItem()->id("ordernumber"));
-    incident newdlg(this, "", TRUE);
+    incident newdlg(this, "", true);
     newdlg.set(params);
 
     if (newdlg.exec() != XDialog::Rejected)
@@ -559,7 +605,7 @@ void dspAROpenItems::sView()
   ParameterList params;
   params.append("mode", "view");
   params.append("aropen_id", list()->id());
-  arOpenItem newdlg(this, "", TRUE);
+  arOpenItem newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -611,14 +657,16 @@ void dspAROpenItems::sVoidCreditMemo()
     if (result < 0)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("voidCreditMemo", result),
-                      __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this,
+                              tr("Error Voiding Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                               storedProcErrorLookup("voidCreditMemo", result),
+                               __FILE__, __LINE__);
       return;
     }
     else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
     {
       rollback.exec();
-      QMessageBox::information( this, tr("Void Credit Memo"), tr("Transaction Canceled") );
+      QMessageBox::information( this, tr("Void Memo"), tr("Transaction Canceled") );
       return;
     }
 
@@ -628,9 +676,9 @@ void dspAROpenItems::sVoidCreditMemo()
   else if (post.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, tr("A System Error occurred voiding Credit Memo.\n%1")
-                .arg(post.lastError().databaseText()),
-                __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg,this,
+                         tr("Error Voiding Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                         post, __FILE__, __LINE__);
   }
 }
 
@@ -642,7 +690,7 @@ void dspAROpenItems::sEnterMiscArCreditMemo()
   if (_customerSelector->isSelectedCust())
     params.append("cust_id", _customerSelector->custId());
 
-  arOpenItem newdlg(this, "", TRUE);
+  arOpenItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
   sFillList();
@@ -656,7 +704,7 @@ void dspAROpenItems::sEnterMiscArDebitMemo()
   if (_customerSelector->isSelectedCust())
     params.append("cust_id", _customerSelector->custId());
 
-  arOpenItem newdlg(this, "", TRUE);
+  arOpenItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
   sFillList();
@@ -765,8 +813,10 @@ void dspAROpenItems::sVoidInvoiceDetails()
     if (result < 0)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("voidInvoice", result),
-                      __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this,
+                           tr("Error Voiding Invoice %1\n").arg(list()->currentItem()->text("docnumber")),
+                           storedProcErrorLookup("voidInvoice", result),
+                           __FILE__, __LINE__);
       return;
     }
     else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
@@ -782,9 +832,9 @@ void dspAROpenItems::sVoidInvoiceDetails()
   else if (post.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, tr("A System Error occurred voiding Invoice.\n%1")
-                .arg(post.lastError().databaseText()),
-                __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Voiding Invoice %1\n").arg(list()->currentItem()->text("docnumber")),
+                         post, __FILE__, __LINE__);
   }
 }
 
@@ -814,7 +864,7 @@ void dspAROpenItems::sIncident()
     params.append("aropen_id", list()->id());
     params.append("crmacct_id", dspIncident.value("crmacct_id"));
     params.append("cntct_id", dspIncident.value("crmacct_cntct_id_1"));
-    incident newdlg(this, 0, TRUE);
+    incident newdlg(this, 0, true);
     newdlg.set(params);
 
     if (newdlg.exec() == XDialog::Accepted)
@@ -827,7 +877,7 @@ void dspAROpenItems::sViewIncident()
   ParameterList params;
   params.append("mode", "view");
   params.append("incdt_id", list()->currentItem()->id("ordernumber"));
-  incident newdlg(this, "", TRUE);
+  incident newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -836,6 +886,9 @@ void dspAROpenItems::sViewIncident()
 
 bool dspAROpenItems::setParams(ParameterList &params)
 {
+  if (!display::setParams(params))
+    return false;
+
   _customerSelector->appendValue(params);
   if (_docDate->isChecked())
     _dates->appendValue(params);
@@ -845,6 +898,7 @@ bool dspAROpenItems::setParams(ParameterList &params)
     params.append("endDueDate", _dates->endDate());
   }
   params.append("invoice", tr("Invoice"));
+  params.append("return", tr("Sales Credit"));
   params.append("creditMemo", tr("Credit Memo"));
   params.append("debitMemo", tr("Debit Memo"));
   params.append("cashdeposit", tr("Customer Deposit"));
@@ -874,6 +928,7 @@ void dspAROpenItems::sPreview()
     ParameterList params;
     params.append("cust_id", _customerSelector->custId());
     params.append("invoice",  tr("Invoice"));
+    params.append("return",   tr("Sales Credit"));
     params.append("debit",    tr("Debit Memo"));
     params.append("credit",   tr("Credit Memo"));
     params.append("deposit",  tr("Deposit"));
@@ -923,7 +978,7 @@ void dspAROpenItems::sPrintItem()
   if (list()->altId() == 0)
   {
     // Print Invoice
-    printInvoice newdlg(this, "", TRUE);
+    printInvoice newdlg(this, "", true);
     
     if (checkInvoiceSitePrivs(list()->currentItem()->id("docnumber")))
     {
@@ -936,10 +991,10 @@ void dspAROpenItems::sPrintItem()
       {
         if(newdlg.exec() == QDialog::Rejected)
           return;
-        newdlg.setSetup(TRUE);
+        newdlg.setSetup(true);
       }
 
-      omfgThis->sInvoicesUpdated(-1, TRUE);
+      omfgThis->sInvoicesUpdated(-1, true);
       return;
     }
     else
@@ -953,14 +1008,14 @@ void dspAROpenItems::sPrintItem()
       params.append("cmhead_id", list()->currentItem()->id("docnumber"));
       params.append("persistentPrint");
 
-      printCreditMemo newdlg(this, "", TRUE);
+      printCreditMemo newdlg(this, "", true);
       newdlg.set(params);
 
       if (!newdlg.isSetup())
       {
         if(newdlg.exec() == QDialog::Rejected)
           return;
-        newdlg.setSetup(TRUE);
+        newdlg.setSetup(true);
       }
       omfgThis->sCreditMemosUpdated();
       return;
@@ -1003,7 +1058,7 @@ void dspAROpenItems::sPostCreditMemo()
 
   if (_privileges->check("ChangeSOMemoPostDate"))
   {
-    getGLDistDate newdlg(this, "", TRUE);
+    getGLDistDate newdlg(this, "", true);
     newdlg.sSetDefaultLit(tr("Credit Memo Date"));
     if (newdlg.exec() == XDialog::Accepted)
     {
@@ -1025,7 +1080,9 @@ void dspAROpenItems::sPostCreditMemo()
     setDate.exec();
     if (setDate.lastError().type() != QSqlError::NoError)
     {
-          systemError(this, setDate.lastError().databaseText(), __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this,
+                               tr("Error Posting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                               setDate, __FILE__, __LINE__);
     }
   }
 
@@ -1046,8 +1103,10 @@ void dspAROpenItems::sPostCreditMemo()
     if (result < 0)
     {
       rollback.exec();
-      systemError( this, storedProcErrorLookup("postCreditMemo", result),
-            __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this,
+                           tr("Error Posting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                           storedProcErrorLookup("postCreditMemo", result),
+                           __FILE__, __LINE__);
       return;
     }
     else
@@ -1066,17 +1125,19 @@ void dspAROpenItems::sPostCreditMemo()
   else if (postq.lastError().databaseText().contains("post to closed period"))
   {
     rollback.exec();
-    systemError(this, tr("Could not post Credit Memo #%1 because of a missing exchange rate.")
-                                      .arg(list()->currentItem()->text("docnumber")));
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Credit Memo %1\n").arg(list()->currentItem()->text("docnumber")),
+                         postq, __FILE__, __LINE__);
     return;
   }
   else if (postq.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, tr("A System Error occurred posting Credit Memo#%1.\n%2")
-            .arg(list()->currentItem()->text("docnumber"))
-            .arg(postq.lastError().databaseText()),
-          __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Credit Memo#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(postq.lastError().databaseText()),
+                         postq, __FILE__, __LINE__);
   }
 
   omfgThis->sCreditMemosUpdated();
@@ -1096,7 +1157,7 @@ void dspAROpenItems::sPostInvoice()
 
   if (_privileges->check("ChangeARInvcDistDate"))
   {
-    getGLDistDate newdlg(this, "", TRUE);
+    getGLDistDate newdlg(this, "", true);
     newdlg.sSetDefaultLit(tr("Invoice Date"));
     if (newdlg.exec() == XDialog::Accepted)
     {
@@ -1114,13 +1175,22 @@ void dspAROpenItems::sPostInvoice()
     journal = dspPostInvoice.value("result").toInt();
     if (journal < 0)
     {
-      systemError(this, storedProcErrorLookup("fetchJournalNumber", journal), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this,
+                           tr("Error Posting Invoice#%1.\n%2")
+                                       .arg(list()->currentItem()->text("docnumber"))
+                                       .arg(dspPostInvoice.lastError().databaseText()),
+                             storedProcErrorLookup("fetchJournalNumber", journal),
+                             __FILE__, __LINE__);
       return;
     }
   }
   else if (dspPostInvoice.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, dspPostInvoice.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(dspPostInvoice.lastError().databaseText()),
+                         dspPostInvoice, __FILE__, __LINE__);
     return;
   }
 
@@ -1150,7 +1220,11 @@ void dspAROpenItems::sPostInvoice()
     setDate.bindValue(":invchead_id", list()->currentItem()->id("docnumber"));
     setDate.exec();
     if (setDate.lastError().type() != QSqlError::NoError)
-      systemError(this, setDate.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                       .arg(list()->currentItem()->text("docnumber"))
+                                       .arg(setDate.lastError().databaseText()),
+                         setDate, __FILE__, __LINE__);
   }
 
   sum.bindValue(":invchead_id", list()->currentItem()->id("docnumber"));
@@ -1174,23 +1248,31 @@ void dspAROpenItems::sPostInvoice()
       xrate.exec();
       if (xrate.lastError().type() != QSqlError::NoError)
       {
-        systemError(this, tr("System Error posting Invoice #%1\n%2")
-                            .arg(list()->currentItem()->text("docnumber"))
-                            .arg(xrate.lastError().databaseText()),
-                        __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this,
+                             tr("Error Posting Invoice#%1.\n%2")
+                                         .arg(list()->currentItem()->text("docnumber"))
+                                         .arg(xrate.lastError().databaseText()),
+                             xrate, __FILE__, __LINE__);
         return;
       }
       else if (!xrate.first() || xrate.value("curr_rate").isNull())
       {
-        systemError(this, tr("Could not post Invoice #%1 because of a missing exchange rate.")
-                                        .arg(list()->currentItem()->text("docnumber")));
+        ErrorReporter::error(QtCriticalMsg, this,
+                             tr("Error Posting Invoice#%1.\n%2")
+                                         .arg(list()->currentItem()->text("docnumber"))
+                                         .arg(xrate.lastError().databaseText()),
+                             xrate, __FILE__, __LINE__);
         return;
       }
     }
   }
   else if (sum.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, sum.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(sum.lastError().databaseText()),
+                         sum, __FILE__, __LINE__);
     return;
   }
 
@@ -1206,8 +1288,12 @@ void dspAROpenItems::sPostInvoice()
     if (result < 0)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("postInvoice", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this,
+                           tr("Error Posting Invoice#%1.\n%2")
+                                       .arg(list()->currentItem()->text("docnumber"))
+                                       .arg(post.lastError().databaseText()),
+                           storedProcErrorLookup("postInvoice", result),
+                           __FILE__, __LINE__);
     }
     else
     {
@@ -1225,19 +1311,23 @@ void dspAROpenItems::sPostInvoice()
   else if (post.lastError().databaseText().contains("post to closed period"))
   {
     rollback.exec();
-    systemError(this, tr("Could not post Invoice #%1 into a closed period.")
-                            .arg(list()->currentItem()->text("docnumber")));
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(post.lastError().databaseText()),
+                         post, __FILE__, __LINE__);
   }
   else if (post.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, tr("A System Error occurred posting Invoice #%1.\n%2")
-                .arg(list()->currentItem()->text("docnumber"))
-                    .arg(post.lastError().databaseText()),
-                    __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this,
+                         tr("Error Posting Invoice#%1.\n%2")
+                                     .arg(list()->currentItem()->text("docnumber"))
+                                     .arg(post.lastError().databaseText()),
+                         post, __FILE__, __LINE__);
   }
 
-  omfgThis->sInvoicesUpdated(-1, TRUE);
+  omfgThis->sInvoicesUpdated(-1, true);
 }
 
 void dspAROpenItems::sShipment()
@@ -1330,6 +1420,6 @@ void dspAROpenItems::sClosedToggled(bool checked)
   if (checked)
     _dates->setStartDate(QDate().currentDate().addDays(-90));
   else
-    _dates->setStartNull(tr("Earliest"), omfgThis->startOfTime(), TRUE);
+    _dates->setStartNull(tr("Earliest"), omfgThis->startOfTime(), true);
 }
 
