@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -30,6 +30,7 @@
 #include "dspGLTransactions.h"
 #include "financialReportNotes.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 #define cFlRoot  0
 #define cFlItem  1
@@ -43,7 +44,7 @@
 #define cBudget   4
 #define cDiff     5
 
-dspFinancialReport::dspFinancialReport(QWidget* parent, const char*, Qt::WFlags fl)
+dspFinancialReport::dspFinancialReport(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "dspFinancialReport", fl)
 {
   setupUi(optionsWidget());
@@ -293,7 +294,7 @@ void dspFinancialReport::sFillListStatement()
         if (dspFillListStatement.value("flcol_prcnt").toBool())
         {
           list()->addColumn(tr("%1\n% of Group").arg(label.value("flstmthead_qtr").toString()),
-                             _bigMoneyColumn, Qt::AlignRight, true, "flstitem_qtrprcent");
+                             _bigMoneyColumn, Qt::AlignRight, true, "flstitem_qtrprcnt");
           qc += ",flstmtitem_qtrprcnt, 'percent' AS flstitem_qtrprcnt_xtnumericrole";
         }
         if (dspFillListStatement.value("flcol_budget").toBool())
@@ -306,7 +307,7 @@ void dspFinancialReport::sFillListStatement()
           {
             list()->addColumn(tr("%1\n% of Group").arg(label.value("flstmthead_qtr").toString()),
                                _bigMoneyColumn, Qt::AlignRight, true, "flstmtitem_qtrbudgetprcnt" );
-            qc += ",flstmtitem_qtrbudgetprcnt, 'percent' AS flstmtitem_qtrbudgetprcent_xtnumericrole";
+            qc += ",flstmtitem_qtrbudgetprcnt, 'percent' AS flstmtitem_qtrbudgetprcnt_xtnumericrole";
           }
           if (dspFillListStatement.value("flcol_budgetdiff").toBool())
           {
@@ -353,8 +354,8 @@ void dspFinancialReport::sFillListStatement()
           if (dspFillListStatement.value("flcol_budgetprcnt").toBool())
           {
             list()->addColumn(tr("%1\n% of Group").arg(label.value("flstmthead_year").toString()),
-                               _bigMoneyColumn, Qt::AlignRight, true, "flstmtitem_yearbudgetprcent");
-            qc += ",flstmtitem_yearbudgetprcnt, 'percent' AS flstmtitem_yearbudgetprcent_xtnumericrole";
+                               _bigMoneyColumn, Qt::AlignRight, true, "flstmtitem_yearbudgetprcnt");
+            qc += ",flstmtitem_yearbudgetprcnt, 'percent' AS flstmtitem_yearbudgetprcnt_xtnumericrole";
           }
           if (dspFillListStatement.value("flcol_budgetdiff").toBool())
           {
@@ -455,10 +456,10 @@ void dspFinancialReport::sFillListStatement()
       dspFillListStatement.bindValue(":prjid", _prjid);
       dspFillListStatement.exec();
       list()->populate(dspFillListStatement, true);
-      if (dspFillListStatement.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Financial Information"),
+                                    dspFillListStatement, __FILE__, __LINE__))
       {
-	systemError(this, dspFillListStatement.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        return;
       }
       list()->expandAll();
     }
@@ -853,9 +854,9 @@ void dspFinancialReport::sFillListTrend()
   dspFillListTrend.bindValue(":spec", cFlSpec);
   dspFillListTrend.exec();
   list()->populate(dspFillListTrend, true);
-  if (dspFillListTrend.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Financial Information"),
+                                dspFillListTrend, __FILE__, __LINE__))
   {
-    systemError(this, dspFillListTrend.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   list()->expandAll();
@@ -1143,13 +1144,15 @@ bool dspFinancialReport::forwardUpdate()
     int result = dspforwardUpdate.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("forwardUpdateTrialBalance", result), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Financial Information"),
+                             storedProcErrorLookup("forwardUpdateTrialBalance", result),
+                             __FILE__, __LINE__);
       return false;
     }
   }
-  else if (dspforwardUpdate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Financial Information"),
+                                dspforwardUpdate, __FILE__, __LINE__))
   {
-    systemError(this, dspforwardUpdate.lastError().databaseText(), __FILE__, __LINE__);
     return false;
   }
   return true;
@@ -1257,7 +1260,7 @@ void dspFinancialReport::sNotes()
   params.append("period_id", _periods->id());
   params.append("flhead_id", _flhead->id());
 
-  financialReportNotes newdlg(this, "", TRUE);
+  financialReportNotes newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
