@@ -1,6 +1,7 @@
 // generated 2009-04-30
 // grep "public X[Dialog,Widget]" *.h | awk '{print "CLASSITEM("$2")"}' > getscreen_classlist.h
 CLASSITEM(absoluteCalendarItem)
+CLASSITEM(account1099)
 CLASSITEM(accountingPeriod)
 CLASSITEM(accountingPeriods)
 CLASSITEM(accountingYearPeriod)
@@ -28,6 +29,7 @@ CLASSITEM(archRestoreSalesHistory)
 CLASSITEM(arCreditMemoApplication)
 CLASSITEM(arOpenItem)
 CLASSITEM(arWorkBench)
+CLASSITEM(assessFinanceCharges)
 CLASSITEM(assignClassCodeToPlannerCode)
 CLASSITEM(assignItemToPlannerCode)
 CLASSITEM(assignLotSerial)
@@ -76,6 +78,7 @@ CLASSITEM(configurePD)
 CLASSITEM(configurePO)
 CLASSITEM(configureSO)
 CLASSITEM(configureSearchPath)
+CLASSITEM(configureWF)
 CLASSITEM(configureWO)
 CLASSITEM(confirmAchOK)
 CLASSITEM(contact)
@@ -231,10 +234,8 @@ CLASSITEM(dspReorderExceptionsByPlannerCode)
 CLASSITEM(dspReservations)
 CLASSITEM(dspRunningAvailability)
 CLASSITEM(dspSalesHistory)
-CLASSITEM(dspSalesOrdersByCustomer)
-CLASSITEM(dspSalesOrdersByCustomerPO)
+CLASSITEM(dspSalesOrders)
 CLASSITEM(dspSalesOrdersByItem)
-CLASSITEM(dspSalesOrdersByParameterList)
 CLASSITEM(dspSalesOrderStatus)
 CLASSITEM(dspShipmentsByDate)
 CLASSITEM(dspShipmentsBySalesOrder)
@@ -249,6 +250,7 @@ CLASSITEM(dspSummarizedBankrecHistory)
 CLASSITEM(dspSummarizedBOM)
 CLASSITEM(dspSummarizedGLTransactions)
 CLASSITEM(dspSummarizedSales)
+CLASSITEM(dspTax1099)
 CLASSITEM(dspTaxHistory)
 CLASSITEM(dspTimePhasedAvailability)
 CLASSITEM(dspTimePhasedBookings)
@@ -262,6 +264,7 @@ CLASSITEM(dspUndefinedManufacturedItems)
 CLASSITEM(dspUninvoicedReceivings)
 CLASSITEM(dspUnusedPurchasedItems)
 CLASSITEM(dspUsageStatistics)
+CLASSITEM(dspUserPrivileges)
 CLASSITEM(dspValidLocationsByItem)
 CLASSITEM(dspVendorAPHistory)
 CLASSITEM(dspVoucherRegister)
@@ -368,6 +371,7 @@ CLASSITEM(itemtax)
 CLASSITEM(itemUOM)
 CLASSITEM(labelForm)
 CLASSITEM(labelForms)
+CLASSITEM(ledgerControl)
 CLASSITEM(listRecurringInvoices)
 CLASSITEM(locales)
 CLASSITEM(location)
@@ -440,6 +444,7 @@ CLASSITEM(printOptions)
 CLASSITEM(printPackingListBatchByShipvia)
 CLASSITEM(printPackingList)
 CLASSITEM(printPoForm)
+CLASSITEM(printToForm)
 CLASSITEM(printPurchaseOrder)
 CLASSITEM(printPurchaseOrdersByAgent)
 CLASSITEM(printRaForm)
@@ -458,6 +463,7 @@ CLASSITEM(profitCenter)
 CLASSITEM(profitCenters)
 CLASSITEM(project)
 CLASSITEM(projects)
+CLASSITEM(projectTypes)
 CLASSITEM(prospect)
 CLASSITEM(prospects)
 CLASSITEM(purchaseOrder)
@@ -468,6 +474,7 @@ CLASSITEM(purgeCreditMemos)
 CLASSITEM(purgeInvoices)
 CLASSITEM(purgePostedCounts)
 CLASSITEM(purgePostedCountSlips)
+CLASSITEM(quickRelocateLot)
 CLASSITEM(quotes)
 CLASSITEM(reasonCode)
 CLASSITEM(reasonCodes)
@@ -512,6 +519,7 @@ CLASSITEM(salesHistoryInformation)
 CLASSITEM(salesOrder)
 CLASSITEM(salesOrderInformation)
 CLASSITEM(salesOrderItem)
+CLASSITEM(salesOrderSimple)
 CLASSITEM(salesRep)
 CLASSITEM(salesReps)
 CLASSITEM(saleType)
@@ -583,6 +591,7 @@ CLASSITEM(thawItemSitesByClassCode)
 CLASSITEM(todoItem)
 CLASSITEM(todoListCalendar)
 CLASSITEM(todoList)
+CLASSITEM(toggleBankrecCleared)
 CLASSITEM(transactionInformation)
 CLASSITEM(transferOrder)
 CLASSITEM(transferOrderItem)
@@ -628,7 +637,6 @@ CLASSITEM(vendors)
 CLASSITEM(vendorPriceList)
 CLASSITEM(vendorType)
 CLASSITEM(vendorTypes)
-CLASSITEM(vendorWorkBench)
 CLASSITEM(viewCheckRun)
 CLASSITEM(voidChecks)
 CLASSITEM(voucher)
