@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -17,13 +17,43 @@
 #include <QVariant>
 
 #include "arOpenItem.h"
+#include "errorReporter.h"
 #include "returnAuthorization.h"
 #include "storedProcErrorLookup.h"
 #include "todoItem.h"
-#include "characteristicAssignment.h"
+
 #include <openreports.h>
 
-incident::incident(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+bool incident::userHasPriv(const int pMode, const int pId)
+{
+  if (_privileges->check("MaintainAllIncidents"))
+    return true;
+  bool personalPriv = _privileges->check("MaintainPersonalIncidents");
+  if(pMode==cView)
+  {
+    if(_privileges->check("ViewAllIncidents"))
+      return true;
+    personalPriv = personalPriv || _privileges->check("ViewPersonalIncidents");
+  }
+
+  if(pMode==cNew)
+    return personalPriv;
+  else
+  {
+    XSqlQuery usernameCheck;
+    usernameCheck.prepare( "SELECT getEffectiveXtUser() IN (incdt_owner_username, incdt_assigned_username) AS canModify "
+                           "FROM incdt "
+                            "WHERE (incdt_id=:incdt_id);" );
+    usernameCheck.bindValue(":incdt_id", pId);
+    usernameCheck.exec();
+
+    if (usernameCheck.first())
+      return usernameCheck.value("canModify").toBool()&&personalPriv;
+    return false;
+  }
+}
+
+incident::incident(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   _statusCodes << "N" << "F" << "C" << "A" << "R" << "L";
@@ -46,7 +76,6 @@ incident::incident(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   connect(_editTodoItem,  SIGNAL(clicked()),        this,       SLOT(sEditTodoItem()));
   connect(_item,          SIGNAL(newId(int)),     _lotserial,   SLOT(setItemId(int)));
   connect(_newTodoItem,   SIGNAL(clicked()),        this,       SLOT(sNewTodoItem()));
-  //connect(_return,      SIGNAL(clicked()),        this, SLOT(sReturn()));
   connect(_buttonBox,     SIGNAL(accepted()),        this,       SLOT(sSave()));
   connect(_print,         SIGNAL(clicked()),        this,       SLOT(sPrint()));
   connect(_todoList,      SIGNAL(itemSelected(int)), _editTodoItem, SLOT(animateClick()));
@@ -54,22 +83,14 @@ incident::incident(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   connect(_todoList,      SIGNAL(valid(bool)),      this, SLOT(sHandleTodoPrivs()));
   connect(_viewAR,        SIGNAL(clicked()),        this, SLOT(sViewAR()));
   connect(_viewTodoItem,  SIGNAL(clicked()),        this,       SLOT(sViewTodoItem()));
-  connect(_deleteCharacteristic,SIGNAL(clicked()), this, SLOT(sDeleteCharacteristic()));
-  connect(_editCharacteristic,	SIGNAL(clicked()), this, SLOT(sEditCharacteristic()));
-  connect(_newCharacteristic,	SIGNAL(clicked()), this, SLOT(sNewCharacteristic()));
-  connect(_charass, SIGNAL(valid(bool)), _editCharacteristic, SLOT(setEnabled(bool)));
-  connect(_charass, SIGNAL(valid(bool)), _deleteCharacteristic, SLOT(setEnabled(bool)));
-  connect(_charass, SIGNAL(itemSelected(int)), _editCharacteristic, SLOT(animateClick()));
 
+  _charass->setType("INCDT");
   _severity->setType(XComboBox::IncidentSeverity);
   _priority->setType(XComboBox::IncidentPriority);
   _resolution->setType(XComboBox::IncidentResolution);
   _category->setType(XComboBox::IncidentCategory);
   _lotserial->setStrict(false);
 
-  _charass->addColumn(tr("Characteristic"), _itemColumn, Qt::AlignLeft, true, "char_name");
-  _charass->addColumn(tr("Value"),          -1,          Qt::AlignLeft, true, "charass_value");
-
   _incdthist->addColumn(tr("Username"),     _userColumn, Qt::AlignLeft, true, "incdthist_username");
   _incdthist->addColumn(tr("Date/Time"),_timeDateColumn, Qt::AlignLeft, true, "incdthist_timestamp");
   _incdthist->addColumn(tr("Description"),           -1, Qt::AlignLeft, true, "incdthist_descrip");
@@ -125,6 +146,7 @@ enum SetResponse incident::set(const ParameterList &pParams)
     _incdtid = param.toInt();
     populate();
     _lotserial->setItemId(_item->id());
+    _charass->setId(_incdtid);
   }
 
   param = pParams.value("mode", &valid);
@@ -142,9 +164,11 @@ enum SetResponse incident::set(const ParameterList &pParams)
         _number->setText(incidentet.value("number").toString());
         _comments->setId(_incdtid);
         _documents->setId(_incdtid);
+        _charass->setId(_incdtid);
         _alarms->setId(_incdtid);
         _recurring->setParent(_incdtid, "INCDT");
         _print->hide();
+        _project->setAllowedStatuses(ProjectLineEdit::Concept |  ProjectLineEdit::InProcess);
       }
       else
       {
@@ -153,6 +177,11 @@ enum SetResponse incident::set(const ParameterList &pParams)
                                    "\n%1" ).arg(incidentet.lastError().text()));
         reject();
       }
+      // Characteristics update incident history so we have to save the incident first
+      // when adding a characteristic in new mode otherwise we get foreign key errors
+      QPushButton *newbutton = _charass->findChild<QPushButton*>("_newCharacteristic");
+      disconnect(newbutton, SIGNAL(clicked()), _charass, SLOT(sNew()));
+      connect(newbutton, SIGNAL(clicked()), this, SLOT(sNewCharacteristic()));
     }
     else if (param.toString() == "edit")
     {
@@ -179,10 +208,7 @@ enum SetResponse incident::set(const ParameterList &pParams)
       _deleteTodoItem->setEnabled(false);
       _editTodoItem->setEnabled(false);
       _newTodoItem->setEnabled(false);
-      disconnect(_charass, SIGNAL(valid(bool)), _editCharacteristic, SLOT(setEnabled(bool)));
-      disconnect(_charass, SIGNAL(valid(bool)), _deleteCharacteristic, SLOT(setEnabled(bool)));
-      disconnect(_charass, SIGNAL(itemSelected(int)), _editCharacteristic, SLOT(animateClick()));
-      _newCharacteristic->setEnabled(false);
+      _charass->setReadOnly(true);
       _owner->setEnabled(false);
 
       _buttonBox->setStandardButtons(QDialogButtonBox::Close);
@@ -205,6 +231,12 @@ enum SetResponse incident::set(const ParameterList &pParams)
     _cntct->setId(param.toInt());
   }
 
+  param = pParams.value("prj_id", &valid);
+  if (valid)
+  {
+    _project->setId(param.toInt());
+  }
+  
   param = pParams.value("aropen_id", &valid);
   if (valid)
   {
@@ -271,14 +303,15 @@ void incident::sCancel()
       int result = incidentCancel.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("releaseNumber", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Incident"),
+                               storedProcErrorLookup("releaseNumber", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (incidentCancel.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Incident"),
+                                  incidentCancel, __FILE__, __LINE__))
     {
-      systemError(this, incidentCancel.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -290,14 +323,15 @@ void incident::sCancel()
       int result = incidentCancel.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("deleteIncident", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Incident"),
+                               storedProcErrorLookup("deleteIncident", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (incidentCancel.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Incident"),
+                                  incidentCancel, __FILE__, __LINE__))
     {
-      systemError(this, incidentCancel.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -359,7 +393,8 @@ bool incident::save(bool partial)
 
   if (!incidentave.exec("BEGIN"))
   {
-    systemError(this, incidentave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Incident"),
+                         incidentave, __FILE__, __LINE__);
     return false;
   }
 
@@ -435,7 +470,8 @@ bool incident::save(bool partial)
   if(!incidentave.exec() && incidentave.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, incidentave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Incident"),
+                         incidentave, __FILE__, __LINE__);
     return false;
   }
 
@@ -443,7 +479,8 @@ bool incident::save(bool partial)
   if (! _recurring->save(true, cp, &errmsg))
   {
     rollback.exec();
-    systemError(this, errmsg, __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Incident"),
+                         rollback, __FILE__, __LINE__);
     return false;
   }
 
@@ -451,7 +488,8 @@ bool incident::save(bool partial)
   if(incidentave.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, incidentave.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Incident"),
+                         incidentave, __FILE__, __LINE__);
     return false;
   }
 
@@ -478,9 +516,9 @@ void incident::sFillHistoryList()
   incidentFillHistoryList.bindValue(":contact", tr("Contact"));
   incidentFillHistoryList.exec();
   _incdthist->populate(incidentFillHistoryList);
-  if (incidentFillHistoryList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Incident History"),
+                                incidentFillHistoryList, __FILE__, __LINE__))
   {
-    systemError(this, incidentFillHistoryList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -551,6 +589,7 @@ void incident::populate()
 
     _comments->setId(_incdtid);
     _documents->setId(_incdtid);
+    _charass->setId(_incdtid);
     _alarms->setId(_incdtid);
 
     _project->setId(incidentpopulate.value("incdt_prj_id").toInt());
@@ -567,7 +606,6 @@ void incident::populate()
                           _incdtid : incidentpopulate.value("incdt_recurring_incdt_id").toInt(),
                           "INCDT");
 
-    sFillCharacteristicsList();
     sFillHistoryList();
     sFillTodoList();
 
@@ -580,6 +618,13 @@ void incident::sCRMAcctChanged(const int newid)
   _cntct->setSearchAcct(newid);
 }
 
+void incident::sNewCharacteristic()
+{
+  if (! save(false))
+    return;
+  _charass->sNew();
+}
+
 void incident::sNewTodoItem()
 {
   if (! save(true))
@@ -630,15 +675,17 @@ void incident::sDeleteTodoItem()
     int result = incidentDeleteTodoItem.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("deleteTodoItem", result));
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting To-Do Item"),
+                             storedProcErrorLookup("deleteTodoItem", result),
+                             __FILE__, __LINE__);
       return;
     }
     else
       sFillTodoList();
     }
-  else if (incidentDeleteTodoItem.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting To-Do Item"),
+                                incidentDeleteTodoItem, __FILE__, __LINE__))
   {
-    systemError(this, incidentDeleteTodoItem.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -646,25 +693,28 @@ void incident::sDeleteTodoItem()
 void incident::sFillTodoList()
 {
   XSqlQuery incidentFillTodoList;
-  incidentFillTodoList.prepare("SELECT todoitem_id, *, "
-            "       firstLine(todoitem_notes) AS todoitem_notes, "
+  incidentFillTodoList.prepare("SELECT todoitem_id, todoitem_owner_username,"
+            "       firstLine(todoitem_notes) AS todoitem_notes,"
+            "       todoitem_username, todoitem_name, todoitem_description,"
+            "       todoitem_status, todoitem_due_date,"
+            "       incdtpriority_name,"
             "       CASE WHEN (todoitem_status != 'C' AND"
             "                  todoitem_due_date < CURRENT_DATE) THEN 'expired'"
             "            WHEN (todoitem_status != 'C' AND"
             "                  todoitem_due_date > CURRENT_DATE) THEN 'future'"
             "       END AS todoitem_due_date_qtforegroundrole "
             "  FROM todoitem "
-            "       LEFT OUTER JOIN incdtpriority ON (incdtpriority_id=todoitem_priority_id) "
-            "WHERE ( (todoitem_incdt_id=:incdt_id) "
-            "  AND   (todoitem_active) ) "
-            "ORDER BY todoitem_due_date, todoitem_username;");
+            "  LEFT OUTER JOIN incdtpriority ON (incdtpriority_id=todoitem_priority_id)"
+            " WHERE ((todoitem_incdt_id=:incdt_id)"
+            "   AND  todoitem_active )"
+            " ORDER BY todoitem_due_date, todoitem_username;");
 
   incidentFillTodoList.bindValue(":incdt_id", _incdtid);
   incidentFillTodoList.exec();
   _todoList->populate(incidentFillTodoList);
-  if (incidentFillTodoList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting To-Do Items"),
+                           incidentFillTodoList, __FILE__, __LINE__))
   {
-    systemError(this, incidentFillTodoList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -805,62 +855,3 @@ void incident::sAssigned()
   if (_status->currentIndex() < 3 && !_assignedTo->username().isEmpty())
     _status->setCurrentIndex(3);
 }
-
-void incident::sNewCharacteristic()
-{
-  if (! save(true))
-    return;
-
-  ParameterList params;
-  params.append("mode", "new");
-  params.append("incdt_id", _incdtid);
-
-  characteristicAssignment newdlg(this, "", TRUE);
-  newdlg.set(params);
-
-  if (newdlg.exec() != XDialog::Rejected)
-    sFillCharacteristicsList();
-}
-
-void incident::sEditCharacteristic()
-{
-  ParameterList params;
-  params.append("mode", "edit");
-  params.append("charass_id", _charass->id());
-
-  characteristicAssignment newdlg(this, "", TRUE);
-  newdlg.set(params);
-
-  if (newdlg.exec() != XDialog::Rejected)
-    sFillCharacteristicsList();
-}
-
-void incident::sDeleteCharacteristic()
-{
-  XSqlQuery incidentDeleteCharacteristic;
-  incidentDeleteCharacteristic.prepare( "DELETE FROM charass "
-             "WHERE (charass_id=:charass_id);" );
-  incidentDeleteCharacteristic.bindValue(":charass_id", _charass->id());
-  incidentDeleteCharacteristic.exec();
-
-  sFillCharacteristicsList();
-}
-
-void incident::sFillCharacteristicsList()
-{
-  XSqlQuery qry;
-  qry.prepare( "SELECT charass_id, char_name, "
-               " CASE WHEN char_type < 2 THEN "
-               "   charass_value "
-               " ELSE "
-               "   formatDate(charass_value::date) "
-               "END AS charass_value "
-               "FROM charass, char "
-               "WHERE ( (charass_target_type='INCDT')"
-               " AND (charass_char_id=char_id)"
-               " AND (charass_target_id=:incdt_id) ) "
-               "ORDER BY char_order, char_name;" );
-  qry.bindValue(":incdt_id", _incdtid);
-  qry.exec();
-  _charass->populate(qry);
-}
