@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,23 +21,27 @@
 #include "distributeInventory.h"
 #include "issueLineToShipping.h"
 #include "mqlutil.h"
+#include "reserveSalesOrderItem.h"
 #include "shipOrder.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
-issueToShipping::issueToShipping(QWidget* parent, const char* name, Qt::WFlags fl)
+issueToShipping::issueToShipping(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl),
       _captive(false)
 {
   setupUi(this);
 
-  connect(_ship, SIGNAL(clicked()), this, SLOT(sShip()));
-  connect(_issueLine, SIGNAL(clicked()), this, SLOT(sIssueLineBalance()));
-  connect(_issueAll, SIGNAL(clicked()), this, SLOT(sIssueAllBalance()));
-  connect(_issueStock, SIGNAL(clicked()), this, SLOT(sIssueStock()));
-  connect(_order,       SIGNAL(valid(bool)), this, SLOT(sFillList()));
-  connect(_returnStock, SIGNAL(clicked()), this, SLOT(sReturnStock()));
-  connect(_bcFind, SIGNAL(clicked()), this, SLOT(sBcFind()));
-  connect(_soitem, SIGNAL(itemSelectionChanged()), this, SLOT(sHandleButtons()));
+  connect(_ship,        SIGNAL(clicked()),                              this,         SLOT(sShip()));
+  connect(_issueLine,   SIGNAL(clicked()),                              this,         SLOT(sIssueLineBalance()));
+  connect(_issueAll,    SIGNAL(clicked()),                              this,         SLOT(sIssueAllBalance()));
+  connect(_issueStock,  SIGNAL(clicked()),                              this,         SLOT(sIssueStock()));
+  connect(_order,       SIGNAL(valid(bool)),                            this,         SLOT(sFillList()));
+  connect(_returnStock, SIGNAL(clicked()),                              this,         SLOT(sReturnStock()));
+  connect(_bcFind,      SIGNAL(clicked()),                              this,         SLOT(sBcFind()));
+  connect(_soitem,      SIGNAL(itemSelectionChanged()),                 this,         SLOT(sHandleButtons()));
+  connect(_soitem,      SIGNAL(populateMenu(QMenu*,QTreeWidgetItem *)), this,         SLOT(sPopulateMenu(QMenu *)));
+  connect(_warehouse,   SIGNAL(newID(int)),                             this,         SLOT(sFillList()));
 
   _order->setAllowedStatuses(OrderLineEdit::Open);
   _order->setAllowedTypes(OrderLineEdit::Sales |
@@ -54,25 +58,53 @@ issueToShipping::issueToShipping(QWidget* parent, const char* name, Qt::WFlags f
   omfgThis->inputManager()->notify(cBCTransferOrderLineItem, this, this, SLOT(sCatchToitemid(int)));
   omfgThis->inputManager()->notify(cBCWorkOrder, this, this, SLOT(sCatchWoid(int)));
 
-  _soitem->addColumn(tr("#"),           _seqColumn,   Qt::AlignCenter, true, "linenumber");
-  _soitem->addColumn(tr("Item Number"), _itemColumn,  Qt::AlignLeft,   true, "item_number");
-  _soitem->addColumn(tr("Description"),  -1,          Qt::AlignLeft,   true, "itemdescrip");
-  _soitem->addColumn(tr("Site"),        _whsColumn,   Qt::AlignCenter, true, "warehous_code");
-  _soitem->addColumn(tr("Sched. Date"), _qtyColumn, Qt::AlignRight,  true, "scheddate");
-  _soitem->addColumn(tr("UOM"),         _uomColumn,   Qt::AlignLeft,   true, "uom_name");
-  _soitem->addColumn(tr("Ordered"),     _qtyColumn,   Qt::AlignRight,  true, "qtyord");
-  _soitem->addColumn(tr("Shipped"),     _qtyColumn,   Qt::AlignRight,  true, "qtyshipped");
-  _soitem->addColumn(tr("Returned"),    _qtyColumn,   Qt::AlignRight,  false, "qtyreturned");
-  _soitem->addColumn(tr("Balance"),     _qtyColumn,   Qt::AlignRight,  false, "balance");
-  _soitem->addColumn(tr("At Shipping"), _qtyColumn,   Qt::AlignRight,  true, "atshipping");
+  if (_metrics->boolean("EnableSOReservationsByLocation"))
+  {
+    _lineItemsLit->setText("Line Items with Reservations:");
+
+    _soitem->addColumn(tr("#"),                _seqColumn,   Qt::AlignCenter, true,  "linenumber");
+    _soitem->addColumn(tr("Item/Location"),    _itemColumn,  Qt::AlignLeft,   true,  "item_number");
+    _soitem->addColumn(tr("Desc./LotSerial"),   -1,          Qt::AlignLeft,   true,  "itemdescrip");
+    _soitem->addColumn(tr("Site"),             _whsColumn,   Qt::AlignCenter, true,  "warehous_code");
+    _soitem->addColumn(tr("Sched. Date"),      _qtyColumn,   Qt::AlignRight,  true,  "scheddate");
+    _soitem->addColumn(tr("UOM"),              _uomColumn,   Qt::AlignLeft,   true,  "uom_name");
+    _soitem->addColumn(tr("Ordered/Reserved"), _qtyColumn,   Qt::AlignRight,  true,  "qtyord");
+    _soitem->addColumn(tr("Shipped"),          _qtyColumn,   Qt::AlignRight,  true,  "qtyshipped");
+    _soitem->addColumn(tr("Returned"),         _qtyColumn,   Qt::AlignRight,  false, "qtyreturned");
+    _soitem->addColumn(tr("Balance"),          _qtyColumn,   Qt::AlignRight,  false, "balance");
+    _soitem->addColumn(tr("At Shipping"),      _qtyColumn,   Qt::AlignRight,  true,  "atshipping");
+  }
+  else
+  {
+    _soitem->addColumn(tr("#"),           _seqColumn,   Qt::AlignCenter, true,  "linenumber");
+    _soitem->addColumn(tr("Item Number"), _itemColumn,  Qt::AlignLeft,   true,  "item_number");
+    _soitem->addColumn(tr("Description"),  -1,          Qt::AlignLeft,   true,  "itemdescrip");
+    _soitem->addColumn(tr("Site"),        _whsColumn,   Qt::AlignCenter, true,  "warehous_code");
+    _soitem->addColumn(tr("Sched. Date"), _qtyColumn,   Qt::AlignRight,  true,  "scheddate");
+    _soitem->addColumn(tr("UOM"),         _uomColumn,   Qt::AlignLeft,   true,  "uom_name");
+    _soitem->addColumn(tr("Ordered"),     _qtyColumn,   Qt::AlignRight,  true,  "qtyord");
+    _soitem->addColumn(tr("Shipped"),     _qtyColumn,   Qt::AlignRight,  true,  "qtyshipped");
+    _soitem->addColumn(tr("Returned"),    _qtyColumn,   Qt::AlignRight,  false, "qtyreturned");
+    _soitem->addColumn(tr("Balance"),     _qtyColumn,   Qt::AlignRight,  false, "balance");
+    _soitem->addColumn(tr("At Shipping"), _qtyColumn,   Qt::AlignRight,  true,  "atshipping");
+  }
+
   _soitem->setSelectionMode(QAbstractItemView::ExtendedSelection);
 
-  _order->setFromSitePrivsEnforced(TRUE);
+  _order->setFromSitePrivsEnforced(true);
+  _order->setFocus();
 
   _bcQty->setValidator(omfgThis->qtyVal());
 
-  if(_metrics->boolean("EnableSOReservations"))
+  if (!_metrics->boolean("MultiWhs"))
   {
+    _warehouseLit->hide();
+    _warehouse->hide();
+  }
+  
+  if(_metrics->boolean("EnableSOReservations") || _metrics->boolean("DisallowNegativeInventory"))
+  {
+    _requireInventory->setForgetful(true);
     _requireInventory->setChecked(true);
     _requireInventory->setEnabled(false);
   }
@@ -155,9 +187,9 @@ void issueToShipping::sCatchSoitemid(int pSoitemid)
     _soitem->setId(pSoitemid);
     sIssueStock();
   }
-  else if (issueCatchSoitemid.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                issueCatchSoitemid, __FILE__, __LINE__))
   {
-    systemError(this, issueCatchSoitemid.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -183,9 +215,9 @@ void issueToShipping::sCatchToitemid(int porderitemid)
     _soitem->setId(porderitemid);
     sIssueStock();
   }
-  else if (issueCatchToitemid.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                issueCatchToitemid, __FILE__, __LINE__))
   {
-    systemError(this, issueCatchToitemid.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -254,9 +286,9 @@ void issueToShipping::sCatchWoid(int pWoid)
       _soitem->setId(issueCatchWoid.value("coitem_id").toInt());
       sIssueStock();
     }
-    else if (issueCatchWoid.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  issueCatchWoid, __FILE__, __LINE__))
     {
-      systemError(this, issueCatchWoid.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -264,9 +296,23 @@ void issueToShipping::sCatchWoid(int pWoid)
     audioReject();
 }
 
+void issueToShipping::sPopulateMenu(QMenu *pMenu)
+{
+  if (_metrics->boolean("EnableSOReservations"))
+  {
+    QAction *menuItem;
+    menuItem = pMenu->addAction(tr("Unreserve Stock"), this, SLOT(sUnreserveStock()));
+    menuItem->setEnabled(_privileges->check("MaintainReservations"));
+    menuItem = pMenu->addAction(tr("Reserve Stock..."), this, SLOT(sReserveStock()));
+    menuItem->setEnabled(_privileges->check("MaintainReservations"));
+    menuItem = pMenu->addAction(tr("Reserve Line Balance"), this, SLOT(sReserveLineBalance()));
+    menuItem->setEnabled(_privileges->check("MaintainReservations"));
+  }
+}
+
 void issueToShipping::sIssueStock()
 {
-  bool update  = FALSE;
+  bool update  = false;
   QList<XTreeWidgetItem*> selected = _soitem->selectedItems();
   for (int i = 0; i < selected.size(); i++)
   {
@@ -278,9 +324,9 @@ void issueToShipping::sIssueStock()
     if(_requireInventory->isChecked())
       params.append("requireInventory");
     
-    issueLineToShipping newdlg(this, "", TRUE);
+    issueLineToShipping newdlg(this, "", true);
     if (newdlg.set(params) == NoError && newdlg.exec() != XDialog::Rejected)
-      update = TRUE;
+      update = true;
   }
 
   if (update)
@@ -322,24 +368,26 @@ bool issueToShipping::sufficientItemInventory(int porderitemid)
                        "   AND  (toitem_tohead_id=tohead_id)"
                        "   AND  (toitem_id=<? value(\"toitem_id\") ?>));"
                        "<? endif ?>" ;
-        MetaSQLQuery errm(errs);
-        issueufficientItemInventory = errm.toQuery(errp);
-        if (! issueufficientItemInventory.first() && issueufficientItemInventory.lastError().type() != QSqlError::NoError)
-          systemError(this, issueufficientItemInventory.lastError().databaseText(), __FILE__, __LINE__);
-        systemError(this,
-                    storedProcErrorLookup("sufficientInventoryToShipItem",
-                                          result)
-                    .arg(issueufficientItemInventory.value("item_number").toString())
-                    .arg(issueufficientItemInventory.value("warehous_code").toString()), __FILE__, __LINE__);
-        return false;
-      }
-    }
-    else if (issueufficientItemInventory.lastError().type() != QSqlError::NoError)
-    {
-      systemError(this, issueufficientItemInventory.lastError().databaseText(), __FILE__, __LINE__);
-      return false;
-    }
-  }
+         MetaSQLQuery errm(errs);
+         issueufficientItemInventory = errm.toQuery(errp);
+         if (! issueufficientItemInventory.first() && issueufficientItemInventory.lastError().type() != QSqlError::NoError)
+           ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                              issueufficientItemInventory, __FILE__, __LINE__);
+
+         ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                                  storedProcErrorLookup("sufficientInventoryToShipItem", result)
+                                  .arg(issueufficientItemInventory.value("item_number").toString())
+                                  .arg(issueufficientItemInventory.value("warehous_code").toString()),
+                                  __FILE__, __LINE__);
+         return false;
+       }
+     }
+     else if (ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                                  issueufficientItemInventory, __FILE__, __LINE__))
+     {
+       return false;
+     }
+   }
 
   return true;
 }
@@ -359,13 +407,15 @@ bool issueToShipping::sufficientInventory(int porderheadid)
       int result = issueufficientInventory.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("sufficientInventoryToShipOrder", result), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                               storedProcErrorLookup("sufficientInventoryToShipOrder", result),
+                               __FILE__, __LINE__);
         return false;
       }
     }
-    else if (issueufficientInventory.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Insufficient Inventory To Ship"),
+                                  issueufficientInventory, __FILE__, __LINE__))
     {
-      systemError(this, issueufficientInventory.lastError().databaseText(), __FILE__, __LINE__);
       return false;
     }
   }
@@ -422,34 +472,47 @@ bool issueToShipping::sIssueLineBalance(int id, int altId)
       if (itemlocSeries < 0)
       {
         rollback.exec();
-        systemError(this, storedProcErrorLookup("postProduction", itemlocSeries),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Production"),
+                               storedProcErrorLookup("postProduction", itemlocSeries),
+                               __FILE__, __LINE__);
         return false;
       }
-      else if (distributeInventory::SeriesAdjust(itemlocSeries, this) == XDialog::Rejected)
+      
+      if (itemlocSeries > 0)
       {
-        rollback.exec();
-        QMessageBox::information( this, tr("Issue to Shipping"), tr("Issue Canceled") );
-        return false;
-      }
-
-      // Need to get the inventory history id so we can auto reverse the distribution when issuing
-      prod.prepare("SELECT invhist_id "
-                   "FROM invhist "
-                   "WHERE ((invhist_series = :itemlocseries) "
-                   " AND (invhist_transtype = 'RM')); ");
-      prod.bindValue(":itemlocseries" , itemlocSeries);
-      prod.exec();
-      if (prod.first())
-        invhistid = prod.value("invhist_id").toInt();
-      else
-      {
-        rollback.exec();
-        systemError(this, tr("Inventory history not found"),
-                    __FILE__, __LINE__);
-        return false;
+        if (distributeInventory::SeriesAdjust(itemlocSeries, this) == XDialog::Rejected)
+        {
+          rollback.exec();
+          QMessageBox::information( this, tr("Issue to Shipping"), tr("Issue Canceled") );
+          return false;
+        }
+        
+        // Need to get the inventory history id so we can auto reverse the distribution when issuing
+        prod.prepare("SELECT invhist_id "
+                     "FROM invhist "
+                     "WHERE ((invhist_series = :itemlocseries) "
+                     " AND (invhist_transtype = 'RM')); ");
+        prod.bindValue(":itemlocseries" , itemlocSeries);
+        prod.exec();
+        if (prod.first())
+          invhistid = prod.value("invhist_id").toInt();
+        else
+        {
+          rollback.exec();
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                               tr("Inventory history not found")
+                               .arg(windowTitle()),__FILE__,__LINE__);
+          return false;
+        }
       }
     }
+    else if (prod.lastError().type() != QSqlError::NoError)
+    {
+      rollback.exec();
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Production"),
+                           prod, __FILE__, __LINE__);
+      return false;
+    }
   }
 
   issue.prepare("SELECT issueLineBalanceToShipping(:ordertype, :soitem_id, :ts, :itemlocseries, :invhist_id) AS result;");
@@ -467,8 +530,9 @@ bool issueToShipping::sIssueLineBalance(int id, int altId)
     if (result < 0)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("issueLineBalanceToShipping", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Item"),
+                             storedProcErrorLookup("issueLineBalanceToShipping", result),
+                             __FILE__, __LINE__);
       return false;
     }
     else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
@@ -485,9 +549,11 @@ bool issueToShipping::sIssueLineBalance(int id, int altId)
       XSqlQuery lsdetail;
       lsdetail.prepare("INSERT INTO lsdetail "
 	                   "            (lsdetail_itemsite_id, lsdetail_created, lsdetail_source_type, "
-	  				   "             lsdetail_source_id, lsdetail_source_number, lsdetail_ls_id, lsdetail_qtytoassign) "
+	  				   "             lsdetail_source_id, lsdetail_source_number, lsdetail_ls_id, lsdetail_qtytoassign, "
+                                           "             lsdetail_expiration, lsdetail_warrpurc ) "
 					   "SELECT invhist_itemsite_id, NOW(), 'TR', "
-					   "       :orderitemid, invhist_ordnumber, invdetail_ls_id, (invdetail_qty * -1.0) "
+					   "       :orderitemid, invhist_ordnumber, invdetail_ls_id, (invdetail_qty * -1.0), "
+                                           "       invdetail_expiration, invdetail_warrpurc "
 					   "FROM invhist JOIN invdetail ON (invdetail_invhist_id=invhist_id) "
 					   "WHERE (invhist_series=:itemlocseries);");
       lsdetail.bindValue(":orderitemid", id);
@@ -496,7 +562,8 @@ bool issueToShipping::sIssueLineBalance(int id, int altId)
       if (lsdetail.lastError().type() != QSqlError::NoError)
       {
         rollback.exec();
-        systemError(this, lsdetail.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Item"),
+                             lsdetail, __FILE__, __LINE__);
         return false;
       }
 	}
@@ -506,7 +573,8 @@ bool issueToShipping::sIssueLineBalance(int id, int altId)
   else if (issue.lastError().type() != QSqlError::NoError)
   {
     rollback.exec();
-    systemError(this, issue.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Item"),
+                         issue, __FILE__, __LINE__);
     return false;
   }
   return true;
@@ -525,8 +593,9 @@ void issueToShipping::sIssueAllBalance()
     XTreeWidgetItem *cursor = (XTreeWidgetItem*)_soitem->topLevelItem(i);
     if (sIssueLineBalance(cursor->id(),cursor->altId()))
       refresh = true;
-    else
-      break;
+    // attempt to issue all lines
+    //else
+    //  break;
   }
 
   if (refresh)
@@ -556,9 +625,10 @@ void issueToShipping::sReturnStock()
       if (result < 0)
       {
         rollback.exec();
-        systemError( this, storedProcErrorLookup("returnItemShipments", result),
-                     __FILE__, __LINE__);
-        return;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Returning Stock"),
+                               storedProcErrorLookup("returnItemShipments", result),
+                               __FILE__, __LINE__);
+          return;
       }
       else if (distributeInventory::SeriesAdjust(result, this) == XDialog::Rejected)
       {
@@ -571,7 +641,8 @@ void issueToShipping::sReturnStock()
     else if (issueReturnStock.lastError().type() != QSqlError::NoError)
     {
       rollback.exec();
-      systemError(this, issueReturnStock.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Returning Stock"),
+                           issueReturnStock, __FILE__, __LINE__);
       return;
     }
   }
@@ -582,38 +653,32 @@ void issueToShipping::sReturnStock()
 void issueToShipping::sShip()
 {
   XSqlQuery issueShip;
-  issueShip.prepare( "SELECT shiphead_id "
-             "FROM shiphead JOIN shipitem ON (shipitem_shiphead_id=shiphead_id) "
-             "WHERE ((NOT shiphead_shipped)"
-             "  AND  (shiphead_order_type=:ordertype)"
-             "  AND  (shiphead_order_id=:order_id) ) "
-             "LIMIT 1;" );
+  issueShip.prepare( "SELECT getOpenShipmentId(:ordertype, :order_id, :warehous_id) AS shiphead_id;" );
   issueShip.bindValue(":order_id",  _order->id());
   issueShip.bindValue(":ordertype", _order->type());
+  issueShip.bindValue(":warehous_id", _warehouse->id());
 
   issueShip.exec();
-  if (issueShip.first())
+  if ( (issueShip.first()) && (issueShip.value("shiphead_id").toInt() > 0) )
   {
     // Reset _order so that lock is released prior to shipping and potentially auto receiving
     // to avoid locking conflicts
-    _transDate->setDate(omfgThis->dbDate());
     _order->setId(-1);
     _order->setFocus();
 
     ParameterList params;
     params.append("shiphead_id", issueShip.value("shiphead_id").toInt());
+    params.append("transdate", _transDate->date());
 
-    shipOrder newdlg(this, "", TRUE);
+    shipOrder newdlg(this, "", true);
     if (newdlg.set(params) == NoError && newdlg.exec() != XDialog::Rejected)
     {
-      //_transDate->setDate(omfgThis->dbDate());
-      //_order->setId(-1);
-      //_order->setFocus();
+      _transDate->setDate(omfgThis->dbDate());
     }
   }
-  else if (issueShip.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Item"),
+                                issueShip, __FILE__, __LINE__))
   {
-    systemError(this, issueShip.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -627,6 +692,82 @@ void issueToShipping::sShip()
     close();
 }
 
+void issueToShipping::sReserveStock()
+{
+  QList<XTreeWidgetItem *> selected = _soitem->selectedItems();
+  for (int i = 0; i < selected.size(); i++)
+  {
+    ParameterList params;
+    params.append("soitem_id", ((XTreeWidgetItem *)(selected[i]))->id());
+    
+    reserveSalesOrderItem newdlg(this, "", true);
+    newdlg.set(params);
+    newdlg.exec();
+  }
+  
+  sFillList();
+}
+
+void issueToShipping::sReserveLineBalance()
+{
+  XSqlQuery reserveSales;
+  reserveSales.prepare("SELECT reserveSoLineBalance(:soitem_id) AS result;");
+  QList<XTreeWidgetItem *> selected = _soitem->selectedItems();
+  for (int i = 0; i < selected.size(); i++)
+  {
+    reserveSales.bindValue(":soitem_id", ((XTreeWidgetItem *)(selected[i]))->id());
+    reserveSales.exec();
+    if (reserveSales.first())
+    {
+      int result = reserveSales.value("result").toInt();
+      if (result < 0)
+      {
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                               storedProcErrorLookup("reserveSoLineBalance", result),
+                               __FILE__, __LINE__);
+        return;
+      }
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  reserveSales, __FILE__, __LINE__))
+    {
+      return;
+    }
+  }
+  
+  sFillList();
+}
+
+void issueToShipping::sUnreserveStock()
+{
+  XSqlQuery unreserveSales;
+  unreserveSales.prepare("SELECT unreserveSoLineQty(:soitem_id) AS result;");
+  QList<XTreeWidgetItem *> selected = _soitem->selectedItems();
+  for (int i = 0; i < selected.size(); i++)
+  {
+    unreserveSales.bindValue(":soitem_id", ((XTreeWidgetItem *)(selected[i]))->id());
+    unreserveSales.exec();
+    if (unreserveSales.first())
+    {
+      int result = unreserveSales.value("result").toInt();
+      if (result < 0)
+      {
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                               storedProcErrorLookup("unreservedSoLineQty", result),
+                               __FILE__, __LINE__);
+        return;
+      }
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  unreserveSales, __FILE__, __LINE__))
+    {
+      return;
+    }
+  }
+  
+  sFillList();
+}
+
 void issueToShipping::sFillList()
 {
   XSqlQuery issueFillList;
@@ -639,38 +780,38 @@ void issueToShipping::sFillList()
   }
   else if (_order->isSO())
   {
-    issueFillList.prepare( "SELECT cohead_holdtype "
-               "FROM cohead "
-               "WHERE (cohead_id=:sohead_id);" );
+    issueFillList.prepare("SELECT soHoldType(cohead_id) AS holdtype "
+                          "FROM cohead "
+                          "WHERE (cohead_id=:sohead_id);" );
     issueFillList.bindValue(":sohead_id", _order->id());
     issueFillList.exec();
     if (issueFillList.first())
     {
-      if (issueFillList.value("cohead_holdtype").toString() == "C")
+      if (issueFillList.value("holdtype").toString() == "C")
       {
         QMessageBox::critical( this, tr("Cannot Issue Stock"),
-                               storedProcErrorLookup("issuetoshipping", -12));
+                              storedProcErrorLookup("issuetoshipping", -12));
         _order->setId(-1);
         _order->setFocus();
       }
-      else if (issueFillList.value("cohead_holdtype").toString() == "P")
+      else if (issueFillList.value("holdtype").toString() == "P")
       {
         QMessageBox::critical( this, tr("Cannot Issue Stock"),
-                               storedProcErrorLookup("issuetoshipping", -13));
+                              storedProcErrorLookup("issuetoshipping", -13));
         _order->setId(-1);
         _order->setFocus();
       }
-      else if (issueFillList.value("cohead_holdtype").toString() == "R")
+      else if (issueFillList.value("holdtype").toString() == "R")
       {
         QMessageBox::critical( this, tr("Cannot Issue Stock"),
-                               storedProcErrorLookup("issuetoshipping", -14));
+                              storedProcErrorLookup("issuetoshipping", -14));
         _order->setId(-1);
         _order->setFocus();
       }
     }
-    else if (issueFillList.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                  issueFillList, __FILE__, __LINE__))
     {
-      systemError(this, issueFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else
@@ -688,24 +829,31 @@ void issueToShipping::sFillList()
   }
   else
   {
-    systemError(this, tr("Unrecognized order type %1").arg(_order->type()),
-                __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("Window:%1:\n Unrecognized order type %2")
+                         .arg(windowTitle())
+                         .arg(_order->type()),__FILE__,__LINE__);
     return;
   }
 
   listp.append("ordertype", _order->type());
+  listp.append("warehous_id", _warehouse->id());
 
+  if (_metrics->boolean("EnableSOReservationsByLocation"))
+    listp.append("includeReservations");
+  
   MetaSQLQuery listm = mqlLoad("issueToShipping", "detail");
   XSqlQuery listq = listm.toQuery(listp);
   _soitem->populate(listq, true);
+  _soitem->expandAll();
 
   if (listq.first())
   {
     _shipment->setText(listq.value("shiphead_number").toString());
   }
-  if (listq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Item Information"),
+                                listq, __FILE__, __LINE__))
   {
-    systemError(this, listq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -771,9 +919,9 @@ void issueToShipping::sBcFind()
 
   if(!issueBcFind.first())
   {
-    if (issueBcFind.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Barcode Information"),
+                                  issueBcFind, __FILE__, __LINE__))
     {
-      systemError(this, issueBcFind.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     XMessageBox::message(this, QMessageBox::Warning, tr("No Match Found"),
@@ -798,7 +946,7 @@ void issueToShipping::sBcFind()
   params.append("issue");
   params.append("snooze");
 
-  issueLineToShipping newdlg(this, "", TRUE);
+  issueLineToShipping newdlg(this, "", true);
   if (newdlg.set(params) != NoError)
     return;
   sFillList();
@@ -851,9 +999,9 @@ void issueToShipping::sBcFind()
   // If there are no records then the order is complete
   if(!issueBcFind.first())
   {
-    if (issueBcFind.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Barcode Information"),
+                                  issueBcFind, __FILE__, __LINE__))
     {
-      systemError(this, issueBcFind.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     XMessageBox::message( this, QMessageBox::Information, tr("Order Complete"),
