@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,7 +22,7 @@ class itemPricingSchedule : public XDialog, public Ui::itemPricingSchedule
     Q_OBJECT
 
 public:
-    itemPricingSchedule(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    itemPricingSchedule(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~itemPricingSchedule();
 
 public slots:
@@ -33,6 +33,7 @@ public slots:
     virtual void sNew();
     virtual void sEdit();
     virtual void sDelete();
+    virtual void sCheckCurrency();
     virtual void sFillList();
     virtual void sFillList( int pIpsitemid );
     virtual void populate();
@@ -46,6 +47,7 @@ private:
     int _mode;
     QDate _updated;
     QString _rejectedMsg;
+    bool _listpricesched;
 
 };
 
