@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -20,10 +20,11 @@ class itemPricingSchedules : public XWidget, public Ui::itemPricingSchedules
     Q_OBJECT
 
 public:
-    itemPricingSchedules(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    itemPricingSchedules(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~itemPricingSchedules();
 
 public slots:
+    virtual SetResponse set( const ParameterList & pParams );
     virtual void sNew();
     virtual void sEdit();
     virtual void sView();
@@ -37,6 +38,9 @@ public slots:
 protected slots:
     virtual void languageChange();
 
+private:
+    bool _listpricesched;
+  
 };
 
 #endif // ITEMPRICINGSCHEDULES_H
