@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -79,6 +79,7 @@
 #include "dspDepositsRegister.h"
 #include "printStatementByCustomer.h"
 #include "printStatementsByCustomerType.h"
+#include "printStatementsByCustomerGroup.h"
 
 // GL
 #include "glTransaction.h"
@@ -102,6 +103,7 @@
 #include "dspGLSeries.h"
 #include "dspTrialBalances.h"
 #include "dspJournals.h"
+#include "ledgerControl.h"
 
 #include "companies.h"
 #include "profitCenters.h"
@@ -117,7 +119,9 @@
 #include "taxAuthorities.h"
 #include "taxAssignments.h"
 #include "taxRegistrations.h"
+#include "dspTax1099.h"
 #include "dspTaxHistory.h"
+#include "dspTaxReturn.h"
 
 #include "reconcileBankaccount.h"
 #include "bankAdjustment.h"
@@ -132,6 +136,7 @@
 #include "vendors.h"
 #include "bankAccounts.h"
 #include "checkFormats.h"
+#include "assessFinanceCharges.h"
 
 #include "customers.h"
 
@@ -226,7 +231,7 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     { "menu", tr("&Voucher"), (char*)apVoucherMenu, apMenu, "true", NULL, NULL, true, NULL },
     { "ar.enterNewVoucher", tr("&New..."), SLOT(sEnterVoucher()), apVoucherMenu, "MaintainVouchers", NULL, NULL, true , NULL },
     { "ar.enterNewMiscVoucher", tr("New &Miscellaneous..."), SLOT(sEnterMiscVoucher()), apVoucherMenu, "MaintainVouchers", NULL, NULL, true , NULL },
-    { "ar.listUnpostedVouchers", tr("&List Unposted..."), SLOT(sUnpostedVouchers()), apVoucherMenu, "MaintainVouchers ViewVouchers", QPixmap(":/images/listUnpostedVouchers.png"), toolBar, true , tr("List Unposted Vouchers") },
+    { "ar.listUnpostedVouchers", tr("&List Unposted..."), SLOT(sUnpostedVouchers()), apVoucherMenu, "MaintainVouchers ViewVouchers", new QPixmap(":/images/listUnpostedVouchers.png"), toolBar, true , tr("List Unposted Vouchers") },
     { "separator", NULL, NULL, apVoucherMenu, "true", NULL, NULL, true, NULL },
     { "ar.postVouchers", tr("&Post..."), SLOT(sPostVouchers()), apVoucherMenu, "PostVouchers", NULL, NULL, true , NULL },
 
@@ -239,37 +244,37 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
 
     // Accounting | Accaunts Payable |  Payments
     { "menu", tr("&Payments"), (char*)apPaymentsMenu, apMenu, "true", NULL, NULL, true, NULL },
-    { "ap.selectPayments", tr("&Select..."), SLOT(sSelectPayments()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , tr("Select Payments") },
-    { "ap.listSelectPayments", tr("&List Selected..."), SLOT(sSelectedPayments()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
+    { "ap.selectPayments", tr("&Approve..."), SLOT(sSelectPayments()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , tr("Approve Payments") },
+    { "ap.listSelectPayments", tr("&List Approved..."), SLOT(sSelectedPayments()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
     { "separator", NULL, NULL, apPaymentsMenu, "true", NULL, NULL, true, NULL },
-    { "ap.prepareCheckRun", tr("&Prepare Check Run..."), SLOT(sPrepareCheckRun()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
-    { "ap.createMiscCheck", tr("Create &Miscellaneous Check..."), SLOT(sCreateMiscCheck()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
-    { "ap.viewCheckRun", tr("Vie&w Check Run..."), SLOT(sViewCheckRun()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
+    { "ap.prepareCheckRun", tr("&Prepare Payment Run..."), SLOT(sPrepareCheckRun()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
+    { "ap.createMiscCheck", tr("Create &Miscellaneous Payment..."), SLOT(sCreateMiscCheck()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
+    { "ap.viewCheckRun", tr("Vie&w Payment Run..."), SLOT(sViewCheckRun()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
     { "separator", NULL, NULL, apPaymentsMenu, "true", NULL, NULL, true, NULL },
-    { "ap.voidCheckRun", tr("&Void Check Run..."), SLOT(sVoidCheckRun()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
+    { "ap.voidCheckRun", tr("&Void Payment Run..."), SLOT(sVoidCheckRun()), apPaymentsMenu, "MaintainPayments", NULL, NULL, true , NULL },
     { "separator", NULL, NULL, apPaymentsMenu, "true", NULL, NULL, true, NULL },
-    { "ap.postCheck", tr("Post &Check..."), SLOT(sPostCheck()), apPaymentsMenu, "PostPayments", NULL, NULL, true , NULL },
-    { "ap.postChecks", tr("P&ost Checks..."), SLOT(sPostChecks()), apPaymentsMenu, "PostPayments", NULL, NULL, true , NULL },
+    { "ap.postCheck", tr("Post &Payment..."), SLOT(sPostCheck()), apPaymentsMenu, "PostPayments", NULL, NULL, true , NULL },
+    { "ap.postChecks", tr("P&ost Payments..."), SLOT(sPostChecks()), apPaymentsMenu, "PostPayments", NULL, NULL, true , NULL },
                        
     { "separator", NULL, NULL, apMenu, "true", NULL, NULL, true, NULL },
-    { "ap.workbench", tr("&Workbench..."), SLOT(sApWorkBench()), apMenu, "MaintainPayments MaintainAPMemos", QPixmap(":/images/viewCheckRun.png"), toolBar, true, tr("Payables Workbench") },
+    { "ap.workbench", tr("&Workbench..."), SLOT(sApWorkBench()), apMenu, "MaintainPayments MaintainAPMemos", new QPixmap(":/images/viewCheckRun.png"), toolBar, true, tr("Payables Workbench") },
     { "separator", NULL, NULL, apMenu, "true", NULL, NULL, true, NULL },
     
     // Accounting | Accaunts Payable | Forms
     { "menu", tr("&Forms"), (char*)apFormsMenu, apMenu, "true", NULL, NULL, true, NULL },
     { "ap.printPurchaseOrder", tr("Print Purchase &Order..."), SLOT(sPrintPurchaseOrder()), apFormsMenu, "PrintPurchaseOrders", NULL, NULL, true , NULL },
     { "separator", NULL, NULL, apFormsMenu, "true", NULL, NULL, true, NULL },
-    { "ap.printCheck", tr("Print &Check..."), SLOT(sPrintCheck()), apFormsMenu, "MaintainPayments", NULL, NULL, true , NULL },
-    { "ap.printCheckRun", tr("Print Check &Run..."), SLOT(sPrintCheckRun()), apFormsMenu, "MaintainPayments", NULL, NULL, true , NULL },
+    { "ap.printCheck", tr("Print &Payment..."), SLOT(sPrintCheck()), apFormsMenu, "MaintainPayments", NULL, NULL, true , NULL },
+    { "ap.printCheckRun", tr("Print Payment &Run..."), SLOT(sPrintCheckRun()), apFormsMenu, "MaintainPayments", NULL, NULL, true , NULL },
     
     // Accounting | Accaunts Payable |  Reports
     { "menu", tr("&Reports"), (char*)apReportsMenu, apMenu, "true", NULL, NULL, true, NULL },
     { "ap.uninvoicedReceipts", tr("&Uninvoiced Receipts and Returns..."), SLOT(sDspUninvoicedReceipts()), apReportsMenu, "ViewUninvoicedReceipts MaintainUninvoicedReceipts", NULL, NULL, true , NULL },
     { "separator", NULL, NULL, apReportsMenu, "true", NULL, NULL, true, NULL },
     { "ap.dspOpenAPItemsByVendor", tr("Open &Payables..."), SLOT(sDspAPOpenItemsByVendor()), apReportsMenu, "ViewAPOpenItems", NULL, NULL, true , NULL },
-    { "ap.dspAPAging", tr("&Aging..."), SLOT(sDspTimePhasedOpenAPItems()), apReportsMenu, "ViewAPOpenItems", QPixmap(":/images/apAging.png"), toolBar, true , tr("Payables Aging") },
+    { "ap.dspAPAging", tr("&Aging..."), SLOT(sDspTimePhasedOpenAPItems()), apReportsMenu, "ViewAPOpenItems", new QPixmap(":/images/apAging.png"), toolBar, true , tr("Payables Aging") },
     { "separator", NULL, NULL, apReportsMenu, "true", NULL, NULL, true, NULL },
-    { "ap.dspCheckRegister", tr("&Check Register..."), SLOT(sDspCheckRegister()), apReportsMenu, "MaintainPayments", NULL, NULL, true , NULL },
+    { "ap.dspCheckRegister", tr("&Payment Register..."), SLOT(sDspCheckRegister()), apReportsMenu, "MaintainPayments", NULL, NULL, true , NULL },
     { "ap.dspVoucherRegister", tr("&Voucher Register..."), SLOT(sDspVoucherRegister()), apReportsMenu, "MaintainVouchers ViewVouchers", NULL, NULL, true , NULL },
     { "separator", NULL, NULL, apReportsMenu, "true", NULL, NULL, true, NULL },
     { "ap.dspAPApplications", tr("&Applications..."), SLOT(sDspAPApplications()), apReportsMenu, "ViewAPOpenItems", NULL, NULL, true , NULL },
@@ -286,9 +291,10 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     { "menu", tr("&Invoice"), (char*)arInvoicesMenu,	arMenu, "true",	 NULL, NULL, true, NULL },
     { "ar.createInvoice", tr("&New..."), SLOT(sCreateInvoice()), arInvoicesMenu, "MaintainMiscInvoices", NULL, NULL, true , NULL },
     { "ar.listRecurringInvoices", tr("&List Recurring Invoices..."),	SLOT(sRecurringInvoices()), arInvoicesMenu, "SelectBilling",	NULL, NULL,  true, NULL },
-    { "ar.listUnpostedInvoices", tr("&List Unposted..."), SLOT(sUnpostedInvoices()), arInvoicesMenu, "SelectBilling", QPixmap(":/images/unpostedInvoices.png"), toolBar, true , tr("List Unposted Invoices") },
+    { "ar.listUnpostedInvoices", tr("&List Unposted..."), SLOT(sUnpostedInvoices()), arInvoicesMenu, "SelectBilling", new QPixmap(":/images/unpostedInvoices.png"), toolBar, true , tr("List Unposted Invoices") },
     { "separator", NULL, NULL, arInvoicesMenu, "true", NULL, NULL, true, NULL },
     { "ar.postInvoices", tr("&Post..."), SLOT(sPostInvoices()), arInvoicesMenu, "PostMiscInvoices", NULL, NULL, true , NULL },
+    { "ar.assessFinanceCharges", tr("&Assess Finance Charges..."), SLOT(sAssessFinanceCharges()), arInvoicesMenu, "PostMiscInvoices", NULL, NULL, true , NULL },
 
     // Accounting | Accounts Receivable | Memos
     { "menu", tr("&Memos"), (char*)arMemosMenu,	arMenu, "true",	 NULL, NULL, true, NULL },
@@ -300,11 +306,11 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     // Accounting | Accounts Receivable | Cash Receipts
     { "menu", tr("C&ash Receipt"), (char*)arCashReceiptsMenu,	arMenu, "true",	 NULL, NULL, true, NULL },
     { "ar.enterCashReceipt", tr("&New..."), SLOT(sEnterCashReceipt()), arCashReceiptsMenu, "MaintainCashReceipts", NULL, NULL, true , NULL },
-    { "ar.cashReceiptEditList", tr("&Edit List..."), SLOT(sCashReceiptEditList()), arCashReceiptsMenu, "MaintainCashReceipts ViewCashReceipt", QPixmap(":/images/editCashReceipts.png"), toolBar, true , tr("Cash Receipt Edit List") },
+    { "ar.cashReceiptEditList", tr("&Edit List..."), SLOT(sCashReceiptEditList()), arCashReceiptsMenu, "MaintainCashReceipts ViewCashReceipt", new QPixmap(":/images/editCashReceipts.png"), toolBar, true , tr("Cash Receipt Edit List") },
     { "ar.postCashReceipts", tr("&Post..."), SLOT(sPostCashReceipts()), arCashReceiptsMenu, "PostCashReceipts", NULL, NULL, true , NULL },
 
     { "separator", NULL, NULL, arMenu, "true", NULL, NULL, true, NULL },
-    { "ar.arWorkBench", tr("&Workbench..."), SLOT(sArWorkBench()), arMenu, "ViewAROpenItems" , QPixmap(":/images/arWorkbench.png"), toolBar, true , tr("Receivables Workbench") },
+    { "ar.arWorkBench", tr("&Workbench..."), SLOT(sArWorkBench()), arMenu, "ViewAROpenItems" , new QPixmap(":/images/arWorkbench.png"), toolBar, true , tr("Receivables Workbench") },
 
     { "separator", NULL, NULL, arMenu, "true", NULL, NULL, true, NULL },
     // Accounting | Accounts Receivable | Forms
@@ -314,13 +320,13 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     { "separator", NULL, NULL, arFormsMenu, "true", NULL, NULL, true, NULL },
     { "ar.printStatementByCustomer", tr("Print S&tatement by Customer..."), SLOT(sPrintStatementByCustomer()), arFormsMenu, "ViewAROpenItems", NULL, NULL, true , NULL },
     { "ar.printStatementsByCustomerType", tr("Print State&ments by Customer Type..."), SLOT(sPrintStatementsByCustomerType()), arFormsMenu, "ViewAROpenItems", NULL, NULL, true , NULL },
-
+    { "ar.printStatementsByCustomerGroup", tr("Print Statement by Customer &Group..."), SLOT(sPrintStatementsByCustomerGroup()), arFormsMenu, "ViewAROpenItems", NULL, NULL, true, NULL },
     // Accounting | Accounts Receivable | Reports
     { "menu", tr("&Reports"), (char*)arReportsMenu,	arMenu, "true",	 NULL, NULL, true, NULL },
     { "ar.dspInvoiceInformation", tr("&Invoice Information..."), SLOT(sDspInvoiceInformation()), arReportsMenu, "ViewAROpenItems", NULL, NULL, true , NULL },
     { "separator", NULL, NULL, arReportsMenu, "true", NULL, NULL, true, NULL },
     { "ar.dspOpenItems", tr("&Open Receivables..."), SLOT(sDspAROpenItems()), arReportsMenu, "ViewAROpenItems", NULL, NULL, true , NULL },
-    { "ar.dspARAging", tr("A&ging..."), SLOT(sDspTimePhasedOpenItems()), arReportsMenu, "ViewAROpenItems", QPixmap(":/images/arAging.png"), toolBar, true , tr("Receivables Aging") },
+    { "ar.dspARAging", tr("A&ging..."), SLOT(sDspTimePhasedOpenItems()), arReportsMenu, "ViewAROpenItems", new QPixmap(":/images/arAging.png"), toolBar, true , tr("Receivables Aging") },
     { "separator", NULL, NULL, arReportsMenu, "true", NULL, NULL, true, NULL }, 
     { "ar.dspInvoiceRegister", tr("In&voice Register..."), SLOT(sDspInvoiceRegister()), arReportsMenu, "ViewInvoiceRegister", NULL, NULL, true , NULL },
     { "ar.dspCashReceipts", tr("Cash &Receipts..."), SLOT(sDspCashReceipts()), arReportsMenu, "ViewAROpenItems", NULL, NULL, true , NULL },
@@ -340,7 +346,7 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     { "gl.simpleEntry",	    tr("S&imple..."),	SLOT(sSimpleEntry()),		glEnterTransactionMenu,	"PostJournalEntries", NULL, NULL, true, NULL },
     { "gl.seriesEntry",     tr("&Series..."),	SLOT(sSeriesEntry()),		glEnterTransactionMenu,	"PostJournalEntries", NULL, NULL, true, NULL },
     { "separator",	    NULL,				NULL,			        glEnterTransactionMenu,   "true",					NULL, NULL, true, NULL },
-    { "gl.unpostedEntries", tr("&List Unposted..."), SLOT(sUnpostedEntries()),	glEnterTransactionMenu,	"PostJournalEntries", QPixmap(":/images/journalEntries.png"), toolBar,  true, tr("List Unposted Journal Entries") },
+    { "gl.unpostedEntries", tr("&List Unposted..."), SLOT(sUnpostedEntries()),	glEnterTransactionMenu,	"PostJournalEntries", new QPixmap(":/images/journalEntries.png"), toolBar,  true, tr("List Unposted Journal Entries") },
 
     // Accounting | G/L | Standard Journals
     { "menu",			     tr("&Standard Journals"),		   (char*)glStandardJournalsMenu,	     glMenu,		   "true",					      NULL, NULL, true, NULL },
@@ -364,10 +370,12 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     { "gl.dspGLSeries",			tr("&Series..."),			SLOT(sDspGLSeries()),			glReportsMenu, "ViewGLTransactions",	NULL, NULL, true, NULL },
     { "gl.dspStandardJournalHistory",	tr("Standard &Journal History..."),	SLOT(sDspStandardJournalHistory()),	glReportsMenu, "ViewGLTransactions",	NULL, NULL, true, NULL },
     { "separator", NULL, NULL, glReportsMenu, "true", NULL, NULL, _metrics->boolean("UseJournals"), NULL },
-    { "gl.dspJournals",	tr("Journals..."),	SLOT(sDspJournals()),	glReportsMenu, "ViewJournals",	NULL, NULL, _metrics->boolean("UseJournals"), NULL },
+    { "gl.dspJournals",	  tr("Journals..."),	SLOT(sDspJournals()),	glReportsMenu, "ViewJournals",	     NULL, NULL, _metrics->boolean("UseJournals"), NULL },
+    { "separator",        NULL,                 NULL,                   glReportsMenu, "true",               NULL, NULL, true, NULL },
+    { "gl.ledgerControl", tr("Ledger Control"), SLOT(sLedgerControl()), glReportsMenu, "ViewGLTransactions", NULL, NULL, true, NULL },
 
     { "menu",			tr("&Bank Reconciliation"), 	(char*)bankrecMenu,		mainMenu,    "true",						NULL, NULL, true, NULL },
-    { "gl.reconcileBankaccnt",	tr("&Reconcile..."),SLOT(sReconcileBankaccount()),	bankrecMenu, "MaintainBankRec", QPixmap(":/images/bankReconciliation.png"), toolBar,  true, tr("Reconcile Bank Account") },
+    { "gl.reconcileBankaccnt",	tr("&Reconcile..."),SLOT(sReconcileBankaccount()),	bankrecMenu, "MaintainBankRec", new QPixmap(":/images/bankReconciliation.png"), toolBar,  true, tr("Reconcile Bank Account") },
     { "separator",		NULL,				NULL,				bankrecMenu, "true",						NULL, NULL, true, NULL },
     { "gl.enterAdjustment",	tr("&New Adjustment..."),	SLOT(sEnterAdjustment()),	bankrecMenu, "MaintainBankAdjustments",	NULL, NULL, true, NULL },
     { "gl.adjustmentEditList",	tr("Adjustment Edit &List..."),	SLOT(sAdjustmentEditList()),	bankrecMenu, "MaintainBankAdjustments ViewBankAdjustments", NULL, NULL, true, NULL },
@@ -386,8 +394,8 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     { "gl.createFinancialReports",tr("&New Financial Report..."),	SLOT(sNewFinancialReport()),		financialReportsMenu,		"MaintainFinancialLayouts", NULL, NULL, true, NULL },
     { "gl.editFinancialReports",  tr("&List Financial Reports..."),	SLOT(sFinancialReports()),		financialReportsMenu,		"MaintainFinancialLayouts", NULL, NULL, true, NULL },
     { "separator",		  NULL,					NULL,					financialReportsMenu,		"true",					       NULL, NULL, true, NULL },
-    { "gl.dspTrialBalances",	  tr("View &Trial Balances..."),		SLOT(sDspTrialBalances()),		financialReportsMenu,		"ViewTrialBalances",	   QPixmap(":/images/viewTrialBalance.png"), toolBar,  true, NULL },
-    { "gl.viewFinancialReport",	  tr("View &Financial Report..."),	SLOT(sViewFinancialReport()),		financialReportsMenu,		"ViewFinancialReports",   QPixmap(":/images/viewFinancialReport.png"), toolBar, true, NULL },
+    { "gl.dspTrialBalances",	  tr("View &Trial Balances..."),		SLOT(sDspTrialBalances()),		financialReportsMenu,		"ViewTrialBalances",	   new QPixmap(":/images/viewTrialBalance.png"), toolBar,  true, NULL },
+    { "gl.viewFinancialReport",	  tr("View &Financial Report..."),	SLOT(sViewFinancialReport()),		financialReportsMenu,		"ViewFinancialReports",   new QPixmap(":/images/viewFinancialReport.png"), toolBar, true, NULL },
 
     { "separator",		  NULL,					NULL,					mainMenu,		"true",					       NULL, NULL, true, NULL },
     
@@ -397,7 +405,7 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     { "gl.accountingPeriods",	tr("Accounting &Periods..."),	SLOT(sAccountingPeriods()),	calendarMenu,	"MaintainAccountingPeriods",	NULL, NULL, true, NULL },
     
     // Accounting | Account
-    { "menu", tr("&Account"), (char*)coaMenu, mainMenu,	"true",	NULL, NULL, true, NULL },
+    { "menu", tr("&Ledger Accounts"), (char*)coaMenu, mainMenu,	"true",	NULL, NULL, true, NULL },
     { "gl.accountNumbers",	tr("&Chart of Accounts..."),	SLOT(sAccountNumbers()), coaMenu,	"MaintainChartOfAccounts",	NULL, NULL, true, NULL },
     { "gl.companies",		tr("C&ompanies..."),		SLOT(sCompanies()),		coaMenu,	"MaintainChartOfAccounts", NULL, NULL, (_metrics->value("GLCompanySize").toInt() > 0), NULL },
     { "gl.profitCenterNumber",	tr("&Profit Center Numbers..."),	SLOT(sProfitCenters()),	coaMenu,	"MaintainChartOfAccounts", NULL, NULL, (_metrics->value("GLProfitSize").toInt() > 0), NULL },
@@ -423,7 +431,9 @@ menuAccounting::menuAccounting(GUIClient *Pparent) :
     // Accounting | Tax | Reports
     { "menu",			tr("&Reports"),	                (char*)taxReportsMenu,		taxMenu,	"true",			    NULL, NULL, true, NULL },
     { "gl.dspTaxHistory",	tr("&Tax History..."),           SLOT(sDspTaxHistory()),        taxReportsMenu, "ViewTaxReconciliations",   NULL, NULL, true, NULL },
-     
+    { "gl.dspTaxReturn",	tr("&Tax Return..."),            SLOT(sDspTaxReturn()),         taxReportsMenu, "ViewTaxReconciliations",   NULL, NULL, true, NULL },
+    { "gl.dspTax1099",	tr("&Info for 1099..."),                 SLOT(sDspTax1099()),           taxReportsMenu, "ViewTaxReconciliations",   NULL, NULL, true, NULL },
+    
     { "separator",		  NULL,					NULL,					mainMenu,		"true",					       NULL, NULL, true, NULL },
 
     // Accounting | Utilities
@@ -454,11 +464,7 @@ void menuAccounting::addActionsToMenu(actionProperties acts[], unsigned int numE
   QAction * m = 0;
   for (unsigned int i = 0; i < numElems; i++)
   {
-    if (! acts[i].visible)
-    {
-      continue;
-    }
-    else if (acts[i].actionName == QString("menu"))
+    if (acts[i].actionName == QString("menu"))
     {
       m = acts[i].menu->addMenu((QMenu*)(acts[i].slot));
       if(m)
@@ -466,11 +472,11 @@ void menuAccounting::addActionsToMenu(actionProperties acts[], unsigned int numE
     }
     else if (acts[i].actionName == QString("separator"))
     {
-      acts[i].menu->addSeparator();
+      m = acts[i].menu->addSeparator();
     }
     else if ((acts[i].toolBar != NULL) && (acts[i].toolBar != NULL))
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -483,7 +489,7 @@ void menuAccounting::addActionsToMenu(actionProperties acts[], unsigned int numE
     }
     else if (acts[i].toolBar != NULL)
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -496,7 +502,7 @@ void menuAccounting::addActionsToMenu(actionProperties acts[], unsigned int numE
     }
     else
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -504,6 +510,7 @@ void menuAccounting::addActionsToMenu(actionProperties acts[], unsigned int numE
                   acts[i].menu,
                   acts[i].priv ) ;
     }
+    if (m) m->setVisible(acts[i].visible);
   }
 }
 
@@ -525,12 +532,12 @@ void menuAccounting::sUnpostedPurchaseOrders()
 
 void menuAccounting::sPrintPurchaseOrder()
 {
-  printPurchaseOrder(parent, "", TRUE).exec();
+  printPurchaseOrder(parent, "", true).exec();
 }
 
 void menuAccounting::sPostPurchaseOrder()
 {
-  postPurchaseOrder(parent, "", TRUE).exec();
+  postPurchaseOrder(parent, "", true).exec();
 }
 
 //  Vouchers
@@ -566,7 +573,7 @@ void menuAccounting::sUnpostedVouchers()
 
 void menuAccounting::sPostVouchers()
 {
-  postVouchers(parent, "", TRUE).exec();
+  postVouchers(parent, "", true).exec();
 }
 
 
@@ -593,7 +600,7 @@ void menuAccounting::sCreateMiscCheck()
 
 void menuAccounting::sPrepareCheckRun()
 {
-  prepareCheckRun(parent, "", TRUE).exec();
+  prepareCheckRun(parent, "", true).exec();
 }
 
 void menuAccounting::sViewCheckRun()
@@ -615,18 +622,18 @@ void menuAccounting::sPrintCheckRun()
 
 void menuAccounting::sVoidCheckRun()
 {
-  voidChecks newdlg(parent, "", TRUE);
+  voidChecks newdlg(parent, "", true);
   newdlg.exec();
 }
 
 void menuAccounting::sPostCheck()
 {
-  postCheck(parent, "", TRUE).exec();
+  postCheck(parent, "", true).exec();
 }
 
 void menuAccounting::sPostChecks()
 {
-  postChecks(parent, "", TRUE).exec();
+  postChecks(parent, "", true).exec();
 }
 
 void menuAccounting::sApWorkBench()
@@ -641,7 +648,7 @@ void menuAccounting::sEnterMiscApCreditMemo()
   params.append("mode", "new");
   params.append("docType", "creditMemo");
 
-  apOpenItem newdlg(parent, "", TRUE);
+  apOpenItem newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -657,7 +664,7 @@ void menuAccounting::sEnterMiscApDebitMemo()
   params.append("mode", "new");
   params.append("docType", "debitMemo");
 
-  apOpenItem newdlg(parent, "", TRUE);
+  apOpenItem newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -711,22 +718,22 @@ void menuAccounting::sUnpostedInvoices()
 
 void menuAccounting::sReprintInvoices()
 {
-  reprintInvoices(parent, "", TRUE).exec();
+  reprintInvoices(parent, "", true).exec();
 }
 
 void menuAccounting::sPrintInvoices()
 {
-  printInvoices(parent, "", TRUE).exec();
+  printInvoices(parent, "", true).exec();
 }
 
 void menuAccounting::sPostInvoices()
 {
-  postInvoices(parent, "", TRUE).exec();
+  postInvoices(parent, "", true).exec();
 }
 
 void menuAccounting::sPurgeInvoices()
 {
-  purgeInvoices(parent, "", TRUE).exec();
+  purgeInvoices(parent, "", true).exec();
 }
 
 
@@ -749,7 +756,7 @@ void menuAccounting::sCashReceiptEditList()
 
 void menuAccounting::sPostCashReceipts()
 {
-  postCashReceipts(parent, "", TRUE).exec();
+  postCashReceipts(parent, "", true).exec();
 }
 
 
@@ -760,7 +767,7 @@ void menuAccounting::sEnterMiscArCreditMemo()
   params.append("mode", "new");
   params.append("docType", "creditMemo");
 
-  arOpenItem newdlg(parent, "", TRUE);
+  arOpenItem newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -776,7 +783,7 @@ void menuAccounting::sEnterMiscArDebitMemo()
   params.append("mode", "new");
   params.append("docType", "debitMemo");
 
-  arOpenItem newdlg(parent, "", TRUE);
+  arOpenItem newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -834,7 +841,7 @@ void menuAccounting::sEnterStandardJournal()
   ParameterList params;
   params.append("mode", "new");
 
-  standardJournal newdlg(parent, "", TRUE);
+  standardJournal newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -851,12 +858,12 @@ void menuAccounting::sStandardJournalGroups()
 
 void menuAccounting::sPostStandardJournal()
 {
-  postStandardJournal(parent, "", TRUE).exec();
+  postStandardJournal(parent, "", true).exec();
 }
 
 void menuAccounting::sPostStandardJournalGroup()
 {
-  postStandardJournalGroup(parent, "", TRUE).exec();
+  postStandardJournalGroup(parent, "", true).exec();
 }
 
 void menuAccounting::sPostJournals()
@@ -869,7 +876,7 @@ void menuAccounting::sSimpleEntry()
   ParameterList params;
   params.append("mode", "new");
 
-  glTransaction newdlg(parent, "", TRUE);
+  glTransaction newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -879,7 +886,7 @@ void menuAccounting::sSeriesEntry()
   ParameterList params;
   params.append("mode", "new");
 
-  glSeries newdlg(parent, "", TRUE);
+  glSeries newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -905,7 +912,7 @@ void menuAccounting::sNewFinancialReport()
   ParameterList params;
   params.append("mode", "new");
 
-  financialLayout newdlg(parent, "", TRUE);
+  financialLayout newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -942,6 +949,11 @@ void menuAccounting::sDspTrialBalances()
   omfgThis->handleNewWindow(new dspTrialBalances());
 }
 
+void menuAccounting::sLedgerControl()
+{
+  omfgThis->handleNewWindow(new ledgerControl());
+}
+
 //  Master Information
 void menuAccounting::sCompanies()
 {
@@ -986,8 +998,10 @@ void menuAccounting::sAccountingYearPeriods()
 void menuAccounting::sReconcileBankaccount()
 {
 //  omfgThis->handleNewWindow(new reconcileBankaccount());
-  reconcileBankaccount *newdlg = new reconcileBankaccount();
-  omfgThis->handleNewWindow(newdlg, Qt::ApplicationModal);
+// Open Application Modal
+//  reconcileBankaccount *newdlg = new reconcileBankaccount();
+//  omfgThis->handleNewWindow(newdlg, Qt::ApplicationModal);
+  reconcileBankaccount::openReconcileBankaccount(-1);
 }
 
 void menuAccounting::sEnterAdjustment()
@@ -1040,11 +1054,21 @@ void menuAccounting::sTaxRegistrations()
   omfgThis->handleNewWindow(new taxRegistrations());
 }
 
+void menuAccounting::sDspTax1099()
+{
+  omfgThis->handleNewWindow(new dspTax1099());
+}
+
 void menuAccounting::sDspTaxHistory()
 {
   omfgThis->handleNewWindow(new dspTaxHistory());
 }
 
+void menuAccounting::sDspTaxReturn()
+{
+  omfgThis->handleNewWindow(new dspTaxReturn());
+}
+
 void menuAccounting::sDspBankrecHistory()
 {
   omfgThis->handleNewWindow(new dspBankrecHistory());
@@ -1073,7 +1097,7 @@ void menuAccounting::sMaintainBudget()
 
 void menuAccounting::sForwardUpdateAccounts()
 {
-  forwardUpdateAccounts(parent, "", TRUE).exec();
+  forwardUpdateAccounts(parent, "", true).exec();
 }
 
 void menuAccounting::sVendors()
@@ -1088,12 +1112,17 @@ void menuAccounting::sCheckFormats()
 
 void menuAccounting::sPrintStatementByCustomer()
 {
-  printStatementByCustomer(parent, "", TRUE).exec();
+  printStatementByCustomer(parent, "", true).exec();
 }
 
 void menuAccounting::sPrintStatementsByCustomerType()
 {
-  printStatementsByCustomerType(parent, "", TRUE).exec();
+  printStatementsByCustomerType(parent, "", true).exec();
+}
+
+void menuAccounting::sPrintStatementsByCustomerGroup()
+{
+  printStatementsByCustomerGroup(parent, "", true).exec();
 }
 
 void menuAccounting::sCustomers()
@@ -1103,13 +1132,13 @@ void menuAccounting::sCustomers()
 
 void menuAccounting::sUpdateLateCustCreditStatus()
 {
-  updateLateCustCreditStatus newdlg(parent, "", TRUE);
+  updateLateCustCreditStatus newdlg(parent, "", true);
   newdlg.exec();
 }
 
 void menuAccounting::sCreateRecurringInvoices()
 {
-  createRecurringInvoices newdlg(parent, "", TRUE);
+  createRecurringInvoices newdlg(parent, "", true);
   newdlg.exec();
 }
 
@@ -1118,6 +1147,10 @@ void menuAccounting::sSyncCompanies()
   omfgThis->handleNewWindow(new syncCompanies());
 }
 
+void menuAccounting::sAssessFinanceCharges()
+{
+  omfgThis->handleNewWindow(new assessFinanceCharges());
+}
 
 void menuAccounting::sSetup()
 {
