@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -37,6 +37,7 @@
 #include "relocateInventory.h"
 
 #include "reassignLotSerial.h"
+#include "quickRelocateLot.h"
 
 #include "createCountTagsByParameterList.h"
 #include "createCountTagsByItem.h"
@@ -90,6 +91,7 @@
 #include "dspBacklog.h"
 #include "dspValidLocationsByItem.h"
 #include "dspQOH.h"
+#include "dspQOHByZone.h"
 #include "dspQOHByLocation.h"
 #include "dspInventoryLocator.h"
 #include "dspSlowMovingInventoryByClassCode.h"
@@ -103,6 +105,7 @@
 #include "dspTimePhasedUsageStatisticsByItem.h"
 
 #include "printItemLabelsByClassCode.h"
+#include "printToForm.h"
 
 #include "warehouses.h"
 #include "warehouse.h"
@@ -247,7 +250,7 @@ menuInventory::menuInventory(GUIClient *Pparent) :
     //  Inventory | Receiving
     { "menu",            tr("R&eceiving"),                 (char*)receivingMenu,  mainMenu,      "true",          NULL, NULL, true, NULL },
     { "sr.enterReceipt", tr("&New Receipt..."),            SLOT(sEnterReceipt()), receivingMenu, "EnterReceipts", NULL, NULL, true, NULL },
-    { "sr.postReceipts", tr("&List Unposted Receipts..."), SLOT(sPostReceipts()), receivingMenu, "EnterReceipts", QPixmap(":/images/postReceipts.png"), toolBar,  true, tr("List Unposted Receipts") },
+    { "sr.postReceipts", tr("&List Unposted Receipts..."), SLOT(sPostReceipts()), receivingMenu, "EnterReceipts", new QPixmap(":/images/postReceipts.png"), toolBar,  true, tr("List Unposted Receipts") },
     { "separator",       NULL,                             NULL,                  receivingMenu, "true",          NULL, NULL, true, NULL },
     { "sr.enterReturn",  tr("Purchase Order &Return..."),  SLOT(sEnterReturn()),  receivingMenu, "EnterReturns",  NULL, NULL, true, NULL },
     { "separator",       NULL,                             NULL,                  receivingMenu, "true",          NULL, NULL, true, NULL },
@@ -258,7 +261,7 @@ menuInventory::menuInventory(GUIClient *Pparent) :
 
     //  Inventory | Shipping
     { "menu",                        tr("&Shipping"),                      (char*)shippingMenu,           mainMenu,     "true",                 NULL, NULL, true, NULL },
-    { "sr.issueToShipping",          tr("&Issue to Shipping..."),          SLOT(sIssueStockToShipping()), shippingMenu, "IssueStockToShipping", QPixmap(":/images/issueStockToShipping.png"), toolBar,  true, tr("Issue to Shipping") },
+    { "sr.issueToShipping",          tr("&Issue to Shipping..."),          SLOT(sIssueStockToShipping()), shippingMenu, "IssueStockToShipping", new QPixmap(":/images/issueStockToShipping.png"), toolBar,  true, tr("Issue to Shipping") },
     { "sr.maintainShippingContents", tr("&Maintain Shipping Contents..."), SLOT(sDspShippingContents()),  shippingMenu, "ViewShipping",         NULL, NULL, true, NULL },
     { "separator",                   NULL,                                 NULL,                          shippingMenu, "true",                 NULL, NULL, true, NULL },
     { "sr.shipOrder",                tr("&Ship Order..."),                 SLOT(sShipOrders()),           shippingMenu, "ShipOrders",           NULL, NULL, true, NULL },
@@ -269,7 +272,7 @@ menuInventory::menuInventory(GUIClient *Pparent) :
     // Inventory | Shipping | Forms
     { "menu",                              tr("&Forms"),                                   (char*)shippingFormsMenu,                shippingMenu,      "true",                                          NULL, NULL, true, NULL },
     { "sr.packingListBatch",               tr("Packing List &Batch..."),                   SLOT(sPackingListBatch()),               shippingFormsMenu, "MaintainPackingListBatch ViewPackingListBatch", NULL, NULL, true, NULL },
-    { "sr.printPackingListBatchByShipvia", tr("Print Packing List Batch by Ship &Via..."), SLOT(sPrintPackingListBatchByShipvia()), shippingFormsMenu, "PrintPackingLists",  NULL, NULL, true, NULL },
+    { "sr.printPackingListBatchByShipvia", tr("Print Packing List Batch..."), SLOT(sPrintPackingListBatchByShipvia()), shippingFormsMenu, "PrintPackingLists",  NULL, NULL, true, NULL },
     { "sr.printPackingList",               tr("&Packing List..."),                         SLOT(sPrintPackingLists()),              shippingFormsMenu, "PrintPackingLists",  NULL, NULL, true, NULL },
     { "separator",                         NULL,                                           NULL,                                    shippingFormsMenu, "true",               NULL, NULL, true, NULL },
     { "sr.printShippingForm",              tr("&Shipping Form..."),                        SLOT(sPrintShippingForm()),              shippingFormsMenu, "PrintBillsOfLading", NULL, NULL, true, NULL },
@@ -314,6 +317,7 @@ menuInventory::menuInventory(GUIClient *Pparent) :
     // Inventory | Forms
     { "menu",                           tr("&Forms"),                (char*)formsMenu,                    mainMenu,  "true",            NULL, NULL, true, NULL },
     { "im.printItemLabelsByClassCode",  tr("Print &Item Labels..."), SLOT(sPrintItemLabelsByClassCode()), formsMenu, "ViewItemSites",   NULL, NULL, true, NULL },
+    { "im.printTOForms",  tr("Print &T/O Forms..."), SLOT(sPrintToForms()), formsMenu, "MaintainTransferOrders",   NULL, NULL, _metrics->boolean("MultiWhs"), NULL },
 
     //  Inventory | Reports
     { "menu",                           tr("&Reports"),                   (char*)reportsMenu,                   mainMenu,       "true", NULL, NULL, true, NULL },
@@ -323,6 +327,7 @@ menuInventory::menuInventory(GUIClient *Pparent) :
 
     //  Inventory | Reports | Quantities On Hand
     { "im.dspQOH",                    tr("&Quantities On Hand..."),                             SLOT(sDspQOH()),                reportsMenu, "ViewQOH",      NULL, NULL, true, NULL },
+    { "im.dspQOHByZone",              tr("&Quantities On Hand By &Zone..."),                    SLOT(sDspQOHByZone()),          reportsMenu, "ViewQOH",      NULL, NULL, true, NULL },
     { "im.dspQOHByLocation",          tr("Quantities On Hand By &Location..."),                 SLOT(sDspQOHByLocation()),      reportsMenu, "ViewQOH",      NULL, NULL, true, NULL },
 
     { "im.dspLocationLotSerialDetail",tr("&Location/Lot/Serial # Detail..."),SLOT(sDspLocationLotSerialDetail()),       reportsMenu,    "ViewQOH",      NULL, NULL, _metrics->boolean("LotSerialControl"), NULL },
@@ -332,7 +337,7 @@ menuInventory::menuInventory(GUIClient *Pparent) :
     {  "separator",                   NULL,                                  NULL,                                      reportsMenu,    "true",         NULL, NULL, true, NULL },
 
     //  Inventory| Reports | Inventory Availability
-    { "im.dspInventoryAvailability",       tr("Inventory &Availability..."), SLOT(sDspInventoryAvailability()), reportsMenu, "ViewInventoryAvailability", QPixmap(":/images/dspInventoryAvailabilityByPlannerCode.png"), toolBar, true, tr("Inventory Availability by Planner Code") },
+    { "im.dspInventoryAvailability",       tr("Inventory &Availability..."), SLOT(sDspInventoryAvailability()), reportsMenu, "ViewInventoryAvailability", new QPixmap(":/images/dspInventoryAvailabilityByPlannerCode.png"), toolBar, true, tr("Inventory Availability by Planner Code") },
     { "im.dspSubstituteAvailabilityByRootItem",         tr("&Substitute Availability..."),       SLOT(sDspSubstituteAvailabilityByRootItem()), reportsMenu, "ViewInventoryAvailability",        NULL, NULL, true, NULL },
     {  "separator",                   NULL,                                  NULL,                                      reportsMenu,    "true",         NULL, NULL, true, NULL },
 
@@ -354,9 +359,9 @@ menuInventory::menuInventory(GUIClient *Pparent) :
     // Inventory | Item Site
     { "menu",                           tr("&Item Site"),       (char*)itemSitesMenu,   mainMenu,       "true",              NULL, NULL, true, NULL },
     { "im.newItemSite",                 tr("&New..."),          SLOT(sNewItemSite()),   itemSitesMenu,  "MaintainItemSites", NULL, NULL, true, NULL },
-    { "im.listItemSites",               tr("&List..."),         SLOT(sItemSites()),     itemSitesMenu,  "MaintainItemSites ViewItemSites", QPixmap(":/images/itemSites.png"), toolBar, true, tr("List Item Sites") },                                                  
+    { "im.listItemSites",               tr("&List..."),         SLOT(sItemSites()),     itemSitesMenu,  "MaintainItemSites ViewItemSites", new QPixmap(":/images/itemSites.png"), toolBar, true, tr("List Item Sites") },
     { "separator", NULL, NULL, itemSitesMenu,   "true", NULL, NULL, true, NULL },
-    { "im.itemAvailabilityWorkbench",   tr("&Workbench..."),    SLOT(sDspItemAvailabilityWorkbench()),  itemSitesMenu, "ViewItemAvailabilityWorkbench", QPixmap(":/images/itemAvailabilityWorkbench.png"), toolBar, true, tr("Item Availability Workbench") },
+    { "im.itemAvailabilityWorkbench",   tr("&Workbench..."),    SLOT(sDspItemAvailabilityWorkbench()),  itemSitesMenu, "ViewItemAvailabilityWorkbench", new QPixmap(":/images/itemAvailabilityWorkbench.png"), toolBar, true, tr("Item Availability Workbench") },
 
     //  Inventory | Lot/Serial Control
     { "menu",                           tr("&Lot/Serial Control"),      (char*)lotSerialControlMenu,    mainMenu, "true",       NULL, NULL, _metrics->boolean("LotSerialControl") , NULL },
@@ -365,13 +370,14 @@ menuInventory::menuInventory(GUIClient *Pparent) :
     { "separator",                      NULL,                           NULL,   lotSerialControlMenu,   "true", NULL, NULL,  _metrics->boolean("LotSerialControl") , NULL },
 
     { "im.reassignLotSerialNumber",     tr("&Reassign Lot/Serial #..."),        SLOT(sReassignLotSerialNumber()), lotSerialControlMenu, "ReassignLotSerial",    NULL, NULL, _metrics->boolean("LotSerialControl"), NULL },
+    { "im.quickRelocateLot",            tr("&Quick Relocate..."),               SLOT(sQuickRelocateLot()),        lotSerialControlMenu, "true",  NULL, NULL, _metrics->boolean("LotSerialControl"), NULL },
 
     { "separator", NULL, NULL, mainMenu,        "true", NULL, NULL, true, NULL },
 
     // Inventory | Utilities
     { "menu",                                     tr("&Utilities"),                       (char*)utilitiesMenu,                     mainMenu,       "true",                    NULL, NULL, true, NULL },
     { "im.dspUnbalancedQOHByClassCode",           tr("U&nbalanced QOH..."),               SLOT(sDspUnbalancedQOHByClassCode()),     utilitiesMenu,  "ViewItemSites",           NULL, NULL, true, NULL },
-    { "im.adjustInvValue",                        tr("Adjust Avg. Cost Value..."),        SLOT(sAdjustInvValue()),                  utilitiesMenu,  "CreateAdjustmentTrans ViewCosts", NULL, NULL, _metrics->boolean("AllowAvgCostMethod"), NULL },
+    { "im.adjustInvValue",                        tr("Adjust Avg. Cost Value..."),        SLOT(sAdjustInvValue()),                  utilitiesMenu,  "CreateAdjustmentTrans", NULL, NULL, _metrics->boolean("AllowAvgCostMethod"), NULL },
     { "separator",                                NULL,                                   NULL,                                     utilitiesMenu,  "true",                    NULL, NULL, true, NULL },
 
     // Inventory | Utilities | Update Item Controls
@@ -424,11 +430,7 @@ void menuInventory::addActionsToMenu(actionProperties acts[], unsigned int numEl
   QAction * m = 0;
   for (unsigned int i = 0; i < numElems; i++)
   {
-    if (! acts[i].visible)
-    {
-      continue;
-    }
-    else if (acts[i].actionName == QString("menu"))
+    if (acts[i].actionName == QString("menu"))
     {
       m = acts[i].menu->addMenu((QMenu*)(acts[i].slot));
       if(m)
@@ -436,11 +438,11 @@ void menuInventory::addActionsToMenu(actionProperties acts[], unsigned int numEl
     }
     else if (acts[i].actionName == QString("separator"))
     {
-      acts[i].menu->addSeparator();
+      m = acts[i].menu->addSeparator();
     }
     else if ((acts[i].toolBar != NULL) && (!acts[i].toolTip.isEmpty()))
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -453,7 +455,7 @@ void menuInventory::addActionsToMenu(actionProperties acts[], unsigned int numEl
     }
     else if (acts[i].toolBar != NULL)
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -466,7 +468,7 @@ void menuInventory::addActionsToMenu(actionProperties acts[], unsigned int numEl
     }
     else
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -474,6 +476,7 @@ void menuInventory::addActionsToMenu(actionProperties acts[], unsigned int numEl
                   acts[i].menu,
                   acts[i].priv ) ;
     }
+    if (m) m->setVisible(acts[i].visible);
   }
 }
 
@@ -482,7 +485,7 @@ void menuInventory::sNewItemSite()
   ParameterList params;
   params.append("mode", "new");
 
-  itemSite newdlg(parent, "", TRUE);
+  itemSite newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -579,29 +582,35 @@ void menuInventory::sExpenseTrans()
 
 void menuInventory::sResetQOHBalances()
 {
-  resetQOHBalances(parent, "", TRUE).exec();
+  resetQOHBalances(parent, "", true).exec();
 }
 
 void menuInventory::sRelocateInventory()
 {
-  relocateInventory(parent, "", TRUE).exec();
+  relocateInventory(parent, "", true).exec();
 }
 
 
 //  Lot/Serial Control
 void menuInventory::sReassignLotSerialNumber()
 {
-  reassignLotSerial newdlg(parent, "", TRUE);
+  reassignLotSerial newdlg(parent, "", true);
   newdlg.exec();
 }
 
+void menuInventory::sQuickRelocateLot()
+{
+    quickRelocateLot newdlg(parent, "", true);
+    newdlg.exec();
+}
+
 
 void menuInventory::sCreateCountTagsByClassCode()
 {
   ParameterList params;
   params.append("classcode");
 
-  createCountTagsByParameterList newdlg(parent, "", TRUE);
+  createCountTagsByParameterList newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -611,19 +620,19 @@ void menuInventory::sCreateCountTagsByPlannerCode()
   ParameterList params;
   params.append("plancode");
 
-  createCountTagsByParameterList newdlg(parent, "", TRUE);
+  createCountTagsByParameterList newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
 
 void menuInventory::sCreateCountTagsByItem()
 {
-  createCountTagsByItem(parent, "", TRUE).exec();
+  createCountTagsByItem(parent, "", true).exec();
 }
 
 void menuInventory::sCreateCycleCountTags()
 {
-  createCycleCountTags(parent, "", TRUE).exec();
+  createCycleCountTags(parent, "", true).exec();
 }
 
 void menuInventory::sEnterCountSlip()
@@ -631,7 +640,7 @@ void menuInventory::sEnterCountSlip()
   ParameterList params;
   params.append("mode", "new");
 
-  countSlip newdlg(parent, "", TRUE);
+  countSlip newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -641,44 +650,44 @@ void menuInventory::sEnterCountTags()
   ParameterList params;
   params.append("mode", "new");
 
-  countTag newdlg(parent, "", TRUE);
+  countTag newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
 
 void menuInventory::sEnterMiscCount()
 {
-  enterMiscCount(parent, "", TRUE).exec();
+  enterMiscCount(parent, "", true).exec();
 }
 
 void menuInventory::sZeroUncountedTagsByWarehouse()
 {
-  zeroUncountedCountTagsByWarehouse(parent, "", TRUE).exec();
+  zeroUncountedCountTagsByWarehouse(parent, "", true).exec();
 }
 
 void menuInventory::sThawItemSitesByClassCode()
 {
-  thawItemSitesByClassCode(parent, "", TRUE).exec();
+  thawItemSitesByClassCode(parent, "", true).exec();
 }
 
 void menuInventory::sPostCountSlipsByWarehouse()
 {
-  postCountSlips(parent, "", TRUE).exec();
+  postCountSlips(parent, "", true).exec();
 }
 
 void menuInventory::sPostCountTags()
 {
-  postCountTags(parent, "", TRUE).exec();
+  postCountTags(parent, "", true).exec();
 }
 
 void menuInventory::sPurgePostedCountSlips()
 {
-  purgePostedCountSlips(parent, "", TRUE).exec();
+  purgePostedCountSlips(parent, "", true).exec();
 }
 
 void menuInventory::sPurgePostedCountTags()
 {
-  purgePostedCounts(parent, "", TRUE).exec();
+  purgePostedCounts(parent, "", true).exec();
 }
 
 
@@ -699,12 +708,12 @@ void menuInventory::sIssueStockToShipping()
 
 void menuInventory::sPrintPackingLists()
 {
-  printPackingList(parent, "", TRUE).exec();
+  printPackingList(parent, "", true).exec();
 }
 
 void menuInventory::sPrintPackingListBatchByShipvia()
 {
-  printPackingListBatchByShipvia(parent, "", TRUE).exec();
+  printPackingListBatchByShipvia(parent, "", true).exec();
 }
 
 void menuInventory::sPrintShippingForm()
@@ -714,33 +723,33 @@ void menuInventory::sPrintShippingForm()
 
 void menuInventory::sPrintShippingForms()
 {
-  printShippingForms(parent, "", TRUE).exec();
+  printShippingForms(parent, "", true).exec();
 }
 
 void menuInventory::sPrintShippingLabelsBySo()
 {
-  printLabelsBySo(parent, "", TRUE).exec();
+  printLabelsBySo(parent, "", true).exec();
 }
 
 void menuInventory::sPrintShippingLabelsByTo()
 {
-  printLabelsByTo(parent, "", TRUE).exec();
+  printLabelsByTo(parent, "", true).exec();
 }
 
 void menuInventory::sPrintShippingLabelsByInvoice()
 {
-  printLabelsByInvoice(parent, "", TRUE).exec();
+  printLabelsByInvoice(parent, "", true).exec();
 }
 
 void menuInventory::sPrintReceivingLabelsByPo()
 {
-  printLabelsByOrder(parent, "", TRUE).exec();
+  printLabelsByOrder(parent, "", true).exec();
 }
 
 
 void menuInventory::sShipOrders()
 {
-  shipOrder(parent, "", TRUE).exec();
+  shipOrder(parent, "", true).exec();
 }
 
 void menuInventory::sRecallOrders()
@@ -861,6 +870,11 @@ void menuInventory::sDspQOH()
   omfgThis->handleNewWindow(new dspQOH());
 }
 
+void menuInventory::sDspQOHByZone()
+{
+  omfgThis->handleNewWindow(new dspQOHByZone());
+}
+
 void menuInventory::sDspQOHByLocation()
 {
   omfgThis->handleNewWindow(new dspQOHByLocation());
@@ -918,7 +932,12 @@ void menuInventory::sDspTimePhasedUsageStatisticsByItem()
 
 void menuInventory::sPrintItemLabelsByClassCode()
 {
-  printItemLabelsByClassCode(parent, "", TRUE).exec();
+  printItemLabelsByClassCode(parent, "", true).exec();
+}
+
+void menuInventory::sPrintToForms()
+{
+  printToForm(parent, "", true).exec();
 }
 
 
@@ -961,17 +980,17 @@ void menuInventory::sDspUnbalancedQOHByClassCode()
 
 void menuInventory::sUpdateABCClass()
 {
-  updateABCClass(parent, "", TRUE).exec();
+  updateABCClass(parent, "", true).exec();
 }
 
 void menuInventory::sUpdateCycleCountFreq()
 {
-  updateCycleCountFrequency(parent, "", TRUE).exec();
+  updateCycleCountFrequency(parent, "", true).exec();
 }
 
 void menuInventory::sUpdateItemSiteLeadTimes()
 {
-  updateItemSiteLeadTimes(parent, "", TRUE).exec();
+  updateItemSiteLeadTimes(parent, "", true).exec();
 }
 
 void menuInventory::sUpdateReorderLevelByItem()
@@ -1006,27 +1025,27 @@ void menuInventory::sUpdateReorderLevelsByClassCode()
 
 void menuInventory::sUpdateOUTLevelByItem()
 {
-  updateOUTLevelByItem(parent, "", TRUE).exec();
+  updateOUTLevelByItem(parent, "", true).exec();
 }
 
 void menuInventory::sUpdateOUTLevelsByPlannerCode()
 {
-  updateOUTLevels(parent, "", TRUE).exec();
+  updateOUTLevels(parent, "", true).exec();
 }
 
 void menuInventory::sUpdateOUTLevelsByClassCode()
 {
-  updateOUTLevelsByClassCode(parent, "", TRUE).exec();
+  updateOUTLevelsByClassCode(parent, "", true).exec();
 }
 
 void menuInventory::sSummarizeInvTransByClassCode()
 {
-  summarizeInvTransByClassCode(parent, "", TRUE).exec();
+  summarizeInvTransByClassCode(parent, "", true).exec();
 }
 
 void menuInventory::sCreateItemSitesByClassCode()
 {
-  createItemSitesByClassCode(parent, "", TRUE).exec();
+  createItemSitesByClassCode(parent, "", true).exec();
 }
 
 //  inputManager SIGNAL handlers
@@ -1047,7 +1066,7 @@ void menuInventory::sCatchCountTag(int pCnttagid)
   params.append("cnttag_id", pCnttagid);
   params.append("mode", "edit");
 
-  countTag newdlg(parent, "", TRUE);
+  countTag newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
