@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -156,7 +156,7 @@ menuManufacture::menuManufacture(GUIClient *Pparent) :
     { "menu",                           tr("&Reports"), (char*)reportsMenu,     mainMenu,       "true", 0, 0,   true, NULL },
     
     //  Production | Reports | Schedule
-    { "wo.dspWoSchedule",  tr("Work Order &Schedule"), SLOT(sDspWoSchedule()),reportsMenu, "MaintainWorkOrders ViewWorkOrders", QPixmap(":/images/dspWoScheduleByPlannerCode.png"), toolBar, true, tr("Work Order Schedule by Planner Code") },
+    { "wo.dspWoSchedule",  tr("Work Order &Schedule"), SLOT(sDspWoSchedule()),reportsMenu, "MaintainWorkOrders ViewWorkOrders", new QPixmap(":/images/dspWoScheduleByPlannerCode.png"), toolBar, true, tr("Work Order Schedule by Planner Code") },
 
     //  Production | Reports | Material Requirements
     { "menu",                                       tr("&Material Requirements"),(char*)reportsMatlReqMenu,              reportsMenu,        "true",                              0, 0, true, NULL },
@@ -204,11 +204,7 @@ void menuManufacture::addActionsToMenu(actionProperties acts[], unsigned int num
   QAction * m = 0;
   for (unsigned int i = 0; i < numElems; i++)
   {
-    if (! acts[i].visible)
-    {
-      continue;
-    }
-    else if (acts[i].actionName == QString("menu"))
+    if (acts[i].actionName == QString("menu"))
     {
       m = acts[i].menu->addMenu((QMenu*)(acts[i].slot));
       if(m)
@@ -216,11 +212,11 @@ void menuManufacture::addActionsToMenu(actionProperties acts[], unsigned int num
     }
     else if (acts[i].actionName == QString("separator"))
     {
-      acts[i].menu->addSeparator();
+      m = acts[i].menu->addSeparator();
     }
     else if ((acts[i].toolBar != NULL) && (!acts[i].toolTip.isEmpty()))
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -233,7 +229,7 @@ void menuManufacture::addActionsToMenu(actionProperties acts[], unsigned int num
     }
     else if (acts[i].toolBar != NULL)
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -246,7 +242,7 @@ void menuManufacture::addActionsToMenu(actionProperties acts[], unsigned int num
     }
     else
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -254,6 +250,7 @@ void menuManufacture::addActionsToMenu(actionProperties acts[], unsigned int num
                   acts[i].menu,
                   acts[i].priv ) ;
     }
+    if (m) m->setVisible(acts[i].visible);
   }
 }
 
@@ -269,52 +266,52 @@ void menuManufacture::sNewWorkOrder()
 
 void menuManufacture::sExplodeWorkOrder()
 {
-  explodeWo(parent, "", TRUE).exec();
+  explodeWo(parent, "", true).exec();
 }
 
 void menuManufacture::sImplodeWorkOrder()
 {
-  implodeWo(parent, "", TRUE).exec();
+  implodeWo(parent, "", true).exec();
 }
 
 void menuManufacture::sPrintTraveler()
 {
-  printWoTraveler(parent, "", TRUE).exec();
+  printWoTraveler(parent, "", true).exec();
 }
 
 void menuManufacture::sPrintPickList()
 {
-  printWoPickList(parent, "", TRUE).exec();
+  printWoPickList(parent, "", true).exec();
 }
 
 void menuManufacture::sCloseWorkOrder()
 {
-  closeWo(parent, "", TRUE).exec();
+  closeWo(parent, "", true).exec();
 }
 
 void menuManufacture::sReleaseWorkOrdersByPlannerCode()
 {
-  releaseWorkOrdersByPlannerCode(parent, "", TRUE).exec();
+  releaseWorkOrdersByPlannerCode(parent, "", true).exec();
 }
 
 void menuManufacture::sReprioritizeWorkOrder()
 {
-  reprioritizeWo(parent, "", TRUE).exec();
+  reprioritizeWo(parent, "", true).exec();
 }
 
 void menuManufacture::sRescheduleWorkOrder()
 {
-  rescheduleWo(parent, "", TRUE).exec();
+  rescheduleWo(parent, "", true).exec();
 }
 
 void menuManufacture::sChangeWorkOrderQty()
 {
-  changeWoQty(parent, "", TRUE).exec();
+  changeWoQty(parent, "", true).exec();
 }
 
 void menuManufacture::sPurgeClosedWorkOrders()
 {
-  purgeClosedWorkOrders(parent, "", TRUE).exec();
+  purgeClosedWorkOrders(parent, "", true).exec();
 }
 
 void menuManufacture::sCreateWoMaterialRequirement()
@@ -322,7 +319,7 @@ void menuManufacture::sCreateWoMaterialRequirement()
   ParameterList params;
   params.append("mode", "new");
 
-  woMaterialItem newdlg(parent, "", TRUE);
+  woMaterialItem newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -334,22 +331,22 @@ void menuManufacture::sMaintainWoMaterials()
 
 void menuManufacture::sIssueWoMaterialBatch()
 {
-  issueWoMaterialBatch(parent, "", TRUE).exec();
+  issueWoMaterialBatch(parent, "", true).exec();
 }
 
 void menuManufacture::sIssueWoMaterialItem()
 {
-  issueWoMaterialItem(parent, "", TRUE).exec();
+  issueWoMaterialItem(parent, "", true).exec();
 }
 
 void menuManufacture::sReturnWoMaterialBatch()
 {
-  returnWoMaterialBatch(parent, "", TRUE).exec();
+  returnWoMaterialBatch(parent, "", true).exec();
 }
 
 void menuManufacture::sReturnWoMaterialItem()
 {
-  returnWoMaterialItem(parent, "", TRUE).exec();
+  returnWoMaterialItem(parent, "", true).exec();
 }
 
 void menuManufacture::sScrapWoMaterialFromWo()
@@ -358,24 +355,24 @@ void menuManufacture::sScrapWoMaterialFromWo()
   if (_privileges->check("PostProduction"))
     params.append("canPostProd", true);
 
-  scrapWoMaterialFromWIP newdlg(parent, "", TRUE);
+  scrapWoMaterialFromWIP newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
 
 void menuManufacture::sPostProduction()
 {
-  postProduction(parent, "", TRUE).exec();
+  postProduction(parent, "", true).exec();
 }
 
 void menuManufacture::sCorrectProductionPosting()
 {
-  correctProductionPosting(parent, "", TRUE).exec();
+  correctProductionPosting(parent, "", true).exec();
 }
 
 void menuManufacture::sPostMiscProduction()
 {
-  postMiscProduction(parent, "", TRUE).exec();
+  postMiscProduction(parent, "", true).exec();
 }
 
 void menuManufacture::sDspWoHistoryByItem()
@@ -450,7 +447,7 @@ void menuManufacture::sDspMaterialUsageVarianceByWarehouse()
 
 void menuManufacture::sPrintWorkOrderForm()
 {
-  printWoForm(parent, "", TRUE).exec();
+  printWoForm(parent, "", true).exec();
 }
 
 void menuManufacture::sSetup()
