@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -117,10 +117,6 @@ menuProducts::menuProducts(GUIClient *Pparent) :
 
   actionProperties acts[] = {
 
-#ifdef Q_WS_MACX
-  { "pd.setup",              tr("&Setup..."),                SLOT(sPreferences()),              mainMenu, "MaintainPreferencesSelf MaintainPreferencesOthers",  NULL,   NULL,   true, NULL },
-#endif
-
   // Product | Reports
   { "menu",	tr("&Reports"), (char*)reportsMenu,	mainMenu, "true", NULL, NULL, true , NULL },
   
@@ -150,7 +146,7 @@ menuProducts::menuProducts(GUIClient *Pparent) :
   // Product | Items
   { "menu",	tr("&Item"), (char*)itemsMenu,	mainMenu, "true", NULL, NULL, true , NULL },
   { "pd.enterNewItem", tr("&New..."), SLOT(sNewItem()), itemsMenu, "MaintainItemMasters", NULL, NULL, true , NULL },
-  { "pd.listItems", tr("&List..."), SLOT(sItems()), itemsMenu, "MaintainItemMasters ViewItemMasters", QPixmap(":/images/items.png"), toolBar, true , tr("List Items") },
+  { "pd.listItems", tr("&List..."), SLOT(sItems()), itemsMenu, "MaintainItemMasters ViewItemMasters", new QPixmap(":/images/items.png"), toolBar, true , tr("List Items") },
   { "pd.copyItem", tr("&Copy..."), SLOT(sCopyItem()), itemsMenu, "MaintainItemMasters" , NULL, NULL, true, NULL },
   { "separator", NULL, NULL, itemsMenu,	"true", NULL, NULL, true , NULL },
   { "pd.itemAvailabilityWorkbench", tr("&Workbench..."), SLOT(sDspItemAvailabilityWorkbench()), itemsMenu, "ViewItemAvailabilityWorkbench", NULL, NULL, true , NULL },
@@ -161,7 +157,7 @@ menuProducts::menuProducts(GUIClient *Pparent) :
   // Product | Bill of Materials
   { "menu",	tr("Bill Of Ma&terials"), (char*)bomMenu,	mainMenu, "true", NULL, NULL, true , NULL },
   { "pd.enterNewBOM", tr("&New..."), SLOT(sNewBOM()), bomMenu, "MaintainBOMs", NULL, NULL, true , NULL },
-  { "pd.listBOMs", tr("&List..."), SLOT(sBOMs()), bomMenu, "MaintainBOMs ViewBOMs", QPixmap(":/images/boms.png"), toolBar, true , tr("List Bill of Materials") },
+  { "pd.listBOMs", tr("&List..."), SLOT(sBOMs()), bomMenu, "MaintainBOMs ViewBOMs", new QPixmap(":/images/boms.png"), toolBar, true , tr("List Bill of Materials") },
   { "pd.copyBOM", tr("&Copy..."), SLOT(sCopyBOM()), bomMenu, "MaintainBOMs", NULL, NULL, true , NULL },
   { "separator", NULL, NULL, bomMenu,	"true", NULL, NULL, true , NULL },
   { "pd.massReplaceComponentItem", tr("Mass &Replace..."), SLOT(sMassReplaceComponent()), bomMenu, "MaintainBOMs", NULL, NULL, true , NULL },
@@ -240,11 +236,7 @@ void menuProducts::addActionsToMenu(actionProperties acts[], unsigned int numEle
   QAction * m = 0;
   for (unsigned int i = 0; i < numElems; i++)
   {
-    if (! acts[i].visible)
-    {
-      continue;
-    }
-    else if (acts[i].actionName == QString("menu"))
+    if (acts[i].actionName == QString("menu"))
     {
       m = acts[i].menu->addMenu((QMenu*)(acts[i].slot));
       if(m)
@@ -256,7 +248,7 @@ void menuProducts::addActionsToMenu(actionProperties acts[], unsigned int numEle
     }
     else if ((acts[i].toolBar != NULL) && (!acts[i].toolTip.isEmpty()))
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -269,7 +261,7 @@ void menuProducts::addActionsToMenu(actionProperties acts[], unsigned int numEle
     }
     else if (acts[i].toolBar != NULL)
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -282,7 +274,7 @@ void menuProducts::addActionsToMenu(actionProperties acts[], unsigned int numEle
     }
     else
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -290,6 +282,7 @@ void menuProducts::addActionsToMenu(actionProperties acts[], unsigned int numEle
                   acts[i].menu,
                   acts[i].priv ) ;
     }
+    if (m) m->setVisible(acts[i].visible);
   }
 }
 
@@ -305,7 +298,7 @@ void menuProducts::sItems()
 
 void menuProducts::sCopyItem()
 {
-  copyItem(parent, "", TRUE).exec();
+  copyItem(parent, "", true).exec();
 }
 
 void menuProducts::sItemGroups()
@@ -335,7 +328,7 @@ void menuProducts::sBOMs()
 
 void menuProducts::sCopyBOM()
 {
-  copyBOM(parent, "", TRUE).exec();
+  copyBOM(parent, "", true).exec();
 }
 
 void menuProducts::sMassReplaceComponent()
@@ -359,7 +352,7 @@ void menuProducts::sUpdateActualCostsByItem()
   ParameterList params;
   params.append("costtype", "actual");
 
-  updateActualCostsByItem *newdlg = new updateActualCostsByItem(parent, "", TRUE);
+  updateActualCostsByItem *newdlg = new updateActualCostsByItem(parent, "", true);
   newdlg->set(params);
   newdlg->exec();
 }
@@ -369,19 +362,19 @@ void menuProducts::sUpdateActualCostsByClassCode()
   ParameterList params;
   params.append("costtype", "actual");
 
-  updateActualCostsByClassCode *newdlg = new updateActualCostsByClassCode(parent, "", TRUE);
+  updateActualCostsByClassCode *newdlg = new updateActualCostsByClassCode(parent, "", true);
   newdlg->set(params);
   newdlg->exec();
 }
 
 void menuProducts::sPostActualCostsByItem()
 {
-  postCostsByItem(parent, "", TRUE).exec();
+  postCostsByItem(parent, "", true).exec();
 }
 
 void menuProducts::sPostActualCostsByClassCode()
 {
-  postCostsByClassCode(parent, "", TRUE).exec();
+  postCostsByClassCode(parent, "", true).exec();
 }
 
 void menuProducts::sUpdateStandardCostsByItem()
@@ -389,7 +382,7 @@ void menuProducts::sUpdateStandardCostsByItem()
   ParameterList params;
   params.append("costtype", "standard");
 
-  updateActualCostsByItem *newdlg = new updateActualCostsByItem(parent, "", TRUE);
+  updateActualCostsByItem *newdlg = new updateActualCostsByItem(parent, "", true);
   newdlg->set(params);
   newdlg->exec();
 }
@@ -399,7 +392,7 @@ void menuProducts::sUpdateStandardCostsByClassCode()
   ParameterList params;
   params.append("costtype", "standard");
 
-  updateActualCostsByClassCode *newdlg = new updateActualCostsByClassCode(parent, "", TRUE);
+  updateActualCostsByClassCode *newdlg = new updateActualCostsByClassCode(parent, "", true);
   newdlg->set(params);
   newdlg->exec();
 }
@@ -441,7 +434,7 @@ void menuProducts::sUserCostingElements()
 
 void menuProducts::sLotSerial()
 {
-  lotSerial newdlg(parent, "", TRUE);
+  lotSerial newdlg(parent, "", true);
   newdlg.exec();
 }
 
@@ -509,12 +502,12 @@ void menuProducts::sDspInvalidBillsOfMaterials()
 
 void menuProducts::sReassignClassCodeByClassCode()
 {
-  reassignClassCodeByClassCode(parent, "", TRUE).exec();
+  reassignClassCodeByClassCode(parent, "", true).exec();
 }
 
 void menuProducts::sReassignProductCategoryByProductCategory()
 {
-  reassignProductCategoryByProductCategory(parent, "", TRUE).exec();
+  reassignProductCategoryByProductCategory(parent, "", true).exec();
 }
 
 void menuProducts::sSetup()
