@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,6 +19,7 @@
 
 #include "guiclient.h"
 
+#include "salesOrderSimple.h"
 #include "salesOrder.h"
 #include "openSalesOrders.h"
 #include "quotes.h"
@@ -34,6 +35,7 @@
 #include "selectShippedOrders.h"
 #include "selectOrderForBilling.h"
 #include "dspBillingSelections.h"
+#include "invoice.h"
 #include "createInvoices.h"
 #include "unpostedInvoices.h"
 #include "reprintInvoices.h"
@@ -63,10 +65,9 @@
 #include "dspFreightPricesByCustomer.h"
 #include "dspFreightPricesByCustomerType.h"
 
-#include "dspSalesOrdersByCustomer.h"
+#include "dspSalesOrders.h"
 #include "dspSalesOrdersByItem.h"
-#include "dspSalesOrdersByCustomerPO.h"
-#include "dspSalesOrdersByParameterList.h"
+#include "dspReturnAuthorizationsByItem.h"
 #include "dspQuotesByCustomer.h"
 #include "dspQuotesByItem.h"
 #include "dspInventoryAvailability.h"
@@ -130,6 +131,7 @@ menuSales::menuSales(GUIClient *pParent) :
   lookupMenu = new QMenu(parent);
   lookupQuoteMenu = new QMenu(parent);
   lookupSoMenu = new QMenu(parent);
+  lookupRaMenu = new QMenu(parent);
   formsMenu = new QMenu(parent);
   reportsMenu = new QMenu(parent);
   analysisMenu = new QMenu(parent);
@@ -151,6 +153,7 @@ menuSales::menuSales(GUIClient *pParent) :
   lookupMenu->setObjectName("menu.sales.lookup");
   lookupQuoteMenu->setObjectName("menu.sales.lookupquote");
   lookupSoMenu->setObjectName("menu.sales.lookupso");
+  lookupRaMenu->setObjectName("menu.sales.lookupra");
   formsMenu->setObjectName("menu.sales.forms");
   reportsMenu->setObjectName("menu.sales.reports");
   analysisMenu->setObjectName("menu.sales.analysis");
@@ -169,26 +172,29 @@ menuSales::menuSales(GUIClient *pParent) :
     
     // Sales | Sales Order
     { "menu",	tr("&Sales Order"),	(char*)ordersMenu,	mainMenu,	"true",	NULL, NULL, true, NULL },
+    { "so.newSalesOrderSimple",  tr("&New Simple..."),		SLOT(sNewSalesOrderSimple()),   ordersMenu, "MaintainSimpleSalesOrders", NULL, NULL, _metrics->boolean("SSOSEnabled"), NULL },
     { "so.newSalesOrder", 	     tr("&New..."),		SLOT(sNewSalesOrder()),   ordersMenu, "MaintainSalesOrders", NULL, NULL,	 true, NULL },
-    { "so.listOpenSalesOrders",      tr("&List Open..."),	SLOT(sOpenSalesOrders()), ordersMenu, "MaintainSalesOrders ViewSalesOrders",	QPixmap(":/images/listOpenSalesOrders.png"), toolBar,  true, tr("List Open Sales Orders") },
+    { "so.listOpenSalesOrders",  tr("&List Open..."),	SLOT(sOpenSalesOrders()), ordersMenu, "MaintainSalesOrders ViewSalesOrders", new QPixmap(":/images/listOpenSalesOrders.png"), toolBar,  true, tr("List Open Sales Orders") },
+    { "so.listSalesOrders",      tr("&Search Orders..."),	SLOT(sSalesOrders()), ordersMenu, "MaintainSalesOrders ViewSalesOrders", NULL, NULL, true, NULL },
 
     // Sales | Billing
     { "menu",	tr("&Billing"),     (char*)billingMenu,		mainMenu,	"true",	NULL, NULL, true, NULL },
     
     // Sales | Billing | Invoice
     { "menu",	tr("&Invoice"),   (char*)billingInvoicesMenu,	billingMenu,	"true",	NULL, NULL, true, NULL },
-    { "so.uninvoicedShipments",		     tr("&Uninvoiced Shipments..."),			SLOT(sUninvoicedShipments()), 		billingInvoicesMenu, "SelectBilling",	 QPixmap(":/images/uninvoicedShipments"), toolBar, true, tr("Uninvoiced Shipments") },
-    { "so.selectAllShippedOrdersForBilling", tr("Select &All Shipped Orders for Billing..."),	SLOT(sSelectShippedOrdersForBilling()), billingInvoicesMenu, "SelectBilling",	NULL, NULL, true, NULL },
-    { "so.selectOrderForBilling",	     tr("Select &Order for Billing..."),			SLOT(sSelectOrderForBilling()),		billingInvoicesMenu, "SelectBilling",	NULL, NULL, true, NULL },
+    { "so.uninvoicedShipments",		     tr("&Uninvoiced Shipments..."),			SLOT(sUninvoicedShipments()), 		billingInvoicesMenu, "SelectBilling",	 new QPixmap(":/images/uninvoicedShipments"), toolBar, true, tr("Uninvoiced Shipments") },
+    { "so.selectAllShippedOrdersForBilling", tr("Approve &All Shipped Orders for Billing..."),	SLOT(sSelectShippedOrdersForBilling()), billingInvoicesMenu, "SelectBilling",	NULL, NULL, true, NULL },
+    { "so.selectOrderForBilling",	     tr("Approve &Order for Billing..."),			SLOT(sSelectOrderForBilling()),		billingInvoicesMenu, "SelectBilling",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	billingInvoicesMenu,	"true",		NULL, NULL, true, NULL },
-    { "so.dspBillingSelections",	     tr("Billing &Selections..."),	SLOT(sDspBillingSelections()), billingInvoicesMenu, "SelectBilling", QPixmap(":/images/billingSelections"), toolBar, true, tr("Billing Selections") },
+    { "so.dspBillingSelections",	     tr("Billing &Approvals..."),	SLOT(sDspBillingSelections()), billingInvoicesMenu, "SelectBilling", new QPixmap(":/images/billingSelections"), toolBar, true, tr("Billing Approvals") },
     { "so.createInvoices",	     tr("&Create Invoices..."),	SLOT(sCreateInvoices()), billingInvoicesMenu, "SelectBilling",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	billingInvoicesMenu,	"true",		NULL, NULL, true, NULL },
+    { "so.createInvoice", tr("&New Invoice..."), SLOT(sCreateInvoice()), billingInvoicesMenu, "MaintainMiscInvoices", NULL, NULL, true , NULL },
     { "so.listUnpostedInvoices",	     tr("&List Unposted Invoices..."),	SLOT(sUnpostedInvoices()), billingInvoicesMenu, "SelectBilling",	NULL, NULL,  true, NULL },
     { "so.postInvoices",		     tr("Post &Invoices..."),		SLOT(sPostInvoices()), billingInvoicesMenu, "PostMiscInvoices",	NULL, NULL, true, NULL },
 
-    // Sales | Billing | Credit Memo
-    { "menu",	tr("&Credit Memo"), (char*)billingCreditMemosMenu,	billingMenu,	"true",	NULL, NULL, true, NULL },
+    // Sales | Billing | Credit Memos
+    { "menu",	tr("&Sales Credit"), (char*)billingCreditMemosMenu,	billingMenu,	"true",	NULL, NULL, true, NULL },
     { "so.newCreditMemo",		     tr("&New..."),		SLOT(sNewCreditMemo()), billingCreditMemosMenu, "MaintainCreditMemos",	NULL, NULL, true, NULL },
     { "so.listUnpostedCreditMemos",	     tr("&List Unposted..."),	SLOT(sUnpostedCreditMemos()), billingCreditMemosMenu, "MaintainCreditMemos ViewCreditMemos",	NULL, NULL, true, NULL },
     { "so.creditMemoEditList",		     tr("&Edit List..."),	SLOT(sCreditMemoEditList()), billingCreditMemosMenu, "MaintainCreditMemos ViewCreditMemos",	NULL, NULL, true, NULL },
@@ -202,11 +208,11 @@ menuSales::menuSales(GUIClient *pParent) :
     { "so.printInvoices",		     tr("&Print Invoices..."),		SLOT(sPrintInvoices()), billingFormsMenu, "PrintInvoices",	NULL, NULL, true, NULL },
     { "so.reprintInvoices",		     tr("&Re-Print Invoices..."),	SLOT(sReprintInvoices()), billingFormsMenu, "PrintInvoices",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	billingFormsMenu,	"true",		NULL, NULL, true , NULL },
-    { "so.printCreditMemos",		     tr("Print &Credit Memos..."),	SLOT(sPrintCreditMemos()), billingFormsMenu, "PrintCreditMemos",	NULL, NULL, true, NULL },
-    { "so.reprintCreditMemos",		     tr("Re-Print Credit &Memos..."),	SLOT(sReprintCreditMemos()), billingFormsMenu, "PrintCreditMemos",	NULL, NULL, true, NULL },
+    { "so.printCreditMemos",		     tr("Print &Sales Credits..."),	SLOT(sPrintCreditMemos()), billingFormsMenu, "PrintCreditMemos",	NULL, NULL, true, NULL },
+    { "so.reprintCreditMemos",		     tr("Re-Print &Sales Credits..."),	SLOT(sReprintCreditMemos()), billingFormsMenu, "PrintCreditMemos",	NULL, NULL, true, NULL },
 
-    // Sales | Returns
-    { "menu",	tr("&Return"),	(char*)returnsMenu,	mainMenu, "true",	NULL, NULL,  _metrics->boolean("EnableReturnAuth"), NULL },
+    // Sales | Return Authorizations
+    { "menu",	tr("&Return Auth."),	(char*)returnsMenu,	mainMenu, "true",	NULL, NULL,  _metrics->boolean("EnableReturnAuth"), NULL },
     { "so.newReturn", tr("&New..."),	SLOT(sNewReturn()), returnsMenu, "MaintainReturns",	NULL, NULL, true, NULL },
     { "so.openReturns", tr("&List Open..."),	SLOT(sOpenReturns()), returnsMenu, "MaintainReturns ViewReturns",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	returnsMenu,	"true",		NULL, NULL, true , NULL },
@@ -224,12 +230,14 @@ menuSales::menuSales(GUIClient *pParent) :
     
     // Sales | Lookup | Sales Order Lookup
     { "menu",	tr("&Sales Order"),           (char*)lookupSoMenu,	lookupMenu,	"true",	NULL, NULL, true, NULL },
-    { "so.dspSalesOrderLookupByCustomerType", tr("by Customer &Type..."),	SLOT(sDspOrderLookupByCustomerType()), lookupSoMenu, "ViewSalesOrders",	NULL, NULL, true, NULL },
-    { "so.dspSalesOrderLookupByCustomer", tr("by &Customer..."),	SLOT(sDspOrderLookupByCustomer()), lookupSoMenu, "ViewSalesOrders",	NULL, NULL, true, NULL },
-    { "so.dspSalesOrderLookupByCustomerPO", tr("by Customer &PO..."),	SLOT(sDspOrderLookupByCustomerPO()), lookupSoMenu, "ViewSalesOrders",	NULL, NULL, true, NULL },
+    { "so.listSalesOrders",      tr("&Search Orders..."),	SLOT(sSalesOrders()), lookupSoMenu, "MaintainSalesOrders ViewSalesOrders", NULL, NULL, true, NULL },
     { "so.dspSalesOrderLookupByItem", tr("by &Item..."),	SLOT(sDspOrderLookupByItem()), lookupSoMenu, "ViewSalesOrders",	NULL, NULL, true, NULL },
     
-    { "separator",	NULL,	NULL,	lookupMenu,	"true",		NULL, NULL, true, NULL }, 
+    // Sales | Lookup | Return Auth. Lookup
+    { "menu",	tr("&Return Auth."),           (char*)lookupRaMenu,	lookupMenu,	"true",	NULL, NULL, _metrics->boolean("EnableReturnAuth"), NULL },
+    { "so.dspReturnAuthLookupByItem", tr("by &Item..."),	SLOT(sDspReturnAuthLookupByItem()), lookupRaMenu, "ViewReturns",	NULL, NULL, true, NULL },
+    
+    { "separator",	NULL,	NULL,	lookupMenu,	"true",		NULL, NULL, true, NULL },
     { "so.dspSalesOrderStatus", tr("Sales Order S&tatus..."),	SLOT(sDspSalesOrderStatus()), lookupMenu, "ViewSalesOrders",	NULL, NULL, true, NULL },   
   
     // Sales | Forms
@@ -243,7 +251,7 @@ menuSales::menuSales(GUIClient *pParent) :
 
     // Sales | Reports
     { "menu",	tr("&Reports"),           (char*)reportsMenu,	mainMenu,	"true",	NULL, NULL, true, NULL },
-    { "so.dspSummarizedBacklogByWarehouse", tr("Su&mmarized Backlog..."),	SLOT(sDspSummarizedBacklogByWarehouse()), reportsMenu, "ViewSalesOrders",	QPixmap(":/images/dspSummarizedBacklogByWhse.png"), toolBar,  true, tr("Summarized Backlog") },
+    { "so.dspSummarizedBacklogByWarehouse", tr("Su&mmarized Backlog..."),	SLOT(sDspSummarizedBacklogByWarehouse()), reportsMenu, "ViewSalesOrders",	new QPixmap(":/images/dspSummarizedBacklogByWhse.png"), toolBar,  true, tr("Summarized Backlog") },
 
     // Sales | Reports | Backlog
     { "so.dspBacklog", tr("&Backlog..."),	SLOT(sDspBacklog()), reportsMenu, "ViewSalesOrders",	NULL, NULL, true, NULL },
@@ -285,7 +293,7 @@ menuSales::menuSales(GUIClient *pParent) :
     { "so.enterNewCustomer", tr("&New..."),	SLOT(sNewCustomer()), customerMenu, "MaintainCustomerMasters",	NULL, NULL, true, NULL },
     { "so.customers", tr("&List..."),	SLOT(sCustomers()), customerMenu, "MaintainCustomerMasters ViewCustomerMasters",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	customerMenu,	"true",		NULL, NULL, true, NULL },
-    { "so.customerWorkbench", tr("&Workbench..."),	SLOT(sCustomerWorkbench()), customerMenu, "MaintainCustomerMasters ViewCustomerMasters",	QPixmap(":/images/customerInformationWorkbench.png"), toolBar,  true, tr("Customer Workbench") }, 
+    { "so.customerWorkbench", tr("&Workbench..."),	SLOT(sCustomerWorkbench()), customerMenu, "MaintainCustomerMasters ViewCustomerMasters",	new QPixmap(":/images/customerInformationWorkbench.png"), toolBar,  true, tr("Customer Workbench") },
     { "separator",	NULL,	NULL,	customerMenu,	"true",		NULL, NULL, true, NULL },
     { "so.customerGroups", tr("&Groups..."),	SLOT(sCustomerGroups()), customerMenu, "MaintainCustomerGroups ViewCustomerGroups",	NULL, NULL, true, NULL },
    
@@ -294,9 +302,14 @@ menuSales::menuSales(GUIClient *pParent) :
     { "so.itemListPrice", tr("Item &List Price..."),	SLOT(sItemListPrice()), pricingMenu, "MaintainListPrices ViewListPrices",	NULL, NULL, true, NULL },
     { "so.updateListPricesByProductCategory", tr("&Update List Prices..."),	SLOT(sUpdateListPricesByProductCategory()), pricingMenu, "MaintainListPrices",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	pricingMenu,	"true",		NULL, NULL, true, NULL },
+    { "so.listPricingSchedules", tr("List Pricing &Schedules..."),	SLOT(sListPricingSchedules()), pricingMenu, "MaintainListPrices ViewListPrices",	NULL, NULL, _metrics->boolean("AllowListPriceSchedules"), NULL },
+    { "so.updateListPricingSchedules", tr("Update List Pricing Schedules..."),	SLOT(sUpdateListPrices()), pricingMenu, "UpdatePricingSchedules",	NULL, NULL, _metrics->boolean("AllowListPriceSchedules"), NULL },
+    { "so.listPricingScheduleAssignments", tr("List Pricing Schedule Assi&gnments..."),	SLOT(sListPricingScheduleAssignments()), pricingMenu, "AssignPricingSchedules",	NULL, NULL, _metrics->boolean("AllowListPriceSchedules"), NULL },
+    { "separator",	NULL,	NULL,	pricingMenu,	"true",		NULL, NULL, _metrics->boolean("AllowListPriceSchedules"), NULL },
     { "so.pricingSchedules", tr("Pricing &Schedules..."),	SLOT(sPricingSchedules()), pricingMenu, "MaintainListPrices ViewListPrices",	NULL, NULL, true, NULL },
     { "so.updatePricingSchedules", tr("Update Pricing Schedules..."),	SLOT(sUpdatePrices()), pricingMenu, "UpdatePricingSchedules",	NULL, NULL, true, NULL },
     { "so.pricingScheduleAssignments", tr("Pricing Schedule Assi&gnments..."),	SLOT(sPricingScheduleAssignments()), pricingMenu, "AssignPricingSchedules",	NULL, NULL, true, NULL },
+    { "separator",	NULL,	NULL,	pricingMenu,	"true",		NULL, NULL, true, NULL },
     { "so.sales", tr("S&ales..."),	SLOT(sSales()), pricingMenu, "CreateSales",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	pricingMenu,	"true",		NULL, NULL, true, NULL },
 
@@ -316,7 +329,7 @@ menuSales::menuSales(GUIClient *pParent) :
     { "so.updateCreditStatusByCustomer", tr("&Update Credit Status by Customer..."),	SLOT(sUpdateCreditStatusByCustomer()), utilitiesMenu, "MaintainCustomerMasters UpdateCustomerCreditStatus",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	utilitiesMenu,	"true",		NULL, NULL, true, NULL },
     { "so.purgeInvoices",		     tr("Purge &Invoices..."),		SLOT(sPurgeInvoices()), utilitiesMenu, "PurgeInvoices",	NULL, NULL, true, NULL },
-    { "so.purgeCreditMemos",		     tr("Purge Credit &Memos..."),	SLOT(sPurgeCreditMemos()), utilitiesMenu, "PurgeCreditMemos",	NULL, NULL, true, NULL },
+    { "so.purgeCreditMemos",		     tr("Purge &Sales Credits..."),	SLOT(sPurgeCreditMemos()), utilitiesMenu, "PurgeCreditMemos",	NULL, NULL, true, NULL },
     { "separator",	NULL,	NULL,	utilitiesMenu,	"true",		NULL, NULL, _metrics->boolean("EnableSOReservations"), NULL },
     { "so.allocateReservations", tr("Allocate Reser&vations..."), SLOT(sAllocateReservations()), utilitiesMenu, "MaintainReservations", NULL, NULL, _metrics->boolean("EnableSOReservations") , NULL },
     { "separator",	NULL,	NULL,	utilitiesMenu,	"true",		NULL, NULL, true, NULL },
@@ -340,11 +353,7 @@ void menuSales::addActionsToMenu(actionProperties acts[], unsigned int numElems)
   QAction * m = 0;
   for (unsigned int i = 0; i < numElems; i++)
   {
-    if (! acts[i].visible)
-    {
-      continue;
-    }
-    else if (acts[i].actionName == QString("menu"))
+    if (acts[i].actionName == QString("menu"))
     {
       m = acts[i].menu->addMenu((QMenu*)(acts[i].slot));
       if(m)
@@ -352,11 +361,11 @@ void menuSales::addActionsToMenu(actionProperties acts[], unsigned int numElems)
     }
     else if (acts[i].actionName == QString("separator"))
     {
-      acts[i].menu->addSeparator();
+      m = acts[i].menu->addSeparator();
     }
     else if ((acts[i].toolBar != NULL) && (!acts[i].toolTip.isEmpty()))
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -369,7 +378,7 @@ void menuSales::addActionsToMenu(actionProperties acts[], unsigned int numElems)
     }
     else if (acts[i].toolBar != NULL)
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -382,7 +391,7 @@ void menuSales::addActionsToMenu(actionProperties acts[], unsigned int numElems)
     }
     else
     {
-      new Action( parent,
+      m = new Action( parent,
                   acts[i].actionName,
                   acts[i].actionTitle,
                   this,
@@ -390,15 +399,26 @@ void menuSales::addActionsToMenu(actionProperties acts[], unsigned int numElems)
                   acts[i].menu,
                   acts[i].priv ) ;
     }
+    if (m) m->setVisible(acts[i].visible);
   }
 }
 
 //  Orders
+void menuSales::sNewSalesOrderSimple()
+{
+  salesOrderSimple::newSalesOrder();
+}
+
 void menuSales::sNewSalesOrder()
 {
   salesOrder::newSalesOrder(-1);
 }
 
+void menuSales::sSalesOrders()
+{
+  omfgThis->handleNewWindow(new dspSalesOrders());
+}
+
 void menuSales::sOpenSalesOrders()
 {
   omfgThis->handleNewWindow(new openSalesOrders());
@@ -411,7 +431,7 @@ void menuSales::sPackingListBatch()
 
 void menuSales::sPrintPackingList()
 {
-  printPackingList(parent, "", TRUE).exec();
+  printPackingList(parent, "", true).exec();
 }
 
 void menuSales::sNewQuote()
@@ -437,7 +457,7 @@ void menuSales::sUninvoicedShipments()
 
 void menuSales::sSelectShippedOrdersForBilling()
 {
-  selectShippedOrders(parent, "", TRUE).exec();
+  selectShippedOrders(parent, "", true).exec();
 }
 
 void menuSales::sSelectOrderForBilling()
@@ -455,9 +475,14 @@ void menuSales::sDspBillingSelections()
   omfgThis->handleNewWindow(new dspBillingSelections());
 }
 
+void menuSales::sCreateInvoice()
+{
+  invoice::newInvoice(-1);
+}
+
 void menuSales::sCreateInvoices()
 {
-  createInvoices(parent, "", TRUE).exec();
+  createInvoices(parent, "", true).exec();
 }
 
 void menuSales::sUnpostedInvoices()
@@ -467,22 +492,22 @@ void menuSales::sUnpostedInvoices()
 
 void menuSales::sPrintInvoices()
 {
-  printInvoices(parent, "", TRUE).exec();
+  printInvoices(parent, "", true).exec();
 }
 
 void menuSales::sReprintInvoices()
 {
-  reprintInvoices(parent, "", TRUE).exec();
+  reprintInvoices(parent, "", true).exec();
 }
 
 void menuSales::sPostInvoices()
 {
-  postInvoices(parent, "", TRUE).exec();
+  postInvoices(parent, "", true).exec();
 }
 
 void menuSales::sPurgeInvoices()
 {
-  purgeInvoices(parent, "", TRUE).exec();
+  purgeInvoices(parent, "", true).exec();
 }
 
 
@@ -508,22 +533,22 @@ void menuSales::sCreditMemoEditList()
 
 void menuSales::sPrintCreditMemos()
 {
-  printCreditMemos(parent, "", TRUE).exec();
+  printCreditMemos(parent, "", true).exec();
 }
 
 void menuSales::sReprintCreditMemos()
 {
-  reprintCreditMemos(parent, "", TRUE).exec();
+  reprintCreditMemos(parent, "", true).exec();
 }
 
 void menuSales::sPostCreditMemos()
 {
-  postCreditMemos(parent, "", TRUE).exec();
+  postCreditMemos(parent, "", true).exec();
 }
 
 void menuSales::sPurgeCreditMemos()
 {
-  purgeCreditMemos(parent, "", TRUE).exec();
+  purgeCreditMemos(parent, "", true).exec();
 }
 
 void menuSales::sNewReturn()
@@ -553,24 +578,52 @@ void menuSales::sReturnsWorkbench()
 //  S/O | Item Pricing
 void menuSales::sItemListPrice()
 {
-  itemListPrice newdlg(parent, "", TRUE);
+  itemListPrice newdlg(parent, "", true);
   newdlg.exec();
 }
 
 void menuSales::sUpdateListPricesByProductCategory()
 {
-  updateListPricesByProductCategory newdlg(parent, "", TRUE);
+  updateListPricesByProductCategory newdlg(parent, "", true);
   newdlg.exec();
 }
 
 void menuSales::sPricingSchedules()
 {
-  omfgThis->handleNewWindow(new itemPricingSchedules());
+  ParameterList params;
+  
+  itemPricingSchedules *newdlg = new itemPricingSchedules();
+  newdlg->set(params);
+  omfgThis->handleNewWindow(newdlg);
+}
+
+void menuSales::sListPricingSchedules()
+{
+  ParameterList params;
+  params.append("listpricesched", true);
+  
+  itemPricingSchedules *newdlg = new itemPricingSchedules();
+  newdlg->set(params);
+  omfgThis->handleNewWindow(newdlg);
 }
 
 void menuSales::sPricingScheduleAssignments()
 {
-  omfgThis->handleNewWindow(new pricingScheduleAssignments());
+  ParameterList params;
+  
+  pricingScheduleAssignments *newdlg = new pricingScheduleAssignments();
+  newdlg->set(params);
+  omfgThis->handleNewWindow(newdlg);
+}
+
+void menuSales::sListPricingScheduleAssignments()
+{
+  ParameterList params;
+  params.append("listpricesched", true);
+  
+  pricingScheduleAssignments *newdlg = new pricingScheduleAssignments();
+  newdlg->set(params);
+  omfgThis->handleNewWindow(newdlg);
 }
 
 void menuSales::sSales()
@@ -580,7 +633,18 @@ void menuSales::sSales()
 
 void menuSales::sUpdatePrices()
 {
-  updatePrices(parent, "", TRUE).exec();
+  updatePrices newdlg(parent, "updatePrices", true);
+  newdlg.exec();
+}
+
+void menuSales::sUpdateListPrices()
+{
+  ParameterList params;
+  params.append("listpricesched", true);
+  
+  updatePrices newdlg(parent, "updatePrices", true);
+  newdlg.set(params);
+  newdlg.exec();
 }
 
 void menuSales::sDspPricesByItem()
@@ -634,29 +698,14 @@ void menuSales::sDspReservations()
   omfgThis->handleNewWindow(new dspReservations());
 }
 
-void menuSales::sDspOrderLookupByCustomer()
-{
-  omfgThis->handleNewWindow(new dspSalesOrdersByCustomer());
-}
-
-void menuSales::sDspOrderLookupByCustomerType()
-{
-  ParameterList params;
-  params.append("custtype");
-
-  dspSalesOrdersByParameterList *newdlg = new dspSalesOrdersByParameterList();
-  newdlg->set(params);
-  omfgThis->handleNewWindow(newdlg);
-}
-
 void menuSales::sDspOrderLookupByItem()
 {
   omfgThis->handleNewWindow(new dspSalesOrdersByItem());
 }
 
-void menuSales::sDspOrderLookupByCustomerPO()
+void menuSales::sDspReturnAuthLookupByItem()
 {
-  omfgThis->handleNewWindow(new dspSalesOrdersByCustomerPO());
+  omfgThis->handleNewWindow(new dspReturnAuthorizationsByItem());
 }
 
 void menuSales::sDspQuoteLookupByCustomer()
@@ -732,12 +781,12 @@ void menuSales::sDspTimePhasedSales()
 
 void menuSales::sPrintSalesOrderForm()
 {
-  printSoForm(parent, "", TRUE).exec();
+  printSoForm(parent, "", true).exec();
 }
 
 void menuSales::sPrintReturnAuthForm()
 {
-  printRaForm(parent, "", TRUE).exec();
+  printRaForm(parent, "", true).exec();
 }
 
 
@@ -772,7 +821,7 @@ void menuSales::sCustomers()
 
 void menuSales::sUpdateCreditStatusByCustomer()
 {
-  updateCreditStatusByCustomer(parent, "", TRUE).exec();
+  updateCreditStatusByCustomer(parent, "", true).exec();
 }
 
 void menuSales::sCustomerGroups()
@@ -797,7 +846,7 @@ void menuSales::sProspects()
 
 void menuSales::sReassignCustomerTypeByCustomerType()
 {
-  reassignCustomerTypeByCustomerType(parent, "", TRUE).exec();
+  reassignCustomerTypeByCustomerType(parent, "", true).exec();
 }
 
 void menuSales::sArchiveSalesHistory()
@@ -805,7 +854,7 @@ void menuSales::sArchiveSalesHistory()
   ParameterList params;
   params.append("archieve");
 
-  archRestoreSalesHistory newdlg(parent, "", TRUE);
+  archRestoreSalesHistory newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -815,14 +864,14 @@ void menuSales::sRestoreSalesHistory()
   ParameterList params;
   params.append("restore");
 
-  archRestoreSalesHistory newdlg(parent, "", TRUE);
+  archRestoreSalesHistory newdlg(parent, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
 
 void menuSales::sAllocateReservations()
 {
-  allocateReservations(parent, "", TRUE).exec();
+  allocateReservations(parent, "", true).exec();
 }
 
 
