@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,7 +22,7 @@ class miscCheck : public XWidget, public Ui::miscCheck
     Q_OBJECT
 
 public:
-    miscCheck(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    miscCheck(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~miscCheck();
 
     virtual void populate();
@@ -30,10 +30,12 @@ public:
 public slots:
     virtual enum SetResponse set( const ParameterList & pParams );
     virtual void sHandleButtons();
+    virtual void sHandleTaxZone();
     virtual void sPopulateBankInfo( int pBankaccntid );
     virtual void sSave();
-	virtual void sCustomerSelected();
-	virtual void sCreditMemoSelected();
+    virtual void sCustomerSelected();
+    virtual void sCreditMemoSelected();
+    virtual void sCalculateTax();
 
 protected slots:
     virtual void languageChange();
