@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -15,38 +15,46 @@
 #include <QMessageBox>
 #include <QSqlError>
 #include <QVariant>
+#include <metasql.h>
+#include <mqlutil.h>
 
 #include "guiErrorCheck.h"
 #include "errorReporter.h"
 #include "voucherMiscDistrib.h"
 
-miscVoucher::miscVoucher(QWidget* parent, const char* name, Qt::WFlags fl)
+miscVoucher::miscVoucher(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
 
-  connect(_amountDistributed, SIGNAL(valueChanged()), this, SLOT(sPopulateBalanceDue()));
-  connect(_amountToDistribute, SIGNAL(valueChanged()), this, SLOT(sPopulateBalanceDue()));
-  connect(_amountToDistribute, SIGNAL(effectiveChanged(const QDate&)), this, SLOT(sFillMiscList()));
-  connect(_amountToDistribute, SIGNAL(idChanged(int)), this, SLOT(sFillMiscList()));
-  connect(_amountToDistribute, SIGNAL(valueChanged()), this, SLOT(sPopulateBalanceDue()));
-  connect(_delete, SIGNAL(clicked()), this, SLOT(sDeleteMiscDistribution()));
-  connect(_edit, SIGNAL(clicked()), this, SLOT(sEditMiscDistribution()));
-  connect(_invoiceDate, SIGNAL(newDate(const QDate&)), this, SLOT(sPopulateDistDate()));
-  connect(_invoiceDate, SIGNAL(newDate(const QDate&)), this, SLOT(sPopulateDueDate()));
-  connect(_terms, SIGNAL(newID(int)), this, SLOT(sPopulateDueDate()));
-  connect(_new, SIGNAL(clicked()), this, SLOT(sNewMiscDistribution()));
-  connect(_save, SIGNAL(clicked()), this, SLOT(sSave()));
-  connect(_voucherNumber, SIGNAL(editingFinished()), this, SLOT(sHandleVoucherNumber()));
+  connect(_amountDistributed,    SIGNAL(valueChanged()),                 this, SLOT(sPopulateBalanceDue()));
+  connect(_amountToDistribute,   SIGNAL(valueChanged()),                 this, SLOT(sPopulateBalanceDue()));
+  connect(_amountToDistribute,   SIGNAL(effectiveChanged(const QDate&)), this, SLOT(sFillMiscList()));
+  connect(_amountToDistribute,   SIGNAL(idChanged(int)),                 this, SLOT(sFillMiscList()));
+  connect(_delete,               SIGNAL(clicked()),                      this, SLOT(sDeleteMiscDistribution()));
+  connect(_edit,                 SIGNAL(clicked()),                      this, SLOT(sEditMiscDistribution()));
+  connect(_invoiceDate,          SIGNAL(newDate(const QDate&)),          this, SLOT(sPopulateDistDate()));
+  connect(_invoiceDate,          SIGNAL(newDate(const QDate&)),          this, SLOT(sPopulateDueDate()));
+  connect(_terms,                SIGNAL(newID(int)),                     this, SLOT(sPopulateDueDate()));
+  connect(_new,                  SIGNAL(clicked()),                      this, SLOT(sNewMiscDistribution()));
+  connect(_save,                 SIGNAL(clicked()),                      this, SLOT(sSave()));
+  connect(_voucherNumber,        SIGNAL(editingFinished()),              this, SLOT(sHandleVoucherNumber()));
+  connect(_taxzone,              SIGNAL(newID(int)),                     this, SLOT(sDistributionDateUpdated()));
+  connect(_distributionDate,     SIGNAL(newDate(const QDate&)),          this, SLOT(sDistributionDateUpdated()));
 
   _terms->setType(XComboBox::APTerms);
 
   _vendor->setShowInactive(false);
 
+  _postVoucher->setEnabled(_privileges->check("PostVouchers"));
+
   _recurring->setParent(-1, "V");
 
-  _miscDistrib->addColumn(tr("Account"), -1,           Qt::AlignLeft,   true,  "account"  );
-  _miscDistrib->addColumn(tr("Amount"),  _moneyColumn, Qt::AlignRight,  true,  "vodist_amount" );
+  _miscDistrib->addColumn(tr("Account"),    -1,           Qt::AlignLeft,   true,  "account"  );
+  _miscDistrib->addColumn(tr("Amount"),     _moneyColumn, Qt::AlignRight,  true,  "vodist_amount" );
+  _miscDistrib->addColumn(tr("Notes"),      -1,           Qt::AlignLeft,   true,  "vodist_notes"  );
+
+  _charass->setType("VCH");
 }
 
 miscVoucher::~miscVoucher()
@@ -65,10 +73,6 @@ enum SetResponse miscVoucher::set(const ParameterList &pParams)
   QVariant param;
   bool     valid;
 
-  param = pParams.value("vend_id", &valid);
-  if (valid)
-    _vendor->setId(param.toInt());
-
   param = pParams.value("mode", &valid);
   if (valid)
   {
@@ -96,6 +100,8 @@ enum SetResponse miscVoucher::set(const ParameterList &pParams)
       {
         _voheadid = insq.value("vohead_id").toInt();
         _recurring->setParent(_voheadid, "V");
+        _documents->setId(_voheadid);
+        _charass->setId(_voheadid);
       }
       else if (ErrorReporter::error(QtCriticalMsg, this, tr("Creating Voucher"),
                                insq, __FILE__, __LINE__))
@@ -128,22 +134,41 @@ enum SetResponse miscVoucher::set(const ParameterList &pParams)
       _new->setEnabled(false);
       _flagFor1099->setEnabled(false);
       _notes->setEnabled(false);
+//    _documents->setReadOnly(true);
+      _charass->setReadOnly(true);
       _close->setText(tr("&Close"));
       _save->hide();
+      _postVoucher->setVisible(false);
 
     }
   }
 
+  param = pParams.value("vend_id", &valid);
+  if (valid)
+    _vendor->setId(param.toInt());
+  
   param = pParams.value("vohead_id", &valid);
   if (valid)
   {
     _voheadid = param.toInt();
+    _documents->setId(_voheadid);
+    _charass->setId(_voheadid);
     populate();
   }
 
   return NoError;
 }
 
+int miscVoucher::id() const
+{
+  return _voheadid;
+}
+
+int miscVoucher::mode() const
+{
+  return _mode;
+}
+
 void miscVoucher::sSave()
 {
   QList<GuiErrorCheck> errors;
@@ -271,6 +296,9 @@ void miscVoucher::sSave()
 
   XSqlQuery commitq("COMMIT;");
 
+  if (_postVoucher->isChecked())
+    postVoucher();
+
   omfgThis->sVouchersUpdated();
 
   _voheadid = -1;
@@ -299,6 +327,36 @@ void miscVoucher::sSave()
   ParameterList params;
   params.append("mode", "new");
   set(params);
+  _vendor->setFocus();
+
+}
+
+void miscVoucher::postVoucher()
+{
+  XSqlQuery post;
+  post.prepare("SELECT fetchJournalNumber('AP-VO') AS result;");
+  post.exec();
+  int journalNumber = 0;
+  if(post.first())
+    journalNumber = post.value("result").toInt();
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Journal Number"),
+                                post, __FILE__, __LINE__))
+    return;
+
+  post.prepare("SELECT postVoucher(:vohead_id, :journalNumber, FALSE) AS result;");
+  post.bindValue(":vohead_id", _voheadid);
+  post.bindValue(":journalNumber", journalNumber);
+  post.exec();
+  if (post.first())
+  {
+    int result = post.value("result").toInt();
+    if (result < 0)
+      ErrorReporter::error(QtCriticalMsg, this, tr("Posting Voucher"),
+                         post, __FILE__, __LINE__);
+  }
+  else
+    ErrorReporter::error(QtCriticalMsg, this, tr("Posting Voucher"),
+                         post, __FILE__, __LINE__);
 }
 
 void miscVoucher::sHandleVoucherNumber()
@@ -378,10 +436,11 @@ void miscVoucher::sNewMiscDistribution()
   if (_taxzone->isValid())
    params.append("taxzone_id", _taxzone->id());
 
-  voucherMiscDistrib newdlg(this, "", TRUE);
+  voucherMiscDistrib newdlg(this, "", true);
   newdlg.set(params);
   if (newdlg.exec() != XDialog::Rejected)
   {
+    sUpdateVoucherTax();
     sFillMiscList();
     sPopulateDistributed();
   }
@@ -399,10 +458,11 @@ void miscVoucher::sEditMiscDistribution()
   if (_taxzone->isValid())
    params.append("taxzone_id", _taxzone->id());
 
-  voucherMiscDistrib newdlg(this, "", TRUE);
+  voucherMiscDistrib newdlg(this, "", true);
   newdlg.set(params);
   if (newdlg.exec() != XDialog::Rejected)
   {
+    sUpdateVoucherTax();
     sFillMiscList();
     sPopulateDistributed();
   }
@@ -418,41 +478,22 @@ void miscVoucher::sDeleteMiscDistribution()
                            delq, __FILE__, __LINE__))
     return;
 
+  sUpdateVoucherTax();
   sFillMiscList();
   sPopulateDistributed();
 }
 
 void miscVoucher::sFillMiscList()
 {
-  XSqlQuery getq;
-  getq.prepare("SELECT vodist_id,"
-             " (formatGLAccount(accnt_id) || ' - ' || accnt_descrip) AS account,"
-             "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-             "FROM vodist, accnt "
-             "WHERE ( (vodist_poitem_id=-1)"
-             " AND (vodist_accnt_id=accnt_id)"
-             " AND (vodist_vohead_id=:vohead_id) ) "
-             "UNION ALL "
-             "SELECT vodist_id, (expcat_code || ' - ' || expcat_descrip) AS account,"
-             "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-             "  FROM vodist, expcat "
-             " WHERE ( (vodist_poitem_id=-1)"
-             "   AND   (vodist_expcat_id=expcat_id)"
-             "   AND   (vodist_vohead_id=:vohead_id) ) "
-             "UNION ALL "
-             "SELECT vodist_id, (tax_code || ' - ' || tax_descrip) AS account,"
-             "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-             "  FROM vodist, tax "
-             " WHERE ( (vodist_poitem_id=-1)"
-             "   AND   (vodist_tax_id=tax_id)"
-             "   AND   (vodist_vohead_id=:vohead_id) ) "
-             "ORDER BY account;" );
-  getq.bindValue(":vohead_id", _voheadid);
-  getq.exec();
-  _miscDistrib->populate(getq);
-  if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Misc. Distributions"),
-                           getq, __FILE__, __LINE__))
-    return;
+  MetaSQLQuery mql = mqlLoad("voucher", "miscDistr");
+  ParameterList params;
+
+  params.append("vohead_id", _voheadid);
+  XSqlQuery miscFillList = mql.toQuery(params);
+  _miscDistrib->populate(miscFillList, true);
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Misc Distributions"),
+                              miscFillList, __FILE__, __LINE__))
+      return;
 }
 
 void miscVoucher::sPopulateDistributed()
@@ -489,7 +530,7 @@ void miscVoucher::populateNumber()
   if (numq.first())
   {
     _voucherNumber->setText(numq.value("vouchernumber").toString());
-    _voucherNumber->setEnabled(FALSE);
+    _voucherNumber->setEnabled(false);
   }
   else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Number"),
                                 numq, __FILE__, __LINE__))
@@ -527,6 +568,8 @@ void miscVoucher::populate()
     else
       _recurring->setParent(_voheadid, "V");
 
+    if(vohead.value("vohead_posted").toBool())
+      _postVoucher->setVisible(false);
 
     sFillMiscList();
     sPopulateDistributed();
@@ -579,6 +622,30 @@ void miscVoucher::sPopulateDueDate()
   }
 }
 
+void miscVoucher::sDistributionDateUpdated()
+{
+  sUpdateVoucherTax();
+  sFillMiscList();
+  sPopulateDistributed();
+}
+
+void miscVoucher::sUpdateVoucherTax()
+{
+  if (_amountToDistribute->localValue() <= 0 || !_taxzone->isValid() || !_distributionDate->isValid())
+    return;
+
+  XSqlQuery updTax;
+  updTax.prepare("SELECT updatemiscvouchertax(:voheadid,:taxzone,:distdate,:curr) as ret;");
+  updTax.bindValue(":voheadid", _voheadid);
+  updTax.bindValue(":taxzone",  _taxzone->id());
+  updTax.bindValue(":distdate", _distributionDate->date());
+  updTax.bindValue(":curr", _amountToDistribute->id());
+  updTax.exec();
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Adding Tax to Voucher"),
+                         updTax, __FILE__, __LINE__))
+    return;
+}
+
 void miscVoucher::keyPressEvent( QKeyEvent * e )
 {
   if(e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter)
