@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,7 +22,7 @@ class openReturnAuthorizations : public XWidget, public Ui::openReturnAuthorizat
     Q_OBJECT
 
 public:
-    openReturnAuthorizations(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    openReturnAuthorizations(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~openReturnAuthorizations();
     
     virtual void setParams(ParameterList &);
@@ -34,6 +34,7 @@ public slots:
     virtual void sNew();
     virtual void sEdit();
     virtual void sView();
+    virtual void sCloseRA();
     virtual void sDelete();
     virtual void sPopulateMenu( QMenu * pMenu );
     virtual void sFillList();
