@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -37,6 +37,7 @@ TODO:	refactor:
 #include "wcombobox.h"
 #include "xtreewidget.h"	// for column widths
 #include "itemSourceSearch.h"
+#include "errorReporter.h"
 
 #define QE_NONINVENTORY
 
@@ -50,7 +51,7 @@ PoitemTableView::PoitemTableView(QWidget* parent) :
   setItemDelegate(delegate);
   setShowGrid(false);
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
   QFont f = font();
   f.setPointSize(f.pointSize() - 2);
   setFont(f);
@@ -275,7 +276,7 @@ QWidget *PoitemTableDelegate::createEditor(QWidget *parent,
 
   if (editor)
   {
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
     // compensate for many custom widgets making themselves smaller on OS X
     QFont f = editor->font();
     f.setPointSize(f.pointSize() + 2);
@@ -407,16 +408,18 @@ void PoitemTableDelegate::setModelData(QWidget *editor, QAbstractItemModel *pMod
 	      model->setData(model->index(index.row(), WAREHOUS_ID_COL), itemq.value("warehous_id").toInt());
 	      model->setData(model->index(index.row(), WAREHOUS_CODE_COL), itemq.value("warehous_code").toString());
 	    }
-	    else if (itemq.lastError().type() != QSqlError::NoError)
-	    {
-	      systemError(0, itemq.lastError().databaseText(), __FILE__, __LINE__);
-	      hitError = true;
-	      break;
-	    }
+        else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving PO Information"),
+                                      itemq, __FILE__, __LINE__))
+        {
+          hitError = true;
+          break;
+        }
 	    else
 	    {
-	      systemError(0, QString("Could not find Item Site for %1 (%2).")
-			      .arg(item->itemNumber()).arg(item->id()));
+          ErrorReporter::error(QtCriticalMsg, 0, tr("Error Occurred"),
+                               tr("Could not find Item Site for %1 (%2).")
+                               .arg(item->itemNumber())
+                               .arg(item->id()),__FILE__,__LINE__);
 	      hitError = true;
 	      break;
 	    }
@@ -441,12 +444,12 @@ void PoitemTableDelegate::setModelData(QWidget *editor, QAbstractItemModel *pMod
 		hitError = true;
 		break;
 	      }
-	      else if (stdcostq.lastError().type() != QSqlError::NoError)
-	      {
-		systemError(0, stdcostq.lastError().databaseText(), __FILE__, __LINE__);
-		hitError = true;
-		break;
-	      }
+          else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Standard Cost Information"),
+                                        stdcostq, __FILE__, __LINE__))
+          {
+            hitError = true;
+            break;
+          }
 	    }
 
 	    XSqlQuery itemsrcq;
@@ -515,12 +518,12 @@ void PoitemTableDelegate::setModelData(QWidget *editor, QAbstractItemModel *pMod
 	      if (_metrics->boolean("UseEarliestAvailDateOnPOItem"))
 		model->setData(model->index(index.row(), POITEM_DUEDATE_COL), itemsrcq.value("earliestdate").toDate());
 	    }
-	    else if (itemsrcq.lastError().type() != QSqlError::NoError)
-	    {
-	      systemError(0, itemsrcq.lastError().databaseText(), __FILE__, __LINE__);
-	      hitError = true;
-	      break;
-	    }
+        else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Item Information"),
+                                      itemsrcq, __FILE__, __LINE__))
+        {
+          hitError = true;
+          break;
+        }
 	  }
 	}
       }
@@ -555,17 +558,19 @@ void PoitemTableDelegate::setModelData(QWidget *editor, QAbstractItemModel *pMod
 	  {
 	    model->setData(model->index(index.row(), POITEM_ITEMSITE_ID_COL), itemq.value("itemsite_id").toInt());
 	  }
-	  else if (itemq.lastError().type() != QSqlError::NoError)
-	  {
-	    systemError(0, itemq.lastError().databaseText(), __FILE__, __LINE__);
-	    hitError = true;
-	    break;
-	  }
+      else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Item Site Information"),
+                                    itemq, __FILE__, __LINE__))
+      {
+        hitError = true;
+        break;
+      }
 	  else
 	  {
-	    systemError(0, QString("Could not find Item Site for %1 in %2.")
-			    .arg(model->index(index.row(), ITEM_NUMBER_COL).data().toString())
-			    .arg(whs->currentText()));
+        ErrorReporter::error(QtCriticalMsg, 0, tr("Error Occurred"),
+                             tr("%1: Could not find Item Site for %1 in %2.")
+                             .arg(model->index(index.row(), ITEM_NUMBER_COL).data().toString())
+                             .arg(whs->currentText())
+                             ,__FILE__,__LINE__);
 	    hitError = true;
 	    break;
 	  }
@@ -632,12 +637,12 @@ void PoitemTableDelegate::setModelData(QWidget *editor, QAbstractItemModel *pMod
 	  {
 	    model->setData(model->index(index.row(), POITEM_UNITPRICE_COL), PoitemetModelData.value("new_itemsrcp_price").toDouble());
 	  }
-	  else if (PoitemetModelData.lastError().type() != QSqlError::NoError)
-	  {
-	    systemError(0, PoitemetModelData.lastError().databaseText(), __FILE__, __LINE__);
-	    hitError = true;
-	    break;
-	  }
+      else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Retrieving Item Information"),
+                                    PoitemetModelData, __FILE__, __LINE__))
+      {
+        hitError = true;
+        break;
+      }
 	  else
 	    model->setData(model->index(index.row(), POITEM_UNITPRICE_COL), 0);
 	}
