@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -25,6 +25,14 @@ public:
     ~priceList();
 
     double _selectedPrice;
+    QString _selectedMethod;
+    QString _selectedType;
+    QString _selectedSale;
+    QString _selectedSchedule;
+    double _selectedBasis;
+    double _selectedModifierPct;
+    double _selectedModifierAmt;
+    double _selectedQtyBreak;
     int _curr_id;
 
 public slots:
@@ -33,11 +41,15 @@ public slots:
     virtual void sFillList();
 
 protected:
+    int _saletypeid;
     int _shiptoid;
+    int _shipzoneid;
     QString _shiptonum;
     int _prodcatid;
     int _custtypeid;
+    double _iteminvpricerat;
     QString _custtypecode;
+    QString _listpriceschedule;
     QDate _effective;
     QDate _asOf;
 
