@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,10 +19,11 @@ class pricingScheduleAssignments : public XWidget, public Ui::pricingScheduleAss
     Q_OBJECT
 
 public:
-    pricingScheduleAssignments(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    pricingScheduleAssignments(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
     ~pricingScheduleAssignments();
 
 public slots:
+    virtual SetResponse set( const ParameterList & pParams );
     virtual void sPrint();
     virtual void sNew();
     virtual void sEdit();
@@ -33,6 +34,9 @@ public slots:
 protected slots:
     virtual void languageChange();
 
+private:
+    bool _listpricesched;
+  
 };
 
 #endif // PRICINGSCHEDULEASSIGNMENTS_H
