@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -26,10 +26,11 @@
 #include "xtsettings.h"
 #include "confirmAchOK.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
 QString printCheck::eftFileDir = QString();
 
-printCheck::printCheck(QWidget* parent, const char* name, Qt::WFlags fl)
+printCheck::printCheck(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
@@ -38,13 +39,17 @@ printCheck::printCheck(QWidget* parent, const char* name, Qt::WFlags fl)
   connect(_check,     SIGNAL(newID(int)), this, SLOT(sEnableCreateEFT()));
   connect(_createEFT, SIGNAL(clicked()),  this, SLOT(sCreateEFT()));
   connect(_print,     SIGNAL(clicked()),  this, SLOT(sPrint()));
+  connect(_printed,   SIGNAL(clicked()),  this, SLOT(sPrintedAlready()));
 
-  _captive = FALSE;
+  _nextCheckNum->setValidator(new QIntValidator(1,999999999));
+
+  _captive = false;
   _setCheckNumber = -1;
 
-  _check->setAllowNull(TRUE);
+  _check->setAllowNull(true);
 
   _bankaccnt->setType(XComboBox::APBankAccounts);
+  sHandleBankAccount(_bankaccnt->id());
 
   _createEFT->setVisible(_metrics->boolean("ACHSupported") && _metrics->boolean("ACHEnabled"));
 }
@@ -62,7 +67,7 @@ void printCheck::languageChange()
 enum SetResponse printCheck::set(const ParameterList &pParams)
 {
   XWidget::set(pParams);
-  _captive = TRUE;
+  _captive = true;
 
   QVariant param;
   bool     valid;
@@ -71,14 +76,24 @@ enum SetResponse printCheck::set(const ParameterList &pParams)
   if (valid)
   {
     populate(param.toInt());
-    _bankaccnt->setEnabled(FALSE);
-    _check->setEnabled(FALSE);
+    _bankaccnt->setEnabled(false);
+    _check->setEnabled(false);
   }
 
   return NoError;
 }
 
 void printCheck::sPrint()
+{
+  sPrintImpl(false);
+}
+
+void printCheck::sPrintedAlready()
+{
+  sPrintImpl(true);
+}
+
+void printCheck::sPrintImpl(bool printedAlready)
 {
   XSqlQuery printPrint;
   if(_setCheckNumber != -1 && _setCheckNumber != _nextCheckNum->text().toInt())
@@ -98,14 +113,15 @@ void printCheck::sPrint()
                     tr("<p>The selected Check Number has already been used.") );
       return;
     }
-    else if (printPrint.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                                  printPrint, __FILE__, __LINE__))
     {
-      systemError(this, printPrint.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
 
   if (_createEFT->isEnabled() &&
+      !printedAlready &&
       QMessageBox::question(this, tr("Print Anyway?"),
                             tr("<p>The recipient of this check has been "
                                "configured for EFT transactions. Do you want "
@@ -152,13 +168,15 @@ void printCheck::sPrint()
   
       if (!docReport.setContent(report.value("report_source").toString(), &errorMessage, &errorLine))
       {
-        systemError(this, errorMessage, __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                             report, __FILE__, __LINE__);
         return;
       }
     }
     else
     {
-      systemError(this, report.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                           report, __FILE__, __LINE__);
       return;
     }
 // end getting the report definition out the database
@@ -174,14 +192,15 @@ void printCheck::sPrint()
         int result = printPrint.value("result").toInt();
         if (result < 0)
         {
-          systemError(this, storedProcErrorLookup("setNextCheckNumber", result),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                                 storedProcErrorLookup("setNextCheckNumber", result),
+                                 __FILE__, __LINE__);
           return;
         }
       }
-      else if (printPrint.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                                    printPrint, __FILE__, __LINE__))
       {
-        systemError(this, printPrint.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
@@ -190,9 +209,15 @@ void printCheck::sPrint()
               " WHERE(checkhead_id=:checkhead_id);");
     printPrint.bindValue(":checkhead_id", _check->id());
     printPrint.exec();
-    if (printPrint.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Check Information"),
+                                  printPrint, __FILE__, __LINE__))
     {
-      systemError(this, printPrint.lastError().databaseText(), __FILE__, __LINE__);
+      return;
+    }
+
+    if(printedAlready)
+    {
+      markCheckAsPrinted(_check->id());
       return;
     }
 
@@ -234,26 +259,28 @@ void printCheck::sPrint()
                  "       checkhead_for, checkhead_notes,"
                  "       checkhead_curr_id, checkhead_deleted) "
                  "SELECT checkhead_recip_id, checkhead_recip_type,"
-                 "       checkhead_bankaccnt_id, TRUE,"
+                 "       checkhead_bankaccnt_id, true,"
                  "       checkhead_checkdate, fetchNextCheckNumber(checkhead_bankaccnt_id),"
-                 "       checkhead_amount, TRUE, TRUE,"
+                 "       checkhead_amount, true, true,"
                  "       'Continuation of Check #'||checkhead_number,"
                  "       'Continuation of Check #'||checkhead_number,"
-                 "       checkhead_curr_id, TRUE"
+                 "       checkhead_curr_id, true"
                  "  FROM checkhead"
                  " WHERE(checkhead_id=:checkhead_id);");
       qq.bindValue(":checkhead_id", _check->id());
       if(!qq.exec())
       {
-        systemError(this, "Received error but will continue anyway:\n"+qq.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Check Information but will continue anyway\n %1")
+                             .arg(qq.lastError().databaseText()),
+                             qq, __FILE__, __LINE__);
       }
     }
 
-    omfgThis->sChecksUpdated(_bankaccnt->id(), _check->id(), TRUE);
+    omfgThis->sChecksUpdated(_bankaccnt->id(), _check->id(), true);
   }
-  else if (printPrint.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Check Information"),
+                                printPrint, __FILE__, __LINE__))
   {
-    systemError(this, printPrint.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else // join failed
@@ -287,14 +314,15 @@ void printCheck::sPrint()
       int result = printPrint.value("result").toInt();
       if (result < 0)
       {
-	systemError(this, storedProcErrorLookup("voidCheck", result),
-		    __FILE__, __LINE__);
-	return;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Voiding Check"),
+                               storedProcErrorLookup("voidCheck", result),
+                               __FILE__, __LINE__);
+        return;
       }
     }
-    else if (printPrint.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Voiding Check"),
+                                  printPrint, __FILE__, __LINE__))
     {
-      systemError(this, printPrint.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -309,19 +337,20 @@ void printCheck::sPrint()
       int result = printPrint.value("result").toInt();
       if (result < 0)
       {
-	systemError(this, storedProcErrorLookup("replaceVoidedCheck", result),
-		    __FILE__, __LINE__);
-	return;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                               storedProcErrorLookup("replaceVoidedCheck", result),
+                               __FILE__, __LINE__);
+        return;
       }
       omfgThis->sChecksUpdated(printPrint.value("checkhead_bankaccnt_id").toInt(),
-			       _check->id(), TRUE);
+			       _check->id(), true);
 
       sHandleBankAccount(_bankaccnt->id());
       _print->setFocus();
     }
-    else if (printPrint.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                                  printPrint, __FILE__, __LINE__))
     {
-      systemError(this, printPrint.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -344,9 +373,9 @@ void printCheck::sHandleBankAccount(int pBankaccntid)
       _createEFT->setEnabled(checkNumber.value("bankaccnt_ach_enabled").toBool());
       _nextCheckNum->setText(checkNumber.value("bankaccnt_nextchknum").toString());
     }
-    else if (printHandleBankAccount.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Information"),
+                                  printHandleBankAccount, __FILE__, __LINE__))
     {
-      systemError(this, printHandleBankAccount.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -372,9 +401,9 @@ void printCheck::sHandleBankAccount(int pBankaccntid)
   printHandleBankAccount.exec();
   _check->populate(printHandleBankAccount);
   _check->setNull();
-  if (printHandleBankAccount.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Information"),
+                                printHandleBankAccount, __FILE__, __LINE__))
   {
-    systemError(this, printHandleBankAccount.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -392,9 +421,9 @@ void printCheck::populate(int pcheckid)
     _bankaccnt->setId(printpopulate.value("checkhead_bankaccnt_id").toInt());
     _check->setId(pcheckid);
   }
-  else if (printpopulate.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                                printpopulate, __FILE__, __LINE__))
   {
-    systemError(this, printpopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -449,7 +478,7 @@ void printCheck::sCreateEFT()
     }
     do
     {
-      eftfile.write(printCreateEFT.value("achline_value").toString().toAscii());
+      eftfile.write(printCreateEFT.value("achline_value").toString().toLatin1());
       eftfile.write("\n");
     } while (printCreateEFT.next());
     eftfile.close();
@@ -457,7 +486,8 @@ void printCheck::sCreateEFT()
     {
       releasenum.exec();
       eftfile.remove();
-      systemError(this, printCreateEFT.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating EFT File"),
+                           printCreateEFT, __FILE__, __LINE__);
       return;
     }
 
@@ -473,16 +503,16 @@ void printCheck::sCreateEFT()
                      "WHERE (checkhead_id=:checkhead_id);");
       clearq.bindValue(":checkhead_id", _check->id());
       clearq.exec();
-      if (clearq.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Receiving ACH Confirmation"),
+                                    clearq, __FILE__, __LINE__))
       {
-        systemError(this, clearq.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
   }
-  else if (printCreateEFT.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retreiving Check Information for EFT Creation"),
+                                printCreateEFT, __FILE__, __LINE__))
   {
-    systemError(this, printCreateEFT.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -506,9 +536,9 @@ void printCheck::sEnableCreateEFT()
     printEnableCreateEFT.exec();
     if (printEnableCreateEFT.first())
       _createEFT->setEnabled(printEnableCreateEFT.value("eftenabled").toBool());
-    else if (printEnableCreateEFT.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Information for EFT Creation"),
+                                  printEnableCreateEFT, __FILE__, __LINE__))
     {
-      systemError(this, printEnableCreateEFT.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else // not found
@@ -547,12 +577,13 @@ void printCheck::markCheckAsPrinted(const int pcheckid)
     int result = markq.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("markCheckAsPrinted", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Marking Check as Printed"),
+                             storedProcErrorLookup("markCheckAsPrinted", markq.value("result").toInt()),
+                             __FILE__, __LINE__);
       return;
     }
     omfgThis->sChecksUpdated(markq.value("checkhead_bankaccnt_id").toInt(),
-                             pcheckid, TRUE);
+                             pcheckid, true);
 
     if (_captive)
       close();
