@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -27,8 +27,9 @@
 #include "printCheck.h"
 #include "printChecksReview.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
-printChecks::printChecks(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+printChecks::printChecks(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
@@ -37,9 +38,11 @@ printChecks::printChecks(QWidget* parent, const char* name, bool modal, Qt::WFla
   connect(_createEFT, SIGNAL(clicked()),  this, SLOT(sCreateEFT()));
   connect(_print,     SIGNAL(clicked()),  this, SLOT(sPrint()));
 
+  _nextCheckNum->setValidator(new QIntValidator(1,999999999));
+
   _setCheckNumber = -1;
 
-  _bankaccnt->setAllowNull(TRUE);
+  _bankaccnt->setAllowNull(true);
   _bankaccnt->setType(XComboBox::APBankAccounts);
 
   _createEFT->setVisible(_metrics->boolean("ACHSupported") && _metrics->boolean("ACHEnabled"));
@@ -58,7 +61,7 @@ void printChecks::languageChange()
 enum SetResponse printChecks::set(const ParameterList & pParams )
 {
   XDialog::set(pParams);
-  _captive = TRUE;
+  _captive = true;
 
   QVariant param;
   bool     valid;
@@ -73,13 +76,56 @@ enum SetResponse printChecks::set(const ParameterList & pParams )
 void printChecks::sPrint()
 {
   XSqlQuery printPrint;
+  
+  if(_setCheckNumber != -1 && _setCheckNumber != _nextCheckNum->text().toInt())
+  {
+    int countCheckNum = _nextCheckNum->text().toInt();
+
+    XSqlQuery checks;
+    MetaSQLQuery mql = mqlLoad("checks", "detail");
+    
+    ParameterList params;
+    params.append("bankaccnt_id", _bankaccnt->id());
+    params.append("toPrintOnly");
+    params.append("numtoprint", _numberOfChecks->value());
+    if (_orderByName->isChecked())
+      params.append("orderByName");
+    
+    checks = mql.toQuery(params);
+    while (checks.next())
+    {
+      printPrint.prepare("SELECT checkhead_id "
+                         "FROM checkhead "
+                         "WHERE ( (checkhead_bankaccnt_id=:bankaccnt_id) "
+                         "  AND   (checkhead_number=:nextCheckNumber));");
+      printPrint.bindValue(":bankaccnt_id", _bankaccnt->id());
+      printPrint.bindValue(":checkhead_id", checks.value("checkhead_id").toInt());
+      printPrint.bindValue(":nextCheckNumber", countCheckNum);
+      printPrint.exec();
+      if (printPrint.first() && !_metrics->boolean("ReprintPaymentNumbers"))
+      {
+        QMessageBox::information( this, tr("Payment Number Already Used"),
+                                 tr("<p>A Payment Number has already been used.") );
+        return;
+      }
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Printing Check"),
+                                    printPrint, __FILE__, __LINE__))
+      {
+        return;
+      }
+      
+      countCheckNum = countCheckNum + 1;
+    }
+  }
+  
+  
   if (_somerecips_eft_enabled &&
       QMessageBox::question(this, tr("Print Anyway?"),
-                            tr("<p>Some of the recipients of checks in this "
-                               "check run have been configured for EFT "
-                               "transactions. Do you want to print checks "
+                            tr("<p>Some of the recipients of payments in this "
+                               "payment run have been configured for EFT "
+                               "transactions. Do you want to print payments "
                                "for them anyway?<p>If you answer 'Yes' then "
-                               "a check will be printed. If you say 'No' then "
+                               "a payment will be printed. If you say 'No' then "
                                "you should click %1 first and <i>then</i> "
                                "click %2.")
                               .arg(_createEFT->text(), _print->text()),
@@ -88,7 +134,7 @@ void printChecks::sPrint()
     return;
 
   QList<int> printedChecks;
-  bool firstRun = TRUE;
+  bool firstRun = true;
 
   ParameterList params;
   params.append("bankaccnt_id", _bankaccnt->id());
@@ -129,13 +175,17 @@ void printChecks::sPrint()
 
         if (!docReport.setContent(report.value("report_source").toString(), &errorMessage, &errorLine))
         {
-          systemError(this, errorMessage, __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                               tr("%1: %2")
+                               .arg(windowTitle())
+                               .arg(errorMessage),__FILE__,__LINE__);
           return;
         }
       }
       else
       {
-        systemError(this, report.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Printing Check"),
+                             report, __FILE__, __LINE__);
         return;
       }
       // end getting the report definition out the database
@@ -151,17 +201,18 @@ void printChecks::sPrint()
           int result = printPrint.value("result").toInt();
           if (result < 0)
           {
-            systemError(this, storedProcErrorLookup("setNextCheckNumber", result),
-                        __FILE__, __LINE__);
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Setting Next Check Number"),
+                                   storedProcErrorLookup("setNextCheckNumber", result),
+                                   __FILE__, __LINE__);
             return;
           }
         }
-        else if (printPrint.lastError().type() != QSqlError::NoError)
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Printing Check"),
+                                      printPrint, __FILE__, __LINE__))
         {
-          systemError(this, printPrint.lastError().databaseText(), __FILE__, __LINE__);
           return;
         }
-       firstRun = FALSE;
+       firstRun = false;
       }
     }
 
@@ -169,9 +220,9 @@ void printChecks::sPrint()
               " WHERE(checkhead_id=:checkhead_id);");
     printPrint.bindValue(":checkhead_id", checks.value("checkhead_id").toInt());
     printPrint.exec();
-    if (printPrint.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Printing Check"),
+                                  printPrint, __FILE__, __LINE__))
     {
-      systemError(this, printPrint.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -200,25 +251,27 @@ void printChecks::sPrint()
                  "       checkhead_for, checkhead_notes,"
                  "       checkhead_curr_id, checkhead_deleted) "
                  "SELECT checkhead_recip_id, checkhead_recip_type,"
-                 "       checkhead_bankaccnt_id, TRUE,"
+                 "       checkhead_bankaccnt_id, true,"
                  "       checkhead_checkdate, fetchNextCheckNumber(checkhead_bankaccnt_id),"
-                 "       checkhead_amount, TRUE, TRUE,"
+                 "       checkhead_amount, true, true,"
                  "       'Continuation of Check #'||checkhead_number,"
                  "       'Continuation of Check #'||checkhead_number,"
-                 "       checkhead_curr_id, TRUE"
+                 "       checkhead_curr_id, true"
                  "  FROM checkhead"
                  " WHERE(checkhead_id=:checkhead_id);");
       qq.bindValue(":checkhead_id", checks.value("checkhead_id").toInt());
       if(!qq.exec())
       {
-        systemError(this, "Received error but will continue anyway:\n"+qq.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Received error but will continue anyway:\n %1")
+                             .arg(qq.lastError().databaseText()),
+                             qq, __FILE__, __LINE__);
       }
     }
     singleCheckPrerendered.append(doc);
   }
-  if (checks.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Printing Check"),
+                                checks, __FILE__, __LINE__))
   {
-    systemError(this, checks.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -245,8 +298,8 @@ void printChecks::sPrint()
 
     QList<int>::iterator it;
 
-    if ( QMessageBox::question(this, tr("All Checks Printed"),
-			       tr("<p>Did all the Checks print successfully?"),
+    if ( QMessageBox::question(this, tr("All Payments Printed"),
+			       tr("<p>Did all the Payments print successfully?"),
 				QMessageBox::Yes,
 				QMessageBox::No | QMessageBox::Default) == QMessageBox::Yes)
     {
@@ -260,22 +313,23 @@ void printChecks::sPrint()
 	{
 	  int result = postCheck.value("result").toInt();
 	  if (result < 0)
-	  {
-	    systemError(this, storedProcErrorLookup("markCheckAsPrinted",
-						   result), __FILE__, __LINE__);
-	    return;
-	  }
-	}
-	else if (postCheck.lastError().type() != QSqlError::NoError)
-	{
-	  systemError(this, postCheck.lastError().databaseText(), __FILE__, __LINE__);
-	  return;
+      {
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Marking Check As Printed"),
+                               storedProcErrorLookup("markCheckAsPrinted", result),
+                               __FILE__, __LINE__);
+        return;
+      }
 	}
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Printing Check"),
+                                  postCheck, __FILE__, __LINE__))
+    {
+      return;
+    }
       }
     }
     else
     {
-      printChecksReview newdlg(this, "", TRUE);
+      printChecksReview newdlg(this, "", true);
       QString query = QString( "SELECT checkhead_id, checkhead_number"
                                "  FROM checkhead"
                                " WHERE (checkhead_id IN (" );
@@ -296,11 +350,11 @@ void printChecks::sPrint()
     }
   }
   else
-    QMessageBox::information( this, tr("No Checks Printed"),
-			     tr("<p>No Checks were printed for the selected "
+    QMessageBox::information( this, tr("No Payments Printed"),
+			     tr("<p>No Payments were printed for the selected "
 				"Bank Account.") );
 
-  omfgThis->sChecksUpdated(_bankaccnt->id(), -1, TRUE);
+  omfgThis->sChecksUpdated(_bankaccnt->id(), -1, true);
   sHandleBankAccount(_bankaccnt->id());
 }
 
@@ -335,9 +389,9 @@ void printChecks::sHandleBankAccount(int pBankaccntid)
                            printHandleBankAccount.value("somerecip_eft_enabled").toBool() &&
                            printHandleBankAccount.value("numofchecks").toInt() > 0);
   }
-  else if (printHandleBankAccount.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Check Information"),
+                                printHandleBankAccount, __FILE__, __LINE__))
   {
-    systemError(this, printHandleBankAccount.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else // not found
@@ -357,15 +411,15 @@ void printChecks::sCreateEFT()
   XSqlQuery printCreateEFT;
   if (_somerecips_eft_enabled && !_allrecips_eft_enabled &&
       QMessageBox::question(this, tr("Print Anyway?"),
-                            tr("<p>Some but not all of the checks in this run "
+                            tr("<p>Some but not all of the Payments in this run "
                                "are for Vendors configured to receive EFT "
                                "transactions. Do you want to create the EFT "
                                "file anyway?<p>If you answer 'Yes' then an "
                                "EFT file will be created but you will have to "
                                "click Print to get the remainder of the "
-                               "checks in this run. If you say 'No' then you "
+                               "Payments in this run. If you say 'No' then you "
                                "will get a warning when you click Print "
-                               "asking whether you want to print checks for "
+                               "asking whether you want to print Payments for "
                                "EFT recipients."),
                             QMessageBox::Yes | QMessageBox::Default,
                             QMessageBox::No) == QMessageBox::No)
@@ -417,7 +471,7 @@ void printChecks::sCreateEFT()
     }
     do
     {
-      eftfile.write(printCreateEFT.value("achline_value").toString().toAscii());
+      eftfile.write(printCreateEFT.value("achline_value").toString().toLatin1());
       eftfile.write("\n");
     } while (printCreateEFT.next());
     eftfile.close();
@@ -425,7 +479,8 @@ void printChecks::sCreateEFT()
     {
       releasenum.exec();
       eftfile.remove();
-      systemError(this, printCreateEFT.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating EFT File"),
+                           printCreateEFT, __FILE__, __LINE__);
       return;
     }
 
@@ -441,17 +496,17 @@ void printChecks::sCreateEFT()
                      "WHERE (checkhead_ach_batch=:checkhead_ach_batch);");
       clearq.bindValue(":checkhead_ach_batch", batch);
       clearq.exec();
-      if (clearq.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Receiving ACH Confirmation"),
+                                    clearq, __FILE__, __LINE__))
       {
-        systemError(this, clearq.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
     sHandleBankAccount(_bankaccnt->id());
   }
-  else if (printCreateEFT.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Creating EFT File"),
+                                printCreateEFT, __FILE__, __LINE__))
   {
-    systemError(this, printCreateEFT.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -470,10 +525,11 @@ void printChecks::markChecksAsPrinted(const QString pbatch)
     int result = markq.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("markCheckAsPrinted", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Marking Check As Printed"),
+                             storedProcErrorLookup("markCheckAsPrinted", result),
+                             __FILE__, __LINE__);
       return;
     }
-    omfgThis->sChecksUpdated(markq.value("bankaccnt_id").toInt(), -1, TRUE);
+    omfgThis->sChecksUpdated(markq.value("bankaccnt_id").toInt(), -1, true);
   }
 }
