@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2017 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,19 +19,19 @@
 #include "errorReporter.h"
 #include "guiErrorCheck.h"
 #include "mqlutil.h"
-#include "taxDetail.h"
+#include "taxBreakdown.h"
 #include "itemCharacteristicDelegate.h"
 #include "itemSourceSearch.h"
 #include "itemSourceList.h"
 #include "vendorPriceList.h"
 
-purchaseOrderItem::purchaseOrderItem(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+purchaseOrderItem::purchaseOrderItem(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   XSqlQuery purchasepurchaseOrderItem;
   setupUi(this);
 
-#ifndef Q_WS_MAC
+#ifndef Q_OS_MAC
   _vendorItemNumberList->setMaximumWidth(25);
 #else
   _listPrices->setMinimumWidth(60);
@@ -45,6 +45,7 @@ purchaseOrderItem::purchaseOrderItem(QWidget* parent, const char* name, bool mod
   _maxCost = 0.0;
   _dropship = false;
   _costmethod = "";
+  _captive = false;
 
   connect(_ordered, SIGNAL(editingFinished()), this, SLOT(sDeterminePrice()));
   connect(_inventoryItem, SIGNAL(toggled(bool)), this, SLOT(sInventoryItemToggled(bool)));
@@ -57,6 +58,7 @@ purchaseOrderItem::purchaseOrderItem(QWidget* parent, const char* name, bool mod
   connect(_listPrices, SIGNAL(clicked()), this, SLOT(sVendorListPrices()));
   connect(_taxLit, SIGNAL(leftClickedURL(QString)), this, SLOT(sTaxDetail()));  // new slot added for tax url //
   connect(_extendedPrice, SIGNAL(valueChanged()), this, SLOT(sCalculateTax()));  // new slot added for price //
+  connect(_freight, SIGNAL(valueChanged()), this, SLOT(sCalculateTax()));  // new slot added for line freight //
   connect(_taxtype, SIGNAL(newID(int)), this, SLOT(sCalculateTax()));            // new slot added for taxtype //
 
   _bomRevision->setMode(RevisionLineEdit::Use);
@@ -67,7 +69,7 @@ purchaseOrderItem::purchaseOrderItem(QWidget* parent, const char* name, bool mod
   _parentwo = -1;
   _parentso = -1;
   _itemsrcid = -1;
-  _taxzoneid = -1;   //  _taxzoneid  added // 
+  _taxzoneid = -1;   //  _taxzoneid  added //
   _orderQtyCache = -1;
 
   _overriddenUnitPrice = false;
@@ -135,10 +137,6 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
   XDialog::set(pParams);
   QVariant param;
   bool     valid;
-  bool     haveQty  = FALSE;
-  bool     haveDate = FALSE;
-
-
 
   param = pParams.value("vend_id", &valid);
   if (valid)
@@ -227,9 +225,8 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
     }
     else
     {
-      systemError(this, tr("A System Error occurred at %1::%2.")
-                        .arg(__FILE__)
-                        .arg(__LINE__) );
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Purchase Order Item Information"),
+                           purchaseet, __FILE__, __LINE__);
       return UndefinedError;
     }
   }
@@ -252,7 +249,7 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
     }
 
     populate();
-	sCalculateTax();
+    sCalculateTax();
   }
   // connect here and not in the .ui to avoid timing issues at initialization
   connect(_unitPrice, SIGNAL(valueChanged()), this, SLOT(sPopulateExtPrice()));
@@ -270,9 +267,8 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
         _poitemid = purchaseet.value("poitem_id").toInt();
       else
       {
-        systemError(this, tr("A System Error occurred at %1::%2.")
-                          .arg(__FILE__)
-                          .arg(__LINE__) );
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Purchase Order Item Information"),
+                             purchaseet, __FILE__, __LINE__);
         return UndefinedError;
       }
 
@@ -313,9 +309,8 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
         _lineNumber->setText(purchaseet.value("_linenumber").toString());
       else
       {
-        systemError(this, tr("A System Error occurred at %1::%2.")
-                          .arg(__FILE__)
-                          .arg(__LINE__) );
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Purchase Order Item Information"),
+                             purchaseet, __FILE__, __LINE__);
 
         return UndefinedError;
       }
@@ -323,34 +318,35 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
       _bomRevision->setEnabled(_privileges->boolean("UseInactiveRevisions"));
       _booRevision->setEnabled(_privileges->boolean("UseInactiveRevisions"));
       _comments->setId(_poitemid);
-      _tab->setTabEnabled(_tab->indexOf(_demandTab), FALSE);
+      _project->setAllowedStatuses(ProjectLineEdit::Concept |  ProjectLineEdit::InProcess);
+      _tab->setTabEnabled(_tab->indexOf(_demandTab), false);
     }
     else if (param.toString() == "edit")
     {
       _mode = cEdit;
 
-      _typeGroup->setEnabled(FALSE);
+      _typeGroup->setEnabled(false);
     }
     else if (param.toString() == "view")
     {
       _mode = cView;
 
-      _typeGroup->setEnabled(FALSE);
-      _vendorItemNumber->setEnabled(FALSE);
-      _vendorItemNumberList->setEnabled(FALSE);
-      _vendorDescrip->setEnabled(FALSE);
-      _warehouse->setEnabled(FALSE);
-      _dueDate->setEnabled(FALSE);
-      _ordered->setEnabled(FALSE);
-      _unitPrice->setEnabled(FALSE);
-      _freight->setEnabled(FALSE);
-      _notes->setReadOnly(TRUE);
-      _comments->setReadOnly(TRUE);
-      _project->setEnabled(FALSE);
-      _taxtype->setEnabled(FALSE);
-      _taxRecoverable->setEnabled(FALSE);
-      _bomRevision->setEnabled(FALSE);
-      _booRevision->setEnabled(FALSE);
+      _typeGroup->setEnabled(false);
+      _vendorItemNumber->setEnabled(false);
+      _vendorItemNumberList->setEnabled(false);
+      _vendorDescrip->setEnabled(false);
+      _warehouse->setEnabled(false);
+      _dueDate->setEnabled(false);
+      _ordered->setEnabled(false);
+      _unitPrice->setEnabled(false);
+      _freight->setEnabled(false);
+      _notes->setReadOnly(true);
+      _comments->setReadOnly(true);
+      _project->setEnabled(false);
+      _taxtype->setEnabled(false);
+      _taxRecoverable->setEnabled(false);
+      _bomRevision->setEnabled(false);
+      _booRevision->setEnabled(false);
 
       _close->setText(tr("&Close"));
       _save->hide();
@@ -361,8 +357,8 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
   if (valid)
   {
     _item->setItemsiteid(param.toInt());
-    _item->setEnabled(FALSE);
-    _warehouse->setEnabled(FALSE);
+    _item->setEnabled(false);
+    _warehouse->setEnabled(false);
   }
   
   param = pParams.value("itemsrc_id", &valid);
@@ -376,15 +372,12 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
 
     if (_item->isValid())
       sDeterminePrice();
-
-    haveQty = TRUE;
   }
 
   param = pParams.value("dueDate", &valid);
   if (valid)
   {
     _dueDate->setDate(param.toDate());
-    haveDate = TRUE;
   }
 
   param = pParams.value("prj_id", &valid);
@@ -411,6 +404,10 @@ enum SetResponse purchaseOrderItem::set(const ParameterList &pParams)
     }
   }
 
+  param = pParams.value("captive", &valid);
+  if (valid)
+    _captive = true;
+  
   return NoError;
 }
 
@@ -424,9 +421,9 @@ void purchaseOrderItem::populate()
   params.append("sonum",     tr("Sales Order #")),
   params.append("wonum",     tr("Work Order #")),
   purchasepopulate = mql.toQuery(params);
-  if (purchasepopulate.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Purchase Order Item Information"),
+                                purchasepopulate, __FILE__, __LINE__))
   {
-    systemError(this, purchasepopulate.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   if (purchasepopulate.first())
@@ -452,18 +449,18 @@ void purchaseOrderItem::populate()
 
     if(purchasepopulate.value("poitem_order_id") != -1)
     {
-      _ordered->setEnabled(FALSE);
-      _dueDate->setEnabled(FALSE);
+      _ordered->setEnabled(false);
+      _dueDate->setEnabled(false);
       _soLit->setText(purchasepopulate.value("demand_type").toString());
       _so->setText(purchasepopulate.value("order_number").toString());
 //      _soLine->setText(purchasepopulate.value("orderline_number").toString());
     }
     else
-      _tab->setTabEnabled(_tab->indexOf(_demandTab), FALSE);
+      _tab->setTabEnabled(_tab->indexOf(_demandTab), false);
 
     if (purchasepopulate.value("poitem_itemsite_id").toInt() == -1)
     {
-      _nonInventoryItem->setChecked(TRUE);
+      _nonInventoryItem->setChecked(true);
       _expcat->setId(purchasepopulate.value("poitem_expcat_id").toInt());
       sPopulateItemSourceInfo(-1);
 
@@ -474,9 +471,9 @@ void purchaseOrderItem::populate()
     }
     else
     {
-      _inventoryItem->setChecked(TRUE);
+      _inventoryItem->setChecked(true);
       _item->setItemsiteid(purchasepopulate.value("poitem_itemsite_id").toInt());
-      sPopulateItemSourceInfo(_item->id());
+      sPopulateItemSourceInfo(purchasepopulate.value("poitem_itemsrc_id").toInt());
       if (_metrics->boolean("RevControl"))
       {
         _bomRevision->setId(purchasepopulate.value("poitem_bom_rev_id").toInt());
@@ -487,6 +484,7 @@ void purchaseOrderItem::populate()
     }
 
     _itemsrcid = purchasepopulate.value("poitem_itemsrc_id").toInt();
+    _contrctNumber->setText(purchasepopulate.value("contrct_number").toString());
     _vendorItemNumber->setText(purchasepopulate.value("poitem_vend_item_number").toString());
     _vendorDescrip->setText(purchasepopulate.value("poitem_vend_item_descrip").toString());
     _vendorUOM->setText(purchasepopulate.value("poitem_vend_uom").toString());
@@ -505,10 +503,10 @@ void purchaseOrderItem::populate()
 
     if (_itemsrcid != -1)
     {
-      _vendorUOM->setEnabled(FALSE);
-      _manufName->setEnabled(FALSE);
-      _manufItemNumber->setEnabled(FALSE);
-      _manufItemDescrip->setEnabled(FALSE);
+      _vendorUOM->setEnabled(false);
+      _manufName->setEnabled(false);
+      _manufItemNumber->setEnabled(false);
+      _manufItemDescrip->setEnabled(false);
 
       if(_vendorItemNumber->text().isEmpty())
         _vendorItemNumber->setText(purchasepopulate.value("itemsrc_vend_item_number").toString());
@@ -528,17 +526,36 @@ void purchaseOrderItem::populate()
         _manufItemDescrip->setText(purchasepopulate.value("itemsrc_manuf_item_descrip").toString());
     }
 
-    purchasepopulate.prepare( "SELECT DISTINCT char_id, char_name,"
-               "       COALESCE(b.charass_value, (SELECT c.charass_value FROM charass c WHERE ((c.charass_target_type='I') AND (c.charass_target_id=:item_id) AND (c.charass_default) AND (c.charass_char_id=char_id)) LIMIT 1)) AS charass_value"
-               "  FROM charass a, char "
-               "    LEFT OUTER JOIN charass b"
-               "      ON (b.charass_target_type='PI'"
-               "      AND b.charass_target_id=:poitem_id"
-               "      AND b.charass_char_id=char_id) "
-               " WHERE ( (a.charass_char_id=char_id)"
-               "   AND   (a.charass_target_type='I')"
-               "   AND   (a.charass_target_id=:item_id) ) "
-               " ORDER BY char_name;" );
+    purchasepopulate.prepare( "SELECT char_id, char_name, "
+             "  CASE WHEN char_type < 2 THEN "
+             "    charass_value "
+             "  ELSE "
+             "    formatDate(charass_value::date) "
+             " END AS f_charass_value, "
+             "  charass_value "
+             " FROM ( "
+             " SELECT char_id, char_type, char_name, "
+             "   COALESCE(pi.charass_value,i2.charass_value) AS charass_value "
+             " FROM "
+             "   (SELECT DISTINCT char_id, char_type, char_name "
+             "    FROM charass, char, charuse "
+             "    WHERE ((charass_char_id=char_id) "
+             "    AND (charuse_char_id=char_id AND charuse_target_type='PI') "
+             "    AND (charass_target_type='I') "
+             "    AND (charass_target_id=:item_id) ) "
+             "    UNION SELECT char_id, char_type, char_name "
+             "    FROM charass, char "
+             "    WHERE ((charass_char_id=char_id) "
+             "    AND  (charass_target_type = 'PI' AND charass_target_id=:poitem_id)) ) AS data "
+             "   LEFT OUTER JOIN charass  pi ON ((:poitem_id=pi.charass_target_id) "
+             "                               AND ('PI'=pi.charass_target_type) "
+             "                               AND (pi.charass_char_id=char_id)) "
+             "   LEFT OUTER JOIN item     i1 ON (i1.item_id=:item_id) "
+             "   LEFT OUTER JOIN charass  i2 ON ((i1.item_id=i2.charass_target_id) "
+             "                               AND ('I'=i2.charass_target_type) "
+             "                               AND (i2.charass_char_id=char_id) "
+             "                               AND (i2.charass_default))) data2 "
+             " ORDER BY char_name;"  );
     purchasepopulate.bindValue(":item_id", _item->id());
     purchasepopulate.bindValue(":poitem_id", _poitemid);
     purchasepopulate.exec();
@@ -561,6 +578,80 @@ void purchaseOrderItem::populate()
   }
 }
 
+void purchaseOrderItem::prepare()
+{
+  XSqlQuery prepareq;
+  //  Grab the next poitem_id
+  prepareq.exec("SELECT NEXTVAL('poitem_poitem_id_seq') AS _poitem_id");
+  if (prepareq.first())
+  {
+    _poitemid = prepareq.value("_poitem_id").toInt();
+    _comments->setId(_poitemid);
+  }
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Purchase Order Item Information"),
+                                prepareq, __FILE__, __LINE__))
+  {
+    return;
+  }
+        
+  prepareq.prepare( "SELECT (COALESCE(MAX(poitem_linenumber), 0) + 1) AS _linenumber "
+                    "FROM poitem "
+                    "WHERE (poitem_pohead_id=:pohead_id)" );
+  prepareq.bindValue(":pohead_id", _poheadid);
+  prepareq.exec();
+  if (prepareq.first())
+    _lineNumber->setText(prepareq.value("_linenumber").toString());
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Purchase Order Item Information"),
+                                prepareq, __FILE__, __LINE__))
+  {
+    return;
+  }
+}
+
+void purchaseOrderItem::clear()
+{
+  _poitemid = -1;
+  _item->setId(-1);
+//  _warehouse->setId(-1);
+  _expcat->setId(-1);
+  _taxtype->setId(-1);
+  _taxRecoverable->setChecked(false);
+  _lineNumber->clear();
+  _ordered->clear();
+  _unitPrice->clear();
+  _freight->clear();
+//  _dueDate->clear();
+  _notes->clear();
+  _project->setId(-1);
+  _bomRevision->setId(-1);
+  _booRevision->setId(-1);
+  _maxCost = 0.0;
+  _dropship = false;
+  _parentwo = -1;
+  _parentso = -1;
+  _costmethod = "";
+  _orderQtyCache = -1;
+  _overriddenUnitPrice = false;
+  _itemchar->removeRows(0, _itemchar->rowCount());
+  _itemcharView->setEnabled(true);
+  _itemsrcid = -1;
+  _vendorItemNumber->clear();
+  _vendorDescrip->clear();
+  _vendorUOM->setText(_item->uom());
+  _uom->setText(_item->uom());
+  _minOrderQty->clear();
+  _orderQtyMult->clear();
+  _invVendorUOMRatio->setDouble(1.0);
+  _earliestDate->setDate(omfgThis->dbDate());
+  _manufName->setId(-1);
+  _manufItemNumber->clear();
+  _manufItemDescrip->clear();
+  _invVendUOMRatio = 1;
+  _minimumOrder = 0;
+  _orderMultiple = 0;
+  _save->setEnabled(false);
+}
+
 void purchaseOrderItem::sSave()
 {
   QList<GuiErrorCheck> errors;
@@ -576,6 +667,8 @@ void purchaseOrderItem::sSave()
                           tr("<p>You must select a Tax Type before you may save." ))
          << GuiErrorCheck(_so->text().length() == 0 && _costmethod == "J", _item,
                           tr("<p>You may not purchase a Job Item without an associated demand."))
+         << GuiErrorCheck(!_project->isValid() && _metrics->boolean("RequireProjectAssignment"), _project,
+                          tr("<p>You must enter a Project for this order item before you may save it."))
      ;
 
   if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Purchase Order Item"), errors))
@@ -623,7 +716,7 @@ void purchaseOrderItem::sSave()
     }
   }
 
-  if (_unitPrice->baseValue() > _maxCost && _maxCost > 0.0)
+  if ((_unitPrice->baseValue() / _invVendUOMRatio) > _maxCost && _maxCost > 0.0)
   {
     if (QMessageBox::critical( this, tr("Invalid Unit Price"),
                                tr( "<p>The Unit Price is above the Maximum Desired Cost for this Item."
@@ -750,9 +843,9 @@ void purchaseOrderItem::sSave()
     purchaseSave.bindValue(":poitem_boo_rev_id", _booRevision->id());
   }
   purchaseSave.exec();
-  if (purchaseSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Purchase Order Item Information"),
+                                         purchaseSave, __FILE__, __LINE__))
   {
-    systemError(this, purchaseSave.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -792,7 +885,17 @@ void purchaseOrderItem::sSave()
     }
   }
 
-  done(_poitemid);
+  
+  if (cNew == _mode && !_captive)
+  {
+    clear();
+    prepare();
+    _item->setFocus();
+    if (_orderRefresh->isChecked())
+      omfgThis->sPurchaseOrdersUpdated(_poheadid, true);
+  }
+  else
+    done(_poitemid);
 }
 
 void purchaseOrderItem::sPopulateExtPrice()
@@ -843,17 +946,36 @@ void purchaseOrderItem::sPopulateItemInfo(int pItemid)
 
         sPopulateItemsiteInfo();
 
-        item.prepare( "SELECT DISTINCT char_id, char_name,"
-                   "       COALESCE(b.charass_value, (SELECT c.charass_value FROM charass c WHERE ((c.charass_target_type='I') AND (c.charass_target_id=:item_id) AND (c.charass_default) AND (c.charass_char_id=char_id)) LIMIT 1)) AS charass_value"
-                   "  FROM charass a, char "
-                   "    LEFT OUTER JOIN charass b"
-                   "      ON (b.charass_target_type='PI'"
-                   "      AND b.charass_target_id=:poitem_id"
-                   "      AND b.charass_char_id=char_id) "
-                   " WHERE ( (a.charass_char_id=char_id)"
-                   "   AND   (a.charass_target_type='I')"
-                   "   AND   (a.charass_target_id=:item_id) ) "
-                   " ORDER BY char_name;" );
+        item.prepare( "SELECT char_id, char_name, "
+             "  CASE WHEN char_type < 2 THEN "
+             "    charass_value "
+             "  ELSE "
+             "    formatDate(charass_value::date) "
+             " END AS f_charass_value, "
+             "  charass_value "
+             " FROM ( "
+             " SELECT char_id, char_type, char_name, "
+             "   COALESCE(pi.charass_value,i2.charass_value) AS charass_value "
+             " FROM "
+             "   (SELECT DISTINCT char_id, char_type, char_name "
+             "    FROM charass, char, charuse "
+             "    WHERE ((charass_char_id=char_id) "
+             "    AND (charuse_char_id=char_id AND charuse_target_type='PI') "
+             "    AND (charass_target_type='I') "
+             "    AND (charass_target_id=:item_id) ) "
+             "    UNION SELECT char_id, char_type, char_name "
+             "    FROM charass, char "
+             "    WHERE ((charass_char_id=char_id) "
+             "    AND  (charass_target_type = 'PI' AND charass_target_id=:poitem_id)) ) AS data "
+             "   LEFT OUTER JOIN charass  pi ON ((:poitem_id=pi.charass_target_id) "
+             "                               AND ('PI'=pi.charass_target_type) "
+             "                               AND (pi.charass_char_id=char_id)) "
+             "   LEFT OUTER JOIN item     i1 ON (i1.item_id=:item_id) "
+             "   LEFT OUTER JOIN charass  i2 ON ((i1.item_id=i2.charass_target_id) "
+             "                               AND ('I'=i2.charass_target_type) "
+             "                               AND (i2.charass_char_id=char_id) "
+             "                               AND (i2.charass_default))) data2 "
+             " ORDER BY char_name;"  );
         item.bindValue(":item_id", pItemid);
         item.bindValue(":poitem_id", _poitemid);
         item.exec();
@@ -968,8 +1090,9 @@ void purchaseOrderItem::sPopulateItemSourceInfo(int pItemsrcid)
                  "       (CURRENT_DATE + itemsrc_leadtime) AS earliestdate, "
                  "       itemsrc_manuf_name, "
                  "       itemsrc_manuf_item_number, "
-                 "       itemsrc_manuf_item_descrip "
-                 "FROM itemsrc "
+                 "       itemsrc_manuf_item_descrip, "
+                 "       contrct_number "
+				 "FROM itemsrc LEFT OUTER JOIN contrct ON (itemsrc_contrct_id = contrct_id) "
                  "WHERE (itemsrc_id=:itemsrc_id);" );
       src.bindValue(":itemsrc_id", pItemsrcid);
       src.exec();
@@ -990,6 +1113,7 @@ void purchaseOrderItem::sPopulateItemSourceInfo(int pItemsrcid)
         _invVendUOMRatio = src.value("itemsrc_invvendoruomratio").toDouble();
         _minimumOrder = src.value("itemsrc_minordqty").toDouble();
         _orderMultiple = src.value("itemsrc_multordqty").toDouble();
+        _contrctNumber->setText(src.value("contrct_number").toString());
         
         _manufName->setCode(src.value("itemsrc_manuf_name").toString());
         _manufItemNumber->setText(src.value("itemsrc_manuf_item_number").toString());
@@ -1000,7 +1124,7 @@ void purchaseOrderItem::sPopulateItemSourceInfo(int pItemsrcid)
 
         _ordered->setFocus();
 
-        if(_metrics->boolean("UseEarliestAvailDateOnPOItem"))
+        if(_metrics->boolean("UseEarliestAvailDateOnPOItem") && _dueDate->date() < _earliestDate->date())
           _dueDate->setDate(_earliestDate->date());
 
         skipClear = true;
@@ -1074,7 +1198,7 @@ void purchaseOrderItem::sDeterminePrice()
 void purchaseOrderItem::sInventoryItemToggled( bool yes )
 {
   if(yes)
-    sPopulateItemSourceInfo(_item->id());
+    sPopulateItemSourceInfo(_itemsrcid);
   else
     sPopulateItemSourceInfo(-1);
 }
@@ -1095,12 +1219,12 @@ void purchaseOrderItem::sVendorItemNumberSearch()
     int itemsrcid = newdlg.itemsrcId();
     if(itemsrcid != -1)
     {
-      _inventoryItem->setChecked(TRUE);
+      _inventoryItem->setChecked(true);
       sPopulateItemSourceInfo(itemsrcid);
     }
     else
     {
-      _nonInventoryItem->setChecked(TRUE);
+      _nonInventoryItem->setChecked(true);
       _expcat->setId(newdlg.expcatId());
       _vendorItemNumber->setText(newdlg.vendItemNumber());
       _vendorDescrip->setText(newdlg.vendItemDescrip());
@@ -1122,7 +1246,7 @@ void purchaseOrderItem::sVendorItemNumberList()
   int itemsrcid = newdlg.exec();
   if(itemsrcid > 0)
   {
-    _inventoryItem->setChecked(TRUE);
+    _inventoryItem->setChecked(true);
     sPopulateItemSourceInfo(itemsrcid);
   }
 }
@@ -1140,7 +1264,7 @@ void purchaseOrderItem::sVendorListPrices()
   ParameterList params;
   params.append("itemsrc_id", _itemsrcid);
   params.append("qty", _ordered->toDouble());
-  vendorPriceList newdlg(this, "", TRUE);
+  vendorPriceList newdlg(this, "", true);
   newdlg.set(params);
   if ( (newdlg.exec() == XDialog::Accepted))
   {
@@ -1161,41 +1285,48 @@ void purchaseOrderItem::sVendorListPrices()
 
 void purchaseOrderItem::sCalculateTax()
 {
-  XSqlQuery calcq;
-
-  calcq.prepare("SELECT calculateTax(pohead_taxzone_id,:taxtype_id,pohead_orderdate,pohead_curr_id,ROUND(:ext,2)) AS tax "
+  QString sql("SELECT COALESCE(calculateTax(pohead_taxzone_id,<? value('taxtype_id') ?>,pohead_orderdate,pohead_curr_id,ROUND(<? value('ext') ?>,2)),0.00) + "
+                "       <? if exists('freight') ?> "
+                "       COALESCE(calculateTax(pohead_taxzone_id,getfreighttaxtypeid(),pohead_orderdate,pohead_curr_id,ROUND(COALESCE(<? value('freight') ?>,0.00),2)), 0.00) "
+                "       <? else ?> 0 <? endif ?> AS tax "
                 "FROM pohead "
-                "WHERE (pohead_id=:pohead_id); " );
+                "WHERE (pohead_id=<? value('pohead_id') ?>); " );
 
-  calcq.bindValue(":pohead_id", _poheadid);
-  calcq.bindValue(":taxtype_id", _taxtype->id());
-  calcq.bindValue(":ext", _extendedPrice->localValue());
+  MetaSQLQuery  mql(sql);
+  ParameterList params;
+  params.append("pohead_id", _poheadid);
+  params.append("taxtype_id", _taxtype->id());
+  params.append("ext", _extendedPrice->localValue());
+  if (_freight->localValue() > 0)
+    params.append("freight", _freight->localValue());
 
-  calcq.exec();
+  XSqlQuery calcq = mql.toQuery(params);
   if (calcq.first())
     _tax->setLocalValue(calcq.value("tax").toDouble());
-
-  else if (calcq.lastError().type() != QSqlError::NoError)
-  {
-    systemError(this, calcq.lastError().databaseText(), __FILE__, __LINE__);
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("P/O Tax Calculation"),
+                         calcq, __FILE__, __LINE__))
     return;
-  } 
 }
 
-void purchaseOrderItem::sTaxDetail()    // new function added from raitem
+void purchaseOrderItem::sTaxDetail()
 {
-  taxDetail newdlg(this, "", true);
-  ParameterList params;
-  params.append("taxzone_id",   _taxzoneid);
-  params.append("taxtype_id",  _taxtype->id());
-  params.append("date", _tax->effective());
-  params.append("curr_id", _tax->id());
-  params.append("subtotal", _extendedPrice->localValue());
-  //params.append("readOnly");
+  if (_poitemid < 0)
+    return;
 
-  if (newdlg.set(params) == NoError && newdlg.exec())
+  if (_mode == cNew)
   {
-    if (_taxtype->id() != newdlg.taxtype())
-      _taxtype->setId(newdlg.taxtype());
+    QMessageBox::information( this, tr("Tax Breakdown"),
+                    tr("<p>Please save the Purchase Order Item before viewing the tax breakdown."));
+    return;
   }
+
+  ParameterList params;
+  params.append("order_id", _poitemid);
+  params.append("order_type", "PI");
+  if (_mode == cView)
+    params.append("mode", "view");
+
+  taxBreakdown newdlg(this, "", true);
+  newdlg.set(params);
+  newdlg.exec();
 }
