@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,9 +22,12 @@
 
 #include "mqlutil.h"
 #include "bankAdjustment.h"
+#include "importData.h"
+#include "toggleBankrecCleared.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
-reconcileBankaccount::reconcileBankaccount(QWidget* parent, const char* name, Qt::WFlags fl)
+reconcileBankaccount::reconcileBankaccount(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
     setupUi(this);
@@ -38,19 +41,20 @@ reconcileBankaccount::reconcileBankaccount(QWidget* parent, const char* name, Qt
     connect(_receipts,	SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)), this, SLOT(sReceiptsToggleCleared()));
     connect(_reconcile,	SIGNAL(clicked()),      this, SLOT(sReconcile()));
     connect(_save,	    SIGNAL(clicked()),      this, SLOT(sSave()));
+    connect(_import,	    SIGNAL(clicked()),      this, SLOT(sImport()));
     connect(_update,	SIGNAL(clicked()),      this, SLOT(populate()));
     connect(_startDate, SIGNAL(newDate(QDate)), this, SLOT(sDateChanged()));
     connect(_endDate,   SIGNAL(newDate(QDate)), this, SLOT(sDateChanged()));
 
-    _receipts->addColumn(tr("Cleared"),       _ynColumn * 2, Qt::AlignCenter );
-    _receipts->addColumn(tr("Date"),            _dateColumn, Qt::AlignCenter );
-    _receipts->addColumn(tr("Doc. Type"),     _ynColumn * 2, Qt::AlignCenter );
-    _receipts->addColumn(tr("Doc. Number"),     _itemColumn, Qt::AlignLeft   );
-    _receipts->addColumn(tr("Notes"),                    -1, Qt::AlignLeft   );
-    _receipts->addColumn(tr("Currency"),    _currencyColumn, Qt::AlignCenter );
-    _receipts->addColumn(tr("Exch. Rate"),  _bigMoneyColumn, Qt::AlignRight  );
-    _receipts->addColumn(tr("Base Amount"), _bigMoneyColumn, Qt::AlignRight  );
-    _receipts->addColumn(tr("Amount"),      _bigMoneyColumn, Qt::AlignRight  );
+    _receipts->addColumn(tr("Cleared"),       _ynColumn * 2, Qt::AlignCenter, true, "cleared");
+    _receipts->addColumn(tr("Date"),            _dateColumn, Qt::AlignCenter, true, "transdate");
+    _receipts->addColumn(tr("Doc. Type"),     _ynColumn * 2, Qt::AlignCenter, true, "doc_type");
+    _receipts->addColumn(tr("Doc. Number"),     _itemColumn, Qt::AlignLeft  , true, "doc_number");
+    _receipts->addColumn(tr("Notes"),                    -1, Qt::AlignLeft  , true, "notes");
+    _receipts->addColumn(tr("Currency"),    _currencyColumn, Qt::AlignCenter, true, "doc_curr");
+    _receipts->addColumn(tr("Exch. Rate"),  _bigMoneyColumn, Qt::AlignRight , true, "doc_exchrate");
+    _receipts->addColumn(tr("Base Amount"), _bigMoneyColumn, Qt::AlignRight , true, "base_amount");
+    _receipts->addColumn(tr("Amount"),      _bigMoneyColumn, Qt::AlignRight , true, "amount");
     
     _checks->addColumn(tr("Cleared"),       _ynColumn * 2, Qt::AlignCenter , true, "cleared");
     _checks->addColumn(tr("Date"),            _dateColumn, Qt::AlignCenter , true, "transdate");
@@ -78,9 +82,15 @@ reconcileBankaccount::reconcileBankaccount(QWidget* parent, const char* name, Qt
 			 "ORDER BY bankaccnt_name;");
     _currency->setLabel(_currencyLit);
 
-    if (!_privileges->check("MaintainBankAdjustments"))
-      _addAdjustment->setEnabled(FALSE);
+    _addAdjustment->setEnabled(_privileges->check("MaintainBankAdjustments"));
 
+    if (_metrics->boolean("CashBasedTax"))
+    {
+      _allowEdit->setText(tr("Exchange Rate/Effective Date Edit"));
+    }
+  
+    _import->setVisible(_metrics->boolean("ImportBankReconciliation"));
+  
     connect(omfgThis, SIGNAL(bankAdjustmentsUpdated(int, bool)), this, SLOT(populate()));
     connect(omfgThis, SIGNAL(checksUpdated(int, int, bool)), this, SLOT(populate()));
     connect(omfgThis, SIGNAL(cashReceiptsUpdated(int, bool)), this, SLOT(populate()));
@@ -124,18 +134,19 @@ void reconcileBankaccount::sCancel()
       reconcileCancel.exec();
       if (reconcileCancel.first())
       {
-	int result = reconcileCancel.value("result").toInt();
-	if (result < 0)
-	{
-	  systemError(this, storedProcErrorLookup("deleteBankReconciliation", result),
-		      __FILE__, __LINE__);
-	  return;
-	}
+        int result = reconcileCancel.value("result").toInt();
+        if (result < 0)
+        {
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Bank Reconciliation"),
+                                 storedProcErrorLookup("deleteBankReconciliation", result),
+                                 __FILE__, __LINE__);
+          return;
+        }
       }
-      else if (reconcileCancel.lastError().type() != QSqlError::NoError)
+       else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Cancelling Bank Reconcilaition"),
+                                    reconcileCancel, __FILE__, __LINE__))
       {
-	systemError(this, reconcileCancel.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        return;
       }
     }
   }
@@ -167,9 +178,9 @@ bool reconcileBankaccount::sSave(bool closeWhenDone)
               "(:bankrecid, :bankaccntid,"
               " :startDate, :endDate,"
               " :openbal, :endbal); ");
-  else if (reconcileSave.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Reconciliation Information"),
+                                reconcileSave, __FILE__, __LINE__))
   {
-    systemError(this, reconcileSave.lastError().databaseText(), __FILE__, __LINE__);
     return false;
   }
 
@@ -180,11 +191,9 @@ bool reconcileBankaccount::sSave(bool closeWhenDone)
   reconcileSave.bindValue(":openbal", _openBal->localValue());
   reconcileSave.bindValue(":endbal", _endBal->localValue());
   reconcileSave.exec();
-  if (reconcileSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Reconciliation Information"),
+                                reconcileSave, __FILE__, __LINE__))
   {
-    systemError(this, tr("<p>There was an error creating records to reconcile "
-			 "this account: <br><pre>%1</pre>")
-			.arg(reconcileSave.lastError().databaseText()), __FILE__, __LINE__);
     return false;
   }
 
@@ -256,7 +265,8 @@ void reconcileBankaccount::sReconcile()
   XSqlQuery bal = mbal.toQuery(params);
   if(!bal.first())
   {
-    systemError(this, bal.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                         bal, __FILE__, __LINE__);
     return;
   }
 
@@ -280,16 +290,17 @@ void reconcileBankaccount::sReconcile()
     int result = reconcileReconcile.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("postBankReconciliation", result),
-		  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                             storedProcErrorLookup("postBankReconciliation",result),
+                             __FILE__, __LINE__);
       return;
     }
     _bankrecid = -1;
     close();
   }
-  else if (reconcileReconcile.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                reconcileReconcile, __FILE__, __LINE__))
   {
-    systemError(this, reconcileReconcile.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -318,8 +329,9 @@ void reconcileBankaccount::populate()
   MetaSQLQuery mrcp = mqlLoad("bankrec", "receipts");
   XSqlQuery rcp = mrcp.toQuery(params);
   if (rcp.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                rcp, __FILE__, __LINE__))
   {
-    systemError(this, rcp.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -327,9 +339,11 @@ void reconcileBankaccount::populate()
   XTreeWidgetItem * parent = 0;
   XTreeWidgetItem * lastChild = 0;
   XTreeWidgetItem * last = 0;
-  bool cleared = TRUE;
+  bool cleared = true;
   double amount = 0.0;
   bool amountNull = true;
+  const QString exchangePrecision = "6";
+
   while (rcp.next())
   {
     if(rcp.value("use").toString() == "C/R")
@@ -339,11 +353,14 @@ void reconcileBankaccount::populate()
         if(parent != 0)
         {
           parent->setText(0, (cleared ? tr("Yes") : tr("No")));
-          parent->setText(8, amountNull ? tr("?????") : formatMoney(amount));
+          parent->setNumber(8, amountNull ? tr("?????") : QVariant(amount), "curr");
         }
         jrnlnum = rcp.value("jrnlnum").toInt();
-        last = new XTreeWidgetItem( _receipts, last,
-          jrnlnum, 9, "", formatDate(rcp.value("f_jrnldate").toDate()), tr("JS"), rcp.value("jrnlnum"));
+        last = new XTreeWidgetItem( _receipts, last, jrnlnum, 9);
+        last->setText(0, "");
+        last->setDate(1, rcp.value("f_jrnldate").toDate());
+        last->setText(2, tr("JS"));
+        last->setText(3, rcp.value("jrnlnum"));
         parent = last;
         cleared = true;
         amount = 0.0;
@@ -354,43 +371,48 @@ void reconcileBankaccount::populate()
       amount += rcp.value("amount").toDouble();
       amountNull = rcp.value("amount").isNull();
       
-      lastChild = new XTreeWidgetItem( parent, lastChild,
-        rcp.value("id").toInt(), rcp.value("altid").toInt(),
-        (rcp.value("cleared").toBool() ? tr("Yes") : tr("No")),
-        formatDate(rcp.value("f_date").toDate()), rcp.value("doc_type"), rcp.value("docnumber"),
-        rcp.value("notes"),
-        rcp.value("doc_curr"),
-        rcp.value("doc_exchrate").isNull() ? tr("?????") : formatUOMRatio(rcp.value("doc_exchrate").toDouble()),
-        rcp.value("base_amount").isNull() ? tr("?????") : formatMoney(rcp.value("base_amount").toDouble()),
-        rcp.value("amount").isNull() ? tr("?????") : formatMoney(rcp.value("amount").toDouble()) );
+      lastChild = new XTreeWidgetItem(parent, lastChild, rcp.value("id").toInt(), rcp.value("altid").toInt());
+
+      lastChild->setText(0, rcp.value("cleared").toBool() ? tr("Yes") : tr("No"));
+      lastChild->setDate(1, rcp.value("f_date").toDate());
+      lastChild->setText(2, rcp.value("doc_type"));
+      lastChild->setText(3, rcp.value("docnumber"));
+      lastChild->setText(4, rcp.value("notes"));
+      lastChild->setText(5, rcp.value("doc_curr"));
+      lastChild->setNumber(6, rcp.value("doc_exchrate").isNull() ? tr("?????") : rcp.value("doc_exchrate"), exchangePrecision);
+      lastChild->setNumber(7, rcp.value("base_amount").isNull() ? tr("?????") : rcp.value("base_amount"), "curr");
+      lastChild->setNumber(8, rcp.value("amount").isNull() ? tr("?????") : rcp.value("amount"), "curr");
     }
     else
     {
       if(parent != 0)
       {
         parent->setText(0, (cleared ? tr("Yes") : tr("No")));
-        parent->setText(8, formatMoney(amount));
+        parent->setNumber(8, QVariant(amount), "curr");
       }
       parent = 0;
       cleared = true;
       amount = 0.0;
       amountNull = true;
       lastChild = 0;
-      last = new XTreeWidgetItem( _receipts, last,
-        rcp.value("id").toInt(), rcp.value("altid").toInt(),
-        (rcp.value("cleared").toBool() ? tr("Yes") : tr("No")),
-        formatDate(rcp.value("f_date").toDate()), rcp.value("doc_type"), rcp.value("docnumber"),
-        rcp.value("notes"),
-        rcp.value("doc_curr"),
-        rcp.value("doc_exchrate").isNull() ? tr("?????") : formatUOMRatio(rcp.value("doc_exchrate").toDouble()),
-        rcp.value("base_amount").isNull() ? tr("?????") : formatMoney(rcp.value("base_amount").toDouble()),
-        rcp.value("amount").isNull() ? tr("?????") : formatMoney(rcp.value("amount").toDouble()) );
+
+      last = new XTreeWidgetItem(_receipts, last, rcp.value("id").toInt(), rcp.value("altid").toInt());
+
+      last->setText(0, rcp.value("cleared").toBool() ? tr("Yes") : tr("No"));
+      last->setDate(1, rcp.value("f_date").toDate());
+      last->setText(2, rcp.value("doc_type"));
+      last->setText(3, rcp.value("docnumber"));
+      last->setText(4, rcp.value("notes"));
+      last->setText(5, rcp.value("doc_curr"));
+      last->setNumber(6, rcp.value("doc_exchrate").isNull() ? tr("?????") : rcp.value("doc_exchrate"), exchangePrecision);
+      last->setNumber(7, rcp.value("base_amount").isNull() ? tr("?????") : rcp.value("base_amount"), "curr");
+      last->setNumber(8, rcp.value("amount").isNull() ? tr("?????") : rcp.value("amount"), "curr");
     }
   }
   if(parent != 0)
   {
     parent->setText(0, (cleared ? tr("Yes") : tr("No")));
-    parent->setText(8, amountNull ? tr("?????") : formatMoney(amount));
+    parent->setNumber(8, amountNull ? tr("?????") : QVariant(amount), "curr");
   }
 
   if(currid != -1)
@@ -403,12 +425,12 @@ void reconcileBankaccount::populate()
   _checks->clear();
   MetaSQLQuery mchk = mqlLoad("bankrec", "checks");
   XSqlQuery chk = mchk.toQuery(params);
-  if (chk.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                chk, __FILE__, __LINE__))
   {
-    systemError(this, chk.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
-  _checks->populate(chk, TRUE);
+  _checks->populate(chk, true);
 
   if(currid != -1)
     _checks->setCurrentItem(_checks->topLevelItem(currid));
@@ -421,9 +443,9 @@ void reconcileBankaccount::populate()
   rcp = mrcp.toQuery(params);
   if (rcp.first())
     _clearedReceipts->setDouble(rcp.value("cleared_amount").toDouble());
-  else if (rcp.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                rcp, __FILE__, __LINE__))
   {
-    systemError(this, rcp.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -431,9 +453,9 @@ void reconcileBankaccount::populate()
   chk = mchk.toQuery(params);
   if (chk.first())
     _clearedChecks->setDouble(chk.value("cleared_amount").toDouble());
-  else if (chk.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                chk, __FILE__, __LINE__))
   {
-    systemError(this, chk.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -445,7 +467,7 @@ void reconcileBankaccount::populate()
   params.append("effective", _startDate->date());
   params.append("expires",   _endDate->date());
   XSqlQuery bal = mbal.toQuery(params);
-  bool enableRec = FALSE;
+
   if(bal.first())
   {
     _clearBal->setDouble(bal.value("cleared_amount").toDouble());
@@ -454,26 +476,39 @@ void reconcileBankaccount::populate()
 
     QString stylesheet;
 
-    if(bal.value("diff_value").toDouble() == 0.0)
-    {
-      if(_startDate->isValid() && _endDate->isValid())
-        enableRec = TRUE;
-    }
-    else
+    if(bal.value("diff_value").toDouble() != 0.0)
       stylesheet = QString("* { color: %1; }").arg(namedColor("error").name());
 
     _diffBal->setStyleSheet(stylesheet);
   }
-  else if (bal.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                bal, __FILE__, __LINE__))
   {
-    systemError(this, bal.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
-  //_reconcile->setEnabled(enableRec);
 
   qApp->restoreOverrideCursor();
 }
 
+void reconcileBankaccount::sImport()
+{
+  XSqlQuery reconcileImport;
+
+  // set the metric
+  reconcileImport.prepare("SELECT setMetric('ImportBankRecId', :bankrecid::TEXT) AS result; ");
+  reconcileImport.bindValue(":bankrecid", _bankrecid);
+  reconcileImport.exec();
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                reconcileImport, __FILE__, __LINE__))
+  {
+    return;
+  }
+
+  // import the reconciliation file
+  importData *newdlg = new importData();
+  omfgThis->handleNewWindow(newdlg, Qt::ApplicationModal);
+}
+
 void reconcileBankaccount::sAddAdjustment()
 {
   ParameterList params;
@@ -482,9 +517,8 @@ void reconcileBankaccount::sAddAdjustment()
   params.append("bankaccnt_id", _bankaccnt->id());
 
   bankAdjustment *newdlg = new bankAdjustment();
-  newdlg->setWindowModality(Qt::WindowModal);
   newdlg->set(params);
-  omfgThis->handleNewWindow(newdlg);
+  omfgThis->handleNewWindow(newdlg, Qt::ApplicationModal);
 }
 
 void reconcileBankaccount::sReceiptsToggleCleared()
@@ -508,94 +542,113 @@ void reconcileBankaccount::sReceiptsToggleCleared()
       if(child->text(0) != (setto ? tr("Yes") : tr("No")))
       {
         double rate = QLocale().toDouble(child->text(6));
-        double baseamount = QLocale().toDouble(child->text(7));
-        double amount = QLocale().toDouble(child->text(8));
+        double amount = QLocale().toDouble(child->text(7));
 
-        if (_rateEdit->isChecked() && child->text(0) != tr("Yes"))
+        if (_allowEdit->isChecked() && child->text(0) != tr("Yes"))
         {
-          bool ok;
-          rate = QInputDialog::getDouble(this, tr("Currency Exchange Rate"),
-                                       tr("New Rate:"),
-                                       rate, 0, 100, 5, &ok);
-          if ( !ok )
+          ParameterList params;
+          params.append("transtype", "receipt");
+          params.append("bankaccntid", _bankaccnt->id());
+          params.append("bankrecid", _bankrecid);
+          params.append("sourceid", child->id());
+          if(child->altId()==1)
+            params.append("source", "GL");
+          else if(child->altId()==2)
+            params.append("source", "SL");
+          else if(child->altId()==3)
+            params.append("source", "AD");
+          toggleBankrecCleared newdlg(this, "", true);
+          newdlg.set(params);
+          newdlg.exec();
+        }
+        else
+        {
+          reconcileReceiptsToggleCleared.prepare("SELECT toggleBankrecCleared(:bankrecid, :source, :sourceid, :currrate, :baseamount) AS cleared");
+          reconcileReceiptsToggleCleared.bindValue(":bankrecid", _bankrecid);
+          reconcileReceiptsToggleCleared.bindValue(":sourceid", child->id());
+          if(child->altId()==1)
+            reconcileReceiptsToggleCleared.bindValue(":source", "GL");
+          else if(child->altId()==2)
+            reconcileReceiptsToggleCleared.bindValue(":source", "SL");
+          else if(child->altId()==3)
+            reconcileReceiptsToggleCleared.bindValue(":source", "AD");
+          reconcileReceiptsToggleCleared.bindValue(":currrate", rate);
+          reconcileReceiptsToggleCleared.bindValue(":baseamount", amount);
+          reconcileReceiptsToggleCleared.exec();
+          if(reconcileReceiptsToggleCleared.first())
+            child->setText(0, (reconcileReceiptsToggleCleared.value("cleared").toBool() ? tr("Yes") : tr("No") ));
+          else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                        reconcileReceiptsToggleCleared, __FILE__, __LINE__))
+          {
             return;
-          amount = rate * baseamount;
+          }
         }
-
-        reconcileReceiptsToggleCleared.prepare("SELECT toggleBankrecCleared(:bankrecid, :source, :sourceid, :currrate, :amount) AS cleared");
-        reconcileReceiptsToggleCleared.bindValue(":bankrecid", _bankrecid);
-        reconcileReceiptsToggleCleared.bindValue(":sourceid", child->id());
-        if(child->altId()==1)
-          reconcileReceiptsToggleCleared.bindValue(":source", "GL");
-        else if(child->altId()==2)
-          reconcileReceiptsToggleCleared.bindValue(":source", "SL");
-        else if(child->altId()==3)
-          reconcileReceiptsToggleCleared.bindValue(":source", "AD");
-        reconcileReceiptsToggleCleared.bindValue(":currrate", rate);
-        reconcileReceiptsToggleCleared.bindValue(":amount", amount);
-        reconcileReceiptsToggleCleared.exec();
-        if(reconcileReceiptsToggleCleared.first())
-          child->setText(0, (reconcileReceiptsToggleCleared.value("cleared").toBool() ? tr("Yes") : tr("No") ));
-	else if (reconcileReceiptsToggleCleared.lastError().type() != QSqlError::NoError)
-	{
-	  systemError(this, reconcileReceiptsToggleCleared.lastError().databaseText(), __FILE__, __LINE__);
-	  return;
-	}
       }
     }
     item->setText(0, (setto ? tr("Yes") : tr("No")));
+    populate();
   }
   else
   {
     double rate = QLocale().toDouble(item->text(6));
-    double baseamount = QLocale().toDouble(item->text(7));
-    double amount = QLocale().toDouble(item->text(8));
-
-    if (_rateEdit->isChecked() && item->text(0) != tr("Yes"))
+    double amount = QLocale().toDouble(item->text(7));
+    
+    if (_allowEdit->isChecked() && item->text(0) != tr("Yes"))
     {
-      bool ok;
-      rate = QInputDialog::getDouble(this, tr("Currency Exchange Rate"),
-                                   tr("New Rate:"),
-                                   rate, 0, 100, 5, &ok);
-      if ( !ok )
-        return;
-      amount = rate * baseamount;
-    }
-
-    reconcileReceiptsToggleCleared.prepare("SELECT toggleBankrecCleared(:bankrecid, :source, :sourceid, :currrate, :amount) AS cleared");
-    reconcileReceiptsToggleCleared.bindValue(":bankrecid", _bankrecid);
-    reconcileReceiptsToggleCleared.bindValue(":sourceid", item->id());
-    if(item->altId()==1)
-      reconcileReceiptsToggleCleared.bindValue(":source", "GL");
-    else if(item->altId()==2)
-      reconcileReceiptsToggleCleared.bindValue(":source", "SL");
-    else if(item->altId()==3)
-      reconcileReceiptsToggleCleared.bindValue(":source", "AD");
-    reconcileReceiptsToggleCleared.bindValue(":currrate", rate);
-    reconcileReceiptsToggleCleared.bindValue(":amount", amount);
-    reconcileReceiptsToggleCleared.exec();
-    if(reconcileReceiptsToggleCleared.first())
-    {
-      item->setText(0, (reconcileReceiptsToggleCleared.value("cleared").toBool() ? tr("Yes") : tr("No") ));
-
-      item = (XTreeWidgetItem*)item->QTreeWidgetItem::parent();
-      if(item != 0 && item->altId() == 9)
-      {
-        setto = true;
-	for (int i = 0; i < item->childCount(); i++)
-        {
-          setto = (setto && (item->child(i)->text(0) == tr("Yes")));
-        }
-        item->setText(0, (setto ? tr("Yes") : tr("No")));
-      }
+      ParameterList params;
+      params.append("transtype", "receipt");
+      params.append("bankaccntid", _bankaccnt->id());
+      params.append("bankrecid", _bankrecid);
+      params.append("sourceid", item->id());
+      if(item->altId()==1)
+        params.append("source", "GL");
+      else if(item->altId()==2)
+        params.append("source", "SL");
+      else if(item->altId()==3)
+        params.append("source", "AD");
+      toggleBankrecCleared newdlg(this, "", true);
+      newdlg.set(params);
+      newdlg.exec();
+      populate();
     }
     else
     {
-      populate();
-      if (reconcileReceiptsToggleCleared.lastError().type() != QSqlError::NoError)
+      reconcileReceiptsToggleCleared.prepare("SELECT toggleBankrecCleared(:bankrecid, :source, :sourceid, :currrate, :baseamount) AS cleared");
+      reconcileReceiptsToggleCleared.bindValue(":bankrecid", _bankrecid);
+      reconcileReceiptsToggleCleared.bindValue(":sourceid", item->id());
+      if(item->altId()==1)
+        reconcileReceiptsToggleCleared.bindValue(":source", "GL");
+      else if(item->altId()==2)
+        reconcileReceiptsToggleCleared.bindValue(":source", "SL");
+      else if(item->altId()==3)
+        reconcileReceiptsToggleCleared.bindValue(":source", "AD");
+      reconcileReceiptsToggleCleared.bindValue(":currrate", rate);
+      reconcileReceiptsToggleCleared.bindValue(":baseamount", amount);
+      reconcileReceiptsToggleCleared.exec();
+      if(reconcileReceiptsToggleCleared.first())
       {
-	systemError(this, reconcileReceiptsToggleCleared.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        item->setText(0, (reconcileReceiptsToggleCleared.value("cleared").toBool() ? tr("Yes") : tr("No") ));
+
+        item = (XTreeWidgetItem*)item->QTreeWidgetItem::parent();
+        if(item != 0 && item->altId() == 9)
+        {
+          setto = true;
+          for (int i = 0; i < item->childCount(); i++)
+          {
+            setto = (setto && (item->child(i)->text(0) == tr("Yes")));
+          }
+          item->setText(0, (setto ? tr("Yes") : tr("No")));
+          populate();
+        }
+      }
+      else
+      {
+        populate();
+        if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                      reconcileReceiptsToggleCleared, __FILE__, __LINE__))
+        {
+          return;
+        }
       }
     }
   }
@@ -612,41 +665,53 @@ void reconcileBankaccount::sChecksToggleCleared()
   _checks->scrollToItem(item);
 
   double rate = item->rawValue("doc_exchrate").toDouble();
-  double baseamount = item->rawValue("base_amount").toDouble();
-  double amount = item->rawValue("amount").toDouble();
-
-  if (_rateEdit->isChecked() && item->text(0) != tr("Yes"))
+  double amount = item->rawValue("base_amount").toDouble();
+  
+  if (_allowEdit->isChecked() && item->text(0) != tr("Yes"))
   {
-    bool ok;
-    rate = QInputDialog::getDouble(this, tr("Currency Exchange Rate"),
-                                 tr("New Rate:"),
-                                 rate, 0, 100, 5, &ok);
-    if ( !ok )
-      return;
-    amount = rate * baseamount;
+    ParameterList params;
+    params.append("transtype", "check");
+    params.append("bankaccntid", _bankaccnt->id());
+    params.append("bankrecid", _bankrecid);
+    params.append("sourceid", item->id());
+    if(item->altId()==1)
+      params.append("source", "GL");
+    else if(item->altId()==2)
+      params.append("source", "SL");
+    else if(item->altId()==3)
+      params.append("source", "AD");
+    toggleBankrecCleared newdlg(this, "", true);
+    newdlg.set(params);
+    newdlg.exec();
+    populate();
   }
-
-  reconcileChecksToggleCleared.prepare("SELECT toggleBankrecCleared(:bankrecid, :source, :sourceid, :currrate, :amount) AS cleared");
-  reconcileChecksToggleCleared.bindValue(":bankrecid", _bankrecid);
-  reconcileChecksToggleCleared.bindValue(":sourceid", item->id());
-  if(item->altId()==1)
-    reconcileChecksToggleCleared.bindValue(":source", "GL");
-  else if(item->altId()==2)
-    reconcileChecksToggleCleared.bindValue(":source", "SL");
-  else if(item->altId()==3)
-    reconcileChecksToggleCleared.bindValue(":source", "AD");
-  reconcileChecksToggleCleared.bindValue(":currrate", rate);
-  reconcileChecksToggleCleared.bindValue(":amount", amount);
-  reconcileChecksToggleCleared.exec();
-  if(reconcileChecksToggleCleared.first())
-    item->setText(0, (reconcileChecksToggleCleared.value("cleared").toBool() ? tr("Yes") : tr("No") ));
   else
   {
-    populate();
-    if (reconcileChecksToggleCleared.lastError().type() != QSqlError::NoError)
+    reconcileChecksToggleCleared.prepare("SELECT toggleBankrecCleared(:bankrecid, :source, :sourceid, :currrate, :baseamount) AS cleared");
+    reconcileChecksToggleCleared.bindValue(":bankrecid", _bankrecid);
+    reconcileChecksToggleCleared.bindValue(":sourceid", item->id());
+    if(item->altId()==1)
+      reconcileChecksToggleCleared.bindValue(":source", "GL");
+    else if(item->altId()==2)
+      reconcileChecksToggleCleared.bindValue(":source", "SL");
+    else if(item->altId()==3)
+      reconcileChecksToggleCleared.bindValue(":source", "AD");
+    reconcileChecksToggleCleared.bindValue(":currrate", rate);
+    reconcileChecksToggleCleared.bindValue(":baseamount", amount);
+    reconcileChecksToggleCleared.exec();
+    if(reconcileChecksToggleCleared.first())
     {
-      systemError(this, reconcileChecksToggleCleared.lastError().databaseText(), __FILE__, __LINE__);
-      return;
+      item->setText(0, (reconcileChecksToggleCleared.value("cleared").toBool() ? tr("Yes") : tr("No") ));
+      populate();
+    }
+    else
+    {
+      populate();
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                    reconcileChecksToggleCleared, __FILE__, __LINE__))
+      {
+        return;
+      }
     }
   }
 }
@@ -679,16 +744,17 @@ void reconcileBankaccount::sBankaccntChanged()
         {
 	      int result = reconcileBankaccntChanged.value("result").toInt();
 	      if (result < 0)
-	      {
-	        systemError(this, storedProcErrorLookup("deleteBankReconciliation", result),
-		                __FILE__, __LINE__);
-	        return;
-	      }
+          {
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                   storedProcErrorLookup("deleteBankReconciliation", result),
+                                   __FILE__, __LINE__);
+            return;
+          }
         }
-        else if (reconcileBankaccntChanged.lastError().type() != QSqlError::NoError)
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                      reconcileBankaccntChanged, __FILE__, __LINE__))
         {
-	      systemError(this, reconcileBankaccntChanged.lastError().databaseText(), __FILE__, __LINE__);
-	      return;
+          return;
         }
       }
 	}
@@ -702,9 +768,9 @@ void reconcileBankaccount::sBankaccntChanged()
   accntq.exec();
   if (accntq.first())
     _currency->setId(accntq.value("bankaccnt_curr_id").toInt());
-  else if (accntq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                accntq, __FILE__, __LINE__))
   {
-    systemError(this, accntq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -721,9 +787,9 @@ void reconcileBankaccount::sBankaccntChanged()
     _openBal->setLocalValue(accntq.value("bankrec_openbal").toDouble());
     _endBal->setLocalValue(accntq.value("bankrec_endbal").toDouble());
   }
-  else if (accntq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                accntq, __FILE__, __LINE__))
   {
-    systemError(this, accntq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -732,9 +798,9 @@ void reconcileBankaccount::sBankaccntChanged()
     accntq.exec();
     if (accntq.first())
       _bankrecid = accntq.value("bankrec_id").toInt();
-    else if (accntq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconcilation Information"),
+                                  accntq, __FILE__, __LINE__))
     {
-      systemError(this, accntq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     
@@ -756,9 +822,9 @@ void reconcileBankaccount::sBankaccntChanged()
       _startDate->clear();
       _openBal->clear();
     }
-    if (accntq.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                  accntq, __FILE__, __LINE__))
     {
-      systemError(this, accntq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -769,7 +835,7 @@ void reconcileBankaccount::sBankaccntChanged()
 void reconcileBankaccount::sDateChanged()
 {
   XSqlQuery reconcileDateChanged;
-  reconcileDateChanged.prepare("SELECT TRUE AS reconciled "
+  reconcileDateChanged.prepare("SELECT true AS reconciled "
             "FROM bankrec "
             "WHERE ((bankrec_bankaccnt_id = :bankaccnt_id) "
             "AND (bankrec_posted) "
@@ -791,14 +857,50 @@ void reconcileBankaccount::sDateChanged()
     _datesAreOK = false;
     return;
   }
-  else if(reconcileDateChanged.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Bank Reconciliation Information"),
+                                reconcileDateChanged, __FILE__, __LINE__))
   {
-    systemError(this, reconcileDateChanged.lastError().databaseText(), __FILE__, __LINE__);
     _datesAreOK = false;
-	return;
+    return;
   }
   else
   {
-	_datesAreOK = true;
+    if (! sSave(false))
+      return;
+    
+    _datesAreOK = true;
   }
 }
+
+void reconcileBankaccount::openReconcileBankaccount(int pBankaccntid, QWidget *parent)
+{
+  // Check for a window.
+  if (pBankaccntid == -1)
+  {
+    QWidgetList list = omfgThis->windowList();
+    for (int i = 0; i < list.size(); i++)
+    {
+      QWidget *w = list.at(i);
+      if (QString::compare(w->objectName(), "reconcileBankaccount")==0)
+      {
+        w->setFocus();
+        if (omfgThis->showTopLevel())
+        {
+          w->raise();
+          w->activateWindow();
+        }
+        return;
+      }
+    }
+  }
+  
+  // If none found then create one.
+  ParameterList params;
+  if (pBankaccntid != -1)
+    params.append("bankaccnt_id", pBankaccntid);
+  
+  reconcileBankaccount *newdlg = new reconcileBankaccount(parent);
+  newdlg->set(params);
+  omfgThis->handleNewWindow(newdlg);
+}
+
