@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,7 +21,7 @@ class reschedulePoitem : public XDialog, public Ui::reschedulePoitem
     Q_OBJECT
 
 public:
-    reschedulePoitem(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    reschedulePoitem(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~reschedulePoitem();
 
 public slots:
@@ -29,6 +29,7 @@ public slots:
     virtual void sPopulatePoitem(int pPoheadid);
     virtual void sPopulate(int pPoheadid);
     virtual void sReschedule();
+    virtual void sAllItems();
 
 protected slots:
     virtual void languageChange();
