@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,7 +22,7 @@ class returnAuthItemLotSerial : public XDialog, public Ui::returnAuthItemLotSeri
     Q_OBJECT
 
 public:
-    returnAuthItemLotSerial(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    returnAuthItemLotSerial(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~returnAuthItemLotSerial();
 
 public slots:
@@ -35,7 +35,7 @@ public slots:
 
 protected slots:
     void languageChange();
-    void closeEvent();
+    void closeEvent(QCloseEvent *);
 
 private:
     int _crmacctid;
