@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,6 +21,7 @@
 #include "enterPoitemReceipt.h"
 #include "enterPoReceipt.h"
 #include "errorReporter.h"
+#include "guiErrorCheck.h"
 #include "mqlutil.h"
 #include "returnAuthorizationItem.h"
 #include "storedProcErrorLookup.h"
@@ -33,7 +34,7 @@
 #include "dspShipmentsBySalesOrder.h"
 #include "dspSalesOrderStatus.h"
 
-returnAuthorization::returnAuthorization(QWidget* parent, const char* name, Qt::WFlags fl)
+returnAuthorization::returnAuthorization(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
@@ -88,10 +89,10 @@ returnAuthorization::returnAuthorization(QWidget* parent, const char* name, Qt::
   _ignoreShiptoSignals = false;
   _ignoreSoSignals = false;
   _ignoreWhsSignals = false;
-  _ffBillto = TRUE;
-  _ffShipto = TRUE;
-  _custEmail = FALSE;
-  _saved = FALSE;
+  _ffBillto = true;
+  _ffShipto = true;
+  _custEmail = false;
+  _saved = false;
   _calcfreight = false;
   _freightCache = 0;
 
@@ -190,10 +191,11 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
       {
         _raheadid = returnet.value("rahead_id").toInt();
         _comments->setId(_raheadid);
+        _documents->setId(_raheadid);
       }
-      else if (returnet.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                    returnet, __FILE__, __LINE__))
       {
-        systemError(this, returnet.lastError().databaseText(), __FILE__, __LINE__);
         return UndefinedError;
       }
 
@@ -210,9 +212,9 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
       if (!_authNumber->text().isEmpty())
         returnet.bindValue(":rahead_number", _authNumber->text());
       returnet.exec();
-      if (returnet.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                    returnet, __FILE__, __LINE__))
       {
-        systemError(this, returnet.lastError().databaseText(), __FILE__, __LINE__);
         return UndefinedError;
       }
 
@@ -257,7 +259,7 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
     {
       _mode = cEdit;
 
-      _authNumber->setEnabled(FALSE);
+      _authNumber->setEnabled(false);
       _cancel->setText("&Close");
 
       connect(_authNumber, SIGNAL(editingFinished()), this, SLOT(sCheckAuthorizationNumber()));
@@ -286,48 +288,50 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
   {
     _raheadid = param.toInt();
     _comments->setId(_raheadid);
+    _documents->setId(_raheadid);
     populate();
   }
 
   if (_mode == cView)
   {
-      _authNumber->setEnabled(FALSE);
-      _authDate->setEnabled(FALSE);
-      _expireDate->setEnabled(FALSE);
-      _salesRep->setEnabled(FALSE);
-      _commission->setEnabled(FALSE);
-      _taxzone->setEnabled(FALSE);
-      _rsnCode->setEnabled(FALSE);
-      _disposition->setEnabled(FALSE);
-      _timing->setEnabled(FALSE);
-      _creditBy->setEnabled(FALSE);
+      _authNumber->setEnabled(false);
+      _authDate->setEnabled(false);
+      _expireDate->setEnabled(false);
+      _salesRep->setEnabled(false);
+      _commission->setEnabled(false);
+      _taxzone->setEnabled(false);
+      _rsnCode->setEnabled(false);
+      _disposition->setEnabled(false);
+      _timing->setEnabled(false);
+      _creditBy->setEnabled(false);
 
-      _origso->setEnabled(FALSE);
-      _newso->setEnabled(FALSE);
-      _incident->setEnabled(FALSE);
-      _project->setEnabled(FALSE);
+      _origso->setEnabled(false);
+      _newso->setEnabled(false);
+      _incident->setEnabled(false);
+      _project->setEnabled(false);
 
-      _cust->setReadOnly(TRUE);
-      _billToName->setEnabled(FALSE);
-      _billToAddr->setEnabled(FALSE);
+      _cust->setReadOnly(true);
+      _billToName->setEnabled(false);
+      _billToAddr->setEnabled(false);
 
-      _customerPO->setEnabled(FALSE);
+      _customerPO->setEnabled(false);
 
-      _miscCharge->setEnabled(FALSE);
-      _miscChargeDescription->setEnabled(FALSE);
-      _miscChargeAccount->setReadOnly(TRUE);
-      _freight->setEnabled(FALSE);
-      _notes->setEnabled(FALSE);
+      _miscCharge->setEnabled(false);
+      _miscChargeDescription->setEnabled(false);
+      _miscChargeAccount->setReadOnly(true);
+      _freight->setEnabled(false);
+      _notes->setEnabled(false);
       _comments->setReadOnly(true);
-      _copyToShipto->setEnabled(FALSE);
-      _shipTo->setEnabled(FALSE);
-      _shipToName->setEnabled(FALSE);
-      _shipToAddr->setEnabled(FALSE);
-      _currency->setEnabled(FALSE);
-      _warehouse->setEnabled(FALSE);
-      _shipWhs->setEnabled(FALSE);
-      _shippingZone->setEnabled(FALSE);
-      _saleType->setEnabled(FALSE);
+      _documents->setReadOnly(true);
+      _copyToShipto->setEnabled(false);
+      _shipTo->setEnabled(false);
+      _shipToName->setEnabled(false);
+      _shipToAddr->setEnabled(false);
+      _currency->setEnabled(false);
+      _warehouse->setEnabled(false);
+      _shipWhs->setEnabled(false);
+      _shippingZone->setEnabled(false);
+      _saleType->setEnabled(false);
       _save->hide();
       _new->hide();
       _delete->hide();
@@ -346,6 +350,16 @@ enum SetResponse returnAuthorization::set(const ParameterList &pParams)
   return NoError;
 }
 
+int returnAuthorization::id() const
+{
+  return _raheadid;
+}
+
+int returnAuthorization::mode() const
+{
+  return _mode;
+}
+
 void returnAuthorization::setNumber()
 {
   XSqlQuery returnetNumber;
@@ -359,11 +373,11 @@ void returnAuthorization::setNumber()
       _authNumber->setText(returnetNumber.value("ranumber").toString());
 
       if (_metrics->value("RANumberGeneration") == "A")
-        _authNumber->setEnabled(FALSE);
+        _authNumber->setEnabled(false);
     }
-    else if (returnetNumber.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  returnetNumber, __FILE__, __LINE__))
     {
-      systemError(this, returnetNumber.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -382,98 +396,46 @@ bool returnAuthorization::sSave(bool partial)
   QString timing = QString(timingTypes[_timing->currentIndex()]);
   QString creditBy = QString(creditMethods[_creditBy->currentIndex()]);
 
-  if ( !partial && (disposition.isEmpty()) )
-  {
-    QMessageBox::warning( this, tr("Invalid Disposition"),
-                         tr("<p>You must enter a Disposition." ) );
-    _disposition->setFocus();
+  QList<GuiErrorCheck> errors;
+  errors
+  << GuiErrorCheck((!partial && disposition.isEmpty()), _disposition,
+                   tr("<p>You must enter a Disposition."))
+  << GuiErrorCheck((!partial && timing.isEmpty()), _timing,
+                   tr("<p>You must enter a Timing."))
+  << GuiErrorCheck((!partial && creditBy.isEmpty()), _creditBy,
+                   tr("<p>You must enter a Credit Method."))
+  << GuiErrorCheck(_authNumber->text().isEmpty(), _authNumber,
+                   tr("You must enter a valid Authorization Number."))
+  << GuiErrorCheck(!_authDate->isValid(), _authDate,
+                   tr("You must enter a valid Authorization Date."))
+  << GuiErrorCheck((!partial && (disposition == "C") && (creditBy == "N")), _creditBy,
+                   tr("<p>You may not enter a Disposition of Credit "
+                      "and a Credit By of None."))
+  << GuiErrorCheck((! _miscCharge->isZero() && (!_miscChargeAccount->isValid())), _miscChargeAccount,
+                   tr("<p>You may not enter a Misc. Charge without "
+                      "indicating the G/L Sales Account number for the "
+                      "charge. Please set the Misc. Charge amount to 0 "
+                      "or select a Misc. Charge Sales Account."))
+  << GuiErrorCheck((!partial && _raitem->topLevelItemCount() == 0), _new,
+                   tr("<p>You must create at least one Line Item for "
+                      "this Return Authorization before you may save it."))
+  << GuiErrorCheck(_total->localValue() < 0, _cust,
+                   tr("<p>The Total must be a positive value."))
+  << GuiErrorCheck((!partial && !_warehouse->isValid()), _warehouse,
+                   tr("<p>You must enter a valid Receiving Site."))
+  << GuiErrorCheck((!partial && !_shipWhs->isValid()), _shipWhs,
+                   tr("<p>You must enter a valid Shipping Site."))
+  ;
+  
+  if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Return Authorization"), errors))
     return false;
-  }
-
-  if ( !partial && (timing.isEmpty()) )
-  {
-    QMessageBox::warning( this, tr("Invalid Timing"),
-                         tr("<p>You must enter a Timing." ) );
-    _timing->setFocus();
-    return false;
-  }
-
-  if ( !partial && (creditBy.isEmpty()) )
-  {
-    QMessageBox::warning( this, tr("Invalid Credit Method"),
-                         tr("<p>You must enter a Credit Method." ) );
-    _creditBy->setFocus();
-    return false;
-  }
-
-  if (_authNumber->text().isEmpty())
-  {
-    if(partial && !isVisible())
-      return false;
-    QMessageBox::critical( this, tr("Cannot Save Return Authorization"),
-      tr("You may not save this Return Authorization until you have entered a valid Authorization Number.") );
-    _authNumber->setFocus();
-    return false;
-  }
-
-  if ( !partial && (disposition == "C") && (creditBy == "N") )
-  {
-    QMessageBox::warning( this, tr("Invalid Credit Method"),
-                         tr("<p>You may not enter a Disposition of Credit "
-                            "and a Credit By of None." ) );
-    _creditBy->setFocus();
-    return false;
-  }
-
-  if ( ! _miscCharge->isZero() && (!_miscChargeAccount->isValid()) )
-  {
-    QMessageBox::warning( this, tr("No Misc. Charge Account Number"),
-                         tr("<p>You may not enter a Misc. Charge without "
-                            "indicating the G/L Sales Account number for the "
-                            "charge. Please set the Misc. Charge amount to 0 "
-                            "or select a Misc. Charge Sales Account." ) );
-    _returnAuthInformation->setCurrentIndex(_returnAuthInformation->indexOf(_lineItemsPage));
-    _miscChargeAccount->setFocus();
-    return false;
-  }
-
-  if (!partial && _raitem->topLevelItemCount() == 0)
-  {
-    QMessageBox::warning( this, tr("Create Line Items for this Order"),
-                          tr("<p>You must create at least one Line Item for "
-                          "this Return Authorization before you may save it."));
-    _new->setFocus();
-    return FALSE;
-  }
-
-  if (_total->localValue() < 0 )
-  {
-    QMessageBox::information(this, tr("Total Less than Zero"),
-                             tr("<p>The Total must be a positive value.") );
-    _cust->setFocus();
-    return false;
-  }
-
-  if ( !partial && !_warehouse->isValid() )
-  {
-    QMessageBox::warning( this, tr("Invalid Receiving Site"),
-                         tr("<p>You must enter a valid Receiving Site." ) );
-    _warehouse->setFocus();
-    return false;
-  }
-
-  if ( !partial && !_shipWhs->isValid() )
-  {
-    QMessageBox::warning( this, tr("Invalid Shipping Site"),
-                         tr("<p>You must enter a valid Shipping Site." ) );
-    _shipWhs->setFocus();
-    return false;
-  }
-
+  
   // save address info in case someone wants to use 'em again later
   // but don't make any global changes to the data and ignore errors
   _billToAddr->save(AddressCluster::CHANGEONE);
+  _ignoreShiptoSignals = true;
   _shipToAddr->save(AddressCluster::CHANGEONE);
+  _ignoreShiptoSignals = false;
 
   returnSave.prepare( "UPDATE rahead "
              "   SET rahead_cust_id=:rahead_cust_id,rahead_number=:rahead_number,"
@@ -557,19 +519,20 @@ bool returnAuthorization::sSave(bool partial)
   returnSave.bindValue(":rahead_calcfreight", _calcfreight);
   returnSave.bindValue(":rahead_warehous_id", _warehouse->id());
   returnSave.bindValue(":rahead_cohead_warehous_id", _shipWhs->id());
-  returnSave.bindValue(":rahead_shipzone_id", _shippingZone->id());
+  if (_shippingZone->id() != -1)
+    returnSave.bindValue(":rahead_shipzone_id", _shippingZone->id());
   returnSave.bindValue(":rahead_saletype_id", _saleType->id());
 
   returnSave.exec();
-  if (returnSave.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving RA Information"),
+                                returnSave, __FILE__, __LINE__))
   {
-    systemError(this, returnSave.lastError().databaseText(), __FILE__, __LINE__);
     return false;
   }
 
   omfgThis->sReturnAuthorizationsUpdated();
   omfgThis->sProjectsUpdated(_project->id());
-  _saved = TRUE;
+  _saved = true;
   _comments->setId(_raheadid);
 
   connect(_authNumber, SIGNAL(editingFinished()), this, SLOT(sCheckAuthorizationNumber()));
@@ -680,16 +643,27 @@ void returnAuthorization::sOrigSoChanged()
       }
 
       XSqlQuery sohead;
-      sohead.prepare( "SELECT cohead.*,custinfo.*, custtype_code, "
-                      "       cohead_commission, "
-                      "       shipto_num "
-                      "FROM cohead "
-                      "  LEFT OUTER JOIN shiptoinfo ON (cohead_shipto_id=shipto_id), "
-                      "  custinfo, custtype "
-                      "WHERE ((cohead_id=:cohead_id) "
-                      "AND (cohead_cust_id=cust_id) "
-                      "AND (cust_custtype_id=custtype_id) ) "
-                      "LIMIT 1;" );
+      sohead.prepare("SELECT cohead_salesrep_id, cohead_commission,"
+                     "       cohead_taxzone_id, cohead_custponumber, cohead_prj_id,"
+                     "       cohead_shipzone_id, cohead_saletype_id, cohead_cust_id,"
+                     "       custtype_code, cohead_billtoname,"
+                     "       cohead_billtoaddress1, cohead_billtoaddress2,"
+                     "       cohead_billtoaddress3, cohead_billtocity,"
+                     "       cohead_billtostate, cohead_billtozipcode,"
+                     "       cohead_billtocountry, cust_ffbillto, cohead_shipto_id,"
+                     "       cohead_shiptoname, cohead_shiptoaddress1,"
+                     "       cohead_shiptoaddress2, cohead_shiptoaddress3,"
+                     "       cohead_shiptocity, cohead_shiptostate,"
+                     "       cohead_shiptozipcode, cohead_shiptocountry,"
+                     "       cust_ffshipto, custtype_code, cohead_commission, "
+                     "       shipto_num"
+                     "  FROM cohead"
+                     "  JOIN custinfo ON (cohead_cust_id=cust_id)"
+                     "  JOIN custtype ON (cust_custtype_id=custtype_id)"
+                     "  LEFT OUTER JOIN shiptoinfo ON (cohead_shipto_id=shipto_id)"
+                     " WHERE (cohead_id=:cohead_id)"
+                     " LIMIT 1;");
+      // TODO: why left outer join shipto if we don't use the shipto_num?
       sohead.bindValue(":cohead_id", _origso->id());
       sohead.exec();
       if (sohead.first())
@@ -704,7 +678,7 @@ void returnAuthorization::sOrigSoChanged()
         _shippingZone->setId(sohead.value("cohead_shipzone_id").toInt());
         _saleType->setId(sohead.value("cohead_saletype_id").toInt());
 
-        _cust->setEnabled(FALSE);
+        _cust->setEnabled(false);
 
         _cust->setId(sohead.value("cohead_cust_id").toInt());
         _custType->setText(sohead.value("custtype_code").toString());
@@ -719,7 +693,7 @@ void returnAuthorization::sOrigSoChanged()
         if ( (_mode == cNew) || (_mode == cEdit) )
           _ffBillto = sohead.value("cust_ffbillto").toBool();
         else
-          _ffBillto = FALSE;
+          _ffBillto = false;
         _billToName->setEnabled(_ffBillto);
         _billToAddr->setEnabled(_ffBillto);
 
@@ -736,18 +710,18 @@ void returnAuthorization::sOrigSoChanged()
         if ( (_mode == cNew) || (_mode == cEdit) )
           _ffShipto = sohead.value("cust_ffshipto").toBool();
         else
-          _ffShipto = FALSE;
+          _ffShipto = false;
         _copyToShipto->setEnabled(_ffShipto);
         _shipToName->setEnabled(_ffShipto);
         _shipToAddr->setEnabled(_ffShipto);
-        _ignoreShiptoSignals = FALSE;
+        _ignoreShiptoSignals = false;
 
         sSave(true);
         sFillList();
       }
-      else if (sohead.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Sales Order"),
+                                    sohead, __FILE__, __LINE__))
       {
-        systemError(this, sohead.lastError().databaseText(), __FILE__, __LINE__);
         _origso->setId(-1);
         return;
       }
@@ -777,9 +751,9 @@ void returnAuthorization::populateShipto(int pShiptoid)
       _commission->setDouble(query.value("shipto_commission").toDouble() * 100);
       _ignoreShiptoSignals = false;
     }
-    else if (query.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Ship To Information"),
+                                  query, __FILE__, __LINE__))
     {
-      systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -830,23 +804,23 @@ void returnAuthorization::sPopulateCustomerInfo()
         if ( (_mode == cNew) || (_mode == cEdit) )
           _ffBillto = query.value("cust_ffbillto").toBool();
         else
-          _ffBillto = FALSE;
+          _ffBillto = false;
         _billToName->setEnabled(_ffBillto);
         _billToAddr->setEnabled(_ffBillto);
 
         if ( (_mode == cNew) || (_mode == cEdit) )
           _ffShipto = query.value("cust_ffshipto").toBool();
         else
-          _ffShipto = FALSE;
+          _ffShipto = false;
         _copyToShipto->setEnabled(_ffShipto);
         _shipToName->setEnabled(_ffShipto);
         _shipToAddr->setEnabled(_ffShipto);
         _custEmail = query.value("cust_soemaildelivery").toBool();
         populateShipto(query.value("shiptoid").toInt());
       }
-      else if (query.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Customer Information"),
+                                    query, __FILE__, __LINE__))
       {
-        systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
@@ -856,13 +830,13 @@ void returnAuthorization::sPopulateCustomerInfo()
       _salesRep->setCurrentIndex(-1);
       _taxzone->setId(-1);
       _custtaxzoneid = -1;
-      _billToName->setEnabled(TRUE);
-      _billToAddr->setEnabled(TRUE);
+      _billToName->setEnabled(true);
+      _billToAddr->setEnabled(true);
       _billToName->clear();
       _billToAddr->clear();
-      _shipTo->setEnabled(TRUE);
-      _shipToName->setEnabled(TRUE);
-      _shipToAddr->setEnabled(TRUE);
+      _shipTo->setEnabled(true);
+      _shipToName->setEnabled(true);
+      _shipToAddr->setEnabled(true);
       _shipTo->setId(-1);
       _shipToName->clear();
       _shipToAddr->clear();
@@ -875,7 +849,7 @@ void returnAuthorization::sCheckAuthorizationNumber()
        ( (_metrics->value("RANumberGeneration") == "O") ||
          (_metrics->value("RANumberGeneration") == "M")   ) )
   {
-    _authNumber->setEnabled(FALSE);
+    _authNumber->setEnabled(false);
 
     XSqlQuery query;
     query.prepare( "SELECT rahead_id "
@@ -891,15 +865,15 @@ void returnAuthorization::sCheckAuthorizationNumber()
       _raheadid = query.value("rahead_id").toInt();
 
       if (_cust->id() > 0)
-        _cust->setReadOnly(TRUE);
+        _cust->setReadOnly(true);
 
       populate();
 
       _mode = cEdit;
     }
-    else if (query.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  query, __FILE__, __LINE__))
     {
-      systemError(this, query.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -950,7 +924,7 @@ void returnAuthorization::sNew()
     if (_shipWhs->isValid())
       params.append("shipwarehous_id", _shipWhs->id());
 
-    returnAuthorizationItem newdlg(this, "", TRUE);
+    returnAuthorizationItem newdlg(this, "", true);
     newdlg.set(params);
 
     if (newdlg.exec() != XDialog::Rejected)
@@ -965,7 +939,7 @@ void returnAuthorization::sEdit()
   if (sSave(true))
   {
     bool fill;
-    fill = FALSE;
+    fill = false;
     QList<XTreeWidgetItem*> selected = _raitem->selectedItems();
     for (int i = 0; i < selected.size(); i++)
     {
@@ -981,11 +955,11 @@ void returnAuthorization::sEdit()
       else
         params.append("mode", "edit");
 
-      returnAuthorizationItem newdlg(this, "", TRUE);
+      returnAuthorizationItem newdlg(this, "", true);
       newdlg.set(params);
 
       if (newdlg.exec() != XDialog::Rejected)
-        fill = TRUE;
+        fill = true;
     }
     if (fill)
     {
@@ -1005,7 +979,7 @@ void returnAuthorization::sView()
     params.append("orig_cohead_id", _origso->id());
     params.append("mode", "view");
 
-    returnAuthorizationItem newdlg(this, "", TRUE);
+    returnAuthorizationItem newdlg(this, "", true);
     newdlg.set(params);
 
     newdlg.exec();
@@ -1052,16 +1026,17 @@ void returnAuthorization::sDelete()
       }
       else if (checkwo.lastError().type() != QSqlError::NoError)
       {
-        systemError(this, checkwo.lastError().databaseText(), __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Information"),
+                             checkwo, __FILE__, __LINE__);
       }
 
       returnDelete.prepare( "DELETE FROM raitem "
                  "WHERE (raitem_id=:raitem_id);" );
       returnDelete.bindValue(":raitem_id", ((XTreeWidgetItem*)(selected[i]))->id());
       returnDelete.exec();
-      if (returnDelete.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting RA Item"),
+                                    returnDelete, __FILE__, __LINE__))
       {
-        systemError(this, returnDelete.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
@@ -1082,7 +1057,9 @@ void returnAuthorization::sFillList()
              "            ELSE -1 END AS disposition_code,"
              "       formatRaLineNumber(raitem_id) AS f_linenumber, raitem_subnumber,"
              "       item_number, uom_name, item_type,"
-             "       (item_descrip1 || ' ' || item_descrip2) AS item_descrip, warehous_code,"
+             "       (item_descrip1 || ' ' || item_descrip2) AS item_descrip,"
+             "       (SELECT warehous_code FROM whsinfo JOIN itemsite ON itemsite_warehous_id=warehous_id" 
+             "            WHERE itemsite_id = COALESCE(raitem_coitem_itemsite_id, raitem_itemsite_id)) AS warehous_code,"
              "       raitem_status, "
              "       CASE WHEN (raitem_disposition='C') THEN :credit "
              "            WHEN (raitem_disposition='R') THEN :return "
@@ -1127,10 +1104,9 @@ void returnAuthorization::sFillList()
              " LEFT OUTER JOIN cohead och ON (och.cohead_id=oc.coitem_cohead_id) "
              " LEFT OUTER JOIN coitem nc ON (raitem_new_coitem_id=nc.coitem_id) "
              " LEFT OUTER JOIN cohead nch ON (nch.cohead_id=nc.coitem_cohead_id),"
-             " itemsite, item, whsinfo, uom "
+             " itemsite, item, uom "
              "WHERE ( (raitem_itemsite_id=itemsite_id)"
              " AND (itemsite_item_id=item_id)"
-             " AND (itemsite_warehous_id=warehous_id)"
              " AND (raitem_rahead_id=:rahead_id) "
              " AND (raitem_qty_uom_id=uom_id) ) "
              "ORDER BY raitem_linenumber, raitem_subnumber;" );
@@ -1147,9 +1123,9 @@ void returnAuthorization::sFillList()
   {
     _cust->setDisabled(_cust->isValid());
   }
-  else if (returnFillList.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                returnFillList, __FILE__, __LINE__))
   {
-    systemError(this, returnFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -1173,9 +1149,9 @@ void returnAuthorization::sFillList()
       _freight->setLocalValue(_freightCache);
       connect(_freight, SIGNAL(valueChanged()), this, SLOT(sFreightChanged()));
     }
-    else if (returnFillList.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  returnFillList, __FILE__, __LINE__))
     {
-      systemError(this, returnFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -1223,7 +1199,8 @@ void returnAuthorization::sFillList()
     _commission->setEnabled(true);
     _taxzone->setEnabled(true);
     _disposition->setEnabled(true);
-    _timing->setEnabled(true);
+// this overrides the setting of _timing in sDispositionChanged
+//    _timing->setEnabled(true);
     _cust->setEnabled(!_origso->isValid());
     _billToName->setEnabled(true);
     _billToAddr->setEnabled(true);
@@ -1315,10 +1292,10 @@ void returnAuthorization::populate()
     else if (rahead.value("rahead_creditmethod").toString() == "C")
       _creditBy->setCurrentIndex(3);
 
-    _ignoreSoSignals = TRUE;
+    _ignoreSoSignals = true;
     _origso->setId(rahead.value("rahead_orig_cohead_id").toInt());
     _newso->setId(rahead.value("rahead_new_cohead_id").toInt(),"SO");
-    _ignoreSoSignals = FALSE;
+    _ignoreSoSignals = false;
     _incident->setId(rahead.value("rahead_incdt_id").toInt());
     _project->setId(rahead.value("rahead_prj_id").toInt());
 
@@ -1366,10 +1343,10 @@ void returnAuthorization::populate()
     }
 
 
-    _ignoreWhsSignals = TRUE;
+    _ignoreWhsSignals = true;
     _warehouse->setId(rahead.value("rahead_warehous_id").toInt());
     _shipWhs->setId(rahead.value("rahead_cohead_warehous_id").toInt());
-    _ignoreWhsSignals = FALSE;
+    _ignoreWhsSignals = false;
 
     _miscCharge->setLocalValue(rahead.value("rahead_misc").toDouble());
     _miscChargeDescription->setText(rahead.value("rahead_misc_descrip"));
@@ -1382,13 +1359,13 @@ void returnAuthorization::populate()
 
     if (rahead.value("rahead_headcredited").toBool())
     {
-      _freight->setEnabled(FALSE);
-      _miscCharge->setEnabled(FALSE);
-      _miscChargeDescription->setEnabled(FALSE);
-      _miscChargeAccount->setEnabled(FALSE);
+      _freight->setEnabled(false);
+      _miscCharge->setEnabled(false);
+      _miscChargeDescription->setEnabled(false);
+      _miscChargeAccount->setEnabled(false);
     }
 
-    _ignoreSoSignals = TRUE;
+    _ignoreSoSignals = true;
     if (rahead.value("rahead_disposition").toString() == "C")
       sDispositionChanged();
     else if (rahead.value("rahead_disposition").toString() == "R")
@@ -1399,13 +1376,13 @@ void returnAuthorization::populate()
       _disposition->setCurrentIndex(3);
     else if (rahead.value("rahead_disposition").toString() == "M")
       _disposition->setCurrentIndex(4);
-    _ignoreSoSignals = FALSE;
-    _saved = TRUE;
+    _ignoreSoSignals = false;
+    _saved = true;
     sFillList();
   }
-  else if (rahead.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                rahead, __FILE__, __LINE__))
   {
-    systemError(this, rahead.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -1422,9 +1399,9 @@ void returnAuthorization::closeEvent(QCloseEvent *pEvent)
                "WHERE (rahead_id=:rahead_id);" );
     returncloseEvent.bindValue(":rahead_id", _raheadid);
     returncloseEvent.exec();
-    if (returncloseEvent.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating RA Item Information"),
+                                  returncloseEvent, __FILE__, __LINE__))
     {
-      systemError(this, returncloseEvent.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else
@@ -1439,9 +1416,9 @@ void returnAuthorization::closeEvent(QCloseEvent *pEvent)
       returncloseEvent.prepare("SELECT releaseRaNumber(:number) AS result;");
       returncloseEvent.bindValue(":number", _authNumber->text());
       returncloseEvent.exec();
-      if (returncloseEvent.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating RA Item Information"),
+                                    returncloseEvent, __FILE__, __LINE__))
       {
-        systemError(this, returncloseEvent.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
     }
@@ -1462,7 +1439,7 @@ void returnAuthorization::sTaxDetail()
   if (_mode == cView)
     params.append("mode", "view");
 
-  taxBreakdown newdlg(this, "", TRUE);
+  taxBreakdown newdlg(this, "", true);
   if (newdlg.set(params) == NoError && newdlg.exec() == XDialog::Accepted)
     populate();
 
@@ -1477,9 +1454,9 @@ void returnAuthorization::sCalculateTax()
   taxq.exec();
   if (taxq.first())
     _tax->setLocalValue(taxq.value("tax").toDouble());
-  else if (taxq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Calculating Tax"),
+                                taxq, __FILE__, __LINE__))
   {
-    systemError(this, taxq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -1551,12 +1528,12 @@ void returnAuthorization::sDispositionChanged()
   if (_disposition->currentIndex() == 0)
   {
     _timing->setCurrentIndex(0);
-    _timing->setEnabled(FALSE);
+    _timing->setEnabled(false);
     if (_creditBy->currentIndex() == 0)
       _creditBy->setCurrentIndex(1);
   }
   else
-    _timing->setEnabled(TRUE);
+    _timing->setEnabled(true);
 
   _refund->setEnabled(_creditBy->currentIndex() == 3);
 
@@ -1570,6 +1547,9 @@ void returnAuthorization::sDispositionChanged()
 
 void returnAuthorization::sCreditByChanged()
 {
+  _new->setEnabled(_cust->isValid() ||
+                   (_disposition->currentIndex() == 1 && _creditBy->currentIndex() == 0));
+  
   if (_creditBy->currentIndex() == 0 && _total->localValue() > 0)
   {
     QMessageBox::information(this, tr("Credit By 'None' not allowed"),
@@ -1606,9 +1586,9 @@ void returnAuthorization::sAuthorizeLine()
     returnAuthorizeLine.prepare(sql);
     returnAuthorizeLine.bindValue(":raitem_id",  ((XTreeWidgetItem*)(selected[i]))->id());
     returnAuthorizeLine.exec();
-    if (returnAuthorizeLine.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  returnAuthorizeLine, __FILE__, __LINE__))
     {
-      systemError(this, returnAuthorizeLine.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -1628,10 +1608,10 @@ void returnAuthorization::sClearAuthorization()
         returnClearAuthorization.prepare ( "SELECT clearReturnItem(:raitem_id) AS result;" );
         returnClearAuthorization.bindValue(":raitem_id",  ((XTreeWidgetItem*)(selected[i]))->id());
         returnClearAuthorization.exec();
-        if (returnClearAuthorization.lastError().type() != QSqlError::NoError)
+        if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                      returnClearAuthorization, __FILE__, __LINE__))
         {
-           systemError(this, returnClearAuthorization.lastError().databaseText(), __FILE__, __LINE__);
-           return;
+          return;
         }
   }
 
@@ -1650,9 +1630,9 @@ void returnAuthorization::sAuthorizeAll()
   returnAuthorizeAll.prepare(sql);
   returnAuthorizeAll.bindValue(":raitem_rahead_id",  _raheadid);
   returnAuthorizeAll.exec();
-  if (returnAuthorizeAll.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                returnAuthorizeAll, __FILE__, __LINE__))
   {
-    systemError(this, returnAuthorizeAll.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -1678,9 +1658,9 @@ void returnAuthorization::sEnterReceipt()
     params.append("recv_id", returnEnterReceipt.value("recv_id"));
     params.append("mode", "edit");
   }
-  else if (returnEnterReceipt.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                returnEnterReceipt, __FILE__, __LINE__))
   {
-    systemError(this, returnEnterReceipt.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -1690,7 +1670,7 @@ void returnAuthorization::sEnterReceipt()
     params.append("mode", "new");
   }
 
-  enterPoitemReceipt newdlg(this, "", TRUE);
+  enterPoitemReceipt newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -1705,7 +1685,7 @@ void returnAuthorization::sReceiveAll()
   params.append("ordertype", "RA");
   params.append("nonInventory", tr("Non-Inventory"));
   params.append("na",           tr("N/A"));
-  params.append("EnableReturnAuth", TRUE);
+  params.append("EnableReturnAuth", true);
   if (_metrics->boolean("MultiWhs"))
     params.append("MultiWhs");
   MetaSQLQuery recvm = mqlLoad("receipt", "receiveAll");
@@ -1716,15 +1696,16 @@ void returnAuthorization::sReceiveAll()
     int result = returnReceiveAll.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("enterReceipt", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Receiving RA Items"),
+                             storedProcErrorLookup("enterReceipt", result),
+                             __FILE__, __LINE__);
       return;
     }
     omfgThis->sPurchaseOrderReceiptsUpdated();
   }
-  if (returnReceiveAll.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Receiving RA Items"),
+                                returnReceiveAll, __FILE__, __LINE__))
   {
-    systemError(this, returnReceiveAll.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -1747,17 +1728,17 @@ void returnAuthorization::sHandleAction()
   if (selected.size() > 1)
   {
     _action->setText("Close");
-    _action->setEnabled(FALSE);
+    _action->setEnabled(false);
   }
   else if (_raitem->altId() == -1)
   {
     _action->setText("Open");
-    _edit->setEnabled(FALSE);
+    _edit->setEnabled(false);
   }
   else
   {
     _action->setText("Close");
-    _edit->setEnabled(TRUE);
+    _edit->setEnabled(true);
   }
 }
 
@@ -1773,9 +1754,9 @@ void returnAuthorization::sAction()
   else
     returnAction.bindValue(":status",QString("C"));
   returnAction.exec();
-  if (returnAction.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating RA Item Information"),
+                                        returnAction, __FILE__, __LINE__))
   {
-    systemError(this, returnAction.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -1817,8 +1798,8 @@ void returnAuthorization::sRefund()
     if (ccq.first())
     {
       int ccpayid = ccq.value("ccpay_id").toInt();
-      QMessageBox::information( this, tr("New Credit Memo Created"),
-                                tr("<p>A new CreditMemo has been created and "
+      QMessageBox::information( this, tr("New Sales Credit Created"),
+                                tr("<p>A new Sales Credit has been created and "
                                    "assigned #%1")
                                    .arg(ccq.value("cmhead_number").toString()));
       CreditCardProcessor *cardproc = CreditCardProcessor::getProcessor();
@@ -1882,9 +1863,10 @@ void returnAuthorization::sRefund()
     }
     else if (ccq.lastError().type() != QSqlError::NoError)
     {
-      systemError(this, ccq.lastError().databaseText(), __FILE__, __LINE__);
-      XSqlQuery rollback("ROLLBACK;");
-      return;
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Credit Memo Information"),
+                                              ccq, __FILE__, __LINE__);
+       XSqlQuery rollback("ROLLBACK;");
+       return;
     }
     else
     {
@@ -1898,7 +1880,7 @@ void returnAuthorization::sRefund()
   else if (cmq.lastError().type() != QSqlError::NoError)
   {
     XSqlQuery rollback("ROLLBACK;");
-    ErrorReporter::error(QtCriticalMsg, this, tr("Creating R/A Credit Memo"),
+    ErrorReporter::error(QtCriticalMsg, this, tr("Creating R/A Sales Credit"),
                          cmq, __FILE__, __LINE__);
     return;
   }
@@ -2051,12 +2033,12 @@ void returnAuthorization::sCheckNumber()
     _mode = cEdit;
     _raheadid = returnCheckNumber.value("rahead_id").toInt();
     populate();
-    _authNumber->setEnabled(FALSE);
-    _cust->setReadOnly(TRUE);
+    _authNumber->setEnabled(false);
+    _cust->setReadOnly(true);
   }
-  else if (returnCheckNumber.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                        returnCheckNumber, __FILE__, __LINE__))
   {
-    systemError(this, returnCheckNumber.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   sSave(true);
@@ -2132,7 +2114,7 @@ void returnAuthorization::sFreightDetail()
   // mode => view since there are no fields to hold modified freight data
   params.append("mode", "view");
 
-  freightBreakdown newdlg(this, "", TRUE);
+  freightBreakdown newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
