@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -25,8 +25,9 @@
 #include "returnAuthorization.h"
 #include "returnAuthCheck.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
-returnAuthorizationWorkbench::returnAuthorizationWorkbench(QWidget* parent, const char* name, Qt::WFlags fl)
+returnAuthorizationWorkbench::returnAuthorizationWorkbench(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
@@ -249,7 +250,9 @@ void returnAuthorizationWorkbench::sProcess()
     int cmheadid = returnProcess.value("result").toInt();
     if (cmheadid < 0)
     {
-      systemError(this, storedProcErrorLookup("createRaCreditMemo", cmheadid), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Credit Memo Information"),
+                             storedProcErrorLookup("createRaCreditMemo", cmheadid),
+                             __FILE__, __LINE__);
       return;
     }
     returnProcess.prepare( "SELECT cmhead_number "
@@ -259,8 +262,8 @@ void returnAuthorizationWorkbench::sProcess()
     returnProcess.exec();
     if (returnProcess.first())
     {
-      QMessageBox::information( this, tr("New Credit Memo Created"),
-                                tr("<p>A new CreditMemo has been created and "
+      QMessageBox::information( this, tr("New Sales Credit Created"),
+                                tr("<p>A new Sales Credit has been created and "
 				                   "assigned #%1")
                                    .arg(returnProcess.value("cmhead_number").toString()));
 	  if (_printmemo->isChecked())
@@ -270,7 +273,7 @@ void returnAuthorizationWorkbench::sProcess()
 		if (_post)
 		  params.append("posted");
 
-		printCreditMemo newdlg(this, "", TRUE);
+		printCreditMemo newdlg(this, "", true);
 		newdlg.set(params);
 		newdlg.exec();
 	  }
@@ -279,7 +282,7 @@ void returnAuthorizationWorkbench::sProcess()
         ParameterList params;
         params.append("cmhead_id", cmheadid);
 
-        returnAuthCheck newdlg(this, "", TRUE);
+        returnAuthCheck newdlg(this, "", true);
         newdlg.set(params);
         if (newdlg.exec() != XDialog::Rejected)
           sFillListDue();
@@ -288,7 +291,7 @@ void returnAuthorizationWorkbench::sProcess()
       {
 	ParameterList ccp;
 	ccp.append("cmhead_id", cmheadid);
-  MetaSQLQuery ccm = mqlLoad("creditMemoCreditCards", "detail");
+        MetaSQLQuery ccm = mqlLoad("creditMemoCreditCards", "detail");
 	XSqlQuery ccq = ccm.toQuery(ccp);
 	if (ccq.first())
 	{
@@ -304,7 +307,12 @@ void returnAuthorizationWorkbench::sProcess()
 	  {
 	    QString docnum = returnProcess.value("cmhead_number").toString();
 	    QString refnum = ccq.value("cohead_number").toString();
-	    int refid = -1;
+            int refid = ccq.value("cohead_id").toInt();
+            QString reftype = "";
+
+            if (refid > 0)
+              reftype = "cohead";
+
 	    int returnValue = cardproc->credit(ccq.value("ccard_id").toInt(),
 					 "-2",
 					 ccq.value("total").toDouble(),
@@ -314,7 +322,7 @@ void returnAuthorizationWorkbench::sProcess()
 					 0,
 					 ccq.value("cmhead_curr_id").toInt(),
 					 docnum, refnum, ccpayid,
-					 QString(), refid);
+                                         reftype, refid);
 	    if (returnValue < 0)
 	      QMessageBox::critical(this, tr("Credit Card Processing Error"),
 				    cardproc->errorMsg());
@@ -325,14 +333,14 @@ void returnAuthorizationWorkbench::sProcess()
 	      QMessageBox::information(this, tr("Credit Card Processing Note"),
 				   cardproc->errorMsg());
 	  }
-	  // requery regardless 'cause the new credit memo means nothing's "due"
+	  // requery regardless 'cause the new return means nothing's "due"
 	  sFillListDue();
 	}
-	else if (ccq.lastError().type() != QSqlError::NoError)
-	{
-	  systemError(this, ccq.lastError().databaseText(), __FILE__, __LINE__);
-	  return;
-	}
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                  ccq, __FILE__, __LINE__))
+    {
+      return;
+    }
 	else
 	{
 	  QMessageBox::critical(this, tr("Credit Card Processing Error"),
@@ -345,15 +353,15 @@ void returnAuthorizationWorkbench::sProcess()
       else
 	sFillListDue();
     }
-    else if (returnProcess.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                           returnProcess, __FILE__, __LINE__))
     {
-      systemError(this, returnProcess.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
-  else if (returnProcess.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                           returnProcess, __FILE__, __LINE__))
   {
-    systemError(this, returnProcess.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -363,7 +371,7 @@ void returnAuthorizationWorkbench::setParams(ParameterList &params)
   _customerSelector->appendValue(params);
 
   params.append("credit",     tr("Credit"));
-  params.append("return",     tr("Return"));
+  params.append("return",     tr("Sales Credit"));
   params.append("replace",    tr("Replace"));
   params.append("service",    tr("Service"));
   params.append("none",       tr("None"));
@@ -443,9 +451,9 @@ void returnAuthorizationWorkbench::sFillListReview()
 
     XSqlQuery rareview = mql.toQuery(params);
     _ra->populate(rareview);
-    if (rareview.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                           rareview, __FILE__, __LINE__))
     {
-      systemError(this, rareview.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -463,10 +471,10 @@ void returnAuthorizationWorkbench::sFillListDue()
     setParams(params);
 
     XSqlQuery radue = mql.toQuery(params);
-    _radue->populate(radue,TRUE);
-    if (radue.lastError().type() != QSqlError::NoError)
+    _radue->populate(radue,true);
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving RA Information"),
+                                           radue, __FILE__, __LINE__))
     {
-      systemError(this, radue.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
