@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -46,6 +46,7 @@
 #include "display.h"
 #include "xuiloader.h"
 #include "getscreen.h"
+#include "errorReporter.h"
 
 /** @ingroup scriptapi
 
@@ -83,6 +84,7 @@
     @see XMainWindow
     @see XWidget
  */
+/** @example dspBacklog.js */
 
 QWidget *ScriptToolbox::_lastWindow = 0;
 
@@ -415,9 +417,9 @@ QObject * ScriptToolbox::createGridLayout()
   that contains the given widget. This lets a %script create a new
   QWidget and insert it into the %user interface near another widget.
 
-  @dontinclude addButton.js
-  @skip var
-  @until widgetGetLayout
+  @dontinclude dspBacklog.js
+  @skip createWidget
+  @until addWidget(_collapse
 
   @param w The widget whose layout you want to find
 
@@ -438,7 +440,6 @@ QObject * ScriptToolbox::widgetGetLayout(QWidget * w)
   }
   return NULL;
 }
-/** @example addButton.js */
 
 /** @brief Insert a QGridLayout into an existing QGridLayout.
     @deprecated Use QGridLayout::addLayout instead
@@ -716,8 +717,9 @@ QString ScriptToolbox::tabtabText(QWidget * tab, int idx)
   This is a wrapper around XUiLoader::createWidget.
 
   The following example creates a new QPushButton:
-  @dontinclude addButton.js
-  @skipline newbutton
+  @dontinclude dspBacklog.js
+  @skip  _expand
+  @until _collapse
 
   @param className The name of the class of the desired widget
   @param parent    The widget that should be set as the parent of the new widget
@@ -892,7 +894,8 @@ bool ScriptToolbox::printReportCopies(const QString & name, const ParameterList
   if (orReport::beginMultiPrint(&printer, userCanceled) == false)
   {
     if(!userCanceled)
-      systemError(NULL, tr("Could not initialize printing system for multiple reports."));
+      ErrorReporter::error(QtCriticalMsg, 0, tr("Error Occurred"),
+                         tr("Could not initialize the printing system for multiple reports."),__FILE__,__LINE__);
     return userCanceled;
   }
   userCanceled = false;
@@ -1016,7 +1019,7 @@ QString ScriptToolbox::fileDialog(QWidget * parent, const QString & caption, con
   if (! filter.isEmpty())
   {
     filters << filter << tr("Any Files (*)");
-    newdlg.setFilters(filters);
+    newdlg.setNameFilters(filters);
   }
   if (newdlg.exec())  path = newdlg.selectedFiles().join(", ");
   else path = "";
@@ -1174,11 +1177,11 @@ bool ScriptToolbox::removePath(const QString & rmPath, const QString & rootPath)
     This writes all of the properties of a QtScript object using qWarning.
     This is a debugging tool, most useful when not using the Qt script debugger.
     The property list is not sorted and is not generally recursive.
-    This method does show prototype properties in a separately labeled list
+    This method can show prototype properties in a separately labeled list
     if the given object has a prototype.
 
-    The output has the name of each property and the result of toString(),
-    so properties which are functions show as something like "function()".
+    The output has the name of each property and the result of @c toString() ,
+    so properties which are functions show as something like @c function() .
     The values which would be returned by these functions are not displayed.
     For example,
     @code
@@ -1188,12 +1191,14 @@ bool ScriptToolbox::removePath(const QString & rmPath, const QString & rootPath)
     @code
       id = "function() returning int"
     @endcode
-    rather than the usually-more-useful
+    rather than the more useful
     @code
       id = -1
     @endcode
 
     @param obj The object whose properties should be listed
+    @param showPrototype true: show the properties of @c obj 's prototype
+                         false: show just the top-level properties of @c obj
   */
 void ScriptToolbox::listProperties(const QScriptValue &obj, const bool showPrototype) const
 {
@@ -1279,10 +1284,7 @@ QWidget *ScriptToolbox::openWindow(QString pname, QWidget *parent, Qt::WindowMod
   if(returnVal)
   {
     if(!returnVal->inherits("QDialog"))
-    {
-      omfgThis->handleNewWindow(returnVal);
-      returnVal->setWindowModality(modality);
-    }
+      omfgThis->handleNewWindow(returnVal, modality);
     _lastWindow = returnVal;
     return returnVal;
   }
@@ -1327,12 +1329,11 @@ QWidget *ScriptToolbox::openWindow(QString pname, QWidget *parent, Qt::WindowMod
     }
 
     XMainWindow *window = new XMainWindow(parent,
-                                          screenq.value("uiform_name").toString().toAscii().data(),
+                                          screenq.value("uiform_name").toString().toLatin1().data(),
                                           flags);
 
     window->setCentralWidget(ui);
     window->setWindowTitle(ui->windowTitle());
-    window->setWindowModality(modality);
     window->resize(size);
 
     if (ui->inherits("QDialog"))
@@ -1353,19 +1354,19 @@ QWidget *ScriptToolbox::openWindow(QString pname, QWidget *parent, Qt::WindowMod
         qWarning("Could not find the script engine to embed a dialog inside "
                "a placeholder window");
 
-      omfgThis->handleNewWindow(window);
+      omfgThis->handleNewWindow(window, modality);
       returnVal = ui;
     }
     else
     {
-      omfgThis->handleNewWindow(window);
+      omfgThis->handleNewWindow(window, modality);
       returnVal = window;
     }
     _lastWindow = window;
   }
-  else if (screenq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, 0, tr("Error Opening New Window"),
+                                screenq, __FILE__, __LINE__))
   {
-    systemError(0, screenq.lastError().databaseText(), __FILE__, __LINE__);
     return 0;
   }
 
@@ -1374,7 +1375,7 @@ QWidget *ScriptToolbox::openWindow(QString pname, QWidget *parent, Qt::WindowMod
 
 QWidget *ScriptToolbox::newDisplay(QString pname, QWidget *parent, Qt::WindowModality modality, Qt::WindowFlags flags)
 {
-  display *window = new display(parent, pname.toAscii().data(), flags);
+  display *window = new display(parent, pname.toLatin1().data(), flags);
   window->setWindowModality(modality);
   omfgThis->handleNewWindow(window);
   _lastWindow = window;
