@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,14 +16,20 @@
 #include <metasql.h>
 #include <openreports.h>
 #include <parameter.h>
+#include <xdateinputdialog.h>
 #include <QMessageBox>
 
+#include "apOpenItem.h"
+#include "dspGLSeries.h"
+#include "errorReporter.h"
 #include "guiclient.h"
+#include "miscVoucher.h"
 #include "selectBankAccount.h"
 #include "selectPayment.h"
 #include "storedProcErrorLookup.h"
+#include "voucher.h"
 
-selectPayments::selectPayments(QWidget* parent, const char* name, Qt::WFlags fl, bool pAutoFill)
+selectPayments::selectPayments(QWidget* parent, const char* name, Qt::WindowFlags fl, bool pAutoFill)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
@@ -58,9 +64,9 @@ selectPayments::selectPayments(QWidget* parent, const char* name, Qt::WFlags fl,
   _apopen->addColumn(tr("Amount"),         _moneyColumn, Qt::AlignRight ,        true, "amount" );
   _apopen->addColumn(tr("Amount (%1)").arg(CurrDisplay::baseCurrAbbr()), _moneyColumn, Qt::AlignRight, false, "base_amount"  );
   _apopen->addColumn(tr("Running Amount (%1)").arg(CurrDisplay::baseCurrAbbr()), _moneyColumn, Qt::AlignRight,         false, "running_amount" );
-  _apopen->addColumn(tr("Selected"),       _moneyColumn, Qt::AlignRight ,        true, "selected" );
-  _apopen->addColumn(tr("Selected (%1)").arg(CurrDisplay::baseCurrAbbr()), _moneyColumn, Qt::AlignRight, false, "base_selected"  );
-  _apopen->addColumn(tr("Running Selected (%1)").arg(CurrDisplay::baseCurrAbbr()), _moneyColumn, Qt::AlignRight,       true, "running_selected"  );
+  _apopen->addColumn(tr("Approved"),       _moneyColumn, Qt::AlignRight ,        true, "selected" );
+  _apopen->addColumn(tr("Approved (%1)").arg(CurrDisplay::baseCurrAbbr()), _moneyColumn, Qt::AlignRight, false, "base_selected"  );
+  _apopen->addColumn(tr("Running Approved (%1)").arg(CurrDisplay::baseCurrAbbr()), _moneyColumn, Qt::AlignRight,       true, "running_selected"  );
   _apopen->addColumn(tr("Discount"),       _moneyColumn, Qt::AlignRight ,        true, "discount" );
   _apopen->addColumn(tr("Discount (%1)").arg(CurrDisplay::baseCurrAbbr()), _moneyColumn, Qt::AlignRight, false, "base_discount"  );
   _apopen->addColumn(tr("Currency"),       _currencyColumn, Qt::AlignLeft,       true, "curr_concat" );
@@ -122,30 +128,14 @@ void selectPayments::sSelectDue()
   int bankaccntid = _bankaccnt->id();
   if(bankaccntid == -1)
   {
-    selectBankAccount newdlg(this, "", TRUE);
+    selectBankAccount newdlg(this, "", true);
     newdlg.set(params);
     bankaccntid = newdlg.exec();
   }
 
   if (bankaccntid >= 0)
   {
-    MetaSQLQuery mql("SELECT selectDueItemsForPayment("
-                     "    <? if exists(\"vend_id\") ?> <? value(\"vend_id\") ?>"
-                     "    <? else ?> vend_id <? endif ?>,"
-                     "    <? value(\"bankaccnt_id\") ?>) AS result "
-                     "<? if exists(\"vend_id\") ?>"
-                     ";"
-                     "<? elseif exists(\"vendtype_id\") ?>"
-                     "FROM vendinfo "
-                     "WHERE (vend_vendtype_id=<? value(\"vendtype_id\") ?>);"
-                     "<? elseif exists(\"vendtype_pattern\") ?>"
-                     "FROM vendinfo "
-                     "WHERE (vend_vendtype_id IN (SELECT vendtype_id"
-                     "                            FROM vendtype"
-                     "                            WHERE (vendtype_code ~ <? value(\"vendtype_pattern\") ?>)));"
-                     "<? else ?>"
-                     "FROM vendinfo;"
-                     "<? endif ?>");
+    MetaSQLQuery mql = mqlLoad("selectPayments", "dueitems");
     ParameterList params;
     if (! setParams(params))
         return;
@@ -156,18 +146,20 @@ void selectPayments::sSelectDue()
       int result = selectSelectDue.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("selectDueItemsForPayment", result),
-                    __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                               storedProcErrorLookup("selectPayment", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (selectSelectDue.lastError().type() != QSqlError::NoError)
+    else
     {
-      systemError(this, selectSelectDue.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                                        selectSelectDue, __FILE__, __LINE__);
       return;
     }
 
-    omfgThis->sPaymentsUpdated(-1, -1, TRUE);
+    omfgThis->sPaymentsUpdated(-1, -1, true);
   }
 }
 
@@ -180,30 +172,14 @@ void selectPayments::sSelectDiscount()
   int bankaccntid = _bankaccnt->id();
   if(bankaccntid == -1)
   {
-    selectBankAccount newdlg(this, "", TRUE);
+    selectBankAccount newdlg(this, "", true);
     newdlg.set(params);
     bankaccntid = newdlg.exec();
   }
 
   if (bankaccntid >= 0)
   {
-    MetaSQLQuery mql("SELECT selectDiscountItemsForPayment("
-                     "    <? if exists(\"vend_id\") ?> <? value(\"vend_id\") ?>"
-                     "    <? else ?> vend_id <? endif ?>,"
-                     "    <? value(\"bankaccnt_id\") ?>) AS result "
-                     "<? if exists(\"vend_id\") ?>"
-                     ";"
-                     "<? elseif exists(\"vendtype_id\") ?>"
-                     "FROM vendinfo "
-                     "WHERE (vend_vendtype_id=<? value(\"vendtype_id\") ?>);"
-                     "<? elseif exists(\"vendtype_pattern\") ?>"
-                     "FROM vendinfo "
-                     "WHERE (vend_vendtype_id IN (SELECT vendtype_id"
-                     "                            FROM vendtype"
-                     "                            WHERE (vendtype_code ~ <? value(\"vendtype_pattern\") ?>)));"
-                     "<? else ?>"
-                     "FROM vendinfo;"
-                     "<? endif ?>");
+    MetaSQLQuery mql = mqlLoad("selectPayments", "discountitems");
     ParameterList params;
     if (! setParams(params))
         return;
@@ -214,69 +190,49 @@ void selectPayments::sSelectDiscount()
       int result = selectSelectDiscount.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, storedProcErrorLookup("selectDiscountItemsForPayment", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                               storedProcErrorLookup("selectPayment", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (selectSelectDiscount.lastError().type() != QSqlError::NoError)
+    else
     {
-      systemError(this, selectSelectDiscount.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                           selectSelectDiscount, __FILE__, __LINE__);
       return;
     }
-    omfgThis->sPaymentsUpdated(-1, -1, TRUE);
+
+    omfgThis->sPaymentsUpdated(-1, -1, true);
   }
 }
 
 void selectPayments::sClearAll()
 {
   XSqlQuery selectClearAll;
-  switch (_vendorgroup->state())
-  {
-    case VendorGroup::All:
-      selectClearAll.prepare( "SELECT clearPayment(apselect_id) AS result "
-                 "FROM apselect;" );
-        break;
-    case VendorGroup::Selected:
-      selectClearAll.prepare( "SELECT clearPayment(apselect_id) AS result "
-                 "FROM apopen JOIN apselect ON (apselect_apopen_id=apopen_id) "
-                 "WHERE (apopen_vend_id=:vend_id);" );
-      break;
-    case VendorGroup::SelectedType:
-      selectClearAll.prepare( "SELECT clearPayment(apselect_id) AS result "
-                 "FROM vendinfo JOIN apopen ON (apopen_vend_id=vend_id) "
-                 "              JOIN apselect ON (apselect_apopen_id=apopen_id) "
-                 "WHERE (vend_vendtype_id=:vendtype_id) ;" );
-      break;
-    case VendorGroup::TypePattern:
-      selectClearAll.prepare( "SELECT clearPayment(apselect_id) AS result "
-                 "FROM vendinfo JOIN apopen ON (apopen_vend_id=vend_id) "
-                 "              JOIN apselect ON (apselect_apopen_id=apopen_id) "
-                 "WHERE (vend_vendtype_id IN (SELECT vendtype_id"
-                 "                            FROM vendtype"
-                 "                            WHERE (vendtype_code ~ :vendtype_pattern)));" );
-        break;
-    }
-
-  _vendorgroup->bindValue(selectClearAll);
-  selectClearAll.exec();
+  MetaSQLQuery mql = mqlLoad("selectPayments", "clearall");
+  ParameterList params;
+  if (! setParams(params))
+    return;
+  selectClearAll = mql.toQuery(params);
   if (selectClearAll.first())
   {
     int result = selectClearAll.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("clearPayment", result),
-                  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Clearing Payment Information"),
+                             storedProcErrorLookup("clearPayment", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (selectClearAll.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Clearing Payment Information"),
+                                selectClearAll, __FILE__, __LINE__))
   {
-    systemError(this, selectClearAll.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
-  omfgThis->sPaymentsUpdated(-1, -1, TRUE);
+  omfgThis->sPaymentsUpdated(-1, -1, true);
 }
 
 void selectPayments::sSelect()
@@ -307,7 +263,7 @@ void selectPayments::sSelect()
         if(_bankaccnt->id() != -1)
           params.append("bankaccnt_id", _bankaccnt->id());
 
-        selectPayment newdlg(this, "", TRUE);
+        selectPayment newdlg(this, "", true);
         newdlg.set(params);
         if(newdlg.exec() != XDialog::Rejected)
           update = true;
@@ -332,14 +288,14 @@ void selectPayments::sSelectLine()
   int bankaccntid = _bankaccnt->id();
   if(bankaccntid == -1)
   {
-    selectBankAccount newdlg(this, "", TRUE);
+    selectBankAccount newdlg(this, "", true);
     newdlg.set(params);
     bankaccntid = newdlg.exec();
   }
 
   if (bankaccntid != -1)
   {
-    bool update = FALSE;
+    bool update = false;
     QList<XTreeWidgetItem*> list = _apopen->selectedItems();
     XTreeWidgetItem * cursor = 0;
     selectSelectLine.prepare("SELECT selectPayment(:apopen_id, :bankaccnt_id) AS result;");
@@ -362,30 +318,30 @@ void selectPayments::sSelectLine()
         int result = selectSelectLine.value("result").toInt();
         if (result < 0)
         {
-          systemError(this, cursor->text(0) + " " + cursor->text(2) + "\n" +
-                            storedProcErrorLookup("selectPayment", result),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                                 storedProcErrorLookup("selectPayment", result),
+                                 __FILE__, __LINE__);
           return;
         }
       }
-      else if (selectSelectLine.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                                    selectSelectLine, __FILE__, __LINE__))
       {
-        systemError(this, selectSelectLine.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
                 }
-      update = TRUE;
+      update = true;
     }
     }
     if(update)
-      omfgThis->sPaymentsUpdated(-1, -1, TRUE);
+      omfgThis->sPaymentsUpdated(-1, -1, true);
   }
 }
 
 void selectPayments::sClear()
 {
   XSqlQuery selectClear;
-  bool update = FALSE;
+  bool update = false;
   QList<XTreeWidgetItem*> list = _apopen->selectedItems();
   XTreeWidgetItem * cursor = 0;
   selectClear.prepare("SELECT clearPayment(:apopen_id) AS result;");
@@ -399,22 +355,23 @@ void selectPayments::sClear()
       int result = selectClear.value("result").toInt();
       if (result < 0)
       {
-        systemError(this, cursor->text(0) + " " + cursor->text(2) + "\n" +
-                          storedProcErrorLookup("clearPayment", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, cursor->text(0) + " " + cursor->text(2) + "\n" +
+                               tr(" Error Clearing Payment Information"),
+                               storedProcErrorLookup("clearPayment", result),
+                               __FILE__, __LINE__);
         return;
       }
     }
-    else if (selectClear.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Clearing Payment Information"),
+                                  selectClear, __FILE__, __LINE__))
     {
-      systemError(this, selectClear.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
-    update = TRUE;
+    update = true;
   }
 
   if(update)
-    omfgThis->sPaymentsUpdated(-1, -1, TRUE);
+    omfgThis->sPaymentsUpdated(-1, -1, true);
 }
 
 void selectPayments::sApplyAllCredits()
@@ -427,9 +384,10 @@ void selectPayments::sApplyAllCredits()
   selectApplyAllCredits = mql.toQuery(params);
   if (selectApplyAllCredits.first())
     sFillList();
-  else if (selectApplyAllCredits.lastError().type() != QSqlError::NoError)
+  else
   {
-    systemError(this, selectApplyAllCredits.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Applying all Credits"),
+                         selectApplyAllCredits, __FILE__, __LINE__);
     return;
   }
 }
@@ -466,6 +424,7 @@ void selectPayments::sFillList()
     return;
   params.append("voucher", tr("Voucher"));
   params.append("debitMemo", tr("Debit Memo"));
+  params.append("creditMemo", tr("Credit Memo"));
   if (_selectDate->currentIndex()==1)
     params.append("olderDate", _onOrBeforeDate->date());
   else if (_selectDate->currentIndex()==2)
@@ -479,9 +438,9 @@ void selectPayments::sFillList()
   MetaSQLQuery mql = mqlLoad("apOpenItems", "selectpayments");
   selectFillList = mql.toQuery(params);
   _apopen->populate(selectFillList,true);
-  if (selectFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                                selectFillList, __FILE__, __LINE__))
   {
-    systemError(this, selectFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -490,12 +449,34 @@ void selectPayments::sPopulateMenu(QMenu *pMenu,QTreeWidgetItem *selected)
 {
   QString status(selected->text(1));
   QAction *menuItem;
+  XTreeWidgetItem * item = (XTreeWidgetItem*)selected;
+
+  if (_apopen->currentItem()->text("doctype") == tr("Voucher"))
+  {
+    menuItem = pMenu->addAction(tr("View Voucher..."), this, SLOT(sViewVoucher()));
+    menuItem->setEnabled(_privileges->check("ViewVouchers") || _privileges->check("MaintainVouchers"));
+
+    if(item->rawValue("selected") == 0.0)
+    {
+      menuItem = pMenu->addAction(tr("Void Voucher..."), this, SLOT(sVoidVoucher()));
+      menuItem->setEnabled(_privileges->check("VoidPostedVouchers"));
+    }
+  }
+  
   XSqlQuery menu;
   menu.prepare( "SELECT apopen_status FROM apopen WHERE apopen_id=:apopen_id;");
   menu.bindValue(":apopen_id", _apopen->id());
   menu.exec();
   if (menu.first())
   {
+    menuItem = pMenu->addAction(tr("Edit A/P Open..."), this, SLOT(sEdit()));
+    menuItem->setEnabled(_privileges->check("EditAPOpenItem"));
+    
+    pMenu->addAction(tr("View A/P Open..."), this, SLOT(sView()));
+    
+    menuItem = pMenu->addAction(tr("View G/L Series..."), this, SLOT(sViewGLSeries()));
+    menuItem->setEnabled(_privileges->check("ViewGLTransactions"));
+
     if(menu.value("apopen_status").toString() == "O")
     {
       menuItem = pMenu->addAction(tr("On Hold"), this, SLOT(sOnHold()));
@@ -509,6 +490,54 @@ void selectPayments::sPopulateMenu(QMenu *pMenu,QTreeWidgetItem *selected)
   }
 }
 
+void selectPayments::sEdit()
+{
+  ParameterList params;
+  params.append("mode", "edit");
+  params.append("apopen_id", _apopen->id());
+  
+  apOpenItem newdlg(this, "", true);
+  newdlg.set(params);
+  newdlg.exec();
+  
+  sFillList();
+}
+
+void selectPayments::sView()
+{
+  ParameterList params;
+  params.append("mode", "view");
+  params.append("apopen_id", _apopen->id());
+  
+  apOpenItem newdlg(this, "", true);
+  newdlg.set(params);
+  newdlg.exec();
+}
+
+void selectPayments::sViewGLSeries()
+{
+  XSqlQuery dspViewGLSeries;
+  dspViewGLSeries.prepare("SELECT apopen_distdate, apopen_journalnumber"
+                          "  FROM apopen"
+                          " WHERE(apopen_id=:apopen_id);");
+  dspViewGLSeries.bindValue(":apopen_id", _apopen->id());
+  dspViewGLSeries.exec();
+  if(dspViewGLSeries.first())
+  {
+    ParameterList params;
+    params.append("startDate", dspViewGLSeries.value("apopen_distdate").toDate());
+    params.append("endDate", dspViewGLSeries.value("apopen_distdate").toDate());
+    params.append("journalnumber", dspViewGLSeries.value("apopen_journalnumber").toInt());
+    
+    dspGLSeries *newdlg = new dspGLSeries();
+    newdlg->set(params);
+    omfgThis->handleNewWindow(newdlg);
+  }
+  else
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Payment Information"),
+                       dspViewGLSeries, __FILE__, __LINE__);
+}
+
 void selectPayments::sOpen()
 {
   XSqlQuery open;
@@ -528,7 +557,7 @@ void selectPayments::sOnHold()
   {
     QMessageBox::critical( this, tr("Can not change Status"),
                           tr("<p>You cannot set this item as On Hold. "
-                             "This Item is already selected for payment."));
+                             "This Item is already approved for payment."));
     return;
   }
 
@@ -538,3 +567,116 @@ void selectPayments::sOnHold()
   onhold.exec();
   sFillList();
 }
+
+void selectPayments::sViewVoucher()
+{
+  int voheadid;
+  int poheadid;
+  XSqlQuery open;
+  open.prepare("SELECT vohead_id, COALESCE(pohead_id, -1) AS pohead_id "
+               "FROM vohead LEFT OUTER JOIN pohead ON (pohead_id=vohead_pohead_id) "
+               "WHERE vohead_number=:vohead_number;");
+  open.bindValue(":vohead_number", _apopen->currentItem()->text("apopen_docnumber"));
+  open.exec();
+  if (open.first())
+  {
+    voheadid = open.value("vohead_id").toInt();
+    poheadid = open.value("pohead_id").toInt();
+  }
+  else
+    return;
+  
+  if (!checkSitePrivs(voheadid))
+    return;
+  
+  ParameterList params;
+  params.append("mode", "view");
+  params.append("vohead_id", voheadid);
+  
+  if (poheadid == -1)
+  {
+    miscVoucher *newdlg = new miscVoucher();
+    newdlg->set(params);
+    omfgThis->handleNewWindow(newdlg);
+  }
+  else
+  {
+    voucher *newdlg = new voucher();
+    newdlg->set(params);
+    omfgThis->handleNewWindow(newdlg);
+  }
+}
+
+void selectPayments::sVoidVoucher()
+{
+  bool update = false;
+  QList<XTreeWidgetItem*> list = _apopen->selectedItems();
+  XTreeWidgetItem * cursor = 0;
+  XSqlQuery dspVoidVoucher;
+  dspVoidVoucher.prepare("SELECT voidApopenVoucher(:apopen_id, :voidDate) AS result;");
+  for(int i = 0; i < list.size(); i++)
+  {
+    cursor = (XTreeWidgetItem*)list.at(i);
+    if ( (cursor->rawValue("doctype") == tr("Voucher")) && (cursor->rawValue("selected") == 0.0) )
+    {
+      XDateInputDialog newdlg(this, "", true);
+      ParameterList params;
+      params.append("label", tr("On what date did you void the Voucher?"));
+      params.append("default", cursor->rawValue("apopen_docdate"));
+      newdlg.set(params);
+      int returnVal = newdlg.exec();
+      if (returnVal == XDialog::Accepted)
+      {
+        QDate voidDate = newdlg.getDate();
+        dspVoidVoucher.bindValue(":apopen_id", cursor->id());
+        dspVoidVoucher.bindValue(":voidDate", voidDate);
+        dspVoidVoucher.exec();
+      
+        if(dspVoidVoucher.first())
+        {
+          if(dspVoidVoucher.value("result").toInt() < 0)
+          {
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Voiding Voucher"),
+                                 dspVoidVoucher, __FILE__, __LINE__);
+            return;
+          }
+        }
+        else
+        {
+          ErrorReporter::error(QtCriticalMsg, this, tr("Voiding Voucher"),
+                               dspVoidVoucher, __FILE__, __LINE__);
+          return;
+        }
+        update = true;
+      }
+    }
+  }
+  if(update)
+    sFillList();
+}
+
+bool selectPayments::checkSitePrivs(int orderid)
+{
+  if (_preferences->boolean("selectedSites"))
+  {
+    XSqlQuery check;
+    check.prepare("SELECT checkVoucherSitePrivs(:voheadid) AS result;");
+    check.bindValue(":voheadid", orderid);
+    check.exec();
+    if (check.first())
+    {
+      if (!check.value("result").toBool())
+      {
+        QMessageBox::critical(this, tr("Access Denied"),
+                              tr("<p>You may not view or edit this Voucher as "
+                                 "it references a Site for which you have not "
+                                 "been granted privileges.")) ;
+        return false;
+      }
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Checking Privileges"),
+                                  check, __FILE__, __LINE__))
+      return false;
+  }
+  return true;
+}
