@@ -1,14 +1,15 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
  * to be bound by its terms.
  */
+#include <QDebug>
 #include <QBuffer>
-#include <QUiLoader>
+#include <QtUiTools/QUiLoader>
 #include <QMessageBox>
 #include <QPushButton>
 #include <QScriptEngine>
@@ -33,11 +34,13 @@ void setupSetupApi(QScriptEngine *engine)
   engine->globalObject().setProperty("setup", obj, QScriptValue::ReadOnly | QScriptValue::Undeletable);
 }
 
-setup::setup(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+setup::setup(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
-
+  if (! name)
+    setObjectName("setup");
+    
   _modules->addItem(tr("All"), Xt::AllModules);
   _modules->addItem(tr("Accounting"), Xt::AccountingModule);
   _modules->addItem(tr("Sales"), Xt::SalesModule);
@@ -73,6 +76,20 @@ setup::setup(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   insert(tr("Purchase"), "configurePO", Configure, Xt::PurchaseModule, mode("ConfigurePO"), 0 );
   insert(tr("Registration"), "registrationKey", Configure, Xt::SystemModule, mode("MaintainRegistrationKey"), 0 );
   insert(tr("Schedule"), "configureMS", Configure, Xt::ScheduleModule, mode("ConfigureMS"), 0 );
+
+  // TODO: remove this 4.10 hack when WF is self-contained
+  XSqlQuery wf("SELECT EXISTS("
+               "  SELECT 1"
+               "    FROM pg_proc"
+               "    JOIN pg_namespace n ON pronamespace = n.oid"
+               "   WHERE nspname = 'xt'"
+               "     AND proname = 'createwf_after_insert'"
+               "  ) AS isInstalled;");
+  if (wf.first() && wf.value("isInstalled").toBool())
+  {
+    insert(tr("Workflow"), "configureWF", Configure, Xt::SystemModule, mode("ConfigureWF"), 0 );
+  }
+
   insert(tr("Search Path"), "configureSearchPath", Configure, Xt::SystemModule, _privileges->isDba() ? cEdit : 0, 0);
 
   // Account Mappings
@@ -177,6 +194,9 @@ setup::setup(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
   modeVal = mode("MaintainOpportunityTypes");
   insert(tr("Opportunity Types"), "opportunityTypes", MasterInformation, Xt::CRMModule, modeVal, modeVal);
 
+  modeVal = mode("MaintainProjectTypes");
+  insert(tr("Project Types"), "projectTypes", MasterInformation, Xt::CRMModule, modeVal, modeVal);
+
   modeVal = mode("MaintainPlannerCodes", "ViewPlannerCodes");
   insert(tr("Planner Codes"), "plannerCodes", MasterInformation, Xt::InventoryModule | Xt::ScheduleModule | Xt::PurchaseModule, modeVal, modeVal);
 
@@ -560,7 +580,6 @@ void setup::setCurrentIndex(XTreeWidgetItem* item)
         if (x)
           x->set(params);
       }
-
       int idx = _stack->count();
       _itemMap[uiName].index = idx;
       _stack->addWidget(w);
