@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,8 +22,9 @@
 #include "printInvoice.h"
 #include "printPackingList.h"
 #include "storedProcErrorLookup.h"
+#include "errorReporter.h"
 
-shipOrder::shipOrder(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)
+shipOrder::shipOrder(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)
     : XDialog(parent, name, modal, fl)
 {
   setupUi(this);
@@ -34,11 +35,12 @@ shipOrder::shipOrder(QWidget* parent, const char* name, bool modal, Qt::WFlags f
   connect(_shipment, SIGNAL(newId(int)),    this, SLOT(sFillList()));
   connect(_shipment, SIGNAL(newId(int)),    this, SLOT(sFillTracknum()));
   connect(_order,    SIGNAL(newId(int,QString)),    this, SLOT(sHandleOrder()));
+  connect(_warehouse,SIGNAL(newID(int)),            this, SLOT(sHandleOrder()));
   connect(_tracknum, SIGNAL(activated(const QString&)), this, SLOT(sFillFreight()));
   connect(_tracknum, SIGNAL(currentIndexChanged(const QString&)), this, SLOT(sFillFreight()));
   connect(_tracknum, SIGNAL(highlighted(const QString&)), this, SLOT(sFillFreight()));
 
-  _captive = FALSE;
+  _captive = false;
 
   _coitem->addColumn( tr("#"),           _whsColumn,  Qt::AlignCenter , true, "linenumber");
   _coitem->addColumn( tr("Item Number"), _itemColumn, Qt::AlignLeft   , true, "item_number");
@@ -50,10 +52,17 @@ shipOrder::shipOrder(QWidget* parent, const char* name, bool modal, Qt::WFlags f
   sHandleButtons();
   _reject = false;
 
+  if (!_metrics->boolean("MultiWhs"))
+  {
+    _warehouseLit->hide();
+    _warehouse->hide();
+  }
+  
   _order->setAllowedStatuses(OrderLineEdit::Open);
   _order->setAllowedTypes(OrderLineEdit::Sales |
                           OrderLineEdit::Transfer);
-  _order->setFromSitePrivsEnforced(TRUE);
+  _order->setFromSitePrivsEnforced(true);
+  _order->setFocus();
 
   _transDate->setEnabled(_privileges->check("AlterTransactionDates"));
   _transDate->setDate(omfgThis->dbDate(), true);
@@ -75,11 +84,18 @@ void shipOrder::languageChange()
 enum SetResponse shipOrder::set(const ParameterList &pParams)
 {
   XSqlQuery shipet;
+  XSqlQuery siteq;
   XDialog::set(pParams);
   QString  returnValue;
   QVariant param;
   bool     valid;
 
+  param = pParams.value("transdate", &valid);
+  if (valid)
+  {
+    _transDate->setDate(param.toDate(), true);
+  }
+  
   param = pParams.value("shiphead_id", &valid);
   if (valid)
   {
@@ -93,16 +109,38 @@ enum SetResponse shipOrder::set(const ParameterList &pParams)
       _captive = true;	// so order handling can reject if necessary
       if (shipet.value("shiphead_order_type").toString() == "SO")
       {
+        siteq.prepare( "SELECT DISTINCT itemsite_warehous_id "
+                       "FROM shiphead JOIN shipitem ON (shipitem_shiphead_id=shiphead_id)"
+                       "              JOIN coitem ON (coitem_id=shipitem_orderitem_id)"
+                       "              JOIN itemsite ON (itemsite_id=coitem_itemsite_id)"
+                       "WHERE (shiphead_id=:shiphead_id);" );
+        siteq.bindValue(":shiphead_id", param.toInt());
+        siteq.exec();
+        if (siteq.first())
+        {
+          _warehouse->setId(siteq.value("itemsite_warehous_id").toInt());
+        }
         _order->setId(param.toInt(), "SO");
       }
       else if (shipet.value("shiphead_order_type").toString() == "TO")
       {
+        siteq.prepare( "SELECT DISTINCT tohead_src_warehous_id "
+                      "FROM shiphead JOIN shipitem ON (shipitem_shiphead_id=shiphead_id)"
+                      "              JOIN toitem ON (toitem_id=shipitem_orderitem_id)"
+                      "              JOIN tohead ON (tohead_id=toitem_tohead_id)"
+                      "WHERE (shiphead_id=:shiphead_id);" );
+        siteq.bindValue(":shiphead_id", param.toInt());
+        siteq.exec();
+        if (siteq.first())
+        {
+          _warehouse->setId(siteq.value("tohead_src_warehous_id").toInt());
+        }
         _order->setId(param.toInt(), "TO");
       }
     }
-    else if (shipet.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                  shipet, __FILE__, __LINE__))
     {
-      systemError(this, shipet.lastError().databaseText(), __FILE__, __LINE__);
       return UndefinedError;
     }
 
@@ -117,6 +155,9 @@ enum SetResponse shipOrder::set(const ParameterList &pParams)
 
 void shipOrder::sShip()
 {
+  // Disconnect button to prevent accidental signals
+  disconnect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
+  
   XSqlQuery shipq;
   shipq.prepare( "UPDATE shiphead "
 	     "   SET shiphead_shipvia=:shiphead_shipvia,"
@@ -130,9 +171,10 @@ void shipOrder::sShip()
   shipq.bindValue(":shiphead_tracknum",	_tracknum->currentText());
   shipq.bindValue(":shiphead_id",		_shipment->id());
   shipq.exec();
-  if (shipq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Shipping Information"),
+                                shipq, __FILE__, __LINE__))
   {
-    systemError(this, shipq.lastError().databaseText(), __FILE__, __LINE__);
+    connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
     return;
   }
 
@@ -166,31 +208,38 @@ void shipOrder::sShip()
       shipq.exec();
       while (shipq.next())
       {
-	// missing transit itemsite is fatal here but not missing dest
-	int transis = itemSite::createItemSite(this,
-					       shipq.value("itemsite_id").toInt(),
-				     shipq.value("tohead_trns_warehous_id").toInt(),
-				     false);
-	int destis  = itemSite::createItemSite(this,
-					       shipq.value("itemsite_id").toInt(),
-				     shipq.value("tohead_dest_warehous_id").toInt(),
-				     true);
-	if (transis <= 0 || destis < 0)
-	  return;
+        // missing transit itemsite is fatal here but not missing dest
+        int transis = itemSite::createItemSite(this,
+                                               shipq.value("itemsite_id").toInt(),
+                                               shipq.value("tohead_trns_warehous_id").toInt(),
+                                               false);
+        int destis  = itemSite::createItemSite(this,
+                                               shipq.value("itemsite_id").toInt(),
+                                               shipq.value("tohead_dest_warehous_id").toInt(),
+                                               true);
+        if (transis <= 0 || destis < 0)
+        {
+          connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
+          return;
+        }
       }
-      if (shipq.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                    shipq, __FILE__, __LINE__))
       {
-        systemError(this, shipq.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
+        return;
       }
+      connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
       sShip();	// beware of endless loop if you change createItemSite err check
       return;	// return because the recursive call cleans up for us
     }
     else if (result < 0)
     {
       rollback.exec();
-      systemError(this, storedProcErrorLookup("shipShipment", result),
-		  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                               storedProcErrorLookup("shipShipment", result),
+                               __FILE__, __LINE__);
+      connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
       return;
     }
   }
@@ -202,14 +251,18 @@ void shipOrder::sShip()
       errorStr = tr("One or more required accounts are not set or set incorrectly."
                     " Please make sure that all your Cost Category and Sales Account Assignments"
                     " are complete and correct.");
-    systemError(this, errorStr, __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                         errorStr, __FILE__, __LINE__);
+    connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
     return;
   }
 
   shipq.exec("COMMIT;");
   if (shipq.lastError().type() != QSqlError::NoError)
   {
-    systemError(this, shipq.lastError().databaseText(), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                         shipq, __FILE__, __LINE__);
+    connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
     return;
   }
   // END the transaction
@@ -220,7 +273,7 @@ void shipOrder::sShip()
     params.append("shiphead_id", _shipment->id());
     params.append("print");
 
-    printPackingList newdlg(this, "", TRUE);
+    printPackingList newdlg(this, "", true);
     newdlg.set(params);
   }
 
@@ -234,66 +287,66 @@ void shipOrder::sShip()
       int cobmiscid = shipq.value("result").toInt();
       if (cobmiscid < 0)
       {
-	systemError(this,
-	      storedProcErrorLookup("selectUninvoicedShipment", cobmiscid),
-	      __FILE__, __LINE__);
-	return;
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                               storedProcErrorLookup("selectUninvoicedShipment", cobmiscid),
+                               __FILE__, __LINE__);
+        connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
+        return;
       }
       else if (0 == cobmiscid)
       {
-	QMessageBox::information(this, tr("Already Invoiced"),
-				 tr("<p>This shipment appears to have been "
-				   "invoiced already. It will not be selected "
-				   "for billing again."));
+        QMessageBox::information(this, tr("Already Invoiced"),
+                                 tr("<p>This shipment appears to have been "
+                                    "invoiced already. It will not be selected "
+                                    "for billing again."));
       }
       else if (_create->isChecked())
       {
         shipq.prepare("SELECT createInvoice(:cobmisc_id) AS result;");
-	shipq.bindValue(":cobmisc_id", cobmiscid);
-	shipq.exec();
-	if (shipq.first())
-	{
-	  int result = shipq.value("result").toInt();
-	  if (result < 0)
-	  {
-	    systemError(this,
-		      storedProcErrorLookup("postBillingSelection", result),
-		      __FILE__, __LINE__);
-	    return;
-	  }
-	  ParameterList params;
-	  params.append("invchead_id", result);
-
-	  printInvoice newdlg(this, "", TRUE);
-	  newdlg.set(params);
-	  newdlg.exec();
-
-	  omfgThis->sInvoicesUpdated(result, TRUE);
-	  omfgThis->sSalesOrdersUpdated(_order->id());
-	}
-	else if (shipq.lastError().type() != QSqlError::NoError)
-	{
-	  systemError(this, shipq.lastError().databaseText() +
-		      tr("<p>Although Sales Order %1 was successfully shipped "
-			 "and selected for billing, the Invoice was not "
-			 "created properly. You may need to create an Invoice "
-			 "manually from the Billing Selection.")
-			.arg(_order->id()),
-		      __FILE__, __LINE__);
-	  return;
-	}
-
-	omfgThis->sBillingSelectionUpdated(_order->id(), TRUE);
+        shipq.bindValue(":cobmisc_id", cobmiscid);
+        shipq.exec();
+        if (shipq.first())
+        {
+          int result = shipq.value("result").toInt();
+          if (result < 0)
+          {
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Posting Billing Information"),
+                                   storedProcErrorLookup("postBillingSelection", result),
+                                   __FILE__, __LINE__);
+            connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
+            return;
+          }
+          ParameterList params;
+          params.append("invchead_id", result);
+          
+          printInvoice newdlg(this, "", true);
+          newdlg.set(params);
+          newdlg.exec();
+          
+          omfgThis->sInvoicesUpdated(result, true);
+          omfgThis->sSalesOrdersUpdated(_order->id());
+        }
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("<p>Although Sales Order %1 was successfully shipped "
+                                                              "and selected for billing, the Invoice was not "
+                                                              "created properly. You may need to create an Invoice "
+                                                              "manually from the Billing Selection.")
+                                      .arg(_order->id()),
+                                      shipq, __FILE__, __LINE__))
+        {
+          connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
+          return;
+        }
+        
+        omfgThis->sBillingSelectionUpdated(_order->id(), true);
       }
     }
-    else if (shipq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("<p>Although Sales Order %1 was successfully shipped, "
+                                                          "it was not selected for billing. You must manually "
+                                                          "select this Sales Order for Billing.")
+                                  .arg(_order->id()),
+                                  shipq, __FILE__, __LINE__))
     {
-      systemError(this, shipq.lastError().databaseText() +
-		  tr("<p>Although Sales Order %1 was successfully shipped, "
-		     "it was not selected for billing. You must manually "
-		     "select this Sales Order for Billing.")
-		    .arg(_order->id()),
-		  __FILE__, __LINE__);
+      connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
       return;
     }
   }
@@ -301,60 +354,62 @@ void shipOrder::sShip()
   if (_order->isTO() && _receive->isChecked())
   {
     QString recverr = tr("<p>Receiving inventory for this Transfer Order "
-			"failed although Shipping the Order succeeded. "
-			"Manually receive this order using the Enter Order "
-			"Receipt window.");
+                         "failed although Shipping the Order succeeded. "
+                         "Manually receive this order using the Enter Order "
+                         "Receipt window.");
     shipq.exec("BEGIN;");
     ParameterList params;
-
+    
     if (_metrics->boolean("MultiWhs"))
       params.append("MultiWhs");
     params.append("tohead_id",   _order->id());
     params.append("orderid",     _order->id());
     params.append("ordertype",   "TO");
     params.append("shiphead_id", _shipment->id());
-
+    
     MetaSQLQuery recvm = mqlLoad("receipt", "receiveAll");
     shipq = recvm.toQuery(params);
-
+    
     while (shipq.next())
     {
       int result = shipq.value("result").toInt();
       if (result < 0)
       {
-	rollback.exec();
-	systemError(this,
-		    recverr + storedProcErrorLookup("enterReceipt", result),
-		    __FILE__, __LINE__);
+        rollback.exec();
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                 storedProcErrorLookup("enterReceipt", result),
+                                 __FILE__, __LINE__);
       }
       omfgThis->sPurchaseOrderReceiptsUpdated();
     }
     if (shipq.lastError().type() != QSqlError::NoError)
     {
       rollback.exec();
-      systemError(this, recverr + "<br>" + shipq.lastError().databaseText(),
-		  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                           shipq, __FILE__, __LINE__);
     }
-
+    
     shipq.exec("COMMIT;");
     if (shipq.lastError().type() != QSqlError::NoError)
     {
-      systemError(this,
-		  recverr + "<br>" + shipq.lastError().databaseText(),
-		  __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                           shipq, __FILE__, __LINE__);
     }
-
+    
     ParameterList recvParams;
     recvParams.append("tohead_id", _order->id());
     enterPoReceipt *newdlg = new enterPoReceipt();
     newdlg->set(recvParams);
-
+    
     // to address bug 5680, replace
     // omfgThis->handleNewWindow(newdlg, Qt::ApplicationModal);
     // with
     newdlg->sPost();
     // end of replacement
   }
+  
+  // Reconnect button
+  connect(_ship,     SIGNAL(clicked()),     this, SLOT(sShip()));
 
   if (_captive)
     accept();
@@ -362,11 +417,10 @@ void shipOrder::sShip()
   {
     _order->setId(-1);
     _order->setEnabled(true);
-    sHandleButtons();
     _billToName->clear();
     _shipToName->clear();
     _shipToAddr1->clear();
-    _freight->setEnabled(TRUE);
+    _freight->setEnabled(true);
     _freight->reset();
     _shipVia->clear();
     _shipment->clear();
@@ -386,16 +440,16 @@ void shipOrder::sShip()
   if (! _shipment->number().isEmpty())
     shipq.bindValue(":shiphead_number", _shipment->number());
   shipq.exec();
-  if (shipq.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Updating Shipping Information"),
+                                shipq, __FILE__, __LINE__))
   {
-    systemError(this, shipq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
 
 void shipOrder::sHandleOrder()
 {
-  if (_order->id() < 0)
+  if (_order->id() < 0 || _warehouse->id() < 0)
   {
     _shipment->clear();
     _billToName->setText("");
@@ -422,23 +476,27 @@ void shipOrder::sHandleSo()
   sHandleButtons();
 
 
-  shipHandleSo.prepare( "SELECT cohead_holdtype, cust_name, cohead_shiptoname, "
-             "       cohead_shiptoaddress1, cohead_curr_id, cohead_freight "
-             "FROM cohead, custinfo "
-             "WHERE ((cohead_cust_id=cust_id) "
-             "  AND  (cohead_id=:sohead_id));" );
+  shipHandleSo.prepare( "SELECT DISTINCT"
+                        "       soHoldType(cohead_id) AS holdtype, cust_name, cohead_shiptoname, "
+                        "       cohead_shiptoaddress1, cohead_curr_id, cohead_freight "
+                        "FROM cohead JOIN custinfo ON (cust_id=cohead_cust_id)"
+                        "            JOIN coitem ON (coitem_cohead_id=cohead_id)"
+                        "            JOIN itemsite ON (itemsite_id=coitem_itemsite_id) "
+                        "WHERE ((cohead_id=:sohead_id)"
+                        "  AND  (itemsite_warehous_id=:warehous_id));" );
   shipHandleSo.bindValue(":sohead_id", _order->id());
+  shipHandleSo.bindValue(":warehous_id", _warehouse->id());
   shipHandleSo.exec();
   if (shipHandleSo.first())
   {
     QString msg;
-    if ( (shipHandleSo.value("cohead_holdtype").toString() == "C"))
+    if ( (shipHandleSo.value("holdtype").toString() == "C"))
       msg = storedProcErrorLookup("shipShipment", -12);
-    else if (shipHandleSo.value("cohead_holdtype").toString() == "P")
+    else if (shipHandleSo.value("holdtype").toString() == "P")
       msg = storedProcErrorLookup("shipShipment", -13);
-    else if (shipHandleSo.value("cohead_holdtype").toString() == "R")
+    else if (shipHandleSo.value("holdtype").toString() == "R")
       msg = storedProcErrorLookup("shipShipment", -14);
-    else if (shipHandleSo.value("cohead_holdtype").toString() == "S")
+    else if (shipHandleSo.value("holdtype").toString() == "S")
       msg = storedProcErrorLookup("shipShipment", -15);
 
     if (! msg.isEmpty())
@@ -462,16 +520,17 @@ void shipOrder::sHandleSo()
     _shipToName->setText(shipHandleSo.value("cohead_shiptoname").toString());
     _shipToAddr1->setText(shipHandleSo.value("cohead_shiptoaddress1").toString());
 
-    QString sql( "SELECT shiphead_id "
-                 "FROM shiphead "
-                 "WHERE ( (NOT shiphead_shipped)"
-                 "<? if exists(\"shiphead_id\") ?>"
-                 " AND (shiphead_id=<? value(\"shiphead_id\") ?>)"
-                 "<? endif ?>"
-                 " AND (shiphead_order_id=<? value(\"sohead_id\") ?>)"
-                 " AND (shiphead_order_type='SO'));" );
+    QString sql("SELECT shiphead_id "
+                "FROM shiphead "
+                "<? if exists(\"shiphead_id\") ?>"
+                "WHERE (shiphead_id=<? value(\"shiphead_id\") ?>)"
+                "<? else ?>"
+                "WHERE (shiphead_number=getOpenShipment('SO', <? value(\"sohead_id\") ?>, <? value(\"warehous_id\") ?>))"
+                "<? endif ?>"
+                ";" );
     ParameterList params;
     params.append("sohead_id", _order->id());
+    params.append("warehous_id", _warehouse->id());
     if (_shipment->isValid())
       params.append("shiphead_id", _shipment->id());
     MetaSQLQuery mql(sql);
@@ -488,15 +547,16 @@ void shipOrder::sHandleSo()
         _shipment->setEnabled(true);
       }
     }
-    else if (shipHandleSo.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                  shipHandleSo, __FILE__, __LINE__))
     {
-      systemError(this, shipHandleSo.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else if (_shipment->isValid())
     {
       params.clear();
       params.append("sohead_id", _order->id());
+      params.append("warehous_id", _warehouse->id());
       MetaSQLQuery mql(sql);
       shipHandleSo = mql.toQuery(params);
       if (shipHandleSo.first())
@@ -509,9 +569,9 @@ void shipOrder::sHandleSo()
           _shipment->setEnabled(true);
         }
       }
-      else if (shipHandleSo.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                    shipHandleSo, __FILE__, __LINE__))
       {
-        systemError(this, shipHandleSo.lastError().databaseText(), __FILE__, __LINE__);
         return;
       }
       else
@@ -526,9 +586,9 @@ void shipOrder::sHandleSo()
       return;
     }
   }
-  else if (shipHandleSo.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                shipHandleSo, __FILE__, __LINE__))
   {
-    systemError(this, shipHandleSo.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -543,15 +603,16 @@ void shipOrder::sHandleTo()
   sHandleButtons();
 
   shipHandleTo.prepare("SELECT tohead_freight_curr_id, tohead_destname,"
-            "       tohead_destaddress1,"
-            "       SUM(toitem_freight) + tohead_freight AS freight "
-            "FROM tohead, toitem "
-            "WHERE ((toitem_tohead_id=tohead_id)"
-            "  AND  (toitem_status<>'X')"
-            "  AND  (tohead_id=:tohead_id)) "
-            "GROUP BY tohead_freight_curr_id, tohead_destname,"
-            "         tohead_destaddress1, tohead_freight;");
+                       "       tohead_destaddress1,"
+                       "       SUM(toitem_freight) + tohead_freight AS freight "
+                       "FROM tohead JOIN toitem ON (toitem_tohead_id=tohead_id) "
+                       "WHERE ((toitem_status<>'X')"
+                       "  AND  (tohead_src_warehous_id=:warehous_id)"
+                       "  AND  (tohead_id=:tohead_id)) "
+                       "GROUP BY tohead_freight_curr_id, tohead_destname,"
+                       "         tohead_destaddress1, tohead_freight;");
   shipHandleTo.bindValue(":tohead_id", _order->id());
+  shipHandleTo.bindValue(":warehous_id", _warehouse->id());
   shipHandleTo.exec();
   if (shipHandleTo.first())
   {
@@ -561,22 +622,23 @@ void shipOrder::sHandleTo()
     _shipToName->setText(shipHandleTo.value("tohead_destname").toString());
     _shipToAddr1->setText(shipHandleTo.value("tohead_destaddress1").toString());
   }
-  else if (shipHandleTo.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                shipHandleTo, __FILE__, __LINE__))
   {
-    systemError(this, shipHandleTo.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
-  QString sql( "SELECT shiphead_id "
-               "FROM shiphead "
-               "WHERE ( (NOT shiphead_shipped)"
-               "<? if exists(\"shiphead_id\") ?>"
-               " AND (shiphead_id=<? value(\"shiphead_id\") ?>)"
-               "<? endif ?>"
-               " AND (shiphead_order_id=<? value(\"tohead_id\") ?>)"
-               " AND (shiphead_order_type='TO'));" );
+  QString sql("SELECT shiphead_id "
+              "FROM shiphead "
+              "<? if exists(\"shiphead_id\") ?>"
+              "WHERE (shiphead_id=<? value(\"shiphead_id\") ?>)"
+              "<? else ?>"
+              "WHERE (shiphead_number=getOpenShipment('TO', <? value(\"tohead_id\") ?>, <? value(\"warehous_id\") ?>))"
+              "<? endif ?>"
+              ";" );
   ParameterList params;
   params.append("tohead_id", _order->id());
+  params.append("warehous_id", _warehouse->id());
   if (_shipment->isValid())
     params.append("shiphead_id", _shipment->id());
   MetaSQLQuery mql(sql);
@@ -593,15 +655,16 @@ void shipOrder::sHandleTo()
       _shipment->setEnabled(true);
     }
   }
-  else if (shipHandleTo.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                shipHandleTo, __FILE__, __LINE__))
   {
-    systemError(this, shipHandleTo.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else if (_shipment->isValid())
   {
     params.clear();
     params.append("tohead_id", _order->id());
+    params.append("warehous_id", _warehouse->id());
     MetaSQLQuery mql(sql);
     shipHandleTo = mql.toQuery(params);
     if (shipHandleTo.first())
@@ -614,9 +677,9 @@ void shipOrder::sHandleTo()
         _shipment->setEnabled(true);
       }
     }
-    else if (shipHandleTo.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                  shipHandleTo, __FILE__, __LINE__))
     {
-      systemError(this, shipHandleTo.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else
@@ -660,12 +723,12 @@ void shipOrder::sFillList()
     XSqlQuery shipq;
     shipq.prepare("SELECT shiphead_order_id, shiphead_shipvia, shiphead_order_type,"
                   "       shiphead_tracknum, shiphead_freight, shiphead_freight_curr_id,"
-		  "       COALESCE(shipchrg_custfreight, TRUE) AS custfreight,"
-		  "       COALESCE(shiphead_shipdate,CURRENT_DATE) AS effective "
-		  "FROM shiphead LEFT OUTER JOIN "
-		  "     shipchrg ON (shiphead_shipchrg_id=shipchrg_id) "
-		  "WHERE ( (NOT shiphead_shipped)"
-		  " AND (shiphead_id=:shiphead_id));" );
+                  "       COALESCE(shipchrg_custfreight, true) AS custfreight,"
+                  "       COALESCE(shiphead_shipdate,CURRENT_DATE) AS effective "
+                  "FROM shiphead LEFT OUTER JOIN "
+                  "     shipchrg ON (shiphead_shipchrg_id=shipchrg_id) "
+                  "WHERE ( (NOT shiphead_shipped)"
+                  " AND (shiphead_id=:shiphead_id));" );
     shipq.bindValue(":shiphead_id", _shipment->id());
     shipq.exec();
     if (shipq.first())
@@ -677,22 +740,22 @@ void shipOrder::sFillList()
 
       if (shipq.value("custfreight").toBool())
       {
-	_freight->setEnabled(TRUE);
+	_freight->setEnabled(true);
 	_freight->set(shipq.value("shiphead_freight").toDouble(),
 		      shipq.value("shiphead_freight_curr_id").toInt(),
 		      shipq.value("effective").toDate(), false);
       }
       else
       {
-	_freight->setEnabled(FALSE);
+	_freight->setEnabled(false);
 	_freight->set(0,
 		      shipq.value("shiphead_freight_curr_id").toInt(),
 		      shipq.value("effective").toDate());
       }
     }
-    else if (shipq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                  shipq, __FILE__, __LINE__))
     {
-      systemError(this, shipq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
     else
@@ -753,9 +816,9 @@ void shipOrder::sFillList()
     MetaSQLQuery itemm(items);
     shipq = itemm.toQuery(itemp);
     _coitem->populate(shipq);
-    if (shipq.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                  shipq, __FILE__, __LINE__))
     {
-      systemError(this, shipq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -776,9 +839,9 @@ void shipOrder::sFillList()
     shipq = valm.toQuery(itemp);	// shared parameters
     if(shipq.first())
       _shipValue->setDouble(shipq.value("value").toDouble());
-    else if (shipq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                  shipq, __FILE__, __LINE__))
     {
-      systemError(this, shipq.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -787,7 +850,7 @@ void shipOrder::sFillList()
     _billToName->clear();
     _shipToName->clear();
     _shipToAddr1->clear();
-    _freight->setEnabled(TRUE);
+    _freight->setEnabled(true);
     _freight->reset();
     _shipVia->clear();
     _tracknum->clear();
@@ -831,13 +894,13 @@ void shipOrder::sFillFreight()
     shipdataQ.exec();
     if (shipdataQ.first())
     {
-      _freight->setEnabled(TRUE);
+      _freight->setEnabled(true);
       _freight->setLocalValue(shipdataQ.value("shipdatasum_base_freight").toDouble());
       _shipVia->setText(shipdataQ.value("shipper_data").toString());
     }
-    else if (shipdataQ.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                  shipdataQ, __FILE__, __LINE__))
     {
-      systemError(this, shipdataQ.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -872,9 +935,9 @@ void shipOrder::sFillTracknum()
     shipdataQ.exec();
     if (shipdataQ.first())
       _tracknum->populate(shipdataQ);
-    else if (shipdataQ.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Shipping Information"),
+                                  shipdataQ, __FILE__, __LINE__))
     {
-      systemError(this, shipdataQ.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
@@ -892,9 +955,9 @@ void shipOrder::calcFreight()
     {
       _freight->setLocalValue(data.value("freight").toDouble());
     }
-    else if (data.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Calculating Shipping Freight"),
+                                  data, __FILE__, __LINE__))
     {
-      systemError(this, data.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
   }
