@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,10 +22,11 @@ class shipTo : public XDialog, public Ui::shipTo
     Q_OBJECT
 
 public:
-    shipTo(QWidget* = 0, const char* = 0, bool = false, Qt::WFlags = 0);
+    shipTo(QWidget* = 0, const char* = 0, bool = false, Qt::WindowFlags = 0);
     ~shipTo();
 
     Q_INVOKABLE virtual int id() const;
+    Q_INVOKABLE virtual int mode() const;
 
 public slots:
     virtual SetResponse set(const ParameterList & pParams );
