@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -25,10 +25,13 @@
 
 #include "login2.h"
 #include "storedProcErrorLookup.h"
+#include "version.h"
+
+#include "errorReporter.h"
 
 #define DEBUG   false
 
-// TODO: XDialog should have a default implementation that returns FALSE
+// TODO: XDialog should have a default implementation that returns false
 bool syncCompanies::userHasPriv(const int pMode)
 {
   if (DEBUG)
@@ -49,16 +52,16 @@ void syncCompanies::setVisible(bool visible)
 
   else if (! userHasPriv())
   {
-    systemError(this,
-		tr("You do not have sufficient privilege to view this window"),
-		__FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: Insufficient privileges to view this window")
+                         .arg(windowTitle()),__FILE__,__LINE__);
     deleteLater();
   }
   else
     XWidget::setVisible(true);
 }
 
-syncCompanies::syncCompanies(QWidget* parent, const char* name, Qt::WFlags fl)
+syncCompanies::syncCompanies(QWidget* parent, const char* name, Qt::WindowFlags fl)
 : XWidget(parent, name, fl)
 {
   setupUi(this);
@@ -100,9 +103,9 @@ void syncCompanies::sFillList()
          "ORDER BY period_start, period_end;");
   syncFillList.exec();
   _period->populate(syncFillList);
-  if (syncFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Accounting Period Information"),
+                                syncFillList, __FILE__, __LINE__))
   {
-    systemError(this, syncFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
@@ -115,9 +118,9 @@ void syncCompanies::sFillList()
 
   syncFillList.exec();
   _company->populate(syncFillList);
-  if (syncFillList.lastError().type() != QSqlError::NoError)
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Company Information"),
+                                syncFillList, __FILE__, __LINE__))
   {
-    systemError(this, syncFillList.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -128,17 +131,13 @@ void syncCompanies::sSync()
   if (DEBUG)
     qDebug("syncCompanies::sSync()");
 
-  QProgressDialog progress;
-  progress.setWindowModality(Qt::ApplicationModal);
-  progress.setAutoClose(false);
-
   XSqlQuery lbaseq;
   lbaseq.exec("SELECT * FROM curr_symbol WHERE curr_base;");
   if (lbaseq.first())
     ; // keep the query results for later comparisons
-  else if (lbaseq.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Currency Information"),
+                                lbaseq, __FILE__, __LINE__))
   {
-    systemError(this, lbaseq.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
   else
@@ -151,6 +150,7 @@ void syncCompanies::sSync()
   }
 
   int errorCount = 0;
+  bool canceled = false;
   QList<XTreeWidgetItem*> company = _company->selectedItems();
   for (int i = 0; i < company.size(); i++)
   {
@@ -193,9 +193,9 @@ void syncCompanies::sSync()
         continue;
       }
     }
-    else if (co.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Company Information"),
+                                  co, __FILE__, __LINE__))
     {
-      systemError(this, co.lastError().databaseText(), __FILE__, __LINE__);
       errorCount++;
       continue;
     }
@@ -220,24 +220,18 @@ void syncCompanies::sSync()
       qDebug() << "";
     }
 
-    if (progress.wasCanceled())
-      break;
-
     buildDatabaseURL(dbURL, protocol, host, db, port);
     if (DEBUG)
       qDebug("syncCompanies::sSync() dbURL before login2 = %s", qPrintable(dbURL));
 
-    progress.setLabelText(tr("Synchronizing Company %1 (%2)")
-                                       .arg(c->rawValue("company_number").toString())
-                                       .arg(dbURL));
-
     ParameterList params;
     params.append("databaseURL", dbURL);
     params.append("multipleConnections");
+    params.append("applicationName", _ConnAppName);
 
     login2 newdlg(this, "testLogin", false);
     // disallow changing connection info
-    newdlg._options->setEnabled(false);
+    //newdlg._options->setEnabled(false);
     //newdlg._demoOption->setEnabled(false);
     //newdlg._otherOption->setEnabled(false);
     newdlg.set(params);
@@ -253,6 +247,14 @@ void syncCompanies::sSync()
     }
 
     dbURL = newdlg._databaseURL;
+
+    QProgressDialog progress;
+    progress.setWindowModality(Qt::ApplicationModal);
+    progress.setAutoClose(false);
+    progress.setLabelText(tr("Synchronizing Company %1 (%2)")
+                                       .arg(c->rawValue("company_number").toString())
+                                       .arg(dbURL));
+
     if (DEBUG)
       qDebug("syncCompanies::sSync() dbURL after login2 = %s", qPrintable(dbURL));
     parseDatabaseURL(dbURL, protocol, host, db, port);
@@ -284,9 +286,9 @@ void syncCompanies::sSync()
       rmq.exec();
       if (rmq.first())
         ; // good - keep going
-      else if (rmq.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Privilege Information"),
+                                    rmq, __FILE__, __LINE__))
       {
-        systemError(this, rmq.lastError().databaseText(), __FILE__, __LINE__);
         errorCount++;
         continue;
       }
@@ -316,9 +318,9 @@ void syncCompanies::sSync()
           continue;
         }
       }
-      else if (rmq.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Metric Information"),
+                                    rmq, __FILE__, __LINE__))
       {
-        systemError(this, rmq.lastError().databaseText(), __FILE__, __LINE__);
         continue;
       }
 
@@ -327,9 +329,9 @@ void syncCompanies::sSync()
       rmq.exec();
       if (rmq.first())
         ; // nothing to do
-      else if (rmq.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Company Information"),
+                                    rmq, __FILE__, __LINE__))
       {
-        systemError(this, rmq.lastError().databaseText(), __FILE__, __LINE__);
         errorCount++;
         continue;
       }
@@ -357,9 +359,9 @@ void syncCompanies::sSync()
         syncSync.exec();
         if (syncSync.first())
           ; // nothing to do
-        else if (syncSync.lastError().type() != QSqlError::NoError)
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Profit Center Information"),
+                                      syncSync, __FILE__, __LINE__))
         {
-          systemError(this, syncSync.lastError().databaseText(), __FILE__, __LINE__);
           errorCount++;
           // don't break/continue - do as much as we can
         }
@@ -370,19 +372,19 @@ void syncCompanies::sSync()
           syncSync.bindValue(":prftcntr_number",  rmq.value("accnt_profit"));
           syncSync.bindValue(":prftcntr_descrip", rmq.value("prftcntr_descrip"));
           syncSync.exec();
-          if (syncSync.lastError().type() != QSqlError::NoError)
+          if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Profit Center Information"),
+                                        syncSync, __FILE__, __LINE__))
           {
-            systemError(this, syncSync.lastError().databaseText(), __FILE__, __LINE__);
             errorCount++;
             // don't break/continue - do as much as we can
           }
         }
       } // next profit center
-      if (rmq.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Profit Center Information "),
+                                    rmq, __FILE__, __LINE__))
       {
-	systemError(this, rmq.lastError().databaseText(), __FILE__, __LINE__);
         errorCount++;
-	return;
+        return;
       }
 
       rmq.prepare("SELECT DISTINCT accnt_sub "
@@ -397,9 +399,9 @@ void syncCompanies::sSync()
         syncSync.exec();
         if (syncSync.first())
           ; // nothing to do
-        else if (syncSync.lastError().type() != QSqlError::NoError)
+        else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Sub Account Information"),
+                                      syncSync, __FILE__, __LINE__))
         {
-          systemError(this, syncSync.lastError().databaseText(), __FILE__, __LINE__);
           errorCount++;
           // don't break/continue - do as much as we can
         }
@@ -410,19 +412,19 @@ void syncCompanies::sSync()
           syncSync.bindValue(":subaccnt_number",  rmq.value("accnt_sub"));
           syncSync.bindValue(":subaccnt_descrip", rmq.value("subaccnt_descrip"));
           syncSync.exec();
-          if (syncSync.lastError().type() != QSqlError::NoError)
+          if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Sub Account Information"),
+                                        syncSync, __FILE__, __LINE__))
           {
-            systemError(this, syncSync.lastError().databaseText(), __FILE__, __LINE__);
             errorCount++;
             // don't break/continue - do as much as we can
           }
         }
       } // next profit center
-      if (rmq.lastError().type() != QSqlError::NoError)
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Sub Account Information"),
+                                    rmq, __FILE__, __LINE__))
       {
-	systemError(this, rmq.lastError().databaseText(), __FILE__, __LINE__);
         errorCount++;
-	return;
+        return;
       }
 
       // Now for what we really want - if the periods match then upsert trialbal
@@ -438,9 +440,9 @@ void syncCompanies::sSync()
       seq.exec("SELECT fetchGLSequence() AS sequence;");
       if (seq.first())
         sequence = seq.value("sequence").toInt();
-      else if (seq.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving GL Sequence"),
+                                    seq, __FILE__, __LINE__))
       {
-        systemError(this, seq.lastError().databaseText(), __FILE__, __LINE__);
         errorCount++;
         return;
       }
@@ -542,8 +544,8 @@ void syncCompanies::sSync()
           if (tbsync.lastError().type() != QSqlError::NoError)
           {
             rollback.exec();
-            systemError(this, rperiod.lastError().databaseText(),
-                        __FILE__, __LINE__);
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Trial Balance Information"),
+                                 rperiod, __FILE__, __LINE__);
             errorCount++;
             break;
           }
@@ -551,8 +553,8 @@ void syncCompanies::sSync()
         else if (rperiod.lastError().type() != QSqlError::NoError)
         {
           rollback.exec();
-          systemError(this, rperiod.lastError().databaseText(),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Accounting Period Information"),
+                               rperiod, __FILE__, __LINE__);
           errorCount++;
           break;
         }
@@ -626,8 +628,8 @@ void syncCompanies::sSync()
           else if (laccnt.lastError().type() != QSqlError::NoError)
           {
             rollback.exec();
-            systemError(this, laccnt.lastError().databaseText(),
-                        __FILE__, __LINE__);
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Savng Account Information"),
+                                 laccnt, __FILE__, __LINE__);
             errorCount++;
             break;
           }
@@ -652,9 +654,9 @@ void syncCompanies::sSync()
               laccntups.bindValue(":accnt_id",	syncSync.value("accnt_id"));
               accntid = syncSync.value("accnt_id").toInt();
             }
-            else if (syncSync.lastError().type() != QSqlError::NoError)
+            else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Account Information"),
+                                          syncSync, __FILE__, __LINE__))
             {
-              systemError(this, syncSync.lastError().databaseText(), __FILE__, __LINE__);
               errorCount++;
               break;
             }
@@ -676,8 +678,8 @@ void syncCompanies::sSync()
           if (laccntups.lastError().type() != QSqlError::NoError)
           {
             rollback.exec();
-            systemError(this, laccntups.lastError().databaseText(),
-                        __FILE__, __LINE__);
+            ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Account Information"),
+                                 laccntups, __FILE__, __LINE__);
             errorCount++;
             break;
           }
@@ -794,8 +796,8 @@ void syncCompanies::sSync()
             if (lgl.lastError().type() != QSqlError::NoError)
             {
               rollback.exec();
-              systemError(this, lgl.lastError().databaseText(),
-                          __FILE__, __LINE__);
+              ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving GL Transaction Information "),
+                                   lgl, __FILE__, __LINE__);
               errorCount++;
               break;
             }
@@ -808,12 +810,12 @@ void syncCompanies::sSync()
           }
 
           progress.setValue(progress.value()+1);
-        } // for each remote g/l account
+        } // for each remote ledger account
         if (raccnt.lastError().type() != QSqlError::NoError)
         {
           rollback.exec();
-          systemError(this, raccnt.lastError().databaseText(),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Account Information "),
+                               raccnt, __FILE__, __LINE__);
           errorCount++;
           break;
         }
@@ -838,8 +840,8 @@ void syncCompanies::sSync()
       if (post.lastError().type() != QSqlError::NoError)
       {
         rollback.exec();
-        systemError(this, post.lastError().databaseText(),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Trial Balance Information"),
+                             post, __FILE__, __LINE__);
         errorCount++;
         break;
       }
@@ -849,8 +851,8 @@ void syncCompanies::sSync()
       if (tbs.lastError().type() != QSqlError::NoError)
       {
         rollback.exec();
-        systemError(this, tbs.lastError().databaseText(),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Trial Balance Information "),
+                             tbs, __FILE__, __LINE__);
         errorCount++;
         break;
       }
@@ -868,8 +870,8 @@ void syncCompanies::sSync()
         if (post.lastError().type() != QSqlError::NoError)
         {
           rollback.exec();
-          systemError(this, post.lastError().databaseText(),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Forward Updating Trial Balances"),
+                               post, __FILE__, __LINE__);
           errorCount++;
           break;
         }
@@ -888,8 +890,8 @@ void syncCompanies::sSync()
       if (post.lastError().type() != QSqlError::NoError)
       {
         rollback.exec();
-        systemError(this, post.lastError().databaseText(),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Forward Updating Trial Balances"),
+                             post, __FILE__, __LINE__);
         errorCount++;
         break;
       }
@@ -903,8 +905,8 @@ void syncCompanies::sSync()
       if (tbs.lastError().type() != QSqlError::NoError)
       {
         rollback.exec();
-        systemError(this, tbs.lastError().databaseText(),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Trial Balance Sync Information"),
+                             tbs, __FILE__, __LINE__);
         errorCount++;
         break;
       }
@@ -925,8 +927,8 @@ void syncCompanies::sSync()
         if (post.lastError().type() != QSqlError::NoError)
         {
           rollback.exec();
-          systemError(this, post.lastError().databaseText(),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Currency Revaluation Adjustments"),
+                               post, __FILE__, __LINE__);
           errorCount++;
           break;
         }
@@ -949,10 +951,12 @@ void syncCompanies::sSync()
       errorCount++;
       continue;
     }
+    progress.accept();
+    if (progress.wasCanceled())
+      canceled = true;
   } // for each selected company
 
-  progress.accept();
-  if (progress.wasCanceled())
+  if (canceled)
     QMessageBox::critical(this, tr("Synchronizing Canceled"),
                           tr("Synchronization Canceled."));
   else
