@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,24 +16,31 @@
 #include <parameter.h>
 #include "ui_systemMessage.h"
 
+#include <QHash>
+
 class systemMessage : public XDialog, public Ui::systemMessage
 {
     Q_OBJECT
 
-public:
-    systemMessage(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+  public:
+    systemMessage(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~systemMessage();
 
-public slots:
+    static systemMessage *windowForId(int id);
+
+  public slots:
     virtual enum SetResponse set( const ParameterList & pParams );
     virtual void sClose();
     virtual void sSave();
     virtual void populate();
 
-protected slots:
+  protected slots:
     virtual void languageChange();
 
-private:
+  protected:
+    static QHash<int, systemMessage*> open;
+
+  private:
     int _mode;
     int _msguserid;
 
