@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,7 +21,7 @@ class taxDetail : public XDialog, public Ui::taxDetail
     Q_OBJECT
 
 public:
-    taxDetail(QWidget* = 0, const char* = 0, bool = false, Qt::WFlags = 0);
+    taxDetail(QWidget* = 0, const char* = 0, bool = false, Qt::WindowFlags = 0);
     ~taxDetail();
 
 	virtual int    taxtype()	const;
