@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,8 +22,9 @@
 #include <QMessageBox>
 #include <QSqlError>
 #include <QToolBar>
+#include "errorReporter.h"
 
-todoList::todoList(QWidget* parent, const char*, Qt::WFlags fl)
+todoList::todoList(QWidget* parent, const char*, Qt::WindowFlags fl)
   : display(parent, "todoList", fl)
 {
   _shown = false;
@@ -41,7 +42,7 @@ todoList::todoList(QWidget* parent, const char*, Qt::WFlags fl)
   parameterWidget()->append(tr("User"), "username", ParameterWidget::User, omfgThis->username());
   parameterWidget()->append(tr("Owner"), "owner_username", ParameterWidget::User);
   parameterWidget()->append(tr("Assigned To"), "assigned_username", ParameterWidget::User);
-  parameterWidget()->append(tr("CRM Account"), "crmacct_id", ParameterWidget::Crmacct);
+  parameterWidget()->append(tr("Account"), "crmacct_id", ParameterWidget::Crmacct);
   parameterWidget()->append(tr("Start Date on or Before"), "startStartDate", ParameterWidget::Date);
   parameterWidget()->append(tr("Start Date on or After"), "startEndDate", ParameterWidget::Date);
   parameterWidget()->append(tr("Due Date on or Before"), "dueStartDate", ParameterWidget::Date);
@@ -76,19 +77,15 @@ todoList::todoList(QWidget* parent, const char*, Qt::WFlags fl)
   QAction *menuItem;
   QMenu * todoMenu = new QMenu;
   menuItem = todoMenu->addAction(tr("To-Do Item"),   this, SLOT(sNew()));
-  if(_privileges->check("MaintainAllToDoItems") || _privileges->check("MaintainPersonalToDoItems"))
+  if(todoItem::userHasPriv(cNew))
     menuItem->setShortcut(QKeySequence::New);
-  menuItem->setEnabled(_privileges->check("MaintainPersonalToDoItems") ||
-                       _privileges->check("MaintainAllToDoItems"));
+  menuItem->setEnabled(todoItem::userHasPriv(cNew));
   menuItem = todoMenu->addAction(tr("Opportunity"), this, SLOT(sNewOpportunity()));
-  menuItem->setEnabled(_privileges->check("MaintainPersonalOpportunities") ||
-                       _privileges->check("MaintainAllOpportunities"));
+  menuItem->setEnabled(opportunity::userHasPriv(cNew));
   menuItem = todoMenu->addAction(tr("Incident"), this, SLOT(sNewIncdt()));
-  menuItem->setEnabled(_privileges->check("MaintainPersonalIncidents") ||
-                       _privileges->check("MaintainAllIncidents"));
+  menuItem->setEnabled(incident::userHasPriv(cNew));
   menuItem = todoMenu->addAction(tr("Project"), this, SLOT(sNewProject()));
-  menuItem->setEnabled(_privileges->check("MaintainPersonalProjects") ||
-                       _privileges->check("MaintainAllProjects"));
+  menuItem->setEnabled(project::userHasPriv(cNew));
   newBtn->setMenu(todoMenu);
 }
 
@@ -108,29 +105,16 @@ void todoList::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
 {
   QAction *menuItem;
 
-  bool editPriv = false;
-  bool viewPriv = false;
-
   if (list()->currentItem()->altId() == 1)
   {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalToDoItems")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalToDoItems")) ||
-        (_privileges->check("MaintainAllToDoItems"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalToDoItems")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalToDoItems")) ||
-        (_privileges->check("ViewAllToDoItems"));
-
     menuItem = pMenu->addAction(tr("Edit To-Do..."), this, SLOT(sEdit()));
-    menuItem->setEnabled(editPriv);
+    menuItem->setEnabled(todoItem::userHasPriv(cEdit, getId(1)));
 
     menuItem = pMenu->addAction(tr("View To-Do..."), this, SLOT(sView()));
-    menuItem->setEnabled(viewPriv);
+    menuItem->setEnabled(todoItem::userHasPriv(cView, getId(1)));
 
     menuItem = pMenu->addAction(tr("Delete To-Do"), this, SLOT(sDelete()));
-    menuItem->setEnabled(editPriv);
+    menuItem->setEnabled(todoItem::userHasPriv(cEdit, getId(1)));
   }
 
   pMenu->addSeparator();
@@ -139,87 +123,47 @@ void todoList::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *, int)
       (list()->currentItem()->altId() == 1 &&
        list()->currentItem()->rawValue("parent") == "INCDT"))
   {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalIncidents")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalIncidents")) ||
-        (_privileges->check("MaintainAllIncidents"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalIncidents")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalIncidents")) ||
-        (_privileges->check("ViewAllIncidents"));
-
     menuItem = pMenu->addAction(tr("Edit Incident"), this, SLOT(sEditIncident()));
-    menuItem->setEnabled(editPriv);
+    menuItem->setEnabled(incident::userHasPriv(cEdit, getId(2)));
     menuItem = pMenu->addAction(tr("View Incident"), this, SLOT(sViewIncident()));
-    menuItem->setEnabled(viewPriv);
+    menuItem->setEnabled(incident::userHasPriv(cView, getId(2)));
   }
   pMenu->addSeparator();
 
   if (list()->altId() == 3)
   {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalProjects")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalProjects")) ||
-        (_privileges->check("MaintainAllProjects"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalProjects")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalProjects")) ||
-        (_privileges->check("ViewAllProjects"));
-
     menuItem = pMenu->addAction(tr("Edit Task"), this, SLOT(sEditTask()));
-    menuItem->setEnabled(editPriv);
+    menuItem->setEnabled(task::userHasPriv(cEdit, getId(3)));
     menuItem = pMenu->addAction(tr("View Task"), this, SLOT(sViewTask()));
-    menuItem->setEnabled(viewPriv);
+    menuItem->setEnabled(task::userHasPriv(cView, getId(3)));
     pMenu->addSeparator();
   }
 
   if (list()->altId() == 3 || list()->altId() == 4)
   {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalProjects")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalProjects")) ||
-        (_privileges->check("MaintainAllProjects"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalProjects")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalProjects")) ||
-        (_privileges->check("ViewAllProjects"));
-
     menuItem = pMenu->addAction(tr("Edit Project"), this, SLOT(sEditProject()));
-    menuItem->setEnabled(editPriv);
+    menuItem->setEnabled(project::userHasPriv(cEdit, getId(4)));
     menuItem = pMenu->addAction(tr("View Project"), this, SLOT(sViewProject()));
-    menuItem->setEnabled(viewPriv);
+    menuItem->setEnabled(project::userHasPriv(cView, getId(4)));
   }
 
   if (list()->altId() == 5  ||
       (list()->currentItem()->altId() == 1 &&
        list()->currentItem()->rawValue("parent") == "OPP"))
   {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalOpportunities")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalOpportunities")) ||
-        (_privileges->check("MaintainAllOpportunities"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalOpportunities")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalOpportunities")) ||
-        (_privileges->check("ViewAllOpportunities"));
-
     menuItem = pMenu->addAction(tr("Edit Opportunity"), this, SLOT(sEditOpportunity()));
-    menuItem->setEnabled(editPriv);
+    menuItem->setEnabled(opportunity::userHasPriv(cEdit, getId(5)));
     menuItem = pMenu->addAction(tr("View Opportunity"), this, SLOT(sViewOpportunity()));
-    menuItem->setEnabled(viewPriv);
+    menuItem->setEnabled(opportunity::userHasPriv(cView, getId(5)));
   }
 
   if (list()->currentItem()->rawValue("cust").toInt() > 0)
   {
     pMenu->addSeparator();
     menuItem = pMenu->addAction(tr("Edit Customer"), this, SLOT(sEditCustomer()));
-    menuItem->setEnabled(_privileges->check("MaintainCustomerMasters") || _privileges->check("ViewCustomerMasters"));
+    menuItem->setEnabled(customer::userHasPriv(cEdit));
     menuItem = pMenu->addAction(tr("View Customer"), this, SLOT(sViewCustomer()));
-    menuItem->setEnabled(_privileges->check("ViewCustomerMasters"));
+    menuItem->setEnabled(customer::userHasPriv(cView));
   }
 }
 
@@ -239,15 +183,14 @@ enum SetResponse todoList::set(const ParameterList& pParams)
 void todoList::sNew()
 {
   //Need an extra priv check because of display trigger
-  if (!_privileges->check("MaintainPersonalToDoItems") &&
-      !_privileges->check("MaintainAllToDoItems"))
+  if (!todoItem::userHasPriv(cNew))
     return;
 
   ParameterList params;
   parameterWidget()->appendValue(params);
   params.append("mode", "new");
 
-  todoItem newdlg(this, "", TRUE);
+  todoItem newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -282,7 +225,7 @@ void todoList::sEdit()
     params.append("mode", "edit");
     params.append("todoitem_id", list()->id());
 
-    todoItem newdlg(this, "", TRUE);
+    todoItem newdlg(this, "", true);
     newdlg.set(params);
 
     if (newdlg.exec() != XDialog::Rejected)
@@ -306,7 +249,7 @@ void todoList::sView()
     params.append("mode", "view");
     params.append("todoitem_id", list()->id());
 
-    todoItem newdlg(this, "", TRUE);
+    todoItem newdlg(this, "", true);
     newdlg.set(params);
 
     newdlg.exec();
@@ -328,7 +271,7 @@ void todoList::sDelete()
   }
 
   bool deleteAll  = false;
-  bool createMore = false;
+  bool deleteOne  = false;
   if (! recurstr.isEmpty())
   {
     XSqlQuery recurq;
@@ -349,13 +292,13 @@ void todoList::sDelete()
         return;
       else if (ret == QMessageBox::YesToAll)
         deleteAll = true;
-      // user said delete one but the only one that exists is the base
-      else if (ret == QMessageBox::Yes && recurq.value("max").isNull())
-        createMore = true;
+      // user said delete one but the only one that exists is the parent ToDo
+      else if (ret == QMessageBox::Yes)
+        deleteOne = true;
     }
-    else if (recurq.lastError().type() != QSqlError::NoError)
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving To Do Item Information"),
+                                  recurq, __FILE__, __LINE__))
     {
-      systemError(this, recurq.lastError().text(), __FILE__, __LINE__);
       return;
     }
     else if (QMessageBox::warning(this, tr("Delete List Item?"),
@@ -372,40 +315,48 @@ void todoList::sDelete()
 			    QMessageBox::No) == QMessageBox::No)
     return;
 
-  QString procname;
   int procresult = 0;
-  if (deleteAll)
+  if (deleteAll)  // Delete all todos in the recurring series
   {
-    procname = "deleteOpenRecurringItems";
-    todoDelete.prepare("SELECT deleteOpenRecurringItems(:id, :type, NULL, TRUE)"
+    todoDelete.prepare("SELECT deleteOpenRecurringItems(:id, :type, NULL, true)"
               "       AS result;");
     todoDelete.bindValue(":id",   list()->id());
     todoDelete.bindValue(":type", recurtype);
     todoDelete.exec();
     if (todoDelete.first())
       procresult = todoDelete.value("result").toInt();
-  }
-  if (procresult >= 0 && createMore)
-  {
-    procname = "createRecurringItems";
-    todoDelete.prepare("SELECT createRecurringItems(:id, :type) AS result;");
-    todoDelete.bindValue(":id",   list()->id());
-    todoDelete.bindValue(":type", recurtype);
-    todoDelete.exec();
-    if (todoDelete.first())
-      procresult = todoDelete.value("result").toInt();
+
+    if (procresult < 0)
+    {
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Recurring To Do Item Information"),
+                             storedProcErrorLookup("deleteOpenRecurringItems", procresult),
+                             __FILE__, __LINE__);
+      return;
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Recurring To Do Item Information"),
+                                  todoDelete, __FILE__, __LINE__))
+    {
+      return;
+    }
   }
 
-  // not elseif - error handling for 1 or 2 queries
-  if (procresult < 0)
+  if (deleteOne) // The base todo in a recurring series has been seleted.  Have to move
+                 // recurrence to the next item else we hit foreign key errors.
+                 // Make the next item on the list the parent in the series
   {
-    systemError(this, storedProcErrorLookup(procname, procresult));
-    return;
-  }
-  else if (todoDelete.lastError().type() != QSqlError::NoError)
-  {
-    systemError(this, todoDelete.lastError().databaseText(), __FILE__, __LINE__);
-    return;
+    todoDelete.prepare("UPDATE todoitem SET todoitem_recurring_todoitem_id =("
+                        "               SELECT MIN(todoitem_id) FROM todoitem"
+                        "                 WHERE todoitem_recurring_todoitem_id=:id"
+                        "                   AND todoitem_id!=:id)"
+                        "  WHERE todoitem_recurring_todoitem_id=:id"
+                        "  AND todoitem_id!=:id;");
+    todoDelete.bindValue(":id",   list()->id());
+    todoDelete.exec();
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Recurring To Do Item Information"),
+                                  todoDelete, __FILE__, __LINE__))
+    {
+      return;
+    }
   }
 
   if (list()->altId() == 1)
@@ -424,14 +375,16 @@ void todoList::sDelete()
     int result = todoDelete.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("deleteTodoItem", result));
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving To Do Item Information"),
+                             storedProcErrorLookup("deleteTodoItem", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (todoDelete.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving To Do Item Information"),
+                                todoDelete, __FILE__, __LINE__))
   {
-    systemError(this, todoDelete.lastError().databaseText(), __FILE__, __LINE__);
-    return;
+     return;
   }
   sFillList();
 }
@@ -493,7 +446,7 @@ void todoList::sEditIncident()
   params.append("mode", "edit");
   params.append("incdt_id", getId(2));
 
-  incident newdlg(this, "", TRUE);
+  incident newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -506,7 +459,7 @@ void todoList::sViewIncident()
   params.append("mode", "view");
   params.append("incdt_id", getId(2));
 
-  incident newdlg(this, "", TRUE);
+  incident newdlg(this, "", true);
   newdlg.set(params);
 
   newdlg.exec();
@@ -531,7 +484,7 @@ void todoList::sEditProject()
   qDebug("project %d", getId(4));
   params.append("prj_id", getId(4));
 
-  project newdlg(this, "", TRUE);
+  project newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -544,7 +497,7 @@ void todoList::sViewProject()
   params.append("mode", "view");
   params.append("prj_id", getId(4));
 
-  project newdlg(this, "", TRUE);
+  project newdlg(this, "", true);
   newdlg.set(params);
 
   newdlg.exec();
@@ -557,7 +510,7 @@ void todoList::sEditTask()
   params.append("mode", "edit");
   params.append("prjtask_id", list()->id());
 
-  task newdlg(this, "", TRUE);
+  task newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -570,7 +523,7 @@ void todoList::sViewTask()
   params.append("mode", "view");
   params.append("prjtask_id", list()->id());
 
-  task newdlg(this, "", TRUE);
+  task newdlg(this, "", true);
   newdlg.set(params);
 
   newdlg.exec();
@@ -617,7 +570,7 @@ void todoList::sEditOpportunity()
   params.append("mode", "edit");
   params.append("ophead_id", getId(5));
 
-  opportunity newdlg(this, "", TRUE);
+  opportunity newdlg(this, "", true);
   newdlg.set(params);
 
   if (newdlg.exec() != XDialog::Rejected)
@@ -630,7 +583,7 @@ void todoList::sViewOpportunity()
   params.append("mode", "view");
   params.append("ophead_id", getId(5));
 
-  opportunity newdlg(this, "", TRUE);
+  opportunity newdlg(this, "", true);
   newdlg.set(params);
 
   newdlg.exec();
@@ -641,79 +594,31 @@ void todoList::sOpen()
   bool editPriv = false;
   bool viewPriv = false;
 
-  if (list()->currentItem()->altId() == 1)
-  {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalToDoItems")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalToDoItems")) ||
-        (_privileges->check("MaintainAllToDoItems"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalToDoItems")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalToDoItems")) ||
-        (_privileges->check("ViewAllToDoItems"));
-
-  }
-
-  if (list()->altId() == 2 ||
-      (list()->currentItem()->altId() == 1 &&
-       list()->currentItem()->rawValue("parent") == "INCDT"))
-  {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalIncidents")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalIncidents")) ||
-        (_privileges->check("MaintainAllIncidents"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalIncidents")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalIncidents")) ||
-        (_privileges->check("ViewAllIncidents"));
-
-  }
-
-  if (list()->altId() == 3)
-  {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalProjects")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalProjects")) ||
-        (_privileges->check("MaintainAllProjects"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalProjects")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalProjects")) ||
-        (_privileges->check("ViewAllProjects"));
-
-  }
-
-  if (list()->altId() == 3 || list()->altId() == 4)
-  {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalProjects")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalProjects")) ||
-        (_privileges->check("MaintainAllProjects"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalProjects")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalProjects")) ||
-        (_privileges->check("ViewAllProjects"));
-
-  }
-
-  if (list()->altId() == 5  ||
-      (list()->currentItem()->altId() == 1 &&
-       list()->currentItem()->rawValue("parent") == "OPP"))
-  {
-    editPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("MaintainPersonalOpportunities")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("MaintainPersonalOpportunities")) ||
-        (_privileges->check("MaintainAllOpportunities"));
-
-    viewPriv =
-        (omfgThis->username() == list()->currentItem()->rawValue("owner") && _privileges->check("ViewPersonalOpportunities")) ||
-        (omfgThis->username() == list()->currentItem()->rawValue("assigned") && _privileges->check("ViewPersonalOpportunities")) ||
-        (_privileges->check("ViewAllOpportunities"));
-
-  }
+    switch (list()->altId())
+    {
+    case 1:
+      editPriv = todoItem::userHasPriv(cEdit, list()->currentItem()->id());
+      viewPriv = todoItem::userHasPriv(cView, list()->currentItem()->id());
+      break;
+    case 2:
+      editPriv = incident::userHasPriv(cEdit, list()->currentItem()->id());
+      viewPriv = incident::userHasPriv(cView, list()->currentItem()->id());
+      break;
+    case 3:
+      editPriv = task::userHasPriv(cEdit, list()->currentItem()->id());
+      viewPriv = task::userHasPriv(cView, list()->currentItem()->id());
+      break;
+    case 4:
+      editPriv = project::userHasPriv(cEdit, list()->currentItem()->id());
+      viewPriv = project::userHasPriv(cView, list()->currentItem()->id());
+      break;
+    case 5:
+      editPriv = opportunity::userHasPriv(cEdit, list()->currentItem()->id());
+      viewPriv = opportunity::userHasPriv(cView, list()->currentItem()->id());
+      break;
+    default:
+      break;
+    }
 
   if(editPriv)
     sEdit();
