@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -36,6 +36,9 @@ class todoListCalendar : public XWidget, public Ui::todoListCalendar
     void sNew();
     void sEdit();
     void sView();
+    void sNewTask();
+    void sEditTask();
+    void sViewTask();
     void sOpen();
     void sDelete();
     void sEditCustomer();
