@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,7 +21,7 @@ class transactionInformation : public XDialog, public Ui::transactionInformation
     Q_OBJECT
 
 public:
-    transactionInformation(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    transactionInformation(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~transactionInformation();
 
 public slots:
@@ -32,8 +32,9 @@ protected slots:
     virtual void languageChange();
 
 private:
-    int _mode;
-    int _invhistid;
+    int  _mode;
+    int  _invhistid;
+    bool _analyzeInit;
 
 };
 
