@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -38,13 +38,12 @@ const char *_statusTypes[] = { "U", "O", "C" };
 #define cCanceled     0x08
 #define cUnreleased   0x16
 
-transferOrder::transferOrder(QWidget* parent, const char* name, Qt::WFlags fl)
+transferOrder::transferOrder(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl),
       _cachedTabIndex(0),
       _captive(false),
       _ignoreSignals(false),
       _lineMode(0),
-      _locked(false),
       _mode(cView),
       _orderNumberGen(0),
       _qeitem(0),
@@ -102,7 +101,10 @@ transferOrder::transferOrder(QWidget* parent, const char* name, Qt::WFlags fl)
   _orderNumber->setValidator(omfgThis->orderVal());
 
   if(!_metrics->boolean("UseProjects"))
+  {
+    _projectLit->hide();
     _project->hide();
+  }
 
   _mode = cView;
 
@@ -126,34 +128,12 @@ void transferOrder::setToheadid(const int pId)
   _toheadid = pId;
   _qeitem->setHeadId(pId);
   _comments->setId(_toheadid);
+  _documents->setId(_toheadid);
        
-  if(_mode == cEdit && !_locked)
+  if (_mode == cEdit
+      && ! _lock.acquire("tohead", _toheadid, AppLock::Interactive))
   {
-     XSqlQuery to;
-     to.prepare("SELECT tryLock(oid::integer, :tohead_id) AS locked "
-                "FROM pg_class "
-                "WHERE relname='tohead';");
-     to.bindValue(":tohead_id", _toheadid);
-     to.exec();
-     if(to.first())
-     {
-       if(to.value("locked").toBool() != true)
-       {
-          QMessageBox::critical( this, tr("Record Currently Being Edited"),
-               tr("<p>The record you are trying to edit is currently being edited "
-               "by another user. Continue in View Mode.") );
-          setViewMode();
-       }
-       else
-         _locked=true;
-     }
-     else
-     {
-       QMessageBox::critical( this, tr("Cannot Lock Record for Editing"),
-             tr("<p>There was an unexpected error while trying to lock the record "
-             "for editing. Please report this to your administator.") );
-       setViewMode();
-     }
+    setViewMode();
   }
 }
 
@@ -233,9 +213,8 @@ enum SetResponse transferOrder::set(const ParameterList &pParams)
       }
       else
       {
-        systemError(this, tr("A System Error occurred at %1::%2.")
-                          .arg(__FILE__)
-                          .arg(__LINE__) );
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Planned Order Information"),
+                             transferet, __FILE__, __LINE__);
         return UndefinedError;
       }
 
@@ -269,8 +248,8 @@ enum SetResponse transferOrder::set(const ParameterList &pParams)
           _mode = cEdit;
 
           setToheadid(transferet.value("tohead_id").toInt());
-          _orderNumber->setEnabled(FALSE);
-          _orderDate->setEnabled(FALSE);
+          _orderNumber->setEnabled(false);
+          _orderDate->setEnabled(false);
           populate();
         }
         else
@@ -299,7 +278,7 @@ enum SetResponse transferOrder::set(const ParameterList &pParams)
       newItemParams.append("taxzone_id",	_taxzone->id());
       newItemParams.append("curr_id",	_freightCurrency->id());
       newItemParams.append("item_id", itemid);
-      newItemParams.append("captive", TRUE);
+      newItemParams.append("captive", true);
 
       if (qty > 0.0)
         newItemParams.append("qty", qty);
@@ -307,13 +286,13 @@ enum SetResponse transferOrder::set(const ParameterList &pParams)
       if (!dueDate.isNull())
         newItemParams.append("dueDate", dueDate);
 
-      transferOrderItem toItem(this, "", TRUE);
+      transferOrderItem toItem(this, "", true);
       toItem.set(newItemParams);
 // TODO
 //      if (toItem.exec() != XDialog::Rejected)
       if (toItem.exec() == XDialog::Rejected)
       {
-        transferet.prepare("SELECT deletePlannedOrder(:planord_id, FALSE) AS _result;");
+        transferet.prepare("SELECT deletePlannedOrder(:planord_id, false) AS _result;");
         transferet.bindValue(":planord_id", _planordid);
         transferet.exec();
 // TODO
@@ -360,20 +339,16 @@ enum SetResponse transferOrder::set(const ParameterList &pParams)
     getWhsInfo(_srcWhs->id(), _srcWhs);
     getWhsInfo(_dstWhs->id(), _dstWhs);
 
-    _captive = FALSE;
-    _edit->setEnabled(FALSE);
-    _action->setEnabled(FALSE);
-    _delete->setEnabled(FALSE);
+    _captive = false;
+    _edit->setEnabled(false);
+    _action->setEnabled(false);
+    _delete->setEnabled(false);
     _close->setText("&Cancel");
   }
   else if (cEdit == _mode)
   {
-    _captive = TRUE;
-    _orderNumber->setEnabled(FALSE);
-  }
-  else if (cView == _mode)
-  {
-    _project->setReadOnly(true);
+    _captive = true;
+    _orderNumber->setEnabled(false);
   }
 
   if( !_metrics->boolean("EnableTOShipping"))
@@ -412,8 +387,8 @@ enum SetResponse transferOrder::set(const ParameterList &pParams)
   }
   else
   {
-    _orderDate->setEnabled(FALSE);
-    _packDate->setEnabled(FALSE);
+    _orderDate->setEnabled(false);
+    _packDate->setEnabled(false);
   }
 
   param = pParams.value("captive", &valid);
@@ -423,6 +398,16 @@ enum SetResponse transferOrder::set(const ParameterList &pParams)
   return NoError;
 }
 
+int transferOrder::id() const
+{
+  return _toheadid;
+}
+
+int transferOrder::mode() const
+{
+  return _mode;
+}
+
 bool transferOrder::insertPlaceholder()
 {
   XSqlQuery transferinsertPlaceholder;
@@ -434,9 +419,9 @@ bool transferOrder::insertPlaceholder()
     return false;
   }
 
-  _ignoreSignals = TRUE;
+  _ignoreSignals = true;
   populateOrderNumber();
-  _ignoreSignals = FALSE;
+  _ignoreSignals = false;
 
   transferinsertPlaceholder.prepare("INSERT INTO tohead ("
         "          tohead_number, tohead_src_warehous_id,"
@@ -549,10 +534,10 @@ bool transferOrder::save(bool partial)
   // save contact and address info in case someone wants to use 'em again later
   // but don't make any global changes to the data and ignore errors
   // TODO: put in real checking - perhaps the address of the warehouse is wrong
-  _ignoreSignals = TRUE;
+  _ignoreSignals = true;
   _srcAddr->save(AddressCluster::CHANGEONE);
   _dstAddr->save(AddressCluster::CHANGEONE);
-  _ignoreSignals = FALSE;
+  _ignoreSignals = false;
 
   XSqlQuery rollback;
   rollback.prepare("ROLLBACK;");
@@ -671,25 +656,14 @@ bool transferOrder::save(bool partial)
     return false;
   }
 
-  // if this is a new to record and we haven't saved already
-  // then we need to lock this record.
-  if((cNew == _mode) && (!_saved))
+  // TODO: should this be done before saving the t/o?
+  if ((cNew == _mode) && (!_saved)
+      && ! _lock.acquire("tohead", _toheadid, AppLock::Silent))
   {
-    // should I bother to check because no one should have this but us?
-    transferave.prepare("SELECT tryLock(oid::integer, :head_id) AS locked "
-              "FROM pg_class "
-              "WHERE relname='tohead';");
-    transferave.bindValue(":head_id", _toheadid);
-    transferave.exec();
-    if (transferave.first())
-      _locked = transferave.value("locked").toBool();
-    else if (transferave.lastError().type() != QSqlError::NoError)
-    {
-      rollback.exec();
-      ErrorReporter::error(QtCriticalMsg, this, tr("Error Saving Transfer Order"),
-                           transferave, __FILE__, __LINE__);
-      return false;
-    }
+    rollback.exec();
+    (void)ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                               _lock.lastError(), __FILE__, __LINE__);
+    return false;
   }
 
   // keep the status of toitems in synch with tohead
@@ -739,7 +713,7 @@ bool transferOrder::save(bool partial)
 
   emit saved(_toheadid);
 
-  return TRUE;
+  return true;
 }
 
 void transferOrder::sPopulateMenu(QMenu *pMenu)
@@ -769,7 +743,8 @@ void transferOrder::sPopulateMenu(QMenu *pMenu)
       }
     }
 
-    if(_metrics->boolean("EnableTOShipping"))
+    if(_metrics->boolean("EnableTOShipping") &&
+       (_lineMode == cActiveOpen || _lineMode == cInactiveOpen))
     {
       if(_numSelected == 1)
         pMenu->addSeparator();
@@ -796,7 +771,7 @@ void transferOrder::populateOrderNumber()
         _orderNumberGen = transferpopulateOrderNumber.value("tonumber").toInt();
 
         if (_metrics->value("TONumberGeneration") == "A")
-          _orderNumber->setEnabled(FALSE);
+          _orderNumber->setEnabled(false);
       }
       else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Getting Number"),
                                     transferpopulateOrderNumber, __FILE__, __LINE__))
@@ -805,12 +780,12 @@ void transferOrder::populateOrderNumber()
     _new->setEnabled(! _orderNumber->text().isEmpty());
   }
 
-  _userEnteredOrderNumber = FALSE;
+  _userEnteredOrderNumber = false;
 }
 
 void transferOrder::sSetUserEnteredOrderNumber()
 {
-  _userEnteredOrderNumber = TRUE;
+  _userEnteredOrderNumber = true;
 }
     
 void transferOrder::sHandleOrderNumber()
@@ -855,7 +830,7 @@ void transferOrder::sHandleOrderNumber()
         _mode = cEdit;
         setToheadid(query.value("tohead_id").toInt());
         populate();
-        _orderNumber->setEnabled(FALSE);
+        _orderNumber->setEnabled(false);
       }
       else
       {
@@ -877,8 +852,8 @@ void transferOrder::sHandleOrderNumber()
 	  return;
 
         _orderNumber->setText(orderNumber);
-        _userEnteredOrderNumber = FALSE;
-        _orderNumber->setEnabled(FALSE);
+        _userEnteredOrderNumber = false;
+        _orderNumber->setEnabled(false);
       }
     }
 
@@ -908,7 +883,7 @@ void transferOrder::sNew()
   if ((_mode == cNew) || (_mode == cEdit))
     params.append("mode", "new");
 
-  transferOrderItem newdlg(this, "", TRUE);
+  transferOrderItem newdlg(this, "", true);
   newdlg.set(params);
   if (newdlg.exec() == XDialog::Accepted)
     sFillItemList();
@@ -924,7 +899,7 @@ void transferOrder::sEdit()
   else if ((_mode == cNew) || (_mode == cEdit))
     params.append("mode", "edit");
 
-  transferOrderItem newdlg(this, "", TRUE);
+  transferOrderItem newdlg(this, "", true);
   newdlg.set(params);
   if (newdlg.exec() == XDialog::Accepted &&
       ((_mode == cNew) || (_mode == cEdit)) )
@@ -950,7 +925,7 @@ void transferOrder::sHandleButtons()
 
     if(_numSelected == 1)
     {
-      _edit->setEnabled(TRUE);
+      _edit->setEnabled(true);
       int lineMode = selected->altId();
 
       if (lineMode == 1)
@@ -958,66 +933,66 @@ void transferOrder::sHandleButtons()
         _lineMode = cClosed;
 
         _action->setText(tr("Open"));
-        _action->setEnabled(TRUE);
-        _delete->setEnabled(FALSE);
+        _action->setEnabled(true);
+        _delete->setEnabled(false);
       }
       else if (lineMode == 2)
       {
         _lineMode = cActiveOpen;
 
         _action->setText(tr("Close"));
-        _action->setEnabled(TRUE);
-        _delete->setEnabled(FALSE);
+        _action->setEnabled(true);
+        _delete->setEnabled(false);
       }
       else if (lineMode == 3)
       {
         _lineMode = cInactiveOpen;
 
         _action->setText(tr("Close"));
-        _action->setEnabled(TRUE);
-        _delete->setEnabled(TRUE);
+        _action->setEnabled(true);
+        _delete->setEnabled(true);
       }
       else if (lineMode == 4)
       {
         _lineMode = cCanceled;
 
-        _action->setEnabled(FALSE);
-        _delete->setEnabled(FALSE);
+        _action->setEnabled(false);
+        _delete->setEnabled(false);
       }
       else if (lineMode == 5)
       {
         _lineMode = cUnreleased;
 
         _action->setText(tr("Close"));
-        _action->setEnabled(FALSE);
-        _delete->setEnabled(TRUE);
+        _action->setEnabled(false);
+        _delete->setEnabled(true);
       }
       else
       {
-        _action->setEnabled(FALSE);
-        _delete->setEnabled(FALSE);
+        _action->setEnabled(false);
+        _delete->setEnabled(false);
       }
 
       if(1 == lineMode || 4 == lineMode || 5 == lineMode)
       {
-        _issueStock->setEnabled(FALSE);
-        _issueLineBalance->setEnabled(FALSE);
+        _issueStock->setEnabled(false);
+        _issueLineBalance->setEnabled(false);
       }
     }
     else
     {
-      _edit->setEnabled(FALSE);
-      _action->setEnabled(FALSE);
-      _delete->setEnabled(FALSE);
+      _edit->setEnabled(false);
+      _action->setEnabled(false);
+      _delete->setEnabled(false);
     }
   }
   else
   {
-    _edit->setEnabled(FALSE);
-    _action->setEnabled(FALSE);
-    _delete->setEnabled(FALSE);
-    _issueStock->setEnabled(FALSE);
-    _issueLineBalance->setEnabled(FALSE);
+    _edit->setEnabled(false);
+    _action->setEnabled(false);
+    _delete->setEnabled(false);
+    _issueStock->setEnabled(false);
+    _issueLineBalance->setEnabled(false);
   }
 }
 
@@ -1051,8 +1026,9 @@ void transferOrder::sAction()
         int result = transferAction.value("result").toInt();
         if (result < 0)
         {
-          systemError(this, storedProcErrorLookup("closeToItem", result),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Transfer Order Information"),
+                                 storedProcErrorLookup("closeToItem", result),
+                                 __FILE__, __LINE__);
           return;
         }
       }
@@ -1116,9 +1092,9 @@ void transferOrder::sDelete()
           {
             int result = transferDelete.value("result").toInt();
             if (result < 0)
-              systemError(this, storedProcErrorLookup("deleteTO", result),
-			  __FILE__, __LINE__);            
-            sReleaseNumber();
+              ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Transfer Order"),
+                                  storedProcErrorLookup("deleteTO", result),
+                                  __FILE__, __LINE__);
           }
           ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Order"),
                                transferDelete, __FILE__, __LINE__);
@@ -1144,7 +1120,7 @@ void transferOrder::populate()
     if (to.first())
     {
       _orderNumber->setText(to.value("tohead_number"));
-      _orderNumber->setEnabled(FALSE);
+      _orderNumber->setEnabled(false);
       _orderDate->setDate(to.value("tohead_orderdate").toDate(), true);
 
       if (to.value("tohead_status").toString() == "U")
@@ -1235,6 +1211,7 @@ void transferOrder::populate()
       _project->setId(to.value("tohead_prj_id").toInt());
 
       _comments->setId(_toheadid);
+      _documents->setId(_toheadid);
       sFillItemList();
     }
     else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Getting Order"),
@@ -1403,7 +1380,7 @@ bool transferOrder::deleteForCancel()
     deleteTo = true;
   }
 
-  if (cNew == _mode && _toitem->topLevelItemCount() > 0)
+  if (cNew == _mode && _toitem->topLevelItemCount() > 0 && !_captive)
   {
     if (QMessageBox::question(this, tr("Delete Transfer Order?"),
                           tr("<p>Are you sure you want to delete this "
@@ -1436,8 +1413,9 @@ bool transferOrder::deleteForCancel()
     {
       int result = query.value("result").toInt();
       if (result < 0)
-        systemError(this, storedProcErrorLookup("deleteTO", result),
-                    __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Transfer Order"),
+                             storedProcErrorLookup("deleteTO", result),
+                             __FILE__, __LINE__);
 
       sReleaseNumber();
     }
@@ -1466,7 +1444,7 @@ void transferOrder::clear()
 
   _tabs->setCurrentIndex(0);
 
-  _orderNumber->setEnabled(TRUE);
+  _orderNumber->setEnabled(true);
   _orderNumberGen = 0;
   _orderNumber->clear();
 
@@ -1552,8 +1530,8 @@ void transferOrder::closeEvent(QCloseEvent *pEvent)
 
 void transferOrder::sHandleShipchrg(int pShipchrgid)
 {
-  if ( (_mode == cView) )
-    _freight->setEnabled(FALSE);
+  if (_mode == cView)
+    _freight->setEnabled(false);
   else
   {
     XSqlQuery query;
@@ -1565,10 +1543,10 @@ void transferOrder::sHandleShipchrg(int pShipchrgid)
     if (query.first())
     {
       if (query.value("shipchrg_custfreight").toBool())
-        _freight->setEnabled(TRUE);
+        _freight->setEnabled(true);
       else
       {
-        _freight->setEnabled(FALSE);
+        _freight->setEnabled(false);
         _freight->clear();
       }
     }
@@ -1606,7 +1584,7 @@ void transferOrder::sTaxDetail()
   params.append("order_type", "TO");
   params.append("mode", "view"); // because tohead has no fields to hold changes
 
-  taxBreakdown newdlg(this, "", TRUE);
+  taxBreakdown newdlg(this, "", true);
   if (newdlg.set(params) == NoError)
   {
     newdlg.exec();
@@ -1628,29 +1606,31 @@ void transferOrder::setViewMode()
   _mode = cView;
   setObjectName(QString("transferOrder view %1").arg(_toheadid));
 
-  _orderNumber->setEnabled(FALSE);
-  _status->setEnabled(FALSE);
-  _packDate->setEnabled(FALSE);
-  _srcWhs->setEnabled(FALSE);
-  _dstWhs->setEnabled(FALSE);
-  _trnsWhs->setEnabled(FALSE);
-  _agent->setEnabled(FALSE);
-  _taxzone->setEnabled(FALSE);
-  _shipVia->setEnabled(FALSE);
-  _shippingForm->setEnabled(FALSE);
-  _freight->setEnabled(FALSE);
-  _orderComments->setEnabled(FALSE);
-  _shippingComments->setEnabled(FALSE);
-  _qeitemView->setEnabled(FALSE);
-  _qesave->setEnabled(FALSE);
-  _qedelete->setEnabled(FALSE);
-  _qecurrency->setEnabled(FALSE);
-  _freightCurrency->setEnabled(FALSE);
-  _srcContact->setEnabled(FALSE);
-  _dstContact->setEnabled(FALSE);
+  _orderNumber->setEnabled(false);
+  _status->setEnabled(false);
+  _packDate->setEnabled(false);
+  _srcWhs->setEnabled(false);
+  _dstWhs->setEnabled(false);
+  _trnsWhs->setEnabled(false);
+  _agent->setEnabled(false);
+  _taxzone->setEnabled(false);
+  _shipVia->setEnabled(false);
+  _shippingForm->setEnabled(false);
+  _freight->setEnabled(false);
+  _orderComments->setEnabled(false);
+  _shippingComments->setEnabled(false);
+  _qeitemView->setEnabled(false);
+  _qesave->setEnabled(false);
+  _qedelete->setEnabled(false);
+  _qecurrency->setEnabled(false);
+  _freightCurrency->setEnabled(false);
+  _srcContact->setEnabled(false);
+  _dstContact->setEnabled(false);
 
   _edit->setText(tr("View"));
   _comments->setReadOnly(true);
+  _project->setReadOnly(true);
+//  _documents->setReadOnly(true);
   _shipComplete->setEnabled(false);
   _save->hide();
   _clear->hide();
@@ -1667,7 +1647,7 @@ void transferOrder::setViewMode()
 
 void transferOrder::keyPressEvent( QKeyEvent * e )
 {
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
   if(e->key() == Qt::Key_N && (e->modifiers() & Qt::ControlModifier))
   {
     _new->animateClick();
@@ -1802,9 +1782,9 @@ void transferOrder::sReturnStock()
       if (result < 0)
       {
         rollback.exec();
-        systemError(this, storedProcErrorLookup("returnItemShipments", result) +
-                          tr("<br>Line Item %1").arg(selected[i]->text(0)),
-                           __FILE__, __LINE__);
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Return Item Information"),
+                               storedProcErrorLookup("returnItemShipments", result),
+                               __FILE__, __LINE__);
         return;
       }
       if (distributeInventory::SeriesAdjust(transferReturnStock.value("result").toInt(), this) == XDialog::Rejected)
@@ -1818,8 +1798,8 @@ void transferOrder::sReturnStock()
     else if (transferReturnStock.lastError().type() != QSqlError::NoError)
     {
       rollback.exec();
-      systemError(this, tr("Line Item %1\n").arg(selected[i]->text(0)) +
-                        transferReturnStock.lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Return Item Information"),
+                           transferReturnStock, __FILE__, __LINE__);
       return;
     }
   }
@@ -1831,7 +1811,7 @@ void transferOrder::sReturnStock()
 
 void transferOrder::sIssueStock()
 {
-  bool update  = FALSE;
+  bool update  = false;
   QList<XTreeWidgetItem*> selected = _toitem->selectedItems();
   for (int i = 0; i < selected.size(); i++)
   {
@@ -1845,10 +1825,10 @@ void transferOrder::sIssueStock()
       if(_requireInventory->isChecked())
         params.append("requireInventory");
 
-      issueLineToShipping newdlg(this, "", TRUE);
+      issueLineToShipping newdlg(this, "", true);
       newdlg.set(params);
       if (newdlg.exec() != XDialog::Rejected)
-        update = TRUE;
+        update = true;
     }
   }
 
@@ -1884,16 +1864,17 @@ void transferOrder::sIssueLineBalance()
                 "   AND  (toitem_id=:toitem_id)); ");
             transferIssueLineBalance.bindValue(":toitem_id", toitem->id());
             transferIssueLineBalance.exec();
-            if (! transferIssueLineBalance.first() && transferIssueLineBalance.lastError().type() != QSqlError::NoError)
+            if (transferIssueLineBalance.first())
             {
-              ErrorReporter::error(QtCriticalMsg, this, tr("Error Getting Item"),
-                                   transferIssueLineBalance, __FILE__, __LINE__);
-              systemError(this,
-                storedProcErrorLookup("sufficientInventoryToShipItem",
-                          result)
-                .arg(transferIssueLineBalance.value("item_number").toString())
-                .arg(transferIssueLineBalance.value("tohead_srcname").toString()), __FILE__, __LINE__);
+              ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Transfer Order Item Information"),
+                                     storedProcErrorLookup("sufficientInventoryToShipItem", result)
+                                     .arg(transferIssueLineBalance.value("item_number").toString())
+                                     .arg(transferIssueLineBalance.value("tohead_srcname").toString()),
+                                     __FILE__, __LINE__);
               return;
+            } else {
+                ErrorReporter::error(QtCriticalMsg, this, tr("Error Checking Item Availability"),
+                                     transferIssueLineBalance, __FILE__, __LINE__);
             }
           }
         }
@@ -1915,9 +1896,10 @@ void transferOrder::sIssueLineBalance()
         if (result < 0)
         {
           rollback.exec();
-          systemError(this, storedProcErrorLookup("issueLineBalance", result) +
-                            tr("<br>Line Item %1").arg(selected[i]->text(0)),
-                      __FILE__, __LINE__);
+          ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Line Item Balance"),
+                               storedProcErrorLookup("issueLineBalance", result) +
+                               tr("<br>Line Item %1").arg(selected[i]->text(0)),
+                               __FILE__, __LINE__);
           return;
         }
         if (distributeInventory::SeriesAdjust(transferIssueLineBalance.value("result").toInt(), this) == XDialog::Rejected)
@@ -1931,9 +1913,9 @@ void transferOrder::sIssueLineBalance()
       }
       else
       {
-        rollback.exec();
-        systemError(this, tr("Line Item %1\n").arg(selected[i]->text(0)) +
-                          transferIssueLineBalance.lastError().databaseText(), __FILE__, __LINE__);
+        rollback.exec();     
+        ErrorReporter::error(QtCriticalMsg, this, tr("Error Issuing Line Item %1\n").arg(selected[i]->text(0)),
+                             transferIssueLineBalance, __FILE__, __LINE__);
         return;
       }
     }
@@ -1974,7 +1956,10 @@ bool transferOrder::sQESave()
   if (! _qeitem->submitAll())
   {
     if (! _qeitem->lastError().databaseText().isEmpty())
-      systemError(this, _qeitem->lastError().databaseText(), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: Error Saving Quick Item Information: \n %2")
+                        .arg(windowTitle())
+                        .arg(_qeitem->lastError().databaseText()),__FILE__,__LINE__);
     return false;
   }
   sFillItemList();
@@ -1985,7 +1970,9 @@ void transferOrder::sQEDelete()
 {
   if (! _qeitem->removeRow(_qeitemView->currentIndex().row()))
   {
-    systemError(this, tr("Removing row from view failed"), __FILE__, __LINE__);
+    ErrorReporter::error(QtCriticalMsg, this, tr("Error Occurred"),
+                         tr("%1: Removing row from view failed ")
+                            .arg(windowTitle()),__FILE__,__LINE__);
     return;
   }
 }
@@ -2085,23 +2072,9 @@ void transferOrder::sHandleTrnsWhs(const int pid)
 
 void transferOrder::sReleaseTohead()
 {
-  if (!_locked)
-    return;
-
-  XSqlQuery query;
-  query.prepare("SELECT pg_advisory_unlock(oid::integer, :tohead_id) AS result "
-                "FROM pg_class "
-                "WHERE relname='tohead';");
-  query.bindValue(":tohead_id", _toheadid);
-  query.exec();
-  if (query.first() && ! query.value("result").toBool())
-    systemError(this, tr("Could not release this Transfer Order record."),
-                __FILE__, __LINE__);
-  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Line"),
-                                query, __FILE__, __LINE__))
-    return;
-  else
-    _locked = false;
+  if (! _lock.release())
+    (void)ErrorReporter::error(QtCriticalMsg, this, tr("Locking Error"),
+                               _lock.lastError(), __FILE__, __LINE__);
 }
 
 void transferOrder::sReleaseNumber()
