@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -20,7 +20,7 @@ class unpostedPurchaseOrders : public display, public Ui::unpostedPurchaseOrders
     Q_OBJECT
 
 public:
-    unpostedPurchaseOrders(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::Window);
+    unpostedPurchaseOrders(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::Window);
 
     virtual bool checkSitePrivs(int orderid);
 
@@ -30,7 +30,9 @@ public slots:
     virtual void sNew();
     virtual void sPopulateMenu(QMenu*, QTreeWidgetItem*, int);
     virtual void sRelease();
+    virtual void sUnrelease();
     virtual void sPrint();
+    virtual void sPrintForms();
     virtual void sView();
     virtual bool setParams(ParameterList &);
 };
