@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -11,8 +11,12 @@
 #ifndef UPDATEITEMSITELEADTIMES_H
 #define UPDATEITEMSITELEADTIMES_H
 
+#include <QProgressDialog>
+
 #include "guiclient.h"
 #include "xdialog.h"
+#include <parameter.h>
+
 #include "ui_updateItemSiteLeadTimes.h"
 
 class updateItemSiteLeadTimes : public XDialog, public Ui::updateItemSiteLeadTimes
@@ -20,7 +24,7 @@ class updateItemSiteLeadTimes : public XDialog, public Ui::updateItemSiteLeadTim
     Q_OBJECT
 
 public:
-    updateItemSiteLeadTimes(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    updateItemSiteLeadTimes(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~updateItemSiteLeadTimes();
 
 public slots:
