@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -19,10 +19,11 @@ class updatePrices : public XDialog, public Ui::updatePrices
   Q_OBJECT
 
 public:
-  updatePrices(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+  updatePrices(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
   ~updatePrices();
 
 public slots:
+  virtual SetResponse set( const ParameterList & pParams );
   virtual void sUpdate();
   virtual void populate();
   virtual void sAdd();
@@ -31,11 +32,13 @@ public slots:
   virtual void sRemoveAll();
   virtual void sHandleBy(bool toggled);
   virtual void sHandleCharPrice();
-  virtual void closeEvent( QCloseEvent * pEvent );
 
 protected slots:
   virtual void languageChange();
 
+private:
+  bool _listpricesched;
+  
 };
 
 #endif // UPDATEPRICES_H
