@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -28,6 +28,7 @@ user::user(QWidget* parent, const char * name, Qt::WindowFlags fl)
   setupUi(this);
 
   _authCache     = false;
+  _windowCache   = "";
   _cUsername     = "";
   _crmacctid     = -1;
   _inTransaction = false;
@@ -55,10 +56,14 @@ user::user(QWidget* parent, const char * name, Qt::WindowFlags fl)
   connect(_revokeSite, SIGNAL(clicked()), this, SLOT(sRevokeSite()));
 
   _available->addColumn("Available Privileges", -1, Qt::AlignLeft);
+  _available->addColumn("Description", -1, Qt::AlignLeft);
   _granted->addColumn("Granted Privileges", -1, Qt::AlignLeft);
+  _granted->addColumn("Description", -1, Qt::AlignLeft);
 
   _availableGroup->addColumn("Available Roles", -1, Qt::AlignLeft);
+  _availableGroup->addColumn("Description", -1, Qt::AlignLeft);
   _grantedGroup->addColumn("Granted Roles", -1, Qt::AlignLeft);
+  _grantedGroup->addColumn("Description", -1, Qt::AlignLeft);
 
   _availableSite->addColumn("Available Sites", -1, Qt::AlignLeft);
   _grantedSite->addColumn("Granted Sites",      -1, Qt::AlignLeft);
@@ -133,7 +138,7 @@ enum SetResponse user::set(const ParameterList &pParams)
     {
       _mode = cEdit;
 
-      _username->setEnabled(FALSE);
+      _username->setEnabled(false);
     }
     else if (param.toString() == "view")
     {
@@ -156,6 +161,7 @@ enum SetResponse user::set(const ParameterList &pParams)
   _email->setEnabled(canEdit);
   _employee->setReadOnly(! canEdit);
   _enhancedAuth->setEnabled(canEdit);
+  _ssosOnly->setEnabled(canEdit);
   _exportContents->setEnabled(canEdit);
   _initials->setEnabled(canEdit);
   _locale->setEnabled(canEdit);
@@ -214,17 +220,17 @@ bool user::save()
 
   QList<GuiErrorCheck> errors;
   errors << GuiErrorCheck(! username.contains(QRegExp("[A-Za-z]")), _username,
-                          tr("You must enter a valid Username before you can save this User."))
+                          tr("You must enter a valid Username before you can save this User Account."))
          << GuiErrorCheck(_username->text().contains(QRegExp("\\s")), _username,
                           tr("The Username cannot include any spaces."))
          << GuiErrorCheck(_passwd->text().isEmpty(), _passwd,
-                          tr("You must enter a valid Password before you can save this User."))
+                          tr("You must enter a valid Password before you can save this User Account."))
          << GuiErrorCheck(_passwd->text() != _verify->text(), _passwd,
                           tr("The entered password and verify do not match. "
                              "Please enter both again carefully."))
    ;
 
-  if (GuiErrorCheck::reportErrors(this, tr("Cannot save User"), errors))
+  if (GuiErrorCheck::reportErrors(this, tr("Cannot save User Account"), errors))
   {
      if (_passwd->text() != _verify->text())
      {
@@ -246,7 +252,7 @@ bool user::save()
 
       if(isCloud || isXtuple)
       {
-        salt = "private";
+        salt = "j3H44uadEI#8#kSmkh#H%JSLAKDOHImklhdfsn3#432?%^kjasdjla3uy989apa3uipoweurw-03235##+=-lhkhdNOHA?%@mxncvbwoiwerNKLJHwe278NH28shNeGc";
       }
       else
       {
@@ -283,7 +289,7 @@ bool user::save()
       usrq.bindValue(":username", username);
       usrq.bindValue(":createUsers", QVariant(_createUsers->isChecked()));
       usrq.exec();
-      if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User"),
+      if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User Account"),
                                usrq, __FILE__, __LINE__))
         return false;
     }
@@ -299,7 +305,7 @@ bool user::save()
       usrq.exec( QString("ALTER GROUP xtrole ADD USER %1;")
               .arg(username) );
     }
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User"),
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User Account"),
                              usrq, __FILE__, __LINE__))
       return false;
   }
@@ -313,6 +319,13 @@ bool user::save()
     if (ErrorReporter::error(QtCriticalMsg, this, tr("Setting Password"),
                              usrq, __FILE__, __LINE__))
       return false;
+    usrq.prepare("SELECT setUserPreference(:username, 'PasswordResetDate', :passdate);");
+    usrq.bindValue(":username", username);
+    usrq.bindValue(":passdate", QDate::currentDate());
+    usrq.exec();
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User Account"),
+                             usrq, __FILE__, __LINE__))
+      return false;
   }
 
   usrq.prepare("SELECT setUserPreference(:username, 'DisableExportContents', :export),"
@@ -335,10 +348,21 @@ bool user::save()
   usrq.bindValue(":agent", (_agent->isChecked() ? "t" : "f"));
   usrq.bindValue(":active", (_active->isChecked() ? "t" : "f"));
   usrq.exec();
-  if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User"),
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User Account"),
                            usrq, __FILE__, __LINE__))
     return false;
 
+  if (_ssosOnly->isChecked() || _windowCache == "salesOrderSimple")
+  {
+    usrq.prepare("SELECT setUserPreference(:username, 'window', :ssosonly);");
+    usrq.bindValue(":username", username);
+    usrq.bindValue(":ssosonly", (_ssosOnly->isChecked() ? "salesOrderSimple" : ""));
+    usrq.exec();
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Saving User Account"),
+                             usrq, __FILE__, __LINE__))
+      return false;
+  }
+
   omfgThis->sUserUpdated(username);
   return true;
 }
@@ -351,7 +375,7 @@ void user::sModuleSelected(const QString &pModule)
   _availableGroup->clear();
   _grantedGroup->clear();
   XSqlQuery groups;
-  groups.prepare("SELECT grp_id, grp_name, usrgrp_id"
+  groups.prepare("SELECT grp_id, grp_name, grp_descrip, usrgrp_id"
                  "  FROM grp LEFT OUTER JOIN usrgrp"
                  "    ON (usrgrp_grp_id=grp_id AND usrgrp_username=:username);");
   groups.bindValue(":username", _cUsername);
@@ -359,9 +383,9 @@ void user::sModuleSelected(const QString &pModule)
   while(groups.next())
   {
     if (groups.value("usrgrp_id").toInt() == 0)
-      available = new XTreeWidgetItem(_availableGroup, available, groups.value("grp_id").toInt(), groups.value("grp_name"));
+      available = new XTreeWidgetItem(_availableGroup, available, groups.value("grp_id").toInt(), groups.value("grp_name"), groups.value("grp_descrip"));
     else
-      granted = new XTreeWidgetItem(_grantedGroup, granted, groups.value("grp_id").toInt(), groups.value("grp_name"));
+      granted = new XTreeWidgetItem(_grantedGroup, granted, groups.value("grp_id").toInt(), groups.value("grp_name"), groups.value("grp_descrip"));
   }
   if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Groups"),
                            groups, __FILE__, __LINE__))
@@ -371,7 +395,7 @@ void user::sModuleSelected(const QString &pModule)
   _granted->clear();
 
   XSqlQuery privs;
-  privs.prepare( "SELECT priv_id, priv_name "
+  privs.prepare( "SELECT priv_id, priv_name, priv_descrip "
                  "FROM priv "
                  "WHERE (priv_module=:priv_module) "
                  "ORDER BY priv_name;" );
@@ -407,10 +431,10 @@ void user::sModuleSelected(const QString &pModule)
     do
     {
       if (usrpriv.findFirst("priv_id", privs.value("priv_id").toInt()) == -1 && grppriv.findFirst("priv_id", privs.value("priv_id").toInt()) == -1)
-        available = new XTreeWidgetItem(_available, available, privs.value("priv_id").toInt(), privs.value("priv_name"));
+        available = new XTreeWidgetItem(_available, available, privs.value("priv_id").toInt(), privs.value("priv_name"), privs.value("priv_descrip"));
       else
       {
-        granted = new XTreeWidgetItem(_granted, granted, privs.value("priv_id").toInt(), privs.value("priv_name"));
+        granted = new XTreeWidgetItem(_granted, granted, privs.value("priv_id").toInt(), privs.value("priv_name"), privs.value("priv_descrip"));
         if(usrpriv.findFirst("priv_id", privs.value("priv_id").toInt()) == -1)
           granted->setTextColor(Qt::gray);
       }
@@ -512,6 +536,12 @@ void user::sRevokeAll()
 
 void user::sAddGroup()
 {
+  if (_availableGroup->id() == -1)
+  {
+      QMessageBox::critical(this, tr("Error"), tr("Please select an Available Role."));
+      return;
+  }
+
   XSqlQuery grpq;
   grpq.prepare("SELECT grantGroup(:username, :grp_id) AS result;");
   grpq.bindValue(":username", _cUsername);
@@ -527,6 +557,12 @@ void user::sAddGroup()
 
 void user::sRevokeGroup()
 {
+  if (_grantedGroup->id() == -1)
+  {
+      QMessageBox::critical(this, tr("Error"), tr("Please select a Granted Role."));
+      return;
+  }
+
   XSqlQuery grpq;
   grpq.prepare("SELECT revokeGroup(:username, :grp_id) AS result;");
   grpq.bindValue(":username", _cUsername);
@@ -545,7 +581,7 @@ void user::sCheck()
   //This regexp checks to make sure the user name starts with a letter.
   QRegExp re("^\\d"); // just digits
   QRegExp re2("^\\w"); // this includes all letters & numbers of all alphabets
-  _cUsername = _username->text().trimmed();
+  _cUsername = _username->text().trimmed().toLower();
   if (((re.indexIn(_cUsername) != -1) || (re2.indexIn(_cUsername) == -1)) && _username->text() != "")
   {
       QMessageBox::critical(this, tr("Error"), tr("User names must begin with a letter."));
@@ -564,12 +600,36 @@ void user::sCheck()
     {
       sPopulate();
       _mode = cEdit;
-      _username->setEnabled(FALSE);
+      _username->setEnabled(false);
       _properName->setFocus();
     }
-    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting User"),
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting User Account"),
                                   usrq, __FILE__, __LINE__))
       return;
+    
+    XSqlQuery dupq;
+    dupq.prepare("SELECT crmacct_id"
+                 "  FROM crmacct "
+                 " WHERE (UPPER(crmacct_number)=UPPER(:username))"
+                 "   AND crmacct_usr_username IS NULL;");
+    dupq.bindValue(":username", _cUsername);
+    dupq.exec();
+    if (dupq.first())
+    {
+      if (QMessageBox::question(this, tr("Convert"),
+                                tr("<p>This number is currently assigned to CRM Account. "
+                                   "Do you want to convert the CRM Account to a User Account?"),
+                                QMessageBox::Yes,
+                                QMessageBox::No | QMessageBox::Default) == QMessageBox::No)
+      {
+        _username->clear();
+        _username->setFocus();
+        return;
+      }
+    }
+    else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting User CRM"),
+                                  dupq, __FILE__, __LINE__))
+      return;
   }
 }
 
@@ -578,7 +638,9 @@ bool user::sPopulate()
   XSqlQuery usrq;
   if (! _cUsername.isEmpty())
   {
-    usrq.prepare("SELECT *, userCanCreateUsers(usr_username) AS createusers,"
+    usrq.prepare("SELECT usr_username, usr_active, usr_propername, usr_initials,"
+                 "       usr_email, usr_locale_id, usr_agent,"
+                 "       userCanCreateUsers(usr_username) AS createusers,"
                  "       userCanCreateUsers(getEffectiveXtUser()) AS enablecreateusers,"
                  "       crmacct_id, crmacct_emp_id, crmacct_owner_username"
                  "  FROM usr"
@@ -594,9 +656,9 @@ bool user::sPopulate()
                  "          FROM locale"
                  "         WHERE locale_code='Default') AS usr_locale_id,"
                  "       NULL  AS usr_passwd,  cntct_initials AS usr_initials,"
-                 "       FALSE AS usr_agent,   crmacct_active AS usr_active,"
+                 "       false AS usr_agent,   crmacct_active AS usr_active,"
                  "       NULL  AS usr_window,  cntct_email AS usr_email,"
-                 "       FALSE AS createusers,"
+                 "       false AS createusers,"
                  "       userCanCreateUsers(getEffectiveXtUser()) AS enablecreateusers,"
                  "       crmacct_id, crmacct_emp_id, crmacct_owner_username"
                  "  FROM crmacct"
@@ -616,10 +678,10 @@ bool user::sPopulate()
       {
         QMessageBox::warning(this, tr("No Spaces Allowed"),
                              tr("<p>Usernames cannot include space characters "
-                                "but must also match the associated CRM Account "
-                                "numbers. Please Cancel the User window and "
-                                "remove the spaces from the CRM Account number "
-                                "before trying to create this User."));
+                                "but must also match the associated Account "
+                                "numbers. Please Cancel the User Account window and "
+                                "remove the spaces from the Account number "
+                                "before trying to create this User Account."));
         return false;
       }
       _username->setEnabled(false);
@@ -635,6 +697,7 @@ bool user::sPopulate()
     _employee->setId(usrq.value("crmacct_emp_id").toInt());
     _crmacctid = usrq.value("crmacct_id").toInt();
     _crmowner = usrq.value("crmacct_owner_username").toString();
+    _cUsername = _username->text().trimmed().toLower();
 
     _passwd->setText("        ");
     _verify->setText("        ");
@@ -648,7 +711,7 @@ bool user::sPopulate()
     if(usrq.first())
       _exportContents->setChecked(usrq.value("usrpref_value").toString()=="t");
     else
-      _exportContents->setChecked(FALSE);
+      _exportContents->setChecked(false);
 
     usrq.prepare( "SELECT usrpref_value "
                "  FROM usrpref "
@@ -661,26 +724,29 @@ bool user::sPopulate()
       _authCache = (usrq.value("usrpref_value").toString()=="t");
     _enhancedAuth->setChecked(_authCache);
 
-    usrq.prepare( "SELECT priv_module "
-               "FROM usrpriv, priv "
-               "WHERE ( (usrpriv_priv_id=priv_id)"
-               " AND (usrpriv_username=:username) ) "
-               "ORDER BY priv_module "
-               "LIMIT 1;" );
-    usrq.bindValue(":username", _cUsername);
-    usrq.exec();
-    if (usrq.first())
+    if (_metrics->boolean("SSOSEnabled"))
     {
-      _module->setCode(usrq.value("priv_module").toString());
-      sModuleSelected(_module->currentText());
+      usrq.prepare( "SELECT usrpref_value "
+                   "  FROM usrpref "
+                   " WHERE ( (usrpref_name = 'window') "
+                   "   AND (usrpref_username=:username) ); ");
+      usrq.bindValue(":username", _cUsername);
+      usrq.exec();
+      _windowCache = "";
+      if(usrq.first())
+        _windowCache = (usrq.value("usrpref_value").toString());
+      _ssosOnly->setChecked(_windowCache=="salesOrderSimple");
     }
     else
     {
-      _module->setCurrentIndex(0);
-      sModuleSelected(_module->itemText(0));
+      _ssosOnly->setChecked(false);
+      _ssosOnly->hide();
     }
+    
+    _module->setCurrentIndex(0);
+    sModuleSelected(_module->itemText(0));
   }
-  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting User"),
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting User Account"),
                                 usrq, __FILE__, __LINE__))
     return false;
 
@@ -692,7 +758,7 @@ bool user::sPopulate()
   usrq.bindValue(":username", _cUsername);
   usrq.exec();
   if(usrq.first())
-    _selectedSites->setChecked(TRUE);
+    _selectedSites->setChecked(true);
   else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting User Sites"),
                                 usrq, __FILE__, __LINE__))
     return false;
@@ -714,7 +780,7 @@ void user::sEnhancedAuthUpdate()
 {
   if((_mode == cEdit) && (_authCache != _enhancedAuth->isChecked()) && (_passwd->text() == "        "))
     QMessageBox::information( this, tr("Enhanced Authentication"),
-      tr("<p>You have changed this user's Enhanced Authentication option. "
+      tr("<p>You have changed this User Account's Enhanced Authentication option. "
          "The password must be updated in order for this change to take "
          "full effect.") );
 }
@@ -846,7 +912,7 @@ void user::sCrmAccount()
     params.append("mode", "edit");
   else
   {
-    qWarning("tried to open CRM Account window without privilege");
+    qWarning("tried to open Account window without privilege");
     return;
   }
 
