@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2015 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -20,37 +20,43 @@
 #include <QSqlError>
 #include <QValidator>
 #include <QVariant>
+#include <metasql.h>
+#include <mqlutil.h>
 
 #include "errorReporter.h"
 #include "guiErrorCheck.h"
 #include "purchaseOrderList.h"
 #include "purchaseOrderItem.h"
 #include "storedProcErrorLookup.h"
+#include "taxBreakdown.h"
 #include "voucherItem.h"
 #include "voucherMiscDistrib.h"
 
-voucher::voucher(QWidget* parent, const char* name, Qt::WFlags fl)
+voucher::voucher(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
 
-  connect(_save,          SIGNAL(clicked()), this, SLOT(sSave()));
-  connect(_distributions, SIGNAL(clicked()), this, SLOT(sDistributions()));
-  connect(_distributeline, SIGNAL(clicked()), this, SLOT(sDistributeLine()));
-  connect(_clear, SIGNAL(clicked()), this, SLOT(sClear()));
-  connect(_distributeall, SIGNAL(clicked()), this, SLOT(sDistributeAll()));
-  connect(_voucherNumber, SIGNAL(editingFinished()), this, SLOT(sHandleVoucherNumber()));
-  connect(_poNumber, SIGNAL(newId(int, const QString&)), this, SLOT(sFillList()));
-  connect(_poNumber, SIGNAL(newId(int, const QString&)), this, SLOT(sPopulatePoInfo()));
-  connect(_amountToDistribute, SIGNAL(valueChanged()), this, SLOT(sPopulateBalanceDue()));
-  connect(_new, SIGNAL(clicked()), this, SLOT(sNewMiscDistribution()));
-  connect(_edit, SIGNAL(clicked()), this, SLOT(sEditMiscDistribution()));
-  connect(_delete, SIGNAL(clicked()), this, SLOT(sDeleteMiscDistribution()));
-  connect(_invoiceDate, SIGNAL(newDate(const QDate&)), this, SLOT(sPopulateDistDate()));
-  connect(_terms, SIGNAL(newID(int)), this, SLOT(sPopulateDueDate()));
-  connect(_poitem, SIGNAL(populateMenu(QMenu*,QTreeWidgetItem*,int)), this, SLOT(sPopulateMenu(QMenu*)));
-  connect(_amountToDistribute, SIGNAL(idChanged(int)), this, SLOT(sFillList()));
-  connect(_amountDistributed, SIGNAL(valueChanged()), this, SLOT(sPopulateBalanceDue()));
+  connect(_save,                     SIGNAL(clicked()),                                 this,          SLOT(sSave()));
+  connect(_distributions,            SIGNAL(clicked()),                                 this,          SLOT(sDistributions()));
+  connect(_distributeline,           SIGNAL(clicked()),                                 this,          SLOT(sDistributeLine()));
+  connect(_clear,                    SIGNAL(clicked()),                                 this,          SLOT(sClear()));
+  connect(_distributeall,            SIGNAL(clicked()),                                 this,          SLOT(sDistributeAll()));
+  connect(_voucherNumber,            SIGNAL(editingFinished()),                         this,          SLOT(sHandleVoucherNumber()));
+  connect(_poNumber,                 SIGNAL(newId(int, const QString&)),                this,          SLOT(sFillList()));
+  connect(_poNumber,                 SIGNAL(newId(int, const QString&)),                this,          SLOT(sPopulatePoInfo()));
+  connect(_amountToDistribute,       SIGNAL(valueChanged()),                            this,          SLOT(sPopulateBalanceDue()));
+  connect(_new,                      SIGNAL(clicked()),                                 this,          SLOT(sNewMiscDistribution()));
+  connect(_edit,                     SIGNAL(clicked()),                                 this,          SLOT(sEditMiscDistribution()));
+  connect(_delete,                   SIGNAL(clicked()),                                 this,          SLOT(sDeleteMiscDistribution()));
+  connect(_invoiceDate,              SIGNAL(newDate(const QDate&)),                     this,          SLOT(sPopulateDistDate()));
+  connect(_distributionDate,         SIGNAL(newDate(const QDate&)),                     this,          SLOT(sNewDistDate()));
+  connect(_taxLit,                   SIGNAL(leftClickedURL(const QString&)),            this,          SLOT(sTaxDetail()));
+  connect(_terms,                    SIGNAL(newID(int)),                                this,          SLOT(sPopulateDueDate()));
+  connect(_poitem,                   SIGNAL(populateMenu(QMenu*,QTreeWidgetItem*,int)), this,          SLOT(sPopulateMenu(QMenu*)));
+  connect(_amountToDistribute,       SIGNAL(idChanged(int)),                            this,          SLOT(sFillList()));
+  connect(_amountDistributed,        SIGNAL(valueChanged()),                            this,          SLOT(sPopulateBalanceDue()));
+  connect(_freight,                  SIGNAL(valueChanged()),                            this,          SLOT(sPopulateDistributed()));
 
   _terms->setType(XComboBox::APTerms);
   _poNumber->setAllowedStatuses(OrderLineEdit::Open);
@@ -82,6 +88,9 @@ voucher::voucher(QWidget* parent, const char* name, Qt::WFlags fl)
 
   _miscDistrib->addColumn(tr("Account"),    -1,           Qt::AlignLeft,   true,  "account"  );
   _miscDistrib->addColumn(tr("Amount"),     _moneyColumn, Qt::AlignRight,  true,  "vodist_amount" );
+  _miscDistrib->addColumn(tr("Notes"),      -1,           Qt::AlignLeft,   true,  "vodist_notes" );
+
+  _charass->setType("VCH");
 
   _vendid = -1;
 
@@ -121,7 +130,11 @@ enum SetResponse voucher::set(const ParameterList &pParams)
       insq.bindValue(":vohead_number", _voucherNumber->text());
       insq.exec();
       if (insq.first())
+      {
         _voheadid = insq.value("vohead_id").toInt();
+        _documents->setId(_voheadid);
+        _charass->setId(_voheadid);
+      }
       else if (ErrorReporter::error(QtCriticalMsg, this, tr("Inserting Voucher"),
                                     insq, __FILE__, __LINE__))
         return UndefinedError;
@@ -132,8 +145,8 @@ enum SetResponse voucher::set(const ParameterList &pParams)
     {
       _mode = cEdit;
 
-      _voucherNumber->setEnabled(FALSE);
-      _poNumber->setEnabled(FALSE);
+      _voucherNumber->setEnabled(false);
+      _poNumber->setEnabled(false);
     }
     else if (param.toString() == "view")
     {
@@ -141,24 +154,29 @@ enum SetResponse voucher::set(const ParameterList &pParams)
 
       _poNumber->setAllowedStatuses(OrderLineEdit::AnyStatus);
 
-      _voucherNumber->setEnabled(FALSE);
-      _poNumber->setEnabled(FALSE);
-      _taxzone->setEnabled(FALSE);
-      _amountToDistribute->setEnabled(FALSE);
-      _distributionDate->setEnabled(FALSE);
-      _invoiceDate->setEnabled(FALSE);
-      _dueDate->setEnabled(FALSE);
-      _invoiceNum->setEnabled(FALSE);
-      _reference->setEnabled(FALSE);
-      _distributions->setEnabled(FALSE);
-      _miscDistrib->setEnabled(FALSE);
-      _new->setEnabled(FALSE);
-      _terms->setEnabled(FALSE);
+      _voucherNumber->setEnabled(false);
+      _poNumber->setEnabled(false);
+      _taxzone->setEnabled(false);
+      _amountToDistribute->setEnabled(false);
+      _distributionDate->setEnabled(false);
+      _invoiceDate->setEnabled(false);
+      _dueDate->setEnabled(false);
+      _invoiceNum->setEnabled(false);
+      _reference->setEnabled(false);
+      _distributions->setEnabled(false);
+      _miscDistrib->setEnabled(false);
+      _new->setEnabled(false);
+      _terms->setEnabled(false);
       _terms->setType(XComboBox::Terms);
-      _flagFor1099->setEnabled(FALSE);
-      _distributeall->setEnabled(FALSE);
+      _flagFor1099->setEnabled(false);
+      _distributeall->setEnabled(false);
       _notes->setEnabled(false);
+//      _documents->setReadOnly(true);
+      _charass->setReadOnly(true);
       _close->setText(tr("&Close"));
+      _freight->setEnabled(false);
+      _freightExpcat->setEnabled(false);
+   
       _save->hide();
 
       disconnect(_poitem, SIGNAL(valid(bool)), _distributions, SLOT(setEnabled(bool)));
@@ -176,6 +194,8 @@ enum SetResponse voucher::set(const ParameterList &pParams)
   if (valid)
   {
     _voheadid = param.toInt();
+    _documents->setId(_voheadid);
+    _charass->setId(_voheadid);
     populate();
     enableWindowModifiedSetting();
   }
@@ -183,6 +203,16 @@ enum SetResponse voucher::set(const ParameterList &pParams)
   return NoError;
 }
 
+int voucher::id() const
+{
+  return _voheadid;
+}
+
+int voucher::mode() const
+{
+  return _mode;
+}
+
 bool voucher::sSave()
 {
   QList<GuiErrorCheck> errors;
@@ -203,7 +233,11 @@ bool voucher::sSave()
                           _invoiceNum,
                           tr("<p>You must enter a Vendor Invoice Number "
                              "before you may save this Voucher."))
-   ;
+         << GuiErrorCheck(_freight->localValue() > 0 && !_freightExpcat->isValid(), _freightExpcat,
+                           tr("<p>You must select an Expense Category to post the  "
+                              "freight value to."))
+  ;
+
   if (GuiErrorCheck::reportErrors(this, tr("Cannot Save Voucher"), errors))
     return false;
 
@@ -247,6 +281,8 @@ bool voucher::sSave()
              "    vohead_invcnumber=:vohead_invcnumber,"
              "    vohead_reference=:vohead_reference,"
              "    vohead_amount=:vohead_amount,"
+             "    vohead_freight=:vohead_freight,"
+             "    vohead_freight_expcat_id=:vohead_freight_expcat,"
              "    vohead_1099=:vohead_1099, "
              "    vohead_curr_id=:vohead_curr_id, "
              "    vohead_notes=:vohead_notes "
@@ -264,6 +300,9 @@ bool voucher::sSave()
   updq.bindValue(":vohead_invcnumber", _invoiceNum->text().trimmed());
   updq.bindValue(":vohead_reference", _reference->text().trimmed());
   updq.bindValue(":vohead_amount", _amountToDistribute->localValue());
+  updq.bindValue(":vohead_freight", _freight->localValue());
+  if (_freightExpcat->isValid())
+    updq.bindValue(":vohead_freight_expcat", _freightExpcat->id());
   updq.bindValue(":vohead_1099", QVariant(_flagFor1099->isChecked()));
   updq.bindValue(":vohead_curr_id", _amountToDistribute->id());
   updq.bindValue(":vohead_notes", _notes->toPlainText());
@@ -297,12 +336,16 @@ bool voucher::sSave()
   _poitem->clear();
   _miscDistrib->clear();
   _notes->setText("");
+  _charass->setId(-1);
+  _freight->clear();
+  _freightExpcat->setId(-1);
 
   setWindowModified(false);
 
   ParameterList params;
   params.append("mode", "new");
   set(params);
+  _poNumber->setFocus();
   return true;
 }
 
@@ -335,8 +378,8 @@ void voucher::sHandleVoucherNumber()
     {
       _voheadid = editq.value("vohead_id").toInt();
 
-      _voucherNumber->setEnabled(FALSE);
-      _poNumber->setEnabled(FALSE);
+      _voucherNumber->setEnabled(false);
+      _poNumber->setEnabled(false);
 
       _mode = cEdit;
       populate();
@@ -367,7 +410,7 @@ void voucher::sDistributions()
     params.append("curr_id", _amountToDistribute->id());
     params.append("effective", _amountToDistribute->effective());
 
-    voucherItem newdlg(this, "", TRUE);
+    voucherItem newdlg(this, "", true);
     newdlg.set(params);
     if (newdlg.exec() == XDialog::Rejected)
     {
@@ -375,7 +418,7 @@ void voucher::sDistributions()
       // voucherItem wraps itself in a transaction and rolls back on reject
     }
     else
-      _poNumber->setEnabled(FALSE);
+      _poNumber->setEnabled(false);
   }
   sFillList();
   sPopulateDistributed();
@@ -402,7 +445,7 @@ void voucher::sDistributeLine()
                     __FILE__, __LINE__);
       else
       {
-        _poNumber->setEnabled(FALSE);
+        _poNumber->setEnabled(false);
       }
     }
     else if (ErrorReporter::error(QtCriticalMsg, this, tr("Distributing Line"),
@@ -472,7 +515,7 @@ void voucher::sDistributeAll()
                              storedProcErrorLookup("distributeVoucherLine", result),
                              __FILE__, __LINE__);
       else
-        _poNumber->setEnabled(FALSE);
+        _poNumber->setEnabled(false);
     }
     else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Distributing"),
                                   distq, __FILE__, __LINE__))
@@ -494,10 +537,11 @@ void voucher::sNewMiscDistribution()
   if (_taxzone->isValid())
     params.append("taxzone_id", _taxzone->id());
 
-  voucherMiscDistrib newdlg(this, "", TRUE);
+  voucherMiscDistrib newdlg(this, "", true);
   newdlg.set(params);
   if (newdlg.exec() != XDialog::Rejected)
   {
+    sUpdateVoucherTax();
     sFillMiscList();
     sPopulateDistributed();
   }
@@ -515,10 +559,11 @@ void voucher::sEditMiscDistribution()
   if (_taxzone->isValid())
     params.append("taxzone_id", _taxzone->id());
 
-  voucherMiscDistrib newdlg(this, "", TRUE);
+  voucherMiscDistrib newdlg(this, "", true);
   newdlg.set(params);
   if (newdlg.exec() != XDialog::Rejected)
   {
+    sUpdateVoucherTax();
     sFillMiscList();
     sPopulateDistributed();
   }
@@ -544,93 +589,36 @@ void voucher::sDeleteMiscDistribution()
   if (ErrorReporter::error(QtCriticalMsg, this, tr("Deleting Distributions"),
                            delq, __FILE__, __LINE__))
     return;
-
-  sFillMiscList();
-  sPopulateDistributed();
+  else
+  {
+    sUpdateVoucherTax();
+    sFillMiscList();
+    sPopulateDistributed();
+  }
 }
 
 void voucher::sFillList()
 {
   if (_poNumber->isValid())
   {
-    XSqlQuery getq;
-    getq.prepare( "SELECT poitem_id, poitem_linenumber,"
-               "       CASE WHEN(poitem_status='C') THEN :closed"
-               "            WHEN(poitem_status='U') THEN :unposted"
-               "            WHEN(poitem_status='O' AND ((poitem_qty_received-poitem_qty_returned) > 0) AND (poitem_qty_ordered>(poitem_qty_received-poitem_qty_returned))) THEN :partial"
-               "            WHEN(poitem_status='O' AND ((poitem_qty_received-poitem_qty_returned) > 0) AND (poitem_qty_ordered<=(poitem_qty_received-poitem_qty_returned))) THEN :received"
-               "            WHEN(poitem_status='O') THEN :open"
-               "            ELSE poitem_status"
-               "       END AS poitemstatus,"
-               "       COALESCE(item_number, poitem_vend_item_number) AS itemnumber,"
-               "       COALESCE(uom_name, poitem_vend_uom) AS uom,"
-               "       poitem_vend_item_number, poitem_vend_uom,"
-               "       poitem_qty_ordered,"
-               "       ( SELECT COALESCE(SUM(recv_qty), 0)"
-               "         FROM recv"
-               "         WHERE ( (recv_posted)"
-               "           AND (recv_invoiced)"
-               "           AND (recv_orderitem_id=poitem_id) ) ) AS qtyinvoiced,"
-               "       ( SELECT COALESCE(SUM(recv_qty), 0)"
-               "         FROM recv"
-               "         WHERE ( (recv_posted)"
-               "           AND (NOT recv_invoiced)"
-               "           AND (recv_vohead_id IS NULL)"
-               "           AND (recv_orderitem_id=poitem_id) ) ) AS qtyreceived,"
-               "       ( SELECT COALESCE(SUM(poreject_qty), 0)"
-               "         FROM poreject"
-               "         WHERE ( (poreject_posted)"
-               "           AND (NOT poreject_invoiced)"
-               "           AND (poreject_vohead_id IS NULL)"
-               "           AND (poreject_poitem_id=poitem_id) ) ) AS qtyrejected,"
-               "       ( SELECT COALESCE(SUM(vodist_qty), 0)"
-               "         FROM vodist"
-               "         WHERE vodist_poitem_id=poitem_id"
-               "           AND vodist_vohead_id=:vohead_id ) AS invoiceqty, "
-               "       ( SELECT COALESCE(SUM(vodist_amount), 0)"
-               "         FROM vodist"
-               "         WHERE vodist_poitem_id=poitem_id"
-               "           AND vodist_vohead_id=:vohead_id ) "
-               "     + ( SELECT COALESCE(SUM(COALESCE(voitem_freight,0)), 0)"
-               "         FROM voitem"
-               "         WHERE voitem_poitem_id=poitem_id"
-               "           AND   voitem_vohead_id=:vohead_id ) AS invoiceamount,"
-               "       poitem_unitprice,"
-               "       (poitem_unitprice * poitem_qty_ordered) AS extprice,"
-               "       poitem_freight,"
-               "       'qty' AS poitem_qty_ordered_xtnumericrole,"
-               "       'qty' AS qtyinvoiced_xtnumericrole,"
-               "       'qty' AS qtyreceived_xtnumericrole,"
-               "       'qty' AS qtyrejected_xtnumericrole,"
-               "       'qty' AS invoiceqty_xtnumericrole,"
-               "       'curr' AS invoiceamount_xtnumericrole,"
-               "       'curr' AS poitem_unitprice_xtnumericrole,"
-               "       'curr' AS extprice_xtnumericrole,"
-               "       'curr' AS poitem_freight_xtnumericrole "
-               "FROM poitem JOIN pohead ON (pohead_id=poitem_pohead_id)"
-               "            LEFT OUTER JOIN ( itemsite JOIN item ON (itemsite_item_id=item_id)"
-               "                                       JOIN uom ON (item_inv_uom_id=uom_id) )"
-               "                                ON (poitem_itemsite_id=itemsite_id) "
-               "WHERE (poitem_pohead_id=:pohead_id) "
-               "GROUP BY poitem_id, poitem_linenumber, poitem_status,"
-               "         item_number, uom_name,"
-               "         poitem_vend_item_number, poitem_vend_uom,"
-               "         poitem_unitprice, poitem_freight,"
-               "         poitem_qty_ordered, poitem_qty_received, poitem_qty_returned,"
-               "         itemsite_id, pohead_curr_id "
-               "ORDER BY poitem_linenumber;" );
-    getq.bindValue(":vohead_id", _voheadid);
-    getq.bindValue(":pohead_id", _poNumber->id());
-    getq.bindValue(":closed", tr("Closed"));
-    getq.bindValue(":unposted", tr("Unposted"));
-    getq.bindValue(":partial", tr("Partial"));
-    getq.bindValue(":received", tr("Received"));
-    getq.bindValue(":open", tr("Open"));
-    getq.exec();
-    _poitem->populate(getq);
-    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting P/O Information"),
-                             getq, __FILE__, __LINE__))
-      return;
+    MetaSQLQuery mql = mqlLoad("voucher", "poitems");
+    ParameterList params;
+
+    params.append("vohead_id", _voheadid);
+    params.append("pohead_id", _poNumber->id());
+    params.append("closed",    tr("Closed"));
+    params.append("unposted",  tr("Unposted"));
+    params.append("partial",   tr("Partial"));
+    params.append("received",  tr("Received"));
+    params.append("open",      tr("Open"));
+
+    XSqlQuery poitemFillList = mql.toQuery(params);
+    _poitem->populate(poitemFillList, true);
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving PO Items"),
+                                poitemFillList, __FILE__, __LINE__))
+        return;
+
+    _freight->setId(_amountToDistribute->id());
   }
   else
     _poitem->clear();
@@ -640,35 +628,15 @@ void voucher::sFillMiscList()
 {
   if (_poNumber->isValid())
   {
-    XSqlQuery getq;
-    getq.prepare( "SELECT vodist_id, (formatGLAccount(accnt_id) || ' - ' || accnt_descrip) AS account,"
-               "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-               "FROM vodist, accnt "
-               "WHERE ( (vodist_poitem_id=-1)"
-               " AND (vodist_accnt_id=accnt_id)"
-               " AND (vodist_vohead_id=:vohead_id) ) "
-               "UNION ALL "
-               "SELECT vodist_id, (expcat_code || ' - ' || expcat_descrip) AS account,"
-               "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-               "  FROM vodist, expcat "
-               " WHERE ( (vodist_poitem_id=-1)"
-               "   AND   (vodist_expcat_id=expcat_id)"
-               "   AND   (vodist_vohead_id=:vohead_id) ) "
-               "UNION ALL "
-               "SELECT vodist_id, (tax_code || ' - ' || tax_descrip) AS account,"
-               "       vodist_amount, 'curr' AS vodist_amount_xtnumericrole "
-               "  FROM vodist, tax "
-               " WHERE ( (vodist_poitem_id=-1)"
-               "   AND   (vodist_tax_id=tax_id)"
-               "   AND   (vodist_vohead_id=:vohead_id) ) "
-               "ORDER BY account;" );
-    getq.bindValue(":vohead_id", _voheadid);
-    getq.exec();
-    _miscDistrib->populate(getq);
-    if (ErrorReporter::error(QtCriticalMsg, this,
-                             tr("Getting Misc. Distributions"),
-                             getq, __FILE__, __LINE__))
-      return;
+    MetaSQLQuery mql = mqlLoad("voucher", "miscDistr");
+    ParameterList params;
+
+    params.append("vohead_id", _voheadid);
+    XSqlQuery miscFillList = mql.toQuery(params);
+    _miscDistrib->populate(miscFillList, true);
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Misc Distributions"),
+                                miscFillList, __FILE__, __LINE__))
+        return;
   }
 }
 
@@ -692,12 +660,15 @@ void voucher::sPopulatePoInfo()
     _terms->setId(po.value("pohead_terms_id").toInt());
     _taxzone->setId(po.value("pohead_taxzone_id").toInt());
     _amountToDistribute->setId(po.value("pohead_curr_id").toInt());
+    _freight->setId(po.value("pohead_curr_id").toInt());
     _vendid = po.value("vend_id").toInt();
     _vendor->setText(po.value("vend_number").toString());
     _vendName->setText(po.value("vend_name").toString());
     // TODO: replace with a compact AddressCluster when such exists
     _vendAddress1->setText(po.value("addr_line1").toString());
     _vendAddress2->setText(po.value("addr_line2").toString());
+    if (_mode != cView)
+      _new->setEnabled(true);
   }
   else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting P/O Information"),
                                 po, __FILE__, __LINE__))
@@ -708,6 +679,7 @@ void voucher::sPopulateDistributed()
 {
   if (_poNumber->isValid())
   {
+    sCalculateTax();
     XSqlQuery getq;
     getq.prepare( "SELECT (COALESCE(dist,0) + COALESCE(freight,0) + COALESCE(tax,0)) AS distrib"
                "  FROM (SELECT SUM(COALESCE(voitem_freight,0)) AS freight"
@@ -727,7 +699,7 @@ void voucher::sPopulateDistributed()
     getq.exec();
     if (getq.first())
     {
-      _amountDistributed->setLocalValue(getq.value("distrib").toDouble());
+      _amountDistributed->setLocalValue(getq.value("distrib").toDouble() + _freight->localValue());
     }
     else if (ErrorReporter::error(QtCriticalMsg, this,
                                   tr("Getting Distributions"),
@@ -738,11 +710,19 @@ void voucher::sPopulateDistributed()
 
 void voucher::sPopulateBalanceDue()
 {
-  _balance->setLocalValue(_amountToDistribute->localValue() - _amountDistributed->localValue());
+  _balance->setLocalValue(_amountToDistribute->localValue() - 
+                          _amountDistributed->localValue());
+
   if (_balance->isZero())
     _balance->setPaletteForegroundColor(QColor("black"));
   else
     _balance->setPaletteForegroundColor(namedColor("error"));
+
+  if (_freight->localValue() <= 0)
+    _freightExpcat->setId(-1);
+
+  _freightExpcat->setEnabled(_freight->localValue() > 0 && _mode != cView);
+ 
 }
 
 void voucher::populateNumber()
@@ -752,7 +732,7 @@ void voucher::populateNumber()
   if (getq.first())
   {
     _voucherNumber->setText(getq.value("vouchernumber").toString());
-    _voucherNumber->setEnabled(FALSE);
+    _voucherNumber->setEnabled(false);
   }
   else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting ID"),
                                 getq, __FILE__, __LINE__))
@@ -764,7 +744,7 @@ void voucher::populate()
   XSqlQuery vohead;
   vohead.prepare( "SELECT vohead_number, vohead_pohead_id, vohead_taxzone_id, vohead_terms_id,"
                   "       vohead_distdate, vohead_docdate, vohead_duedate,"
-                  "       vohead_invcnumber, vohead_reference,"
+                  "       vohead_invcnumber, vohead_reference, vohead_freight, vohead_freight_expcat_id,"
                   "       vohead_1099, vohead_amount, vohead_curr_id, vohead_notes "
                   "FROM vohead "
                   "WHERE (vohead_id=:vohead_id);" );
@@ -779,6 +759,10 @@ void voucher::populate()
     _amountToDistribute->set(vohead.value("vohead_amount").toDouble(),
                              vohead.value("vohead_curr_id").toInt(),
                              vohead.value("vohead_docdate").toDate(), false);
+    _freight->set(vohead.value("vohead_freight").toDouble(),
+                             vohead.value("vohead_curr_id").toInt(),
+                             vohead.value("vohead_docdate").toDate(), false);
+    _freightExpcat->setId(vohead.value("vohead_freight_expcat_id").toInt());
 
     _distributionDate->setDate(vohead.value("vohead_distdate").toDate(), true);
     _invoiceDate->setDate(vohead.value("vohead_docdate").toDate());
@@ -791,6 +775,7 @@ void voucher::populate()
     sFillList();
     sFillMiscList();
     sPopulateDistributed();
+    _charass->setId(_voheadid);
   }
   else if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Voucher"),
                                 vohead, __FILE__, __LINE__))
@@ -882,7 +867,7 @@ void voucher::sView()
   params.append("poitem_id", _poitem->id());
   params.append("mode", "view");
 
-  purchaseOrderItem newdlg(this, "", TRUE);
+  purchaseOrderItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -898,7 +883,7 @@ void voucher::keyPressEvent( QKeyEvent * e )
     e->ignore();
 }
 
-void voucher::saveDetail()
+bool voucher::saveDetail()
 {
   if (_mode != cView)
   {
@@ -914,6 +899,8 @@ void voucher::saveDetail()
                "    vohead_invcnumber=:vohead_invcnumber,"
                "    vohead_reference=:vohead_reference,"
                "    vohead_amount=:vohead_amount,"
+               "    vohead_freight=:vohead_freight,"
+               "    vohead_freight_expcat_id=:vohead_freightexpcat,"
                "    vohead_1099=:vohead_1099, "
                "    vohead_curr_id=:vohead_curr_id, "
                "    vohead_notes=:vohead_notes "
@@ -931,19 +918,91 @@ void voucher::saveDetail()
     updq.bindValue(":vohead_invcnumber", _invoiceNum->text().trimmed());
     updq.bindValue(":vohead_reference", _reference->text().trimmed());
     updq.bindValue(":vohead_amount", _amountToDistribute->localValue());
+    updq.bindValue(":vohead_freight", _freight->localValue());
+    if (_freightExpcat->isValid())
+      updq.bindValue(":vohead_freightexpcat", _freightExpcat->id());
     updq.bindValue(":vohead_1099", QVariant(_flagFor1099->isChecked()));
     updq.bindValue(":vohead_curr_id", _amountToDistribute->id());
     updq.bindValue(":vohead_notes", _notes->toPlainText());
     updq.exec();
     if (ErrorReporter::error(QtCriticalMsg, this, tr("Updating Voucher"),
                              updq, __FILE__, __LINE__))
-      return;
+      return false;
+  }
+  return true;
+}
+
+void voucher::sCalculateTax()
+{
+  if (_vendid == -1)
+    return;
+
+  if (!saveDetail())
+    return;
+
+  XSqlQuery taxq;
+  taxq.prepare( "SELECT ABS(SUM(tax)) AS tax "
+                "FROM ("
+                "SELECT ROUND(SUM(taxdetail_tax),2) AS tax "
+                "FROM tax "
+                " JOIN calculateTaxDetailSummary('VO', :vohead_id, 'T') ON (taxdetail_tax_id=tax_id)"
+                "GROUP BY tax_id) AS data;" );
+  taxq.bindValue(":vohead_id", _voheadid);
+  taxq.exec();
+  if (taxq.first())
+    _tax->setLocalValue(taxq.value("tax").toDouble());
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Calculating Voucher Tax"),
+                                    taxq, __FILE__, __LINE__))
+        return;
+}
+
+void voucher::sTaxDetail()
+{
+  if (!saveDetail())
+    return;
+
+  ParameterList params;
+  params.append("order_id", _voheadid);
+  params.append("order_type", "VO");
+  // mode => view since there are no fields to hold modified tax data
+  if (_mode == cView)
+    params.append("mode", "view");
+
+  taxBreakdown newdlg(this, "", true);
+  newdlg.set(params);
+  newdlg.exec();
+}
+
+void voucher::sUpdateVoucherTax()
+{
+  if (!_taxzone->isValid() || !_distributionDate->isValid())
+    return;
+
+  XSqlQuery updTax;
+  updTax.prepare("SELECT updatemiscvouchertax(:voheadid,:taxzone,:distdate,:curr) as ret;");
+  updTax.bindValue(":voheadid", _voheadid);
+  updTax.bindValue(":taxzone",  _taxzone->id());
+  updTax.bindValue(":distdate", _distributionDate->date());
+  updTax.bindValue(":curr", _amountToDistribute->id());
+  updTax.exec();
+  if (ErrorReporter::error(QtCriticalMsg, this, tr("Adding Tax to Voucher"),
+                         updTax, __FILE__, __LINE__))
+    return;
+}
+
+void voucher::sNewDistDate()
+{
+  if (_miscDistrib->topLevelItemCount())
+  {
+     sUpdateVoucherTax();
+     sFillMiscList();
   }
 }
 
 void voucher::enableWindowModifiedSetting()
 {
   connect(_amountToDistribute,     SIGNAL(valueChanged()), this, SLOT(sDataChanged()));
+  connect(_freight,                SIGNAL(valueChanged()), this, SLOT(sDataChanged()));
   connect(_distributionDate,SIGNAL(newDate(const QDate&)), this, SLOT(sDataChanged()));
   connect(_dueDate,         SIGNAL(newDate(const QDate&)), this, SLOT(sDataChanged()));
   connect(_flagFor1099,             SIGNAL(toggled(bool)), this, SLOT(sDataChanged()));
