@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -21,7 +21,7 @@ class voucherMiscDistrib : public XDialog, public Ui::voucherMiscDistrib
     Q_OBJECT
 
 public:
-    voucherMiscDistrib(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    voucherMiscDistrib(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~voucherMiscDistrib();
 
 public slots:
@@ -29,6 +29,7 @@ public slots:
     virtual void populate();
     virtual void sSave();
     virtual void sCheck();
+    virtual void sHandleSelection();
     virtual void sPopulateVendorInfo(int);
 
 protected slots:
