@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -22,7 +22,7 @@ class woMaterialItem : public XDialog, public Ui::woMaterialItem
     Q_OBJECT
 
 public:
-    woMaterialItem(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    woMaterialItem(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~woMaterialItem();
 
 public slots:
@@ -35,12 +35,16 @@ protected slots:
     virtual void sSave();
     virtual void sUpdateQtyRequired();
     virtual void sItemIdChanged();
+    virtual void sPopulateUOM();
+    virtual void sUOMChanged();
+    virtual void sEnableSave();
 
 private:
     int _bomitemid;
     int _mode;
     int _womatlid;
     int _wooperid;
+    int _invuomid;
     bool _captive;
 
 };
