@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,13 +18,14 @@
 
 #include "dspInventoryAvailability.h"
 #include "dspSubstituteAvailabilityByItem.h"
+#include "errorReporter.h"
 #include "inputManager.h"
 #include "returnWoMaterialItem.h"
 #include "storedProcErrorLookup.h"
 #include "substituteList.h"
 #include "woMaterialItem.h"
 
-workOrderMaterials::workOrderMaterials(QWidget* parent, const char* name, Qt::WFlags fl)
+workOrderMaterials::workOrderMaterials(QWidget* parent, const char* name, Qt::WindowFlags fl)
     : XWidget(parent, name, fl)
 {
   setupUi(this);
@@ -73,7 +74,7 @@ workOrderMaterials::workOrderMaterials(QWidget* parent, const char* name, Qt::WF
   }
   else
   {
-    _new->setEnabled(FALSE);
+    _new->setEnabled(false);
     connect(_womatl, SIGNAL(itemSelected(int)), _view, SLOT(animateClick()));
   }
 
@@ -118,21 +119,24 @@ enum SetResponse workOrderMaterials::set(const ParameterList &pParams)
 
 void workOrderMaterials::sPopulateMenu(QMenu *pMenu, QTreeWidgetItem *)
 {
-  QAction *menuItem;
+  double qtyiss = _womatl->rawValue("womatl_qtyiss").toDouble();
+  //QAction *menuItem;
 
-  menuItem = pMenu->addAction(tr("Edit..."), this, SLOT(sEdit()));
-  menuItem = pMenu->addAction(tr("View..."), this, SLOT(sView()));
+  (void)pMenu->addAction(tr("Edit..."), this, SLOT(sEdit()));
+  (void)pMenu->addAction(tr("View..."), this, SLOT(sView()));
   
-  menuItem = pMenu->addAction(tr("Delete..."), this, SLOT(sDelete()));
+  if (qtyiss == 0.0)
+    (void)pMenu->addAction(tr("Delete..."), this, SLOT(sDelete()));
   
   pMenu->addSeparator();
   
-  menuItem = pMenu->addAction(tr("View Availability..."), this, SLOT(sViewAvailability()));
+  (void)pMenu->addAction(tr("View Availability..."), this, SLOT(sViewAvailability()));
   
   pMenu->addSeparator();
   
-  menuItem = pMenu->addAction(tr("View Item-Defined Subsitute Availability..."), this, SLOT(sViewSubstituteAvailability()));
-  menuItem = pMenu->addAction(tr("Substitute..."), this, SLOT(sSubstitute()));
+  (void)pMenu->addAction(tr("View Item-Defined Subsitute Availability..."), this, SLOT(sViewSubstituteAvailability()));
+  if (qtyiss == 0.0)
+    (void)pMenu->addAction(tr("Substitute..."), this, SLOT(sSubstitute()));
 }
 
 void workOrderMaterials::sNew()
@@ -141,7 +145,7 @@ void workOrderMaterials::sNew()
   params.append("mode", "new");
   params.append("wo_id", _wo->id());
 
-  woMaterialItem newdlg(this, "", TRUE);
+  woMaterialItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -152,7 +156,7 @@ void workOrderMaterials::sEdit()
   params.append("mode", "edit");
   params.append("womatl_id", _womatl->id());
 
-  woMaterialItem newdlg(this, "", TRUE);
+  woMaterialItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -163,7 +167,7 @@ void workOrderMaterials::sView()
   params.append("mode", "view");
   params.append("womatl_id", _womatl->id());
 
-  woMaterialItem newdlg(this, "", TRUE);
+  woMaterialItem newdlg(this, "", true);
   newdlg.set(params);
   newdlg.exec();
 }
@@ -189,7 +193,7 @@ void workOrderMaterials::sDelete()
         ParameterList params;
         params.append("womatl_id", womatlid);
 
-        returnWoMaterialItem newdlg(omfgThis, "", TRUE);
+        returnWoMaterialItem newdlg(omfgThis, "", true);
         newdlg.set(params);
 
         newdlg.exec();
@@ -225,17 +229,19 @@ void workOrderMaterials::sDelete()
     int result = workDelete.value("result").toInt();
     if (result < 0)
     {
-      systemError(this, storedProcErrorLookup("deleteWoMaterial", result), __FILE__, __LINE__);
+      ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Work Order Material Information"),
+                             storedProcErrorLookup("deleteWoMaterial", result),
+                             __FILE__, __LINE__);
       return;
     }
   }
-  else if (workDelete.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Deleting Work Order Material Information"),
+                                workDelete, __FILE__, __LINE__))
   {
-    systemError(this, workDelete.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 
-  omfgThis->sWorkOrderMaterialsUpdated(_wo->id(), _womatl->id(), TRUE);
+  omfgThis->sWorkOrderMaterialsUpdated(_wo->id(), _womatl->id(), true);
 }
 
 void workOrderMaterials::sViewAvailability()
@@ -257,9 +263,9 @@ void workOrderMaterials::sViewAvailability()
     newdlg->set(params);
     omfgThis->handleNewWindow(newdlg);
   }
-  else if (workViewAvailability.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Material Information"),
+                                workViewAvailability, __FILE__, __LINE__))
   {
-    systemError(this, workViewAvailability.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -283,9 +289,9 @@ void workOrderMaterials::sViewSubstituteAvailability()
     newdlg->set(params);
     omfgThis->handleNewWindow(newdlg);
   }
-  else if (workViewSubstituteAvailability.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Material Information"),
+                                workViewSubstituteAvailability, __FILE__, __LINE__))
   {
-    systemError(this, workViewSubstituteAvailability.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -310,7 +316,7 @@ void workOrderMaterials::sSubstitute()
     params.append("byDate", sub.value("womatl_duedate"));
     params.append("run");
 
-    substituteList substitute(this, "", TRUE);
+    substituteList substitute(this, "", true);
     substitute.set(params);
     int result = substitute.exec();
     if (result != XDialog::Rejected)
@@ -334,7 +340,7 @@ void workOrderMaterials::sSubstitute()
       else if (sub.value("womatl_issuemethod").toString() == "M")
         params.append("issueMethod", "mixed");
 
-      woMaterialItem newdlg(this, "", TRUE);
+      woMaterialItem newdlg(this, "", true);
       newdlg.set(params);
       if (newdlg.exec() != XDialog::Rejected)
       {
@@ -343,13 +349,13 @@ void workOrderMaterials::sSubstitute()
         workSubstitute.bindValue(":womatl_id", womatlid);
         workSubstitute.exec();
 
-        omfgThis->sWorkOrderMaterialsUpdated(_wo->id(), _womatl->id(), TRUE);
+        omfgThis->sWorkOrderMaterialsUpdated(_wo->id(), _womatl->id(), true);
       }
     }
   }
-  else if (workSubstitute.lastError().type() != QSqlError::NoError)
+  else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Material Information"),
+                                workSubstitute, __FILE__, __LINE__))
   {
-    systemError(this, workSubstitute.lastError().databaseText(), __FILE__, __LINE__);
     return;
   }
 }
@@ -365,7 +371,10 @@ void workOrderMaterials::sFillList()
   XSqlQuery workFillList;
   if (_wo->isValid())
   {
-    workFillList.prepare( "SELECT womatl_id, *,"
+    workFillList.prepare( "SELECT womatl_id, item_number, uom_name,"
+               "       womatl_qtyfxd, womatl_qtyper, womatl_scrap,"
+               "       womatl_qtyreq, womatl_qtyiss, womatl_qtywipscrap,"
+               "       womatl_duedate, womatl_notes, womatl_ref,"
                "       (item_descrip1 || ' ' || item_descrip2) AS description,"
                "       CASE WHEN (womatl_issuemethod = 'S') THEN :push"
                "            WHEN (womatl_issuemethod = 'L') THEN :pull"
@@ -382,12 +391,12 @@ void workOrderMaterials::sFillList()
                "       'qty' AS balance_xtnumericrole,"
                "       CASE WHEN (womatl_duedate <= CURRENT_DATE) THEN 'expired'"
                "       END AS womatl_duedate_qtforegroundrole "
-               "FROM wo, womatl, itemsite, item, uom "
-               "WHERE ( (womatl_wo_id=wo_id)"
-               " AND (womatl_uom_id=uom_id)"
-               " AND (womatl_itemsite_id=itemsite_id)"
-               " AND (itemsite_item_id=item_id)"
-               " AND (wo_id=:wo_id) ) "
+               "  FROM wo"
+               "  JOIN womatl   ON (wo_id=womatl_wo_id)"
+               "  JOIN itemsite ON (womatl_itemsite_id=itemsite_id)"
+               "  JOIN item     ON (itemsite_item_id=item_id)"
+               "  JOIN uom      ON (womatl_uom_id=uom_id)"
+               " WHERE (wo_id=:wo_id) "
                "ORDER BY item_number;" );
     workFillList.bindValue(":wo_id", _wo->id());
     workFillList.bindValue(":push",  tr("Push"));
@@ -396,9 +405,9 @@ void workOrderMaterials::sFillList()
     workFillList.bindValue(":error", tr("Error"));
     workFillList.exec();
     _womatl->populate(workFillList);
-    if (workFillList.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Materials"),
+                             workFillList, __FILE__, __LINE__))
     {
-      systemError(this, workFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -412,8 +421,8 @@ void workOrderMaterials::sFillList()
                "GROUP BY item_picklist;" );
     workFillList.bindValue(":wo_id", _wo->id());
     workFillList.exec();
-    bool   foundPick    = FALSE;
-    bool   foundNonPick = FALSE;
+    bool   foundPick    = false;
+    bool   foundNonPick = false;
     int    totalNumber  = 0;
     double totalQtyPer  = 0.0;
     while (workFillList.next())
@@ -423,20 +432,20 @@ void workOrderMaterials::sFillList()
 
       if (workFillList.value("item_picklist").toBool())
       {
-        foundPick = TRUE;
+        foundPick = true;
         _pickNumber->setText(workFillList.value("total").toDouble());
         _pickQtyPer->setText(formatQtyPer(workFillList.value("qtyper").toDouble()));
       }
       else
       {
-        foundNonPick = TRUE;
+        foundNonPick = true;
         _nonPickNumber->setText(workFillList.value("total").toDouble());
         _nonPickQtyPer->setText(formatQtyPer(workFillList.value("qtyper").toDouble()));
       }
     }
-    if (workFillList.lastError().type() != QSqlError::NoError)
+    if (ErrorReporter::error(QtCriticalMsg, this, tr("Getting Pick Info"),
+                             workFillList, __FILE__, __LINE__))
     {
-      systemError(this, workFillList.lastError().databaseText(), __FILE__, __LINE__);
       return;
     }
 
@@ -476,15 +485,16 @@ void workOrderMaterials::sFillList()
         _currentActCost->setText(workFillList.value("f_actcost").toDouble());
         _maxCost->setText(workFillList.value("f_maxcost").toDouble());
       }
-      else if (workFillList.lastError().type() != QSqlError::NoError)
+      else if (ErrorReporter::error(QtCriticalMsg, this, tr("Error Retrieving Work Order Material Information"),
+                                    workFillList, __FILE__, __LINE__))
       {
-	systemError(this, workFillList.lastError().databaseText(), __FILE__, __LINE__);
-	return;
+        return;
       }
     }
   }
   else
   {
+    _womatl->clear();
     _pickNumber->clear();
     _pickQtyPer->clear();
     _nonPickNumber->clear();
