@@ -1,16 +1,18 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
  * to be bound by its terms.
  */
 
-
 #include <QFileInfo>
+#include <QMessageBox>
 #include <QSqlError>
+#include <QSslCertificate>
+#include <QSslConfiguration>
 
 #include <currcluster.h>
 
@@ -76,6 +78,52 @@ YourPayProcessor::YourPayProcessor() : CreditCardProcessor()
   _msgHash.insert( 100, tr("Received unexpected approval code %1. This "
                            "transaction has been treated as approved by "
                            "YourPay."));
+
+#ifdef Q_OS_WIN
+  _pemfile = _metrics->value("CCYPWinPathPEM");
+#elif defined Q_OS_MAC
+  _pemfile = _metrics->value("CCYPMacPathPEM");
+#elif defined Q_OS_LINUX
+  _pemfile = _metrics->value("CCYPLinPathPEM");
+#endif
+  QFile pemio(_pemfile);
+  if (! pemio.exists())
+    QMessageBox::warning(0, tr("Could not find PEM file"),
+                         tr("<p>Failed to find the PEM file %1")
+                         .arg(_pemfile));
+  else
+  {
+    QList<QSslCertificate> certlist = QSslCertificate::fromPath(_pemfile);
+    if (DEBUG) qDebug("%d certificates", certlist.size());
+    if (certlist.isEmpty())
+      QMessageBox::warning(0, tr("Failed to load Certificate"),
+                           tr("<p>There are no Certificates in %1. "
+                              "This may cause communication problems.")
+                           .arg(_pemfile));
+    else if (certlist.at(0).isNull())
+      QMessageBox::warning(0, tr("Failed to load Certificate"),
+                           tr("<p>Failed to load a Certificate from "
+                              "the PEM file %1. "
+                              "This may cause communication problems.")
+                           .arg(_pemfile));
+    else
+    {
+      QSslConfiguration config = QSslConfiguration::defaultConfiguration();
+      QList<QSslCertificate> certs = config.caCertificates();
+      foreach(QSslCertificate c, certlist)
+      {
+        if (certificateIsValid(&c) && ! certs.contains(c))
+          certs.append(c);
+        else
+          QMessageBox::warning(0, tr("Invalid Certificate"),
+                               tr("<p>The Certificate in %1 appears to be invalid. "
+                                  "This may cause communication problems.")
+                               .arg(_pemfile));
+      }
+      config.setCaCertificates(certs);
+      QSslConfiguration::setDefaultConfiguration(config);
+    }
+  }
 }
 
 int YourPayProcessor::buildCommon(const int pccardid, const QString &pcvv, const double pamount, QDomDocument &prequest, QString pordertype)
@@ -199,7 +247,7 @@ int YourPayProcessor::buildCommon(const int pccardid, const QString &pcvv, const
 		    ypq.value("ccard_address1").toString().section(" ", 0, 0));
 
   if (DEBUG)
-    qDebug("YP:buildCommon built %s", prequest.toString().toAscii().data());
+    qDebug("YP:buildCommon built %s", prequest.toString().toLatin1().data());
   return 0;
 }
 
@@ -208,7 +256,7 @@ int YourPayProcessor::doAuthorize(const int pccardid, const QString &pcvv, doubl
   if (DEBUG)
     qDebug("YP:doAuthorize(%d, pcvv, %f, %f, %d, %f, %f, %d, %s, %s, %d)",
 	   pccardid, pamount, ptax, ptaxexempt, pfreight, pduty, pcurrid,
-	   pneworder.toAscii().data(), preforder.toAscii().data(), pccpayid);
+	   pneworder.toLatin1().data(), preforder.toLatin1().data(), pccpayid);
 
   int returnValue = 0;
   double amount = currToCurr(pcurrid, _ypcurrid, pamount, &returnValue);
@@ -280,7 +328,7 @@ int  YourPayProcessor::doCharge(const int pccardid, const QString &pcvv, const d
   if (DEBUG)
     qDebug("YP:doCharge(%d, pcvv, %f, %f, %d, %f, %f, %d, %s, %s, %d)",
 	   pccardid, pamount, ptax, ptaxexempt, pfreight, pduty, pcurrid,
-	   pneworder.toAscii().data(), preforder.toAscii().data(), pccpayid);
+	   pneworder.toLatin1().data(), preforder.toLatin1().data(), pccpayid);
 
   int returnValue = 0;
   double amount = currToCurr(pcurrid, _ypcurrid, pamount, &returnValue);
@@ -464,9 +512,9 @@ int YourPayProcessor::handleResponse(const QString &presponse, const int pccardi
 {
   if (DEBUG)
     qDebug("YP::handleResponse(%s, %d, %s, %f, %d, %s, %d, pparams)",
-	   presponse.toAscii().data(), pccardid,
-	   ptype.toAscii().data(), pamount, pcurrid,
-	   preforder.toAscii().data(), pccpayid);
+	   presponse.toLatin1().data(), pccardid,
+	   ptype.toLatin1().data(), pamount, pcurrid,
+	   preforder.toLatin1().data(), pccpayid);
 
   QDomDocument response;
   // YP doesn't even send back a valid XML doc!
@@ -654,7 +702,7 @@ int YourPayProcessor::handleResponse(const QString &presponse, const int pccardi
 
   if (DEBUG)
     qDebug("YP:%s\t_passedAvs %d\t_passedCvv %d\t_passedLinkShield %d",
-	    r_avs.toAscii().data(), _passedAvs, _passedCvv, _passedLinkShield);
+	    r_avs.toLatin1().data(), _passedAvs, _passedCvv, _passedLinkShield);
 
   pparams.append("ccard_id",    pccardid);
   pparams.append("auth_charge", ptype);
@@ -709,21 +757,13 @@ int YourPayProcessor::doTestConfiguration()
   if (DEBUG)
     qDebug("YP:doTestConfiguration()");
 
-#ifdef Q_WS_WIN
-   QString pemfile = _metrics->value("CCYPWinPathPEM");
-#elif defined Q_WS_MACX
-   QString pemfile = _metrics->value("CCYPMacPathPEM");
-#elif defined Q_WS_X11
-   QString pemfile = _metrics->value("CCYPLinPathPEM");
-#endif
-
-  if (pemfile.isEmpty())
+  if (_pemfile.isEmpty())
   {
     _errorMsg = errorMsg(-15);
     return -15;
   }
 
-  QFileInfo fileinfo(pemfile.toLatin1());
+  QFileInfo fileinfo(_pemfile.toLatin1());
   if (!fileinfo.isFile())
   {
     _errorMsg = errorMsg(-16).arg(fileinfo.fileName());
