@@ -1,18 +1,20 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
  * to be bound by its terms.
  */
-
 #ifndef __ADDRESSCLUSTERSETUP_H__
 #define __ADDRESSCLUSTERSETUP_H__
+#include "addresscluster.h"
 
 class QScriptEngine;
 
+Q_DECLARE_METATYPE(AddressCluster*)
+
 void setupAddressCluster(QScriptEngine *engine);
 
 #endif
