@@ -1,7 +1,7 @@
 /*
  *This file is part of the xTuple ERP: PostBooks Edition, a free and
  *open source Enterprise Resource Planning software suite,
- *Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ *Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  *It is licensed to you under the Common Public Attribution License
  *version 1.0, the full text of which(including xTuple-specific Exhibits)
  *is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,7 +16,7 @@
 
 QScriptValue scriptFind(QScriptContext *context, QScriptEngine  * engine)
 {
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
   if (context->argumentCount() >= 1 &&
       qscriptvalue_cast<WId>(context->argument(0)))
     return engine->toScriptValue(QWidget::find(qscriptvalue_cast<WId>(context->argument(0))));
@@ -171,7 +171,7 @@ QRect QWidgetProto::contentsRect() const
   return QRect();
 }
 
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
 WId QWidgetProto::effectiveWinId() const
 {
   QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
@@ -298,6 +298,7 @@ int QWidgetProto::heightForWidth(int w) const
   return 0;
 }
 
+#if QT_VERSION < 0x050000
 QInputContext *QWidgetProto::inputContext()
 {
   QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
@@ -305,6 +306,8 @@ QInputContext *QWidgetProto::inputContext()
     return item->inputContext();
   return 0;
 }
+#endif
+
 
 QVariant QWidgetProto::inputMethodQuery(int query) const
 {
@@ -637,12 +640,14 @@ void QWidgetProto::setForegroundRole(int role)
     item->setForegroundRole((QPalette::ColorRole)role);
 }
 
+#if QT_VERSION < 0x050000
 void QWidgetProto::setInputContext(QInputContext *context)
 {
   QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
   if (item)
     item->setInputContext(context);
 }
+#endif
 
 void QWidgetProto::setLayout(QLayout *layout)
 {
@@ -788,7 +793,7 @@ QRegion QWidgetProto::visibleRegion() const
   return QRegion();
 }
 
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
 WId QWidgetProto::winId() const
 {
   QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
@@ -838,3 +843,136 @@ QString QWidgetProto::toString() const
   return QString();
 }
 
+bool QWidgetProto::close()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->close();
+  return false;
+}
+
+void QWidgetProto::hide()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->hide();
+}
+
+void QWidgetProto::lower()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->hide();
+}
+
+void QWidgetProto::raise()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->raise();
+}
+
+void QWidgetProto::repaint()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->repaint();
+}
+
+void QWidgetProto::setDisabled(bool disable)
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->setDisabled(disable);
+}
+
+void QWidgetProto::setEnabled(bool enable)
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->setEnabled(enable);
+}
+
+void QWidgetProto::setFocus()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->setFocus();
+}
+
+void QWidgetProto::setHidden(bool hidden)
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->setHidden(hidden);
+}
+
+void QWidgetProto::setStyleSheet(const QString & styleSheet)
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->setStyleSheet(styleSheet);
+}
+
+void QWidgetProto::setVisible(bool visible)
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->setVisible(visible);
+}
+
+void QWidgetProto::setWindowModified(bool mod)
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->setWindowModified(mod);
+}
+
+void QWidgetProto::setWindowTitle(const QString & title)
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->setWindowTitle(title);
+}
+
+void QWidgetProto::show()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->show();
+}
+
+void QWidgetProto::showFullScreen()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->showFullScreen();
+}
+
+void QWidgetProto::showMaximized()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->showMaximized();
+}
+
+void QWidgetProto::showMinimized()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->showMinimized();
+}
+
+void QWidgetProto::showNormal()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->showNormal();
+}
+
+void QWidgetProto::update()
+{
+  QWidget *item = qscriptvalue_cast<QWidget*>(thisObject());
+  if (item)
+    return item->update();
+}
