@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -28,7 +28,7 @@ void AddressCluster::init()
     _list->setObjectName("_list");
     _list->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
-#ifndef Q_WS_MAC
+#ifndef Q_OS_MAC
     _list->setMaximumWidth(25);
 #else
     _list->setMinimumWidth(60);
@@ -115,7 +115,7 @@ void AddressCluster::init()
     _grid->setColumnStretch(4, 0);
     _grid->setColumnStretch(5, 2);
 
-#if defined Q_WS_MAC
+#if defined Q_OS_MAC
     setMinimumSize(_grid->columnCount() * 60, 132);
 #endif
 
@@ -267,7 +267,7 @@ void AddressCluster::setNumber(QString pNumber)
     return;
 }
 
-void AddressCluster::setId(const int pId)
+void AddressCluster::setId(const int pId, const QString&)
 {
   if (pId == _id)
     return;
@@ -337,7 +337,7 @@ void AddressCluster::silentSetId(const int pId)
         return;
   }
 
-    // _parsed = TRUE;
+    // _parsed = true;
 }
 
 void AddressCluster::setState(const QString &p)
@@ -509,7 +509,7 @@ int AddressCluster::save(enum SaveFlags flag)
     if (datamodQ.value("result").toInt() > 0)
     {
       _id=datamodQ.value("result").toInt();
-      _selected = FALSE;
+      _selected = false;
       _valid = true;
       silentSetId(id());
       return id();
@@ -665,7 +665,7 @@ void AddressCluster::setAddrChange(QString p)
 
 ///////////////////////////////////////////////////////////////////////////
 
-AddressList::AddressList(QWidget* pParent, const char* pName, bool, Qt::WFlags)
+AddressList::AddressList(QWidget* pParent, const char* pName, bool, Qt::WindowFlags)
   : VirtualList(pParent, 0)
 {
     _parent = (AddressCluster*)(pParent);
@@ -678,7 +678,7 @@ AddressList::AddressList(QWidget* pParent, const char* pName, bool, Qt::WFlags)
 
     _listTab->setColumnCount(0);
 
-    _listTab->addColumn(tr("CRM Account"), -1, Qt::AlignLeft, true, "crmacct_name");
+    _listTab->addColumn(tr("Account"), -1, Qt::AlignLeft, true, "crmacct_name");
     _listTab->addColumn(tr("Line 1"),      -1, Qt::AlignLeft, true, "addr_line1");
     _listTab->addColumn(tr("Line 2"),      75, Qt::AlignLeft, true, "addr_line2");
     _listTab->addColumn(tr("Line 3"),      75, Qt::AlignLeft, true, "addr_line3");
@@ -814,7 +814,7 @@ AddressSearch::AddressSearch(QWidget* pParent, Qt::WindowFlags pFlags)
     connect(_searchPostalCode, SIGNAL(toggled(bool)), this, SLOT(sFillList()));
     connect(_searchInactive,   SIGNAL(toggled(bool)), this, SLOT(sFillList()));
 
-    _listTab->addColumn(tr("CRM Account"), -1, Qt::AlignLeft, true, "crmacct_name");
+    _listTab->addColumn(tr("Account"), -1, Qt::AlignLeft, true, "crmacct_name");
     _listTab->addColumn(tr("Line 1"),      -1, Qt::AlignLeft, true, "addr_line1");
     _listTab->addColumn(tr("Line 2"),      75, Qt::AlignLeft, true, "addr_line2");
     _listTab->addColumn(tr("Line 3"),      75, Qt::AlignLeft, true, "addr_line3");
