@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -52,7 +52,7 @@ static QString _listAndSearchQueryString(
       "<? elseif exists('taxauth') ?>"
       "    SELECT taxauth_id AS id,     taxauth_code AS number,"
       "           taxauth_name AS name, NULL AS cntct_id,"
-      "           TRUE AS active,       taxauth_addr_id AS addr_id"
+      "           true AS active,       taxauth_addr_id AS addr_id"
       "      FROM taxauth"
       "<? elseif exists('user') ?>"
       "    SELECT usr_id AS id,           usr_username AS number,"
@@ -62,7 +62,7 @@ static QString _listAndSearchQueryString(
       "<? elseif exists('vendor') ?>"
       "    SELECT vend_id AS id,         vend_number AS number,"
       "           vend_name AS name,     vend_cntct1_id AS cntct_id,"
-      "           vend_active AS active, cntct_addr_id AS addr_id,"
+      "           vend_active AS active, COALESCE(cntct_addr_id, vend_addr_id) AS addr_id,"
       "           vendtype_code AS type"
       "      FROM vendinfo"
       "      JOIN vendtype ON (vend_vendtype_id=vendtype_id)"
@@ -157,7 +157,7 @@ CRMAcctLineEdit::CRMAcctLineEdit(QWidget* pParent, const char* pName) :
     CrmClusterLineEdit(pParent, "crmacct", "crmacct_id", "crmacct_number", "crmacct_name", 0, "crmacct_owner_username", 0, 0, pName, "crmacct_active")
 {
   _objtype = "CRMA";
-  setTitles(tr("CRM Account"), tr("CRM Accounts"));
+  setTitles(tr("Account"), tr("Accounts"));
   setUiName("crmaccount");
   setEditPriv("MaintainAllCRMAccounts");
   setViewPriv("ViewAllCRMAccounts");
@@ -193,7 +193,7 @@ CRMAcctLineEdit::CRMAcctSubtype CRMAcctLineEdit::subtype() const
 
 ///////////////////////////////////////////////////////////////////////
 
-CRMAcctList::CRMAcctList(QWidget* pParent, const char* pName, bool, Qt::WFlags pFlags) :
+CRMAcctList::CRMAcctList(QWidget* pParent, const char* pName, bool, Qt::WindowFlags pFlags) :
   VirtualList(pParent, pFlags)
 {
   _parent = pParent;
@@ -276,7 +276,7 @@ CRMAcctList::CRMAcctList(QWidget* pParent, const char* pName, bool, Qt::WFlags p
   else
     setSubtype(CRMAcctLineEdit::Crmacct);
 
-  resize(800, size().height());
+  resize(800, 600);
 }
 
 void CRMAcctList::setId(const int id)
@@ -338,7 +338,7 @@ void CRMAcctList::setSubtype(const CRMAcctLineEdit::CRMAcctSubtype subtype)
     break;
 
   case CRMAcctLineEdit::User:
-    setWindowTitle(tr("Search For User"));
+    setWindowTitle(tr("Search For User Account"));
     _queryParams->append("user");
     hasContact = false;
     hasAddress = false;
@@ -360,7 +360,7 @@ void CRMAcctList::setSubtype(const CRMAcctLineEdit::CRMAcctSubtype subtype)
   case CRMAcctLineEdit::Competitor:
   case CRMAcctLineEdit::Partner:
   default:
-    setWindowTitle(tr("Search For CRM Account"));
+    setWindowTitle(tr("Search For Account"));
     _queryParams->append("crmaccount");
     break;
   }
@@ -487,7 +487,7 @@ CRMAcctSearch::CRMAcctSearch(QWidget* pParent, Qt::WindowFlags pFlags) :
   setTabOrder(_listTab,		_buttonBox);
   setTabOrder(_buttonBox,	_search);
 
-  resize(800, size().height());
+  resize(800, 600);
   
   _parent = pParent;
   setObjectName("crmacctSearch");
@@ -645,7 +645,7 @@ void CRMAcctSearch::setSubtype(const CRMAcctLineEdit::CRMAcctSubtype subtype)
     break;
 
   case CRMAcctLineEdit::User:
-    setWindowTitle(tr("Search For User"));
+    setWindowTitle(tr("Search For User Account"));
     _queryParams->append("user");
 
     _searchNumber->setText(tr("Username"));
@@ -684,11 +684,11 @@ void CRMAcctSearch::setSubtype(const CRMAcctLineEdit::CRMAcctSubtype subtype)
   case CRMAcctLineEdit::Competitor:
   case CRMAcctLineEdit::Partner:
   default:
-    setWindowTitle(tr("Search For CRM Account"));
+    setWindowTitle(tr("Search For Account"));
     _queryParams->append("crmaccount");
 
-    _searchNumber->setText(tr("CRM Account Number"));
-    _searchName->setText(tr("CRM Account Name"));
+    _searchNumber->setText(tr("Account Number"));
+    _searchName->setText(tr("Account Name"));
     _searchContact->setText(tr("Primary Contact Name"));
     _searchPhone->setText(tr("Primary Contact Phone #"));
     _searchEmail->setText(tr("Primary Contact Email"));
