@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -27,6 +27,7 @@
 
 #include "datecluster.h"
 #include "dcalendarpopup.h"
+#include "errorReporter.h"
 #include "format.h"
 
 #define DEBUG false
@@ -113,11 +114,11 @@ XDateEdit::XDateEdit(QWidget *parent, const char *name) :
   else
     setObjectName(metaObject()->className());
 
-  _allowNull   = FALSE;
+  _allowNull   = false;
   _default     = Empty;
-  _parsed      = FALSE;
+  _parsed      = false;
   _nullString  = QString::null;
-  _valid       = FALSE;
+  _valid       = false;
   _siteId      = -1;
 }
 
@@ -152,7 +153,9 @@ void XDateEdit::parseDate()
 
   _valid = false;
 
-  if (dateString.contains(QRegExp("[0-9]+[-][0-9]+"))) //user enters hyphens instead of slashes
+  if (dateString.contains(QRegExp("[0-9]+[-][0-9]+")) &&
+      QLocale().dateFormat(QLocale::ShortFormat).contains(QRegExp("(M+|y+|d+)/(M+|y+|d+)")))
+  //user enters hyphens instead of slashes and locale uses slashes
   {
     dateString.replace("-", "/");
   }
@@ -173,8 +176,12 @@ void XDateEdit::parseDate()
   else if (dateString[0] == '#')                        // julian day
   {
     int offset = dateString.right(dateString.length() - 1).toInt(&isNumeric);
-    if (isNumeric)
-      checkDate(QDate(today.year(), 1, 1).addDays(offset - 1));
+    if (isNumeric) {
+      if (offset < 0)
+        checkDate(QDate(today.year(), 12, 31).addDays(offset + 1));
+      else
+        checkDate(QDate(today.year(), 1, 1).addDays(offset - 1));
+    }
   }
 
   else if (dateString.contains(QRegExp("^[0-9][0-9]?$"))) // date in month
@@ -385,6 +392,18 @@ void XDateEdit::setDataWidgetMap(XDataWidgetMapper* m)
   m->addMapping(this, _fieldName, "date", "currentDefault");
 }
 
+void XDateEdit::setCalendarSiteId(int siteId)
+{
+  int tmpSite = _siteId;
+  _siteId = siteId;
+  if(siteId!=tmpSite&&tmpSite!=-1)
+  {
+    QDate tmpDate = _currentDate;
+    setNull();
+    checkDate(tmpDate);
+  }
+}
+
 void XDateEdit::setNull()
 {
   if (DEBUG)
@@ -394,16 +413,16 @@ void XDateEdit::setNull()
            _allowNull);
   if (_allowNull)
   {
-    _valid  = TRUE;
-    _parsed = TRUE;
+    _valid  = true;
+    _parsed = true;
     setText(_nullString);
     _currentDate = _nullDate;
     emit newDate(_currentDate);
   }
   else
   {
-    _valid = FALSE;
-    _parsed = TRUE;
+    _valid = false;
+    _parsed = true;
     setText("");
     _currentDate = QDate();
   }
@@ -460,7 +479,7 @@ void XDateEdit::checkDate(const QDate &pDate)
 {
   QDate nextWorkDate = pDate;
 
-  if(determineIfStd() && (_siteId != -1))
+  if(determineIfStd() && (_siteId != -1) && (pDate != _currentDate))
   {
     XSqlQuery workday;
 
@@ -470,19 +489,18 @@ void XDateEdit::checkDate(const QDate &pDate)
     workday.bindValue(":desired", 0);
     workday.exec();
     if (workday.first())
-      nextWorkDate = workday.value("result").toDate();
-    else if (workday.lastError().type() != QSqlError::NoError)
     {
-      QMessageBox::warning(this, tr("No work week calendar found"),
-                            tr("<p>The selected Site has no work week defined. "
-                               "Please go to Schedule Setup and define "
-                               "the working days for this site."));
+      nextWorkDate = workday.value("result").toDate();
+    }
+    else if (ErrorReporter::error(QtWarningMsg, this, tr("No work week calendar found"),
+                                  workday, __FILE__, __LINE__))
+    {
       return;
     }
   }
 
   if (nextWorkDate == pDate)
-    setDate(pDate, TRUE);
+    setDate(pDate, true);
   else
   {
     if (QMessageBox::question(this, tr("Non-Working Day Entered"),
@@ -491,9 +509,9 @@ void XDateEdit::checkDate(const QDate &pDate)
                                 "automatically select the next working day?"),
                              QMessageBox::Yes | QMessageBox::Default,
                              QMessageBox::No  | QMessageBox::Escape) == QMessageBox::Yes)
-      setDate(nextWorkDate, TRUE);
+      setDate(nextWorkDate, true);
     else
-      setDate(pDate, TRUE);
+      setDate(pDate, true);
   }
 }
 
@@ -703,7 +721,7 @@ void DateCluster::setEndDate(const QVariant &pDate)
 
 void DateCluster::setStartNull(const QString &pString, const QDate &pDate, bool pSetNull)
 {
-  _startDate->setAllowNullDate(TRUE);
+  _startDate->setAllowNullDate(true);
   _startDate->setNullString(pString);
   _startDate->setNullDate(pDate);
 
@@ -713,7 +731,7 @@ void DateCluster::setStartNull(const QString &pString, const QDate &pDate, bool
 
 void DateCluster::setEndNull(const QString &pString, const QDate &pDate, bool pSetNull)
 {
-  _endDate->setAllowNullDate(TRUE);
+  _endDate->setAllowNullDate(true);
   _endDate->setNullString(pString);
   _endDate->setNullDate(pDate);
 
