@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -24,7 +24,7 @@ class FileMoveSelector : public QWidget, public Ui::FileMoveSelector
   public:
     enum FileMoveOption { Nothing, Suffix, ChangeDir, Delete };
 
-    FileMoveSelector(QWidget* parent = 0, Qt::WFlags fl = 0);
+    FileMoveSelector(QWidget* parent = 0, Qt::WindowFlags fl = 0);
     ~FileMoveSelector();
 
     Q_INVOKABLE QString        code()           const;
