@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -16,7 +16,10 @@
 #include "glcluster.h"
 
 static QString _listAndSearchQueryString(
-  "SELECT accnt_id, *,"
+  "SELECT accnt_id, accnt_number, accnt_descrip, accnt_comments, accnt_profit,"
+  "       accnt_sub, accnt_type, accnt_extref, accnt_company,"
+  "       accnt_subaccnttype_code, accnt_name,"
+  "       subaccnttype_code, subaccnttype_descrip,"
   "       CASE accnt_type WHEN 'A' THEN <? value('asset') ?>"
   "                       WHEN 'E' THEN <? value('expense') ?>"
   "                       WHEN 'L' THEN <? value('liability') ?>"
@@ -300,7 +303,7 @@ bool GLCluster::projectVisible()
   return _project->isVisible();
 }
 
-void GLCluster::setId(const int p)
+void GLCluster::setId(const int p, const QString&)
 {
   int id = p;
 
@@ -309,14 +312,14 @@ void GLCluster::setId(const int p)
     if (_x_metrics->boolean("EnableProjectAccounting"))
     {
       XSqlQuery qry;
-      qry.prepare("SELECT accnt_id, prjaccnt_prj_id "
+      qry.prepare("SELECT prjaccnt_accnt_id, prjaccnt_prj_id "
                   "FROM xtprjaccnt.prjaccnt "
                   "WHERE (accnt_id=:accnt_id); ");
       qry.bindValue(":accnt_id", p);
       qry.exec();
       if (qry.first())
       {
-        id = qry.value("accnt_id").toInt();
+        id = qry.value("prjaccnt_accnt_id").toInt();
         if (qry.value("prjaccnt_prj_id").toInt() != -1 &&
             !_projectVisible)
           setProjectVisible(true);
