@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,7 +18,7 @@
 #include <QTableWidgetItem>
 #include <QtScript>
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
 #include <QFont>
 #endif
 
@@ -264,7 +264,7 @@ void ParameterWidget::addParam()
 
   toolButton->setObjectName("button" + currRow);
   toolButton->setText(tr("-"));
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
   toolButton->setMinimumWidth(25);
 #else
   toolButton->setMinimumWidth(21);
@@ -462,9 +462,7 @@ void ParameterWidget::applySaved(int pId, int filter_id)
   QString query;
   QString filterValue;
   QDate today = QDate::currentDate();
-  int xid, init_filter_id;
-
-  init_filter_id = filter_id;
+  int xid;
 
   clearFilters();
 
@@ -519,6 +517,8 @@ void ParameterWidget::applySaved(int pId, int filter_id)
     if ( !(tempFilter.isEmpty()) )
     {
       //0 is filterType, 1 is filterValue, 2 is parameterwidgettype
+      //Text fields may have : in them, in this case, the last value
+      // is parameterwidgettype, and filterValue spans all middle indices
       QStringList tempFilterList = tempFilter.split(":");
       QString key = this->getParameterTypeKey(tempFilterList[0]);
       if (key.isEmpty())
@@ -550,10 +550,10 @@ void ParameterWidget::applySaved(int pId, int filter_id)
         XComboBox *mybox = qobject_cast<XComboBox*>(test->widget());
         if (mybox)
           mybox->setCurrentIndex(mybox->findText(key));
-	
+
         found = getFilterWidget(windowIdx);
 
-        int widgetType = tempFilterList[2].toInt();
+        int widgetType = tempFilterList[tempFilterList.size()-1].toInt();
 
         switch (widgetType)
         {
@@ -724,7 +724,7 @@ void ParameterWidget::applySaved(int pId, int filter_id)
             QLineEdit *lineEdit = qobject_cast<QLineEdit*>(found);
             if (lineEdit != 0)
             {
-              lineEdit->setText(tempFilterList[1]);
+              lineEdit->setText(QStringList(tempFilterList.mid(1, tempFilterList.size()-2)).join(":"));
               storeFilterValue(-1, lineEdit);
             }
           }
@@ -1207,7 +1207,7 @@ void ParameterWidget::changeFilterObject(int index)
   case Multiselect:
     {
       QTableWidget *tab = new QTableWidget(_filterGroup);
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
       QFont f = tab->font();
       f.setPointSize(f.pointSize() - 2);
       tab->setFont(f);
@@ -1468,7 +1468,8 @@ void ParameterWidget::save()
   params.append("classname", classname);
   if (_shared)
     params.append("shared", true);
-
+  if (!_x_privileges->check("AllowSharedFilterEdit"))
+    params.append("disableshare", true);
   filterSave newdlg(this);
   newdlg.set(params);
   filter_id = newdlg.exec();
@@ -2007,4 +2008,3 @@ void setupParameterWidget(QScriptEngine *engine)
 
   engine->globalObject().setProperty("ParameterWidget", widget, QScriptValue::ReadOnly | QScriptValue::Undeletable);
 }
-
