@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -41,7 +41,7 @@ class XComboBoxEditorDescrip {
     }
 
     XComboBoxEditorDescrip(int   pType,       QObject *pObj,
-                           char *pSlot, const QString &pPriv)
+                           const char *pSlot, const QString &pPriv)
     {
       type = pType;
       obj  = pObj;
@@ -49,11 +49,11 @@ class XComboBoxEditorDescrip {
       priv = pPriv;
     }
 
-    int      type;   // XComboBoxTypes + possible out-of-range
-    QString  priv;
-    QString  uiName;
-    QObject *obj;
-    char    *slot;
+    int         type;   // XComboBoxTypes + possible out-of-range
+    QString     priv;
+    QString     uiName;
+    QObject     *obj;
+    const char  *slot;
 };
 
 XComboBoxPrivate::XComboBoxPrivate(XComboBox *pParent)
@@ -61,7 +61,6 @@ XComboBoxPrivate::XComboBoxPrivate(XComboBox *pParent)
     _default(XComboBox::First),
     _editButton(0),
     _label(0),
-    _lastId(-1),
     _parent(pParent),
     _popupCounter(0),
     _mapper(0)
@@ -175,11 +174,10 @@ void XComboBox::init()
   _data = new XComboBoxPrivate(this);
 
   setAllowNull(false);
-  setMaximumWidth(200);
-
+  setSizeAdjustPolicy(AdjustToContents);
   connect(this, SIGNAL(activated(int)), this, SLOT(sHandleNewIndex(int)));
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
   QFont f = font();
   f.setPointSize(f.pointSize() - 2);
   setFont(f);
@@ -217,6 +215,7 @@ void XComboBox::init()
   insertEditor(CustomerGroups,"customerGroups","MaintainCustomerGroups");
   insertEditor(CustomerTypes,"customerTypes","MaintainCustomerTypes");
   insertEditor(EmployeeCommentTypes,"commentTypes","MaintainCommentTypes");
+  insertEditor(ExchangeRateCommentTypes,"commentTypes","MaintainCommentTypes");
   insertEditor(ExpenseCategories,"expenseCategories","MaintainCustomerTypes");
   insertEditor(FinancialLayouts,"financialLayouts","MaintainFinancialLayouts");
   insertEditor(FiscalYears,"accountingYearPeriods","MaintainAccountingPeriods");
@@ -276,6 +275,7 @@ void XComboBox::init()
   insertEditor(TaxTypes,"taxTypes","MaintainTaxTypes");
   insertEditor(Terms,"terms","MaintainTerms");
   insertEditor(TaskCommentTypes,"commentTypes","MaintainCommentTypes");
+  //insertEditor(TimeAttendanceCommentTypes,"commentTypes","MaintainCommentTypes");
   insertEditor(TodoItemCommentTypes,"commentTypes","MaintainCommentTypes");
   insertEditor(TransferOrderCommentTypes,"commentTypes","MaintainCommentTypes");
   insertEditor(TransferOrderItemCommentTypes,"commentTypes","MaintainCommentTypes");
@@ -288,6 +288,7 @@ void XComboBox::init()
   insertEditor(WarehouseCommentTypes,"commentTypes","MaintainCommentTypes");
   insertEditor(WoProjects,"projects","MaintainAllProjects");
   insertEditor(WorkCenters,"workCenters","MaintainWorkCenters");
+  insertEditor(WorkCentersActive,"workCenters","MaintainWorkCenters");
   insertEditor(WorkOrderCommentTypes,"commentTypes","MaintainCommentTypes");
 }
 
@@ -457,7 +458,7 @@ void XComboBox::setType(XComboBoxTypes pType)
       break;
 
     case UOMs:
-      setAllowNull(TRUE);
+      setAllowNull(true);
       query.exec( "SELECT uom_id, uom_name, uom_name "
                   "FROM uom "
                   "ORDER BY uom_name;" );
@@ -531,8 +532,8 @@ void XComboBox::setType(XComboBoxTypes pType)
       break;
 
     case ShipVias:
-      setAllowNull(TRUE);
-      setEditable(TRUE);
+      setAllowNull(true);
+      setEditable(true);
       query.exec( "SELECT shipvia_id, (shipvia_code || '-' || shipvia_descrip), shipvia_code "
                   "FROM shipvia "
                   "ORDER BY shipvia_code;" );
@@ -547,7 +548,7 @@ void XComboBox::setType(XComboBoxTypes pType)
     case SaleTypes:
       query.exec( "SELECT saletype_id, (saletype_code || '-' || saletype_descr), saletype_code "
                   "FROM saletype "
-                  "ORDER BY saletype_code;" );
+                  "ORDER BY saletype_default DESC, saletype_code;" );
       break;
 
     case ShippingCharges:
@@ -621,7 +622,7 @@ void XComboBox::setType(XComboBoxTypes pType)
       break;
 
     case SoProjects:
-      setAllowNull(TRUE);
+      setAllowNull(true);
       query.exec( "SELECT prj_id, (prj_number || '-' || prj_name), prj_number "
                   "FROM prj "
                   "WHERE (prj_so) "
@@ -629,7 +630,7 @@ void XComboBox::setType(XComboBoxTypes pType)
       break;
 
     case WoProjects:
-      setAllowNull(TRUE);
+      setAllowNull(true);
       query.exec( "SELECT prj_id, (prj_number || '-' || prj_name), prj_number "
                   "FROM prj "
                   "WHERE (prj_wo) "
@@ -637,7 +638,7 @@ void XComboBox::setType(XComboBoxTypes pType)
       break;
 
     case PoProjects:
-      setAllowNull(TRUE);
+      setAllowNull(true);
       query.exec( "SELECT prj_id, (prj_number || '-' || prj_name), prj_number "
                   "FROM prj "
                   "WHERE (prj_po) "
@@ -653,7 +654,7 @@ void XComboBox::setType(XComboBoxTypes pType)
     case CurrenciesNotBase:
       query.exec( "SELECT curr_id, currConcat(curr_abbr, curr_symbol), curr_abbr"
                   " FROM curr_symbol "
-                  " WHERE curr_base = FALSE "
+                  " WHERE curr_base = false "
                   "ORDER BY curr_abbr;" );
       break;
 
@@ -765,6 +766,14 @@ void XComboBox::setType(XComboBoxTypes pType)
                   "ORDER BY cmnttype_order, cmnttype_name;" );
       break;
 
+    case ExchangeRateCommentTypes:
+      query.exec( "SELECT cmnttype_id, cmnttype_name, cmnttype_name "
+                  "FROM cmnttype JOIN cmnttypesource ON (cmnttypesource_cmnttype_id=cmnttype_id)"
+                  "              JOIN source ON (source_id=cmnttypesource_source_id) "
+                  "WHERE (source_name='FX')"
+                  "ORDER BY cmnttype_order, cmnttype_name;" );
+      break;
+
     case IncidentCommentTypes:
       query.exec( "SELECT cmnttype_id, cmnttype_name, cmnttype_name "
                   "FROM cmnttype JOIN cmnttypesource ON (cmnttypesource_cmnttype_id=cmnttype_id)"
@@ -900,6 +909,14 @@ void XComboBox::setType(XComboBoxTypes pType)
                   "WHERE (source_name='TA')"
                   "ORDER BY cmnttype_order, cmnttype_name;" );
       break;
+      
+     case TimeAttendanceCommentTypes:
+      query.exec( "SELECT cmnttype_id, cmnttype_name, cmnttype_name "
+                  "FROM cmnttype JOIN cmnttypesource ON (cmnttypesource_cmnttype_id=cmnttype_id)"
+                  "              JOIN source ON (source_id=cmnttypesource_source_id) "
+                  "WHERE (source_name='TATC')"
+                  "ORDER BY cmnttype_order, cmnttype_name;" );
+      break;     
 
     case TodoItemCommentTypes:
       query.exec( "SELECT cmnttype_id, cmnttype_name, cmnttype_name "
@@ -1032,15 +1049,22 @@ void XComboBox::setType(XComboBoxTypes pType)
                   "ORDER BY wrkcnt_code;" );
       break;
 
+    case WorkCentersActive:
+      query.exec( "SELECT wrkcnt_id, (wrkcnt_code || '-' || wrkcnt_descrip), wrkcnt_code"
+                 "  FROM xtmfg.wrkcnt "
+                 " WHERE (wrkcnt_active) "
+                 "ORDER BY wrkcnt_code;" );
+      break;
+      
     case CRMAccounts:
-      setAllowNull(TRUE);
+      setAllowNull(true);
       query.exec( "SELECT crmacct_id, (crmacct_number || '-' || crmacct_name), crmacct_number"
                   "  FROM crmacct "
                   "ORDER BY crmacct_number;" );
       break;
 
     case Honorifics:
-      setAllowNull(TRUE);
+      setAllowNull(true);
       query.exec( "SELECT hnfc_id, hnfc_code, hnfc_code"
                   "  FROM hnfc "
                   "ORDER BY hnfc_code;" );
@@ -1223,8 +1247,8 @@ void XComboBox::setCode(const QString &pString)
 {
   if (DEBUG)
     qDebug("%s::setCode(%d %d %s) with _codes.count %d and _ids.count %d",
-           objectName().toAscii().data(), pString.isNull(), pString.isEmpty(),
-           pString.toAscii().data(), _data->_codes.count(), _data->_ids.count());
+           objectName().toLatin1().data(), pString.isNull(), pString.isEmpty(),
+           pString.toLatin1().data(), _data->_codes.count(), _data->_ids.count());
 
   if (pString.isEmpty())
   {
@@ -1238,12 +1262,11 @@ void XComboBox::setCode(const QString &pString)
       if (_data->_codes.at(counter) == pString)
       {
         if (DEBUG)
-          qDebug("%s::setCode(%s) found at %d with _ids.count %d & _lastId %d",
-                 objectName().toAscii().data(), pString.toAscii().data(),
-                 counter, _data->_ids.count(), _data->_lastId);
-        setCurrentIndex(counter);
+          qDebug("%s::setCode(%s) found at %d with _ids.count %d & id %d",
+                 qPrintable(objectName()), qPrintable(pString),
+                 counter, _data->_ids.count(), id());
 
-        if (_data->_ids.count() && _data->_lastId!=_data->_ids.at(counter))
+        if (_data->_ids.count() && id()!=_data->_ids.at(counter))
           setId(_data->_ids.at(counter));
 
         return;
@@ -1259,13 +1282,13 @@ void XComboBox::setCode(const QString &pString)
     setCurrentIndex(findText(pString));
     if (DEBUG)
       qDebug("%s::setCode(%s) set current item to %d using findData()",
-             objectName().toAscii().data(), pString.toAscii().data(),
+             objectName().toLatin1().data(), pString.toLatin1().data(),
              currentIndex());
     if (_data->_ids.count() > currentIndex())
       setId(_data->_ids.at(currentIndex()));
     if (DEBUG)
       qDebug("%s::setCode(%s) current item is %d after setId",
-             objectName().toAscii().data(), pString.toAscii().data(),
+             objectName().toLatin1().data(), pString.toLatin1().data(),
              currentIndex());
   }
 
@@ -1296,17 +1319,15 @@ void XComboBox::setId(int pTarget)
       {
         if (_data->_ids.at(counter) == id)
         {
-          setCurrentIndex(counter);
-
-          if(_data->_lastId!=id)
+          if(this->id()!=id)
           {
-            _data->_lastId = id;
+            setCurrentIndex(counter);
             updateMapperData();
             emit newID(pTarget);
-            emit valid(TRUE);
+            emit valid(true);
 
             if (allowNull())
-              emit notNull(TRUE);
+              emit notNull(true);
           }
 
           return;
@@ -1320,17 +1341,15 @@ void XComboBox::setId(int pTarget)
     {
       if (_data->_ids.at(counter) == pTarget)
       {
-        setCurrentIndex(counter);
-
-        if(_data->_lastId!=pTarget)
+        if(id()!=pTarget)
         {
-          _data->_lastId = pTarget;
+          setCurrentIndex(counter);
           updateMapperData();
           emit newID(pTarget);
-          emit valid(TRUE);
+          emit valid(true);
 
           if (allowNull())
-            emit notNull(TRUE);
+            emit notNull(true);
         }
 
         return;
@@ -1389,13 +1408,11 @@ void XComboBox::setNull()
 {
   if (allowNull())
   {
-    _data->_lastId = -1;
-
     setCurrentIndex(0);
     updateMapperData();
     emit newID(-1);
-    emit valid(FALSE);
-    emit notNull(FALSE);
+    emit valid(false);
+    emit notNull(false);
   }
 }
 
@@ -1456,8 +1473,8 @@ void XComboBox::populate(XSqlQuery pQuery, int pSelected)
 
   // TODO: why doesn't setId() handle the following as expected? {
   updateMapperData();
-  emit newID(_data->_lastId);
-  emit valid((_data->_lastId != -1));
+  emit newID(id());
+  emit valid(isValid());
   // } end why
 }
 
@@ -1518,7 +1535,7 @@ QString XComboBox::code() const
 {
   if (DEBUG)
     qDebug("%s::code() with currentIndex %d, allowNull %d, and _codes.count %d",
-           objectName().toAscii().data(), currentIndex(), allowNull(),
+           objectName().toLatin1().data(), currentIndex(), allowNull(),
            _data->_codes.count());
 
   QString returnValue;
@@ -1534,33 +1551,31 @@ QString XComboBox::code() const
 
   if (DEBUG)
     qDebug("%s::code() returning %s",
-           objectName().toAscii().data(), returnValue.toAscii().data());
+           objectName().toLatin1().data(), returnValue.toLatin1().data());
   return returnValue;
 }
 
 bool XComboBox::isValid() const
 {
   if ((allowNull()) && (id() == -1))
-    return FALSE;
+    return false;
   else
-    return TRUE;
+    return true;
 }
 
 void XComboBox::sHandleNewIndex(int pIndex)
 {
   if (DEBUG)
-    qDebug("%s::sHandleNewIndex(%d)",objectName().toAscii().data(), pIndex);
+    qDebug() << objectName() << "::sHandleNewIndex" << pIndex << "/" << id();
 
-  if ((pIndex >= 0) && (pIndex < _data->_ids.count()) &&
-      (_data->_ids.at(pIndex) != _data->_lastId))
+  if (pIndex >= 0 && pIndex < _data->_ids.count())
   {
-    _data->_lastId = _data->_ids.at(pIndex);
     updateMapperData();
-    emit newID(_data->_lastId);
+    emit newID(_data->_ids.at(pIndex));
 
     if (DEBUG)
-      qDebug("%s::sHandleNewIndex() emitted %d",
-             objectName().toAscii().data(), _data->_lastId);
+      qDebug() << objectName() << "::sHandleNewIndex emitted"
+               << _data->_ids.at(pIndex);
 
     if (allowNull())
     {
@@ -1570,8 +1585,7 @@ void XComboBox::sHandleNewIndex(int pIndex)
   }
 
   if (DEBUG)
-    qDebug("%s::sHandleNewIndex() returning",
-           objectName().toAscii().data());
+    qDebug() << objectName() << "::sHandleNewIndex() returning";
 }
 
 void XComboBox::mousePressEvent(QMouseEvent *event)
@@ -1581,6 +1595,15 @@ void XComboBox::mousePressEvent(QMouseEvent *event)
   QComboBox::mousePressEvent(event);
 }
 
+void XComboBox::wheelEvent(QWheelEvent *event)
+{
+  if (_x_preferences)
+    if (_x_preferences->boolean("DisableXComboBoxWheelEvent"))
+      return;
+
+  QComboBox::wheelEvent(event);
+}
+
 void XComboBox::showPopup()
 {
   QComboBox::showPopup();
@@ -1595,7 +1618,7 @@ void XComboBox::showPopup()
 QSize XComboBox::sizeHint() const
 {
   QSize s = QComboBox::sizeHint();
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
   s.setWidth(s.width() + 12);
 #endif
   return s;
@@ -1624,7 +1647,7 @@ void XComboBox::insertEditor(XComboBoxTypes type, const QString &uiName,
 }
 
 void XComboBox::insertEditor(XComboBoxTypes type, QObject *obj,
-                             char *slot, const QString &privilege)
+                             const char *slot, const QString &privilege)
 {
   _data->_editorMap.insert(type, new XComboBoxEditorDescrip(type, obj,
                                                             slot, privilege));
@@ -1678,7 +1701,7 @@ QScriptValue constructXComboBox(QScriptContext *context,
            qscriptvalue_cast<QWidget*>(context->argument(0)) &&
            context->argument(1).isString())
     cbox = new XComboBox(qscriptvalue_cast<QWidget*>(context->argument(0)),
-                         context->argument(1).toString().toAscii().data());
+                         context->argument(1).toString().toLatin1().data());
 
   else if (context->argumentCount() >= 3  &&
            context->argument(0).isBool() &&
@@ -1686,7 +1709,7 @@ QScriptValue constructXComboBox(QScriptContext *context,
   {
     cbox = new XComboBox(context->argument(0).toBool(),
                          qscriptvalue_cast<QWidget*>(context->argument(1)),
-                         context->argument(2).toString().toAscii().data());
+                         context->argument(2).toString().toLatin1().data());
   }
 
   else
@@ -1775,6 +1798,7 @@ void setupXComboBox(QScriptEngine *engine)
   widget.setProperty("VendorTypes",          QScriptValue(engine, XComboBox::VendorTypes),          QScriptValue::ReadOnly | QScriptValue::Undeletable);
   widget.setProperty("WoProjects",           QScriptValue(engine, XComboBox::WoProjects),           QScriptValue::ReadOnly | QScriptValue::Undeletable);
   widget.setProperty("WorkCenters",          QScriptValue(engine, XComboBox::WorkCenters),          QScriptValue::ReadOnly | QScriptValue::Undeletable);
+  widget.setProperty("WorkCentersActive",    QScriptValue(engine, XComboBox::WorkCentersActive),    QScriptValue::ReadOnly | QScriptValue::Undeletable);
 
   engine->globalObject().setProperty("XComboBox", widget, QScriptValue::ReadOnly | QScriptValue::Undeletable);
 }
