@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -48,7 +48,11 @@ XTableView::XTableView(QWidget *parent) :
   setContextMenuPolicy(Qt::CustomContextMenu);
   verticalHeader()->setDefaultSectionSize(20);
   horizontalHeader()->setStretchLastSection(false);
+#if QT_VERSION >= 0x050000
+  horizontalHeader()->setSectionsClickable(true);
+#else
   horizontalHeader()->setClickable(true);
+#endif
   horizontalHeader()->setContextMenuPolicy(Qt::CustomContextMenu); 
 
   setAlternatingRowColors(true);
@@ -215,7 +219,7 @@ void XTableView::select()
   setTable();
   _model->select();
   
-  emit valid(FALSE);
+  emit valid(false);
 
 }
 
@@ -273,7 +277,7 @@ void XTableView::selectionChanged(const QItemSelection & selected, const QItemSe
     emit valid(true);
   }
   else
-    emit valid(FALSE);
+    emit valid(false);
   QTableView::selectionChanged(selected, deselected);
 }
 
@@ -339,8 +343,11 @@ void XTableView::setColumn(const QString &label, int width, int alignment, bool
       break;
     }
   }
-
+#if QT_VERSION >= 0x050000
+  horizontalHeader()->setSectionResizeMode(colnum, QHeaderView::Interactive);
+#else
   horizontalHeader()->setResizeMode(colnum, QHeaderView::Interactive);
+#endif
 
   if (! cp->fromSettings)
     setColumnVisible(colnum, visible);
@@ -425,9 +432,14 @@ void XTableView::setForegroundColor(int row, int col, QString color)
   _model->setData(_model->index(row,col), namedColor(color), Qt::ForegroundRole);
 }
 
-void XTableView::setModel(XSqlTableModel * model)
+void XTableView::setModel(QAbstractItemModel * model)
 {
-  QTableView::setModel(model);
+  XSqlTableModel *matchType = qobject_cast<XSqlTableModel *>(model);
+
+  if(matchType == 0)
+    return;
+
+  QTableView::setModel(matchType);
 
   for (int i = 0; i < QTableView::model()->columnCount(); ++i)
   {
@@ -529,7 +541,7 @@ void XTableView::setModel(XSqlTableModel * model)
       }
     }
   }
-  emit newModel(model);
+  emit newModel(matchType);
 }
 
 void XTableView::setRowForegroundColor(int row, QString color)
@@ -548,7 +560,8 @@ void XTableView::setTable()
     QString tablename=_tableName;
     if (!_schemaName.isEmpty())
       tablename = _schemaName + "." + tablename;
-    _model->setTable(tablename,_keyColumns);
+    _model->setTable(tablename);
+    _model->setKeys(_keyColumns);
     
     setModel(_model);
     setItemDelegate(new XItemDelegate(this));
@@ -651,7 +664,7 @@ void XTableView::sShowMenu(const QPoint &pntThis)
     _menu->clear();
     emit populateMenu(_menu, item);
 
-    bool disableExport = FALSE;
+    bool disableExport = false;
     if(_x_preferences)
       disableExport = (_x_preferences->value("DisableExportContents")=="t");
     if(!disableExport)
