@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -48,7 +48,11 @@ XTreeView::XTreeView(QWidget *parent) :
 
   setContextMenuPolicy(Qt::CustomContextMenu);
   header()->setStretchLastSection(false);
+#if QT_VERSION >= 0x050000
+  header()->setSectionsClickable(true);
+#else
   header()->setClickable(true);
+#endif
   header()->setContextMenuPolicy(Qt::CustomContextMenu);
 
   if (_x_preferences)
@@ -223,7 +227,7 @@ void XTreeView::select()
   setTable();
   _model->select();
   
-  emit valid(FALSE);
+  emit valid(false);
 
 }
 
@@ -281,7 +285,7 @@ void XTreeView::selectionChanged(const QItemSelection & selected, const QItemSel
     emit valid(true);
   }
   else
-    emit valid(FALSE);
+    emit valid(false);
   QTreeView::selectionChanged(selected, deselected);
 }
 
@@ -347,8 +351,11 @@ void XTreeView::setColumn(const QString &label, int width, int alignment, bool v
       break;
     }
   }
-
+#if QT_VERSION >= 0x050000
+  header()->setSectionResizeMode(colnum, QHeaderView::Interactive);
+#else
   header()->setResizeMode(colnum, QHeaderView::Interactive);
+#endif
 
   if (! cp->fromSettings)
     setColumnVisible(colnum, visible);
@@ -436,9 +443,14 @@ void XTreeView::setForegroundColor(int row, int col, QString color)
   _model->setData(_model->index(row,col), namedColor(color), Qt::ForegroundRole);
 }
 
-void XTreeView::setModel(XSqlTableModel * model)
+void XTreeView::setModel(QAbstractItemModel * model)
 {
-  QTreeView::setModel(model);
+  XSqlTableModel *matchType = qobject_cast<XSqlTableModel *>(model);
+
+  if(matchType == 0)
+    return;
+
+  QTreeView::setModel(matchType);
 
   for (int i = 0; i < QTreeView::model()->columnCount(); ++i)
   {
@@ -540,7 +552,7 @@ void XTreeView::setModel(XSqlTableModel * model)
       }
     }
   }
-  emit newModel(model);
+  emit newModel(matchType);
 }
 
 void XTreeView::setRowForegroundColor(int row, QString color)
@@ -559,7 +571,8 @@ void XTreeView::setTable()
     QString tablename=_tableName;
     if (!_schemaName.isEmpty())
       tablename = _schemaName + "." + tablename;
-    _model->setTable(tablename,_keyColumns);
+    _model->setTable(tablename);
+    _model->setKeys(_keyColumns);
     
     setModel(_model);
   }
@@ -660,7 +673,7 @@ void XTreeView::sShowMenu(const QPoint &pntThis)
     _menu->clear();
     emit populateMenu(_menu, item);
 
-    bool disableExport = FALSE;
+    bool disableExport = false;
     if(_x_preferences)
       disableExport = (_x_preferences->value("DisableExportContents")=="t");
     if(!disableExport)
