@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2016 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -127,7 +127,11 @@ XTreeWidget::XTreeWidget(QWidget *pParent) :
   setContextMenuPolicy(Qt::CustomContextMenu);
   setSelectionBehavior(QAbstractItemView::SelectRows);
   header()->setStretchLastSection(false);
+#if QT_VERSION >= 0x050000
+  header()->setSectionsClickable(true);
+#else
   header()->setClickable(true);
+#endif
   if (_x_preferences)
     setAlternatingRowColors(!_x_preferences->boolean("NoAlternatingRowColors"));
 
@@ -143,12 +147,12 @@ XTreeWidget::XTreeWidget(QWidget *pParent) :
   connect(this,           SIGNAL(itemClicked(QTreeWidgetItem*, int)),                       SLOT(sItemClicked(QTreeWidgetItem*, int)));
   connect(&_workingTimer, SIGNAL(timeout()), this, SLOT(populateWorker()));
 
-  emit valid(FALSE);
+  emit valid(false);
   setColumnCount(0);
 
   header()->setContextMenuPolicy(Qt::CustomContextMenu);
 
-#ifdef Q_WS_MAC
+#ifdef Q_OS_MAC
   QFont f = font();
   f.setPointSize(f.pointSize() - 2);
   setFont(f);
@@ -208,7 +212,7 @@ XTreeWidget::~XTreeWidget()
                     (header()->sortIndicatorOrder() == Qt::AscendingOrder ? "ASC" : "DESC" );
     else
       savedString = "-1,ASC";
-      xtsettingsSetValue(_settingsName + "/sortOrder", savedString);
+    xtsettingsSetValue(_settingsName + "/sortOrder", savedString);
   }
 
   for (int i = 0; i < _roles.size(); i++)
@@ -900,11 +904,19 @@ void XTreeWidget::addColumn(const QString &pString, int pWidth, int pAlignment,
   if (pWidth >= 0)
   {
     header()->resizeSection(column, pWidth);
+    #if QT_VERSION >= 0x050000
+    header()->setSectionResizeMode(column, QHeaderView::Interactive);
+    #else
     header()->setResizeMode(column, QHeaderView::Interactive);
+    #endif
   }
   else
   {
+    #if QT_VERSION >= 0x050000
+    header()->setSectionResizeMode(column, QHeaderView::Interactive);
+    #else
     header()->setResizeMode(column, QHeaderView::Interactive);
+    #endif
     _stretch.append(column);
   }
   bool forgetCache = _forgetful;
@@ -1419,7 +1431,7 @@ void XTreeWidget::clear()
     delete _subtotals;
     _subtotals = 0;
   }
-  emit valid(FALSE);
+  emit valid(false);
   _savedId = false; // was -1;
 
   QTreeWidget::clear();
@@ -1480,7 +1492,7 @@ void XTreeWidget::sShowMenu(const QPoint &pntThis)
       emit  populateMenu(_menu, item, logicalColumn);
     }
 
-    bool disableExport = FALSE;
+    bool disableExport = false;
     if (_x_preferences)
       disableExport = (_x_preferences->value("DisableExportContents")=="t");
     if (!disableExport)
@@ -1490,6 +1502,7 @@ void XTreeWidget::sShowMenu(const QPoint &pntThis)
       copyMenu->addAction(tr("All"),  this, SLOT(sCopyVisibleToClipboard()));
       copyMenu->addAction(tr("Row"),  this, SLOT(sCopyRowToClipboard()));
       copyMenu->addAction(tr("Cell"),  this, SLOT(sCopyCellToClipboard()));
+      copyMenu->addAction(tr("Column"),this, SLOT(sCopyColumnToClipboard()));
       _menu->addSeparator();
       _menu->addAction(tr("Export As..."),  this, SLOT(sExport()));
     }
@@ -1552,10 +1565,12 @@ void XTreeWidget::sExport()
   QString   path = xtsettingsValue(_settingsName + "/exportPath").toString();
   QString selectedFilter;
   QFileInfo fi(QFileDialog::getSaveFileName(this, tr("Export Save Filename"), path,
-                                            tr("Text CSV (*.csv);;Text (*.txt);;ODF Text Document (*.odt);;HTML Document (*.html)"), &selectedFilter));
+                                            tr("Text CSV (*.csv);;Text VCF (*.vcf);;Text (*.txt);;ODF Text Document (*.odt);;HTML Document (*.html)"), &selectedFilter));
   QString defaultSuffix;
   if(selectedFilter.contains("csv"))
     defaultSuffix = ".csv";
+  else if(selectedFilter.contains("vcf"))
+    defaultSuffix = ".vcf";
   else if(selectedFilter.contains("odt"))
     defaultSuffix = ".odt";
   else if(selectedFilter.contains("html"))
@@ -1582,6 +1597,11 @@ void XTreeWidget::sExport()
       doc->setPlainText(toCsv());
       writer.setFormat("plaintext");
     }
+    else if (fi.suffix() == "vcf")
+    {
+      doc->setPlainText(toVcf());
+      writer.setFormat("plaintext");
+    }
     else if (fi.suffix() == "odt")
     {
       doc->setHtml(toHtml());
@@ -1955,6 +1975,38 @@ QVariant XTreeWidgetItem::rawValue(const QString pName)
     return data(colIdx, Xt::RawRole);
 }
 
+void XTreeWidgetItem::setNumber(int pColumn, const QVariant pValue, const QString pNumericRole)
+{
+  setData(pColumn, Xt::RawRole, pValue);
+  if (!setNumericRole(pColumn, pNumericRole))
+  {
+    // pValue is not a valid number
+    setText(pColumn, pValue);
+  }
+}
+
+void XTreeWidgetItem::setDate(int pColumn, const QVariant pDate)
+{
+  setData(pColumn, Xt::RawRole, pDate);
+  setData(pColumn, Qt::DisplayRole, formatDate(pDate.toDate()));
+}
+
+bool XTreeWidgetItem::setNumericRole(int pColIdx, const QString pRole)
+{
+  bool canConvert = false;
+
+  if (pColIdx >= 0)
+  {
+    double value = data(pColIdx, Xt::RawRole).toDouble(&canConvert);
+
+    if (canConvert)
+    {
+      setData(pColIdx, Qt::DisplayRole, QLocale().toString(value, 'f', decimalPlaces(pRole)));
+    }
+  }
+  return canConvert;
+}
+
 /* Calculate the total for a particular XTreeWidgetItem, including any children.
    pcol is the column for which we want the total.
    prole is the value of xttotalrole for which we want the total.
@@ -2259,6 +2311,28 @@ void XTreeWidget::sCopyVisibleToClipboard()
   clipboard->setMimeData(mime);
 }
 
+void XTreeWidget::sCopyColumnToClipboard()
+{
+  QTextEdit       text;
+  QMimeData       *mime      = new QMimeData();
+  QClipboard      *clipboard = QApplication::clipboard();
+  QString opText = "";
+  XTreeWidgetItem *item = topLevelItem(0);
+  if (item)
+  {
+    for (QModelIndex idx = indexFromItem(item); idx.isValid(); idx = indexBelow(idx))
+    {
+      item = (XTreeWidgetItem *)itemFromIndex(idx);
+      if (item)
+        opText = opText + item->text(currentColumn()) + "\t\r\n";
+    }
+  }
+  text.setText(opText);
+  mime->setText(text.toPlainText());
+  mime->setHtml(text.toHtml());
+  clipboard->setMimeData(mime);
+}
+
 void XTreeWidget::sSearch(const QString &pTarget)
 {
   clearSelection();
@@ -2294,8 +2368,7 @@ QString XTreeWidget::toTxt() const
   XTreeWidgetItem *item = topLevelItem(0);
   if (item)
   {
-    QModelIndex idx = indexFromItem(item);
-    while (idx.isValid())
+    for (QModelIndex idx = indexFromItem(item); idx.isValid(); idx = indexBelow(idx))
     {
       item = (XTreeWidgetItem *)itemFromIndex(idx);
       if (item)
@@ -2308,7 +2381,6 @@ QString XTreeWidget::toTxt() const
         }
       }
       opText = opText + line + "\r\n";
-      idx    = indexBelow(idx);
     }
   }
   return opText;
@@ -2337,8 +2409,7 @@ QString XTreeWidget::toCsv() const
   XTreeWidgetItem *item = topLevelItem(0);
   if (item)
   {
-    QModelIndex idx = indexFromItem(item);
-    while (idx.isValid())
+    for (QModelIndex idx = indexFromItem(item); idx.isValid(); idx = indexBelow(idx))
     {
       colcount = 0;
       item     = (XTreeWidgetItem *)itemFromIndex(idx);
@@ -2361,12 +2432,112 @@ QString XTreeWidget::toCsv() const
         }
       }
       opText = opText + line + "\r\n";
-      idx    = indexBelow(idx);
     }
   }
   return opText;
 }
 
+QString XTreeWidget::toVcf() const
+{
+  XSqlQuery qry;
+  qry.prepare("SELECT * FROM cntct WHERE (cntct_id=:cntct_id);");
+  qry.bindValue(":cntct_id", this->selectedItems().at(0)->id());
+  qry.exec();
+  if (qry.first()) {
+    QString name;
+    QString fullName;
+    QString first = qry.value("cntct_first_name").toString();
+    QString middle = qry.value("cntct_middle").toString();
+    QString last = qry.value("cntct_last_name").toString();
+    if (!last.isEmpty()) {
+      name = last;
+      fullName = last;
+    }
+    if (!middle.isEmpty()) {
+      name = name + ";" + middle;
+      fullName = middle + " " + fullName;
+    }
+    if (!first.isEmpty()) {
+      name = name + ";" + first;
+      fullName = first + " " + fullName;
+    }
+    QString begin = "VCARD";
+    QString version = "3.0";
+    QString org = "";
+    QString title = qry.value("cntct_title").toString();
+    QString photo = "";
+    QString phoneWork = qry.value("cntct_phone").toString();
+    QString phoneHome = qry.value("cntct_phone2").toString();
+    QString addressId = qry.value("cntct_addr_id").toString();
+    XSqlQuery qry2;
+    qry2.prepare("SELECT * FROM addr WHERE (addr_id=:addr_id);");
+    qry2.bindValue(":addr_id", addressId);
+    qry2.exec();
+    QStringList address;
+    QString addressWork= "";
+    QString labelWork = "";
+    if (qry2.first()) {
+      /*
+      * this is tricky, because sometimes (in the test database at least)
+      * our addr_line1 is the company name, and sometimes it's really the
+      * first line of the actual address.  For the former, we can use
+      * the addr_line1 as the organization name.
+      */
+      if(qry2.value("addr_line1").toString().at(0).isDigit())
+        address.append(qry2.value("addr_line1").toString());
+      else
+        org = qry2.value("addr_line1").toString();
+      address.append(qry2.value("addr_line2").toString());
+      address.append(qry2.value("addr_line3").toString());
+      address.append(qry2.value("addr_city").toString());
+      address.append(qry2.value("addr_state").toString());
+      address.append(qry2.value("addr_postalcode").toString());
+      address.append(qry2.value("addr_country").toString());
+    }
+    //for address, set address with semicolon delimiters
+    //for label, set address with ESCAPED newline delimiters
+    for (int i = 0; i < address.length(); i++) {
+      if(!address.at(i).isEmpty()) {
+        addressWork = addressWork + address.at(i) + ";";
+        labelWork = labelWork + address.at(i) + "\\n";
+      }
+    }
+    QString addressHome = "";
+    QString labelHome = "";
+    QString email = qry.value("cntct_email").toString();
+    QDate date = QDate::currentDate();
+    QString revision = date.toString(Qt::ISODate);
+    QString end = "VCARD";
+
+    QString stringToSave;
+
+    stringToSave.append("BEGIN:" + begin + "\n");
+    stringToSave.append("VERSION:" + version + "\n");
+    stringToSave.append("N:" + name + "\n");
+    stringToSave.append("FN:" + fullName + "\n");
+    if (!org.isEmpty())
+      stringToSave.append("ORG:" + org + "\n");
+    if (!title.isEmpty())
+      stringToSave.append("TITLE:" + title + "\n");
+    if (!phoneWork.isEmpty())
+      stringToSave.append("TEL;TYPE=WORK,VOICE:" + phoneWork + "\n");
+    if (!phoneHome.isEmpty())
+      stringToSave.append("TEL;TYPE=HOME,VOICE:" + phoneHome + "\n");
+    if (!addressWork.isEmpty())
+      stringToSave.append("ADR;TYPE=WORK:;;" + addressWork + "\n");
+    if (!labelWork.isEmpty())
+      stringToSave.append("LABEL;TYPE=WORK:;;" + labelWork + "\n");
+    if (!email.isEmpty())
+      stringToSave.append("EMAIL;TYPE=PREF,INTERNET:" + email + "\n");
+    stringToSave.append("REV:" + revision + "\n");
+    stringToSave.append("END:" + end + "\n");
+
+    return stringToSave;
+  }
+  else
+    return "failed to select contact for export";
+}
+
 QString XTreeWidget::toHtml() const
 {
   QTextDocument     *doc     = new QTextDocument();
@@ -2377,6 +2548,17 @@ QString XTreeWidget::toHtml() const
   QString font;
   int     colcnt = 0;
   int     rowcnt = 0;
+  int     row = 0;
+  double  limit = 0;
+  qlonglong maxDataCount = 0;
+  qlonglong dataCount = 0;
+
+  if (_x_preferences)
+  {
+    limit = _x_preferences->value("XTreeWidgetDataLimit").toDouble();
+    if (limit > 0)
+      maxDataCount = (qlonglong)(limit * 1e9);
+  }
 
   tableFormat.setHeaderRowCount(1);
 
@@ -2388,13 +2570,11 @@ QString XTreeWidget::toHtml() const
   XTreeWidgetItem *item = topLevelItem(0);
   if (item)
   {
-    QModelIndex idx = indexFromItem(item);
-    while (idx.isValid())
+    for (QModelIndex idx = indexFromItem(item); idx.isValid(); idx = indexBelow(idx))
     {
       item = (XTreeWidgetItem *)itemFromIndex(idx);
       if (item)
         rowcnt++;
-      idx = indexBelow(idx);
     }
   }
 
@@ -2410,14 +2590,15 @@ QString XTreeWidget::toHtml() const
       cell.setFormat(format);
       cursor->insertText(header->text(counter));
       cursor->movePosition(QTextCursor::NextCell);
+
+      dataCount += (qlonglong)(header->text(counter).size());
     }
   }
 
   item = topLevelItem(0);
   if (item)
   {
-    QModelIndex idx = indexFromItem(item);
-    while (idx.isValid())
+    for (QModelIndex idx = indexFromItem(item); idx.isValid() && (maxDataCount <= 0 || dataCount < maxDataCount); idx = indexBelow(idx), row++)
     {
       item = (XTreeWidgetItem *)itemFromIndex(idx);
       if (item)
@@ -2443,10 +2624,17 @@ QString XTreeWidget::toHtml() const
             cell.setFormat(format);
             cursor->insertText(item->text(counter));
             cursor->movePosition(QTextCursor::NextCell);
+
+            dataCount += (qlonglong)(item->text(counter).size());
           }
         }
       }
-      idx = indexBelow(idx);
+    }
+
+    if (maxDataCount > 0 && dataCount > maxDataCount)
+    {
+      QString overflowMsg = tr("Maximum data limit was encountered.  Only %1 of %2 rows could be processed.");
+      QMessageBox::warning(NULL, tr("Data Limit Reached"), overflowMsg.arg(row).arg(rowcnt));
     }
   }
   return doc->toHtml();
