@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -134,6 +134,10 @@ class XTUPLEWIDGETS_EXPORT XTreeWidgetItem : public QObject, public QTreeWidgetI
     Q_INVOKABLE virtual QVariant        rawValue(const QString colname);
     Q_INVOKABLE virtual int             id(const QString);
 
+    Q_INVOKABLE virtual void            setDate(int pColIdx, const QVariant pDate);
+    Q_INVOKABLE virtual void            setNumber(int pColIdx, const QVariant pValue, const QString pRole);
+    Q_INVOKABLE virtual bool            setNumericRole(int pColIdx, const QString pRole);
+
     virtual bool operator               <(const XTreeWidgetItem &other) const;
     virtual bool operator               ==(const XTreeWidgetItem &other) const;
 
@@ -175,10 +179,10 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
     XTreeWidget(QWidget *);
     ~XTreeWidget();
 
-    Q_INVOKABLE void  populate(XSqlQuery, bool = FALSE, PopulateStyle = Replace);
-    Q_INVOKABLE void  populate(XSqlQuery, int, bool = FALSE, PopulateStyle = Replace);
-    void    populate(const QString&, bool = FALSE);
-    void    populate(const QString&, int, bool = FALSE);
+    Q_INVOKABLE void  populate(XSqlQuery, bool = false, PopulateStyle = Replace);
+    Q_INVOKABLE void  populate(XSqlQuery, int, bool = false, PopulateStyle = Replace);
+    void    populate(const QString&, bool = false);
+    void    populate(const QString&, int, bool = false);
 
     QString dragString() const;
     void    setDragString(QString);
@@ -206,6 +210,7 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
 
     Q_INVOKABLE QString toTxt() const;
     Q_INVOKABLE QString toCsv() const;
+    Q_INVOKABLE QString toVcf() const;
     Q_INVOKABLE QString toHtml() const;
 
     // just for scripting exposure:
@@ -260,6 +265,7 @@ class XTUPLEWIDGETS_EXPORT XTreeWidget : public QTreeWidget
     void  sCopyVisibleToClipboard();
     void  sCopyRowToClipboard();
     void  sCopyCellToClipboard();
+    void  sCopyColumnToClipboard();
     void  sSearch(const QString&);
 
   signals:
