@@ -1,7 +1,7 @@
 /*
  * This file is part of the xTuple ERP: PostBooks Edition, a free and
  * open source Enterprise Resource Planning software suite,
- * Copyright (c) 1999-2012 by OpenMFG LLC, d/b/a xTuple.
+ * Copyright (c) 1999-2014 by OpenMFG LLC, d/b/a xTuple.
  * It is licensed to you under the Common Public Attribution License
  * version 1.0, the full text of which (including xTuple-specific Exhibits)
  * is available at www.xtuple.com/CPAL.  By using this software, you agree
@@ -18,6 +18,10 @@
 class xTuplePlugin : public QObject, public QDesignerCustomWidgetCollectionInterface
 {
   Q_OBJECT
+
+#if QT_VERSION > 0x050000
+  Q_PLUGIN_METADATA(IID "com.xtuple.plugins.xTuplePlugin")
+#endif
   Q_INTERFACES(QDesignerCustomWidgetCollectionInterface)
 
   public:
